package heasarc.w3browse.plot;

/**
 *           W3Browse Plotting Package                          **
 *                                                              **
 * SOURCE: Histogram.java                                       **
 *                                                              **
 * @author Saima Zobair (RITSS)   saima@lheamail.gsfc.nasa.gov  **
 *                                                              **
 *****************************************************************/

import java.awt.*;
import java.awt.event.*;
import java.util.*;
//import heasarc.w3browse.Column;

/** This class creates histogram plots.
 */
public class Histogram {
  /////////////////////////////////////////////////////////////////
  ///                   class variables                         ///
  /** Number of bins in histogram */
  public int bins = 10;
  /** Column to plot */
  protected Column col;
  /** Histogram ranges */
  protected double range[];
  /** Histogram frequency */
  protected long frequency[];
  /** x and y coordinates for histogram plot */
  protected double[] yval,xval;
  /** bin width*/
  public double bin_width = 0;

  /////////////////////////////////////////////////////////////////
  ///                      constructor                          ///
  /** Constructor for Histogram */
  public Histogram(Column col,int nrows,int bin) {
    this.col = col;
    if (bin > 0)
      this.bins = bin;
    this.range = new double[this.bins + 1];
    setHistRanges();
    setFrequency(nrows);
    getHistPoints();
  }

  /** Constructor creates empty Histogram object */
  public Histogram() {
  }
  /////////////////////////////////////////////////////////////////
  ///                      public methods                       ///
  /** returns xaxis array */
  public double[] getX(){
    return(this.xval);
  }
  /** returns yaxis array */
  public double[] getY(){
    return(this.yval);
  }
  /////////////////////////////////////////////////////////////////
  ///                      protected methods                    ///
  protected void setHistRanges() {

    double min = col.getMin();
    double max = col.getMax();
    bin_width = (max - min)/bins;

    for(int i=0;i<=bins;i++) {
      range[i] = min + (i*bin_width);
    }
    range[bins] = range[bins] + 0.01;
  }

  /** for each range, counts the number of data points
   */
  protected void setFrequency(int nrows) {
    frequency = new long[bins];

    double data[] = col.getData();

    for (int i=0;i<this.bins;i++) {
      frequency[i] = 0;
      for(int j=0;j<nrows;j++){
         if(Double.isNaN(data[j]))
           continue;
         if(data[j] >= range[i] && (data[j] < range[i+1]))
           frequency[i]++;
      }
    }
  } //setFrequency()

  /** gets histogram points
   */
  protected void getHistPoints(){
    xval = new double[bins];
    yval = new double[bins];

    for(int i=0; i< bins;i++) {
      yval[i] = (double) frequency[i];
      xval[i] = range[i];

      //if (isXLog) {
         //double r =range[i];
         //if (Double.isNaN(r) || r <= 0) {
	      //xval[i] = Double.NaN;
          ////} else {
	      //xval[i] = Math.log(r)/Math.log(10);
          //}
      //} else  {
      //}
      
      //System.out.println(" hist " + xval[i]+", "+ yval[i]);
    }
    return;
  }
}

