package heasarc.w3browse.plot;

/**
 *           W3Browse Plotting Package                          **
 *                                                              **
 * SOURCE: 
 *                                                              **
 *                                                              **
 *****************************************************************/

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.applet.*;
import heasarc.w3browse.*;
import ptolemy.plot.*;
import ptolemy.gui.*;
import ptolemy.util.*;
import javax.swing.*;
import flanagan.math.Fmath;


/** CardPanel class creates the panel which switches between plot
 *  options and the actual plot
 */
public class PlotPane extends JPanel  implements ActionListener {

  /////////////////////////////////////////////////////////////////
  ///                   class variables                         ///
     JPanel plotP;
  /** The Plot object from ptolemy package */
  public Plot myPlot;

  /** Error Dialog */
  public  ErrorDialog errDialog;

  private JLabel     ptypeLabel; 
  private boolean    isSample;
  private boolean    isApplet;

  private boolean    isHistogram = false;
  private boolean    isCumulative = false;
  private boolean    isAitoff = false;
  private boolean    isScatter = false;
  private boolean    isLine = false;
  private Column     scol=null;
  private double []  sdata;
  private String     old_criteria;
  private String []  criteriaList;
  private String     xaxis;

  /** max and min values for entire plot */
  private double     ymin,ymax;   
  private double     xmin,xmax;

  private java.util.List       colNames = null;
  private java.util.HashMap    yInRange = null;
  private java.util.HashMap    yaxisdata = null;
  private java.util.HashMap    criteriaRange;
  private int        dataset=0;
  private PlotValues pValues;
  /** Options selected for plotting */
  private PlotSpecs myplotspec;
  private AppletContext appletContext;


  /** constructor to create CardPanel */
  public PlotPane (boolean isApplet) { 

     this.isApplet = isApplet;

     setLayout(new BorderLayout());
     setBackground(Color.white);
     plotP=new JPanel();

     if (myPlot == null) {
        myPlot = newPlot();
     }
     plotP.add(myPlot);

     //--- Plot area
     JPanel valuesP = new JPanel();

     //--- Add a message
     ptypeLabel = new JLabel("");
     ptypeLabel.setText("Press the Plot Button on the Plot Options Panel");
     valuesP.add(ptypeLabel);
     add(valuesP,BorderLayout.NORTH);

     //--- Add a blank plot
     add(plotP,BorderLayout.CENTER);

     //--- Add buttons to Display and Save values
     JPanel saveValuesP = new JPanel();
     JButton saveValuesB = new JButton("Display/Save Plot Values");
     saveValuesP.add(saveValuesB);
     saveValuesB.addActionListener(this);
     add(saveValuesP,BorderLayout.SOUTH);

     errDialog = new ErrorDialog(isApplet);
     old_criteria = new String();

  }

  /** Display plot values on a new web page                */
  public void actionPerformed(java.awt.event.ActionEvent e) {
   if (e.getActionCommand() == "Display/Save Plot Values") {

      try {
       pValues.displayValuesHTML();
      } catch(Exception ee) {
        errDialog.display("Error displaying plot values::" +ee.getMessage());
      }

    }
  }

    /** Creates a new Plot object for the applet */
    public Plot newPlot() {
        return new Plot();
    }

    /** Prepares Plot.  */
    private void preparePlot() {
        myPlot.resetAxes();
        myPlot.clear(true);
        myPlot.setMarksStyle("various");
        myPlot.setButtons(true);
        myPlot.setBackground(new Color(210,200,200));
        myPlot.setForeground(Color.black);
        myPlot.setXLog(false);
        myPlot.setYLog(false);
        //myPlot.repaint();
    }

    public void plot (PlotSpecs pSpecs ) {
         this.plotStart(pSpecs);
    }
  
    /** Starts a thread that plots selected parameters
     */
    public void plotStart(PlotSpecs plotspecs) {
        this.myplotspec = plotspecs;
        ptypeLabel.setText("");
        Container cont = this.getParent()  ;
	while (!(cont instanceof JTabbedPane)) { 
           cont = cont.getParent();  
        }
        ((JTabbedPane) cont).setSelectedComponent(this) ;
        this.plot();
    }


    //** Generate a user requested plot */
    public void plot() {

        preparePlot();
        dataset = 0;

        boolean connect = false;
        isSample = false;

        //get the selected X and Y axes
        xaxis = myplotspec.xaxis;
        String yaxis = myplotspec.yaxis;

        String yaxis2=null;
        if (myplotspec.isYOption2) {yaxis2 = myplotspec.yaxis2;}

        String yaxis3=null;
        if (myplotspec.isYOption3) {yaxis3 = myplotspec.yaxis3;}

        String ptype = myplotspec.plottype;
        HeasarcQuery query = myplotspec.hquery;

        setPlottypes(ptype); 

        // sampling is only done for scatter plots
        if (isScatter || isAitoff|| isLine) {
	    isSample = myplotspec.isSample; 
            if (myplotspec.showErrorBars) {
               //---- get all column names
               try {
                  String [] names = query.getNames(true);
                  colNames = new ArrayList ();
                  for (int i=0; i<names.length; i++) {
                     colNames.add(names[i]);
                  }
               } catch (Exception e) {
                   errDialog.display("Unable to get column names from query. " +
			"Please try again.");
               } 
            }
	}
	
        int nrows = 0;
        try {
 
            nrows = query.getNrow();
             
            if (nrows < 1) {
                errDialog.display("Database returns no query results. " +
			"Please modify the query.");
            }
        } catch (Exception e) {
            errDialog.display("Unable to get number of rows from query. " +
			"Please try again.");
        } 

        Column [] columns = null;
        try {
            columns = query.getNumericColumns();
        } catch (Exception e) {
            errDialog.display("Unable to get numeric columns from query. " +
		"Please try again.");


        } 

        Column xcol   = null;
	Column ycol   = null;
        Column ycol2  = null;
        Column ycol3  = null;
        Column error  = null;
        Column error2 = null;
        Column error3 = null;
	
        if (!isAitoff) { 
            try {
                xcol = query.getColumn(xaxis);  // get xaxis column
            } catch (Exception e) {
                errDialog.display("Unable to get column " + xaxis + 
		" from query. Please try again.");
            }
	    if (!isHistogram && !isCumulative) {
                try {
                    ycol = query.getColumn(yaxis);  // get yaxis column
                    //--- if error bars requested get column after checking
                    //---  if y field error bars exist 
                    if (myplotspec.showErrorBars) {
                       if (colNames.contains(yaxis + "_error")) {
                          error =query.getColumn(yaxis + "_error"); //get error
                       }
                    }
                } catch (Exception e) {
                    errDialog.display("Unable to get column " + yaxis + 
			" from query. Please try again.");
                }
                if (myplotspec.isYOption2) {
                   try {
                       ycol2 = query.getColumn(yaxis2);  // get yaxis column
                       //--- if error bars requested get column after checking
                       //---  if y field error bars exist 
                       if (myplotspec.showErrorBars) {
                          if (colNames.contains(yaxis2 + "_error")) {
			     // get error
                             error2 =query.getColumn(yaxis2 + "_error");  
                          }
                       }
                   } catch (Exception e) {
                       errDialog.display("Unable to get column " + yaxis2 + 
			   " from query. Please try again.");
                   }
                }
                if (myplotspec.isYOption3) {
                   try {
                       ycol3 = query.getColumn(yaxis3);  // get yaxis column
                       //--- if error bars requested get column after checking
                       //---  if y field error bars exist 
                       if (myplotspec.showErrorBars) {
                          if (colNames.contains(yaxis3 + "_error")) {
                             error3 =query.getColumn(yaxis3 + "_error");  
                          }
                       }
                   } catch (Exception e) {
                       errDialog.display("Unable to get column " + yaxis3 + 
			   " from query. Please try again.");
                   }
                }
	    }
        }

        //--- holds x and y axes data points
        double[] xdata  = null;
	double[] ydata  = null;
	double[] ydata2 = null;
	double[] ydata3 = null;

        //--- holds error data for y axes data points
	double[] errordata = null;
	double[] errordata2 = null;
	double[] errordata3 = null;

        //--- y axes data ranges
        double   ydatamin=myplotspec.ymin;
        double   ydatamax=myplotspec.ymax;
        double   y2datamin=Double.NaN; 
        double   y2datamax=Double.NaN; 
        double   y3datamin=Double.NaN; 
        double   y3datamax=Double.NaN; 

        if (isAitoff) {
	    
            Aitoff aitoff  = new Aitoff(myPlot);
            isAitoff = true;
            dataset = 1;
	    
            try { 
                if (myplotspec.ra_name != null && myplotspec.dec_name != null) { 
                    aitoff.convertData(myplotspec.ra_name,
			myplotspec.dec_name, query, nrows);
                } else {
		    throw new Exception("Table does not contain positional " +
			"columns");
		}
            } catch (Exception e) {
                errDialog.display("PlotPane::Aitoff error:" +e.getMessage());
            }
	    
            //--- ignore any other parameters selected
            xcol = aitoff.xcol;
            ycol = aitoff.ycol;
            xaxis = "L";
            yaxis = "B";
        }

        if (isScatter) {
            if (isLine) { connect=true;} 
	
            //--- save min and max values for later filtering and 
            //--- set new plot range
            if (myplotspec.isYOption2) {
               y2datamin=myplotspec.y2min;
               y2datamax=myplotspec.y2max;
            }
            if (myplotspec.isYOption3) {
               y3datamin=myplotspec.y3min;
               y3datamax=myplotspec.y3max;
            }
            if ( !myplotspec.isxlog && !myplotspec.isylog) {
                
                setRange(xcol,ycol);
                if (myplotspec.isYOption2) {
                   setNewYRange (y2datamin,y2datamax);
                }

                if (myplotspec.isYOption3) {
                   setNewYRange (y3datamin,y3datamax);
                }
            }
	
            if (myplotspec.isxlog) {
                setNewXRange(xcol);
                myPlot.setXLog(true);
	   
            } else {
                myPlot.setXLog(false);
            }
	
	
            if (myplotspec.isylog) {
                myPlot.setYLog(true);
	   
            } else {
                myPlot.setYLog(false);
            }
	
         }

         if (isAitoff || isScatter) {
	
             if (isSample) {
		 
		 String mysample = myplotspec.sample_param;
		 String criteria = myplotspec.criteria; 
		 
                 try {
                    //--- sample data
                    scol=query.getColumn(mysample);
                    Sampler sampler=new Sampler(scol, criteria);
                    sdata = sampler.sample();

                    //--- store sampling values to display as plot legend
                    criteriaList=sampler.getCriteria();
                    criteriaRange= new HashMap();
                 } catch (Exception e) {
                     errDialog.display("Unable to sample on "+mysample);
                 }
		 
             } else {
                 sdata = null; 
             }
         }

         if (isHistogram) {
             int bin = myplotspec.bin;

             // instantiate histogram and set myPlot for bar graphs
             Histogram histogram=null;
             Column hcol=null; 
             if (myplotspec.isxlog) {
                hcol = new Column("hcol", "5",xcol.getData(),true);
                histogram  = new Histogram(Column.logColumn(hcol),nrows,bin);
                //histogram  = new Histogram(xcol,nrows,bin);
                xcol = new Column(xcol.getField(), "Double", histogram.getX(), 
		  true);
                myPlot.setXLog(true);

                //--- width of bars need to be adjusted for x log
                double width= (Math.log(xcol.getMax())/Math.log(10)-
			Math.log(xcol.getMin())/Math.log(10)) /histogram.bins;
                myPlot.setBars(width,0);

                //System.out.println("max="+
		   //Math.log(xcol.getMax())/Math.log(10)+
		   //" min="+Math.log(xcol.getMin())/Math.log(10)+
		   //" bin="+histogram.bins+
 		   //" bin_width=" + width); 
                
                
             } else {
                histogram  = new Histogram(xcol,nrows,bin);
                xcol = new Column(xcol.getField(), "Double", histogram.getX(), 
		  true);
                myPlot.setBars(histogram.bin_width,0);
             }

             myPlot.setMarksStyle("none");
             nrows = histogram.bins;
             ycol = new Column("Number", "Double", histogram.getY(), true);
        }

        if (isCumulative) {

            // instantiate cumulative histogram and set myPlot for bar graphs
             Column hcol=null; 
             Cumulative cumulative  = null;
             if (myplotspec.isxlog) {
                hcol = new Column("hcol", "5",xcol.getData(),true);
                cumulative  = new Cumulative(Column.logColumn(hcol),nrows,15);
                xcol = new Column(xcol.getField(), "Double", cumulative.getX(), 
		  true);
                myPlot.setXLog(true);

                //--- width of bars need to be adjusted for x log
                double width= (Math.log(xcol.getMax())/Math.log(10)-
			Math.log(xcol.getMin())/Math.log(10)) /cumulative.bins;
                myPlot.setBars(width,0);
            } else {
               cumulative  = new Cumulative(xcol,nrows,15);
               myPlot.setBars(cumulative.bin_width,0);
            }

            myPlot.setMarksStyle("none");
            nrows = cumulative.bins;
            xcol = new Column(xcol.getField(),"Double",cumulative.getX(),true);
            ycol = new Column("Cumulative Number","Double",
		cumulative.getY(),true);
        }

        //--- set up to save data for display/save option
        yaxisdata = new HashMap();
        yInRange  = new HashMap();

        //--- get data points for plot
        try {
            xdata = xcol.getData();
            //System.out.println("x data " + xdata[0] + ", " +xdata[1]);
            ydata = ycol.getData();
            if (myplotspec.showErrorBars && error != null) {
               errordata = error.getData();
            }
        } catch (Exception e){
            errDialog.display("PlotPane::not able to get column data"+
		e.getMessage());
        }

        if (myplotspec.isYOption2 && ycol2 !=null) {
           try {
               ydata2 = ycol2.getData();
               if (myplotspec.showErrorBars && error2 != null) {
                  errordata2 = error2.getData();
               }
           } catch (Exception e){
               errDialog.display("PlotPane::2nd optional y data:"+e.getMessage());
           }
        }

        if (myplotspec.isYOption3 && ycol3 !=null) {
           try {
               ydata3 = ycol3.getData();
               if (myplotspec.showErrorBars && error3 != null) {
                  errordata3 = error3.getData();
               }
           } catch (Exception e){
               errDialog.display("PlotPane::3rd optional y data:" +
		  e.getMessage());
           }
        }
        setTitle(xcol,ycol);

        goPlot(nrows,xdata,ydata,connect,yaxis,errordata,ydatamin,ydatamax);
        //--- save the data for Plot Values option
        if (!yaxisdata.containsKey(yaxis)) {
           yaxisdata.put(yaxis, new Column []{ycol,error}); 
        } 

        if (ydata2 != null && isScatter) {
           ++dataset;
           goPlot(nrows,xdata,ydata2,connect,yaxis2,errordata2,
		y2datamin,y2datamax);
           //--- save the data for Plot Values option
           if (!yaxisdata.containsKey(yaxis2)) {
              yaxisdata.put(yaxis2, new Column []{ycol2,error2}); 
           } 
        }
        if (ydata3 != null && isScatter) {
           ++dataset;
           goPlot(nrows,xdata,ydata3,connect,yaxis3,errordata3,
		y3datamin,y3datamax);
           //--- save the data for Plot Values option
           if (!yaxisdata.containsKey(yaxis3)) {
              yaxisdata.put(yaxis3, new Column []{ycol3,error3}); 
           } 
        }

        //--- display legend with sampling ranges 
        if (isSample) {
           //--- sort the axis and sampling values before displaying Legend
           TreeMap sorted= new TreeMap(criteriaRange);
           Set keys= sorted.keySet();
           Iterator it = keys.iterator();
           int count =0; 
           //--- Go through ranges and add legend
           while (it.hasNext()) {
              Integer key = (Integer) it.next();
              myPlot.addLegend(key.intValue(), (String) criteriaRange.get(key));
           }
        }

        //--- generate and save values in case user
        //--- decides to display or print
        if (isApplet) {
           Container cont = this.getParent()  ;
           while (!(cont instanceof JApplet)) { 
              cont = cont.getParent();  
           }
           this.appletContext=((JApplet) cont).getAppletContext();
        }

        //--- final set up for display/save option
        pValues = new PlotValues(appletContext,myplotspec.getURL());
        if (isHistogram || isCumulative) {
           pValues.setHistogram(true);
        }
        pValues.updateValues(xcol,yaxisdata,yInRange,sdata,scol);
        
    }

    /** If new criteria has been set, we need to calculate
     *  the sample column.
     */
    public double[] sampleQuery() {
      
        String new_criteria = myplotspec.criteria;

        if (!old_criteria.equals(new_criteria)) {
            try {
                HeasarcQuery squery = 
		new HeasarcQuery(myplotspec.getURL(), myplotspec.getPostData());
            } catch (Exception e) {
                errDialog.display("Unable to run sample HEASARC query:  " + e);
            }
            old_criteria = new_criteria;
        }
	// This needs to be fixed.....
        return null;
    }
 
    private void setPlottypes(String plottype) {

        isHistogram = false;
        isCumulative = false;
        isAitoff = false;
        isScatter = false;
        isLine = false;

        if (plottype.equals("histogram")) {
            isHistogram = true;
        } else if (plottype.equals("scatter") || plottype.equals("lineplot")) {
            isScatter = true;
            if (plottype.equals("lineplot")) {
               isLine=true;
            }
        } else if (plottype.equals("aitoff")) {
            isAitoff = true; 
        } else if (plottype.equals("cumulative")) {
            isCumulative = true;
        } else {
            errDialog.display("You are requesting an unknown plot type.");
        }
    }
    
    /** setRange() routine sets the plot range if min/max are defined.
     *  This routine is used only for 'scatter' plots
     */
    public void setRange(Column xcol,Column ycol) {
       
        double xmin = ifNaN(myplotspec.xmin, xcol.getMin());
        double xmax = ifNaN(myplotspec.xmax, xcol.getMax());
        double ymin = ifNaN(myplotspec.ymin, ycol.getMin());
        double ymax = ifNaN(myplotspec.ymax, ycol.getMax());
      
        if (!Double.isNaN(xmin) && ! Double.isNaN(xmax)) {
            myPlot.setXRange(xmin, xmax);
            //System.out.println("PlotPane::setting x "  + xmin +", " +xmax);
            this.xmax=xmax;
            this.xmin=xmin;
        }
      
        if (!Double.isNaN(ymin) && ! Double.isNaN(ymax)) {
            //System.out.println("PlotPane::setting y "  + ymin +", " +ymax);
            myPlot.setYRange(ymin, ymax);
            this.ymax=ymax;
            this.ymin=ymin;
        }
    }

    
    /** setNewYRange() routine updates the plot Y range if min/max are defined.
     *  This routine is used only for 'scatter' plots
     */
    public void setNewYRange(double newYmin,double newYmax) {
    
        if (newYmin < this.ymin) { 
           if (!Double.isNaN(newYmin)) {
              this.ymin=newYmin;
              //System.out.println("PlotPane::update y min=" + ymin);
           }
        }
        if (newYmax > this.ymax) { 
           if (!Double.isNaN(newYmax)) {
              this.ymax=newYmax;
              //System.out.println("PlotPane::update y max=" + ymax);
           }
        }
            myPlot.setYRange(ymin, ymax);
    }
    
    /** setRange() routine sets the plot range if min/max are defined.</Image>
     *  This routine is used only for 'scatter' plots
     */
    public void setNewXRange(Column xcol) {
       
        double newXmin = ifNaN(myplotspec.xmin, xcol.getMin());
        double newXmax = ifNaN(myplotspec.xmax, xcol.getMax());
      
        if (newXmin < this.xmin) { 
           if (!Double.isNaN(newXmin)) {
              this.xmin=newXmin;
              //System.out.println("PlotPane::update x min=" + xmin);
           }
        }
        if (newXmax > this.xmax) { 
           if (!Double.isNaN(newXmax)) {
              this.xmax=newXmax;
              //System.out.println("PlotPane::update x max=" + xmax);
           }
        }
    }

    /** Check if a value is NaN */
    private double ifNaN(double a, double b) {
        if (!Double.isNaN(a)) {
	    return a;
        } else {
	    return b;
        }
    }

    /** Plots xdata[] and ydata[] points
     */
    public void goPlot(int nrows,double[] xdata, double[] ydata, 
                       boolean connect, String axis, double [] errordata,
                       double ydatamin, double ydatamax){
	
        if (isLine) {  //sort data points
           Vector  sortedData=Fmath.selectSortVector(xdata);
           double [] newxdata = (double []) sortedData.elementAt(1);
           double [] newydata = new double [ydata.length];
           int [] indices = (int [] ) sortedData.elementAt(2);
           for (int i=0; i<xdata.length; i++) {
               newydata [i] = ydata[indices[i]];
           }
           xdata=newxdata;
           ydata=newydata;

        }
        //--- this will contain flag indicating if point is within 
        //--- min and max y values
        double [] yinclude=new double [ydata.length];

        //--- Legend for sampling is not written until after all points 
        //--- are plotted
        if (isScatter && !isSample) {myPlot.addLegend(dataset,axis);}

        int plotset=dataset;   //--- used to determine plot symbol
        int sampval=0;
        int maxsamp=0;
        for (int i=0; i<nrows; i++) {
	
           //System.out.println("PlotPane:: x data="+ xdata[i] + " xmin=" + 
		//xmin + " max=" +xmax);
           //System.out.println("PlotPane:: y data="+ ydata[i] + " ymin=" + 
		//ydatamin + " max=" +ydatamax);

           if (isScatter ) {
              //--- Skip plotting if point is not in x or y range
              if (xdata[i] < xmin || xdata[i] > xmax) {continue;}

              //--- ydata may have been converted to histogram  data
              if (ydata[i] < ydatamin || ydata[i] > ydatamax) {
                 //-- set include flag for value display/save 
                 yinclude[i]=-1;
                 continue;
              }
           }
           
           yinclude[i]=1;   
           if (isSample) {
              sampval = (int) sdata[i]-1 ;

              //--- Save maximum sampling dataset #.
              if (sampval > maxsamp) {maxsamp=sampval;}

              //--- plotset determines plot symbol
              plotset = dataset + sampval;
                

              //System.out.println("PlotPane:: goPlot samp: " + i + " deset="+ 
	          //dataset + " x=" +xdata[i] + " y=" + ydata[i] + 
                  //" plotset=" +plotset + " sdata=" + sdata[i] + " max =" + 
		  //maxsamp + " axis=" +axis);

	      if (sampval < 0) {
                 //--- not included in sample range so go to next point
	         continue;
	      }  else {
                 //--- save y axis name and sampling range for this plot point
                 //--- for plot legend
                 Integer keyval = new Integer(plotset);
                 String currval= axis +", " +criteriaList[sampval];
                 if (!criteriaRange.containsKey(keyval)) {
                    criteriaRange.put(keyval, currval);
                 }
              }
           }  //--- end sample processing
	
           if(Double.isNaN(xdata[i])) {
              continue;
	   }
	    
           if (Double.isNaN(ydata[i])) {
              continue;
	   }
	    
           //--- plot point with errors if option selected
           if (myplotspec.showErrorBars && errordata != null) {
	      if (Double.isNaN(errordata[i])) {
                 continue;
              }

              myPlot.addPointWithErrorBars(plotset,xdata[i],ydata[i], 
		   ydata[i]+errordata[i],ydata[i]-errordata[i], connect);
           } else {
               //System.out.println("PrintPane::adding no error points " 
		//+ xdata[i] + ", " + ydata[i] + " dataset=" + dataset);

               myPlot.addPoint(plotset,xdata[i],ydata[i],connect);
           }
        }
        //--- save the data for Plot Values option
        if (!yInRange.containsKey(axis)) {
           yInRange.put(axis, yinclude); 
        } 
        //--- save the data for Plot Values option
        //if (!yaxisdata.containsKey(axis)) {
           //yaxisdata.put(axis, new double [][]{ydata,errordata,yinclude}); 
        //} 

        dataset += maxsamp;   //--- if sampling, adjust dataset for next plot

        myPlot.repaint();
	
        if (isApplet) {
	    
            CreateGif cgif = new CreateGif(myPlot);
	    
            try {
                cgif.makeGif(myplotspec.procnum);
                ////System.exit(1);
            } catch (Exception e) {
                errDialog.display("CreateGif problem: " +e.getMessage());
            } 
        } else {
            myPlot.setVisible(true);
        }
    }  //--- end goPlot

   /** Sets up the titles on the plot
     */
    public void setTitle (Column xcol, Column ycol) {
        String xname,yname;

        //get actual field names for x and y axis and the title
        xname = xcol.getField().toUpperCase();
        yname = ycol.getField().toUpperCase();

        if (isHistogram) {  //histogram
            myPlot.setTitle("Histogram for " + xname);
        } else if (isCumulative) {  //cumulative histogram
            myPlot.setTitle("Cumulative Histogram for " + xname);
        } else if (isAitoff) {
            if (isSample) {
               myPlot.setTitle("Aitoff projection with sampling on "+
			myplotspec.sample_param);
	    } else {
               myPlot.setTitle("Aitoff projection");
	    }
        
	    myPlot.setXLabel("L");
            if (!isSample) { myPlot.addLegend(1,"B");}
	    
        } else {
	  
            if (isSample) {
               myPlot.setTitle("Sampling on " + 
                               myplotspec.sample_param);
            } else { 
               //myPlot.setTitle(xname + " versus " + yname);
            }
	    
            myPlot.setXLabel(xname);
        }
    }
}
