
package heasarc.w3browse.plot;

/**
 *           W3Browse Plotting Package                          **
 *                                                              **
 * SOURCE: Aitoff.java                                          **
 *                                                              **
 * @author Saima Zobair (RITSS)   saima@lheamail.gsfc.nasa.gov  **
 *                                                              **
 *****************************************************************/

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import ptolemy.plot.*;
import heasarc.w3browse.*;

/** This class creates histogram plots.
 */
public class Aitoff {
  /////////////////////////////////////////////////////////////////
  ///                   class variables                         ///
  /** the plot */
  private Plot myPlot;
    
  public Column xcol;
  public Column ycol;

  /////////////////////////////////////////////////////////////////
  ///                      constructor                          ///
  /** Constructor for Aitoff */
  public Aitoff(Plot myPlot) {
      
    this.myPlot = myPlot;
    myPlot.setMarksStyle("none",0);
    myPlot.setMarksStyle("dots",1);
    myPlot.addXTick("  ",0);
    myPlot.addYTick("  ",0);
    myPlot.setGrid(false);
    grid();
  }
  /////////////////////////////////////////////////////////////////
  ///                      private methods                      ///
  /** Sets up the axis grid on plot
   */
  private void grid () {

    myPlot.setXRange(-3.15,3.15);
    myPlot.setYRange(-1.5,1.5);

    double[] point = new double[2];
    boolean first;
    for (int ra = -180; ra <= 180; ra +=60) {
      first = true;
      for(int dec = -90; dec <= 90; dec +=2) {
        point = convert(ra,dec);
        myPlot.addPoint(0,point[0],point[1],!first);
        first = false;
      }
    }
    for(int dec = -90; dec < 90; dec +=30) {
      first = true;
      for (int ra = -180; ra <= 180; ra +=3) {
        point = convert(ra,dec);
        myPlot.addPoint(0,point[0],point[1],!first);
        first = false;
      }
    }
  }
    
  /** converts to aitoff values
   */
  public double[] convert(double ra, double dec) {
      
    double[] point = new double[]{ra,dec};
    convert(point);
    return point;
    
  }
    
  public void convert(double[] point) {

    //convert to radians
    double dtor = Math.PI/180;
    double l = point[0] * dtor;
    double b = point[1] * dtor;

    double z1 = 1.;

    double z2 = 1+ Math.cos(b) * Math.cos(l/2.);
    if (z2 < .000001) {
       z2 = .000001;
    }
    z2 = 2/z2;

    double alp = z1 * Math.sqrt(z2);
    double x = 2 * alp * Math.cos(b) * Math.sin(l/2.);
    double y = alp * Math.sin(b);

    point[0] = -x; //Invert X to be standard astronomical frame
    point[1] =  y;
  }

  /** converts RA, Dec  axis data by calling convert()
   */
  public void convertData(String ra,String dec,HeasarcQuery plotquery,
         int nrows) throws InputException {
    
     double[] xdata;
     double[] ydata;
     double[] point;

     if(ra == null) 
        throw new InputException("RA column is not defined! Can't do aitoff projection.");
     if (dec == null)
        throw new InputException("Dec column is not defined! Can't do aitoff projection.");

     try {
        xcol = plotquery.getColumn(ra);  // get ra column
     } catch (Exception e) {
        throw new InputException("Unable to get column " + ra + " from query. Please try again.");
     }
      
     try {
        ycol = plotquery.getColumn(dec); // get dec column
     } catch (Exception e) {
        throw new InputException("Unable to get column " + dec + " from query. Please try again.");
     }

     try {
	 
        xdata = xcol.getData();
        ydata = ycol.getData();
	 
     } catch (Exception e){
        throw new InputException(e.getMessage());
     }

     for (int i =0; i<nrows; i++) {
	 
	// Convert ra,dec to l,b
	point = heasarc.util.Sla.eqgal(toRadians(xdata[i]), toRadians(ydata[i]));
	point[0] = toDegrees(point[0]);
	point[1] = toDegrees(point[1]);
        if (point[0] > 180) point[0] -= 360; 
	 
        convert(point);
	 
        xdata[i] = point[0];
        ydata[i] = point[1];
     }

     xcol = new Column(xdata, xcol);
     ycol = new Column(ydata, ycol);

  }
  
  /** These functions in Math library not available in Java 1.0 */
  private static double toRadians(double x) {
      return x*Math.PI/180;
  }
    
  private static double toDegrees(double x) {
      return x*180/Math.PI;
  }
}
