# $Id: Config.pm,v 1.39 2008/09/11 16:57:29 lmm Exp $
#=======================================================================#
# Module Name:    Config (was w3config.pl)
# Module Author:  Kressel, McGlynn, Sabol, Zobair, Heikkila
# Date:           Dec, 95
# Project:        Browse : HEASARC
#=======================================================================#
# Module Description:
# Define W3Browse program variables.
#  This module should be 'use'd by all W3Browse perl scripts.
#  The $::variables are in the main package.
#
#=======================================================================#
# High Energy Astrophysics Science Archive Research Center (HEASARC),
# NASA's Goddard Space Flight Center, Greenbelt, MD 20771, USA.
#
# You are free to use this code as you wish, but unless you change
#  it significantly, please leave the above name & address intact.
#=======================================================================#

package main;


# These are things you won't need to change.
$::version       = $VERSION = "8.3"; 
$::batch_version = "2.0";
$::heasarc       = "http://heasarc.gsfc.nasa.gov";

# These are the VizieR sources - uncomment the one you want to use, or add
# more if you want to use others.
$::vizier      = "http://vizier.u-strasbg.fr"; # CDS in Strasbourg, France
#$::vizier      = "http://vizier.cfa.harvard.edu/"; #CFA at Harvard
#$::vizier      = "http://adc.gsfc.nasa.gov"; # ADC: slow, maybe inactive?
#$::vizier      = "http://vizier.hia.nrc.ca";  # CADC in Canada
#$::vizier      = "http://archive.ast.cam.ac.uk/";  # Cambridge
#$::vizier      = "http://data.bao.ac.cn/"; # Bejing Obs. in China, slow
#$::vizier      = "http://vizier.nao.ac.jp/"; # ADAC in Tokyo, Japan
#$::vizier      = "http://urania.iucaa.ernet.in/"; IUCAA in Pune, India
#$::vizier      = "http://www.inasan.rssi.ru/"; INASAN in Russia




# Set Perl path for required libraries... 
# The first and second paths are where your XML, DBI, and other nonstandard 
# libraries live.  The third: './lib' should always be listed, since that
# is where the Browse:: package lives.
BEGIN{
  @::browselibs = ("/www/htdocs/cgi-bin/lib/",
                   "/www/htdocs/cgi-bin/lib/heasarc/",
                   "./lib");
  unshift(@INC,@::browselibs);
}

# dflib is the directory where you will find sys/statvfs.ph and sys/syscall.ph.
#  This is needed for calling Filesys::Diskspace in Products.pm.
  $::dflib = "/usr1/local/lib/perl5";

# Map of table name patterns to data products modules:
#   e.g.:  table names matching "heasarc_" will cause data product query objects
#          to be created with type HDPQuery.
# The prods hash doesn't use patterns, just replace the H* modules with yours.
%::dpq = (
    "heasarc_" => "HDPQuery",
    "zz"       => "HDPQuery",
    "\/"       => "HDPQuery",
);
%::prods = (
    "view" => "HProdDisplay",
   "write" => "HProdGen",
   #"write" => "HProdGenChoice",
   "script" => "HProdScriptGen",
   "scriptfile" => "DPScriptToFile",
   "analyze" => "ProdHera",
);

# map old tables that are no longer used to their new equivalents
%::maptables = (
    "BATCHRETRIEVALCATALOG_2.0 heasarc_usnoa2" => 
                        "BATCHRETRIEVALCATALOG_2.0 I/252",
    "usnoa2" => "I/252",
    "heasarc_usnoa2" => "I/252",
);

# commonly-used modules
use Browse::Utils;
use Browse::MailMessage;
use Browse::HDB;
use Browse::DPQuery;
use Browse::DBQ;
use Browse::Decor;
use Browse::CGI;
use CGI::Carp qw(fatalsToBrowser);
use Sys::Hostname;

#--- Name resolver setup no longer needed 7/2006
# Name resolver setup:
# the location of the NED name-resolver script supplied by HEASARC:
#$Astro::Object::Resolver::NED::NED = 
     #'/www/htdocs/cgi-bin/W3Browse/bin/NED_name';
# the location of the SIMBAD client
#$Astro::Object::Resolver::SIMBAD::SNR = 
     #'/www/htdocs/cgi-bin/W3Browse/bin/snr';
# the username and password for SIMBAD - please change these to your own.
$Astro::Object::Resolver::SIMBAD::USER = 'u616idsv';
$Astro::Object::Resolver::SIMBAD::PASSWORD = 'yjqjmzex';

# Maintainer name and email is printed at the bottom of each page.  Change it
# to whoever is maintaining this installation of Browse.
$::maintainer_name  = "Browse Software Development Team";
$::maintainer_email = "request\@athena.gsfc.nasa.gov"; 

# The custom is to prefix your local tables with this origin label: 
#   e.g.: heasarc_rospublic, heasarc_xray
# When displaying the table name to the user, this origin is removed.
$::local_origin= "heasarc_";

# The footer is printed at the bottom of each page, and the menubar at the top.
# These are not in the distribution - change them to your own footer/header.
$::inc_summary ="inc_summary.html";
$::swiftbanner ="swiftbanner.html";
$::swiftburstfile ="/www/htdocs/W3Browse/swiftburst.txt";

$::footer  = "/www/htdocs/docs/lhea/lhea_footer.html";
$::menubar = "/www/htdocs/docs/browsemenu.html";

# temp file directory (used for writing temp files when Excel output or file
# uploads are chosen

$::tempdir = "/tmp";

# At HEASARC, our tarfiles are deposited in /FTP/retrieve/w3browse/ and are 
#  available at the url ftp://legacy.gsfc.nasa.gov/retrieve/w3browse/
#  and at the url http://legacy.gsfc.nasa.gov/FTP/retrieve/w3browse/
# The tar size limit is the maximum estimated tarfile size you want to 
#  allow users to request.  The limit is set in bytes.
$::tarurlbase = "retrieve/w3browse";
#$::tarurlbase = "retrieve/w3browse";
#$::tarurlbase = "retrieve/w3browse" if hostname() =~ /heasarcdev/;
#$::tarurlbase = "retrieve/w3browse" if hostname() =~ /heasarc2/;
$::tarftpbase = "/FTP";
#$::tarsizelimit = 1*1024*1024;  # small test limit
#$::tarsizeasklimit = 1*1024*1024;  # small test limit
#$::tarsizelimit = 5*1024*1024*1024;  # 5 GB
$::tarsizelimit = 2*1024*1024*1024;  # 2 GB  until server issues resolved
#$::tarsizeasklimit = 3*1024*1024*1024;  # 3 GB (commented until server issues resolved)

# How do you want the 'tar' binary called? (this is called in Products.pm)
$::tar = "nice tar";

# What is your df binary?  Pls leave the '-k', or whatever you need to get 
# the disk space in kilobytes
$::df = "df -k";

# We write logs for certain Browse actions.
$::logDir   = "/Apache.Logs";
$::queryLog = "$::logDir/w3browse_query.log";
$::tarLog   = "$::logDir/w3browse_tar.log";
$::sqlLog   = "$::logDir/w3browse_sql.log";
$::VizLog   = "$::logDir/w3browse_vizierquery.log";
$::hera_DPLog  = "$::logDir/w3browse_hera_dp.log";

# root is the path to just above cgi-bin.
$::root    = "/www/htdocs";

# w3bdocs is the directory under $::root where the Browse html pages live.
$::w3bdocs = "/W3Browse";
$::mainbrowse = "/docs/archive.html";

# bulletin (caveats, notes, warnings) directory 
$::bulletins = "$::root/$::w3bdocs/bulletins/";

# images is the URL where your Browse images live.
$::images  = "/Images/w3b";

# Java is used for plotting.
# Set this to the directory under the web tree containing your 
#   java classes for Browse
if ($ENV{'SCRIPT_NAME'} && $ENV{'SCRIPT_NAME'} =~ /w3bdev/) {
   $::java = "/classesnew";
} else {
   $::java = "/classesnew";
}

#$::java = "/xclasses/w3bx";

# Your java binary, rt.jar, and fonts dir.
#$::javaroot     = "/usr1/local/java122";
$::javaroot     = "/usr/java/j2sdk1.4.0_01";
$::jbin         = "$::javaroot/bin/java";
$::rtjar        = "$::javaroot/jre/lib/rt.jar";
$::jsystemfonts = "$::javaroot/jre/lib/fonts";

# Max # of preview images you want displayed on the products list page
$::MaxGIFs = 25;

# You may not have to change anything in this group, since these build on 
#   prior definitions.
$::w3bhelp     = "$::w3bdocs/w3browse-help.html"; # Browse help doc
$::helpdir     =  $::root . $::w3bdocs; # mission help pages
$::vizgif      = "$::images/vizier_logo.gif";

# We check to see if the server is swamped with requests and print error
$::maxprocesses = 15; # if there are more than this, print error
$::ps = 'ps -A';  # your ps command - see Utils.pm if more changes needed

# This is the maximum angular separation that two resolved positions can
# differ by before we report a warning.  
$::radius_tolerance = 0.167;  # arcminutes = 10 arcsec

# These are strings to use as the start of saved filed names.
$::savedQueryFile="BrowseQuery";
$::savedTargetsFile="BrowseTargets";
$::savedPlotValuesFile="BrowseValues";

##--- For master table  (get from index page)
%::missiontypes=(
'ARIEL V'          => 'archived',
'ASCA'             => 'active',
'BBXRT'            => 'archived',
'BEPPOSAX'         => 'archived',
'CGRO'             => 'archived',
'CHANDRA'          => 'active',
'COPERNICUS'       => 'archived',
'COS B'            => 'archived',
'EINSTEIN'         => 'archived',
'EUVE'             => 'archived',
'EXOSAT'           => 'archived',
'FAUST'            => 'archived',
'FUSE'             => 'archived',
'GALAXY CATALOG'   => 'catalog',
'GAMMA-RAY BURSTS' => 'archived',
'GENERAL CATALOG'  => 'catalog',
'GINGA'            => 'archived',
'GROUND-BASED'     => 'archived',
'HEAO1'            => 'archived',
'HETE-2'           => 'active',
'HST'              => 'archived',
'INTEGRAL'         => 'active',
'IRAS'             => 'archived',
'ISO'              => 'archived',
'IUE'              => 'archived',
'MASTER CATALOG'   => 'catalog',
'MSX'              => 'archived',
'NEBULA CATALOG'   => 'catalog',
'OSO8'             => 'archived',
'PHYSICS DATA'     => 'archived',
'RADIO CATALOG'    => 'catalog',
'ROSAT'            => 'archived',
'RXTE'             => 'active',
'SAS-2'            => 'archived',
'SAS-3'            => 'archived',
'SPITZER'          => 'active',
'STAR CATALOG'     => 'catalog',
'SUZAKU'           => 'active',
'SWIFT'            => 'active',
'TD1'              => 'archived',
'UHURU'            => 'archived',
'UIT'              => 'archived',
'VELA 5B'          => 'archived',
'XMM-NEWTON'       => 'active',
);
##
@::fovtables = ('heasarc_chanmaster');
@::fovstatus = ('archived');

# ** There are more settings to change in the next subroutine.... **

#------------------------------------------------------------------------

package Browse::Config;

sub AlwaysRun() {
   # this code should always be run for each request, even under mod_perl

   # can also put the string 'debug' in the query string.
   $::debug = 1 if $ENV{'QUERY_STRING'} =~ /debug/;
   $::wgacatLink = 1; 

   #if ($ENV{'SCRIPT_NAME'} =~ /db-perl/) {
      #($::cgiurl) = $ENV{'SCRIPT_NAME'} =~ /\/(db-perl\/.*)\/.+$/;
   #} else {
      #($::cgiurl) = $ENV{'SCRIPT_NAME'} =~ /\/(cgi-bin\/.*)\/.+$/;
   #}

   ($::cgiurl) = $ENV{'SCRIPT_NAME'} =~ 
	/\/((cgi-bin|cgi-perl|db-perl)\/.+)\/.+$/;

   ($::cgi = $::cgiurl) =~ s/db-perl/cgi-bin/;

   $::writeHeader = 1;
   $::lib         = "$::root/$::cgi/lib";
   $::bin         = "$::root/$::cgi/bin";
   $::basicData   = "$::root/$::cgi/w3basic.data";

   $::server      = $ENV{'SERVER_NAME'};

   # This section expands the server name to one of two servers, with the full
   #  domain name. (The java plotting will not work if the server name is not
   #  fully qualified.)
   if ($::server =~ /^(dbms1|heasarcdev|olegacy)/) {
       $::host = "heasarcdev.gsfc.nasa.gov";
   } else { 
       $::host = "heasarc.gsfc.nasa.gov";
   }

   $::ftphost = $::host;
   $::ftphost = "legacy.gsfc.nasa.gov" if $::host =~ /heasarc/;
   $::serverURL   = "http://$::host/$::cgiurl/w3server.pl";

   # Sybase/installation specific stuff 
   if (-e "/heasarc/sybase") {
       $ENV{SYBASE} = '/heasarc/sybase';
       $ENV{SYBASE_OCS} = 'OCS';
   } elsif (-e "/wwwroot/mnt/heasarc/sybase") {
       $ENV{SYBASE} = '/wwwroot/mnt/heasarc/sybase';
   }
   $ENV{LD_LIBRARY_PATH} = "$ENV{SYBASE}/$ENV{SYBASE_OCS}/lib";

   # Set path for binary executables...
   $ENV{'PATH'} = "/bin";

   # Set Perl path for required libraries... 
   # keep these as unshifts, not use's, because of mod_perl
   unshift(@INC,@::browselibs);

   # Get Remote Host
   if ($ENV{'HTTP_X_FORWARDED_FOR'}) {
         $::REMOTE_HOST = $ENV{'HTTP_X_FORWARDED_FOR'};
   } elsif ($ENV{'REMOTE_HOST'}) {
        $::REMOTE_HOST = $ENV{'REMOTE_HOST'};
   } elsif ($ENV{'REMOTE_ADDR'}) {
        $::REMOTE_HOST = $ENV{'REMOTE_ADDR'};
   } else {
        $::REMOTE_HOST = "Unknown";
   }

   # At HEASARC, if the url has db-perl instead of cgi-bin, the mod_perl server
   #   handles the request.  The files still reside in cgi-bin.
   if ($ENV{'SCRIPT_NAME'} =~ /db-perl/) {
      $::REMOTE_HOST = Browse::Utils::getREMOTEHOST();
   }

   # These are the emergency addresses mailed to by MailMessage().
   if($::REMOTE_HOST =~ /^savannah/){
       $::addr = 'lmm@milkyway.gsfc.nasa.gov';
   } elsif($::cgi =~ /lmm/ || $::cgi =~ /w3bdev/){
       $::addr = 'lmm@milkyway.gsfc.nasa.gov';
   } elsif ($::REMOTE_HOST =~ /^silk/) {
       $::addr = 'tam@silk.gsfc.nasa.gov';
   } elsif ($::server =~ /^dbms1|^olegacy|^heasarcdev/) {
       $::addr = 'lmm@milkyway.gsfc.nasa.gov,tam@silk.gsfc.nasa.gov';
   } else {
       $::addr = 'w3browse@bigbang.gsfc.nasa.gov';
   }

   # counts the number of times this process has been called
   $::pcount += 1;

   if ($::debug) {
       print "Content-type: text/html\n\n";
       print 'in Config: @INC=',join('<br>',@INC),"<br>\n";
       print "\$ENV\{\'SCRIPT_NAME\'\}=$ENV{'SCRIPT_NAME'}<br>\n";
       print "AlwaysRun: proc_id=$$, $::pcount<br>\n";
       print "REMOTE_HOST :: $::REMOTE_HOST<br>";
   }
   # Initialize database tables.
   Browse::DPQuery::clear();
   Browse::DBQ::Registry::init("databases.dat");

}

1;
#########
#  EOF  #
#########
