#
# $Id: Cross.pm,v 1.30 2003/08/28 21:33:19 lmm Exp $
# $Log: Cross.pm,v $
# Revision 1.30  2003/08/28 21:33:19  lmm
# Define text and style for page header and remove HEASARC footer
#
# Revision 1.29  2003/05/20 17:05:28  cwh
# Added missing dl tags.
#
# Revision 1.28  2002/12/10 21:41:27  cwh
# Added quotes where needed.
# Added labels and ids to form elements.
# Changed bolding which didn't work right in Mozilla.
#
# Revision 1.27  2002/09/13 21:05:58  cwh
# Changed default rows to 0 (unlimited).
# Changed explanation of default radii - now larger of the two tables' radii
# Added a name to the form for use in javascript.
# Added javascript functions for the table interface.
#
# Revision 1.26  2002/03/13 20:53:36  cwh
# Tom fixed some minor problems with the html in the explanation section.
#
# Revision 1.25  2001/12/06 16:44:28  cwh
# Encoding table name in req_table.
#
# Revision 1.24  2001/11/20 15:45:10  cwh
# Don't want to display tablename to user as "heasarc_tablename".
#
# Revision 1.23  2001/11/07 23:34:16  cwh
# Moved everything except DBD/ into lib/Browse/, so as not to pollute the module
# namespace with our Sort, Search, CGI, etc.  This required putting Browse:: in
# front of every module call and package declaration.  Also any mod::new() had
# to be changed to mod->new().
#
# Revision 1.22  2001/10/04 22:10:21  cwh
# Added checkbox so users can ask for all fields to be returned.
#
# Revision 1.21  2001/09/15 03:00:05  cwh
# Modified the form to Cornerstone style.
# Added output format selection menu.
# Changed radius correlation flag from C to R.
# Passing item number to CrossChoices.
#
# Revision 1.20  2001/07/26 19:22:43  cwh
# Fixed error in javascript check that didn't let the user select the first table.
#
# Revision 1.19  2001/06/20 22:57:31  cwh
# Got rid of $::writeHeader in calls to ::Error.
#
# Revision 1.18  2001/06/08 19:46:37  cwh
# Now passing $::writeHeader to ::Error calls, so we don't get repeating headers.
#
# Revision 1.17  2001/05/17 22:08:39  cwh
# Added check for selection of second table using javascript.
#
# Revision 1.16  2001/05/16 18:10:28  tam
# Cross-correlation capabilities now largely handled in CrossRequest.
# Support for VizieR queries added.  The multi-line output option
# has been disabled.
#
#
# Eearlier comments deleted... Functionality of this module
# has been largely transferred into CrossRequest and Display
#
# Revision 1.1  1997/03/21 00:21:28  sabol
# Initial addition to CVS repository.
#
#=========================================================================#
# High Energy Astrophysics Science Archive Research Center (HEASARC),
# NASA's Goddard Space Flight Center, Greenbelt, MD 20771, USA.
#
# You are free to use this code as you wish, but unless you change
#  it significantly, please leave the above name & address intact.
#=========================================================================#
# Module Name:    Cone.pm
# Module Authors: Edward J. Sabol (HSTX), Saima Zobair (RITSS)
# Date:           February 1997
# Project:        Browse : HEASARC
#=========================================================================#
# Module Description:
#   Rountines for executing a object name/coordinates cone
#   cross-correlation search and displaying the results to the user.
#
#=========================================================================#

use strict;

use Browse::Options;

package Browse::Cross;

sub CrossChoices{
  my ($space, $time,$num) = @_;
  my($state) = {};
print <<EOT;
<p>
<strong>Correlation Flags:</strong>
<dl>
<dd>R - Correlation by Radius is possible 
<dd>T - Correlation by Time is possible
</dl>
<p>
EOT
    print "<p><strong>".$num++ .". Do you want to correlate on: </strong><dl>";
    if ($space) {
      print "<dd><input type='checkbox' checked id='CrossRadius' name='CrossRadius'> <label for='CrossRadius'><a href='#radius'>radius?</a></label> ";
      print  Browse::Options::RadiusSelection("XRadius"),"\n";
    }
  
    if ($time) {
      print "<dd><input type='checkbox' name='CrossTime' id='CrossTime'> <label for='CrossTime'><a href='#time'>delta time?</a></label> \n";
      print  Browse::Options::TimeSelection(),"\n";
    }
  
  print qq(
    </dl>
    <p><b>),$num++,qq(. Do you want to 
    </b><input type="checkbox" name="Anticorrelation" id="Anticorrelation"> <label for="Anticorrelation"><a href="#anti"><b>anticorrelate?</b></a> (find entries in the primary table that are not in the secondary)</label>
    <p><strong>),$num++,qq(. Do you want to change the display of the output?</strong><dl>
    <dd><input type="checkbox" name="ShowSecond" id="ShowSecond" checked> <label for="ShowSecond"><a href="#secondary">Show columns of secondary table</a> as well as primary</label>
    <dd><input type="checkbox" name="Minimal" id="Minimal"> <label for="Minimal"><a href="#minimize">Minimize data</a>: show only a critical subset of the table columns in the output</label>
    <dd><label for="ResultMax"><a href="#maxrows">Max Rows:</a></label>
);
  
  $state->{'ResultMax_0'} = 'selected'; #setting default
  print  Browse::Options::ResultLimit($state)," rows\n";

  print "<dd><label for='displaymode'><a href='#format'>Output format:</a></label> ";

  print Browse::Options::OutputFormat($state),"\n";
  $state->{'Fields'} = "";
  print "<br><input type='checkbox' id='Fields' name=\"Fields\" $state->{'Fields'} value=\"All\"> <label for='Fields'>Return all fields?</label>";
  
#         <dd><input type='checkbox' name='Multiline' id='Multiline'> <label for='Multiline'><a href='#multiline'>Multiline format</a></label>

print qq(
    </dl>
    <p><strong>),$num++,qq(. <input type=submit value="Begin Search">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp<input type=reset value="Reset"></strong>
);

} #eosub--CrossChoices
#--------------------------------------------------------------------------
sub Explain {

print <<EOT;

<p>
<strong>Explanation of Options:</strong>
<dl>
<a name="radius"></a>
  <dt> Radius
  <dd> Check the box if you wish to correlate by position.  Set
       the value to the radius to which observations should match.  
       The default is the larger of the two tables' individual default 
       search radii.
<a name="time"></a>
  <dt> Delta Time
  <dd> Check the box if you wish to correlate by time.  Set the value
       to the time interval for which entries are to match.  The default
       is 1 day.
<a name="anti"></a>
  <dt> Anticorrelation
  <dd> Anticorrelations ask for entries in the primary table for which
       there is no match in the secondary table.
<a name="secondary"></a>
  <dt> Show Second Table Data
  <dd> Show the data from the seconary
       table in the results.
<a name="maxrows"></a>
  <dt> Max Rows
  <dd> Set the maximum number of rows to be returned.
<a name="minimize"></a>
  <dt> Minimize data
  <dd> Use a critical subset of fields in
       the tables.  This will not override fields selected in a prior query.
<a name="format"></a>
  <dt> Output format
  <dd> Show the results as HTML (default), as a simple ASCII table without HTML formatting, or as a FITS table.
<a name="multiline"></a>
</dl>
EOT
#  <dt> Multiline format
#  <dd> Show the results with the primary table and secondary table data
#       displayed on separate lines.  Fonts or indentation are used
#       so distinguish the tables.
} #eosub--Explain
#'
#--------------------------------------------------------------------------
# Generate page for selection of tables for results cross-correlations.
sub CrossCorrelate{

$0 = 'Cross.pm';        # required for MailMessage
    my ($table, $in) = @_;

    my %tinfo = Browse::HDB::TableInfo($table);
    my $hasRadius = $tinfo{"default_search_radius"};
    my $hasTime   = $tinfo{"start_time"};

    if (!$hasRadius && !$hasTime) {
        Browse::Utils::Error("Cannot correlate with this table.  No position or time information. ");
    }

    (my $ptable = $table) =~ s/$::local_origin//ig;
    #Browse::Decor::Header("Cross-Correlate $ptable Results");
    my($stylecolor) = "xcortable";
    Browse::Decor::Header("Cross-Correlate $ptable Results",0,0,0,$stylecolor);

    # Build the form.
    my($method) = "POST";
    print "<form method=$method name='fullcross' action=\"w3cross.pl\" onSubmit=\"if (this.Table2.selectedIndex < 0) {alert('Please choose a table to correlate with.'); return false;}\">\n";

    my $navtrail = $in->{'navtrail'};
    print "$navtrail &gt; <span class=navcurrent>Choose Cross-correlation table</span> &gt; <span class=navfuture>Search Results &gt; Choose Data Products</span><br>\n";
    $navtrail .= " &gt; Choose Cross-correlation table";
    $navtrail =~ s/\</&lt;/g;
    $navtrail =~ s/\>/&gt;/g;
    my($popupRef)="Results from Cross-Correlation";
    print "<input type='hidden' name='navtrail' value=\"$navtrail\"><p>";
    print "<input type='hidden' name='popupFrom' value=\"$popupRef\"><p>";

    # Get the list of tables.
    my ($tables) = Browse::HDB::GetCrossTables();

print qq|
<SCRIPT TYPE="text/javascript" LANGUAGE="JavaScript"><!--
function reshowTable2(object,table) {
    missionTable2 = object.options[object.selectedIndex].text;
    for (var i = table.length;i > 0;i--)
        table.options[0] = null;
    reloading = true;
    showlinksTable2(table);
    table.options[0].selected = true;
    return false;
}

function opt(href,text,table) {
    if (reloading)  {
        var optionName = new Option(text, href, false, false)
        var length = table.length;
        table.options[length] = optionName;
    }
    else
        document.write('<OPTION VALUE="',href,'">',text,'<\/OPTION>');

}

//--></SCRIPT>

|;


    print Browse::Options::SelectTable('table_list' => $tables,
                                'prompt' => '1. Which table do you want to cross-correlate with your results?',
                                'CGI_name' => 'Table2',
                                'multiple' => 0,
                                'no_radius' => 0,
                                'code' => 1,
                                'hint' => 0,
                                'size' => 5);

    CrossChoices($hasRadius, $hasTime,2);

    # Include hidden parameters.
    print "\n";
    print "<input type='hidden' name='Table1' value='$table'>\n";
    my $etable = Browse::CGI::Encode($table);
    my $reqstr = $$in{"req_$etable"};
    print "<input type='hidden' name='Info1' value='$reqstr'>\n";

    print "</form>\n";
    Explain();
    #Browse::Decor::Footer();
    Browse::Decor::EmailFooter();
    Browse::Decor::EndTags();

    exit;

} #eosub--CrossCorrelate
#--------------------------------------------------------------------------

1;
#########
#  EOF  #
#########

