# $Id: DBQ.pm,v 1.18 2002/09/05 18:10:49 cwh Exp $
# $Log: DBQ.pm,v $
# Revision 1.18  2002/09/05 18:10:49  cwh
# Made changes to SYBASE setup.
#
# Revision 1.17  2002/03/15 23:40:22  cwh
# Moved call to fixQuery from execute to oexecute, since the first calls the
#   second.
# Added lines needed for ISDC (Oracle).
#
# Revision 1.16  2001/11/07 23:34:17  cwh
# Moved everything except DBD/ into lib/Browse/, so as not to pollute the module
# namespace with our Sort, Search, CGI, etc.  This required putting Browse:: in
# front of every module call and package declaration.  Also any mod::new() had
# to be changed to mod->new().
#
# Revision 1.15  2001/07/18 19:43:38  cwh
# Added back in 'use DBD::HCS' - Vizier xcorr failed without it.
#
# Revision 1.14  2001/05/21 15:45:07  cwh
# removed some debug statements.
#
# Revision 1.13  2001/05/17 02:25:27  tam
# Added defaultID package variable.
#
# Revision 1.12  2001/04/25 16:49:44  cwh
# Added a little error checking to init.
#
# Revision 1.11  2001/02/15 21:17:57  cwh
# Changed debug info: now inserts newline after every 70 chars in sql echo, so
# we can read the thing!
#
# Revision 1.10  2000/05/26 18:46:02  saima
# Modified so that the DBI fetchrow_array() call is made instead of
#   fetchall_array(). With very large set of results, this was crashing.
#
# Revision 1.9  2000/01/27 18:38:35  saima
# Minor changes to debug mode display.
#
# Revision 1.8  2000/01/27 18:34:17  saima
# Modified DBQ::execute() so that it calls fixQuery() to fix any
#   database server specific sql statements. DBQ::fixQuery() was
#   also modified so that 'set rowcount' query is not sent to the HCS server.
#
# Revision 1.6  1999/10/25 20:10:04  saima
# Forgot to comment out a debug statement.
#
# Revision 1.5  1999/10/21 17:53:12  saima
# Modified code so that it works with Oracle. The returned parameter
#   names are converted to lower case. Also, the 'convert()' syntax is
#   replaced by 'to_char()' syntax for Oracle database.
#
# Revision 1.4  1999/09/03 20:33:21  saima
# Sybase files are now in a standard location.
#
# Revision 1.3  1999/08/26 17:21:03  saima
# Added the types information.
#
# Revision 1.2  1999/05/27 15:30:40  tam
# Initial Revision
#
# The DBQ library allows for convenient access to a 
# database using DBI connections from HEASARC applications.
# The DBQ library establishes the environment needed to
# make connections to the HEASARC's Sybase server, and
# provides simply wrappers for the DBI calls to execute
# queries or database commands.
#
# The DBQ module allows the generation of DBQ objects, but
# it is anticipated that users will typically access the
# static methods of the package and allow the package to
# automatically create or find the appropriate database connection.
#
# The DBI::Registry command allows persistent connections
# to the database and provides an aliasing mechanism
# whereby a given DBI connection can be given a simple name.

# # Developed 1999 May by T. McGlynn based on earlier implementations. #


package Browse::DBQ;

use strict;
use DBI;
use DBD::HCS;

my %dbTypes = (

     1,	"CHAR",
     2, "NUMERIC",
     3, "DECIMAL",
     4, "INTEGER",
     5, "SMALLINT",
     6, "FLOAT",
     7, "REAL",
     8, "DOUBLE",
     9, "DATE",
    10, "TIME",
    11, "TIMESTAMP",
    12, "VARCHAR",
    -1, "LONGVARCHAR",
    -2, "BINARY",
    -3, "VARBINARY",
    -4, "LONGVARBINARY",
    -5, "BIGINT",
    -6, "TINYINT");
    
my %dbPadding = (
     1,	0,
     2, 1,
     3, 1,
     4, 1,
     5, 1,
     6, 1,
     7, 1,
     8, 1,
     9, 0,
    10, 0,
    11, 0,
    12, 0,
    -1, 0,
    -2, 0,
    -3, 0,
    -4, 0,
    -5, 1,
    -6, 1);

### Sybase/installation specific stuff ###
BEGIN {

   # Anything that needs to be done at compile time
   # should go in here.
   if (-e "/heasarc/sybase") {
       $ENV{SYBASE} = '/heasarc/sybase';
       $ENV{SYBASE_OCS} = 'OCS';
   } elsif (-e "/wwwroot/mnt/heasarc/sybase") {
       $ENV{SYBASE} = '/wwwroot/mnt/heasarc/sybase';
   }
   $ENV{LD_LIBRARY_PATH} = "$ENV{SYBASE}/$ENV{SYBASE_OCS}/lib";
}

### Static methods

sub getConnection {  
    return Browse::DBQ::Registry::getConnection(@_);
}

# Send a query and retrieve results.
#
#     ($results, $names, $typesm $precisions, $error) = 
#            Browse::DBQ::query($SQL, @connection)
#
#          $sql        -- The text of the query.
#          @connection -- Either an alias or three element description
#                         of the desired connection (Optional).
#          $results -- A reference pointing to the results.
#                      E.g., $row = $$results[0];
#                            $elementOfResults = $$results[0][0];
#          $names   -- A reference to an array of column names.
#          $types   -- A reference to an array of column types.
#                      The types may vary from DB to DB.
#          $precisions-A reference to an array of column precisions.
#                      These are currently unused and often not
#                      supported in a particular implementation.
#          $error   -- Any error status returned.
#

sub query {

    my ($sql) = shift(@_);
    my $db = Browse::DBQ::Registry::getConnection(@_);
    if (defined($db)) {
        return $db->oquery($sql);
    } else {
        return undef;
    }
}

# Execute a non-query database command.
#
#   $status = Browse::DBQ::execute($sql);
#

sub execute {
    my ($sql) = shift(@_);
    my $db = Browse::DBQ::Registry::getConnection(@_);


    if (defined($db) && $sql ne '') {
        return $db->oexecute($sql);
    } else {
        return undef;
    }
}

# Set the database to be queried (Sybase specific)
# 

sub database {
    excute("use $_[0]");
}

# Retrieve results as columns rather than rows.
# This runs the query and reorganizes the results.
#
# Arguments:
#   $query -- The query to be run.
#
# Returns:
#   $columns  -- A reference to an array of results.  This
#                is the transpose of the $results returned
#                by the Browse::DBQ::query call.
#   $names    -- A reference to an array of column names.
#   $types    -- A reference to an array of column types.
sub cols {

    my ($obj, $query) = @_;
    
    my ($results, $names, $types) = query($obj, $query);
    if ($#$names < 0) {
        return undef;
    }
    my $data;

    my ($i, $j);
    for ($j=0; $j<= $#$names; $j += 1) {
        push(@$data, []);
    }
    
    for($i=0; $i <= $#$results; $i += 1) {
        my $row = $$results[$i];
        for ($j=0; $j <= $#$names; $j += 1) {	
	    $$data[$j]->[$i] = $$row[$j];
	}
    }
    return ($data, $names, $types);
}


#### Constructor for DBQ objects.
#
#   $ref = Browse::DBQ->new($dsn, $name, $pwd);
#
#   Note the use of $ref in the descriptions below


sub new {

    my $type = shift(@_);
    my $conn = {};
    bless $conn, $type;
    my ($dsn, $name, $pwd) = @_;

    my $db;

    eval { $db = DBI->connect($dsn, $name, $pwd,
              {"PrintError" => 1}        );};
    
    if (defined($db)) {
        $$conn{connection} = $db;
	
	return $conn;
    } else {
        return undef;
    }
}

#### Object methods.
#
#
#   Submit a query (see query above)
#
#   ($results, $names, $precisions, $error) = $ref->oquery($sql);

sub oquery {

    my ($obj, $sql) = @_;
    
    my $db = $$obj{connection};
    
    # Make any DB dependent modifications required.
    my $osql = $sql;

    $sql = fixQuery($sql, $db->{Driver}->{Name});

    if ($::debug) {
      # insert newline after every 70 chars, at next space, for readability 
      my ($debugsql) = $sql;
      $debugsql =~ s/(.{70} )/$1<br>/g; 
    }
    
    my $sth = $db->prepare( $sql );
    if (!$sth) {
        return undef;
    }
    
    
    my ($key, $val);
    my $rc = $sth->execute();
    if (!$rc) {
        return undef;
    }
    
    
#    my $data = $sth->fetchall_arrayref;

    my $data = [];
    while (1) {
        my @row = $sth->fetchrow_array;
	if (scalar(@row)) {
            push(@$data, \@row);
	} else {
	    last;
	}
    }
	
   
    if($DBI::err) {
      warn "DBI reporting error in DBQ::oquery(): $DBI::errstr\n";
    } elsif ($::debug && $DBI::err) {
      print "DBI reporting error in DBQ::oquery(): $DBI::errstr\n";
    }
     
    my $names = $sth->{NAME};
    my $types = $sth->{TYPE};
    my $precisions = $sth->{PRECISION};
    
    # Close out the query.
    $sth->finish;
    
    return ($data, $names, $types, $precisions, $DBI::err);
}

# Make any database dependent modifications to a query.
# This allows us to accommodate differences in the SQL
# supported by different machines.
sub fixQuery {

    my ($sql, $db) = @_;

    if ($db eq "HCS") { #rowcount queries not allowed on HCS server
       $sql = '' if ($sql =~ /set\s+rowcount/);
    }
    
    if ($db eq "Oracle") {
    
        # Sybase uses convert(varchar, field) to convert a field to
	# a string.  The roughly equivalent Oracle syntax is to_char(field).
	
        $sql =~ s/convert\s*\(\s*varchar\s*,/to_char(/ig;

        # next 2 lines added by Mohamed Meharga (ISDC)
        $sql =~ s/ _([a-z_0-9]*)/ \"_$1\"/gi;
        $sql =~ s/,([^,]*) as/, to_char\($1\) as/ig ;

    }
    return $sql;
}

# Initiate a query but do not retrieve the results.
#
#   $status = $ref->query_by_row($sql);
#
#       $sql -- The query text
#

sub query_by_row {

    my ($obj, $sql) = @_;

    #  next line added by Mohamed Meharga (ISDC)
    $sql = fixQuery($sql, $$obj{connection}->{Driver}->{Name});

    my $sth = $$obj{connection}->prepare( $sql );
    if (!$sth) {
        return undef;
    }
    
    my ($key, $val);
    my $rc = $sth->execute;
    if (!$rc) {
        return undef;
    }
    
    $$obj{statement} = $sth;
    
    return 1;
						     
}

# Get the next row of a query initiated by query_by_row.
#
#   $row = $ref->nextRow();
#
#   $row:  A reference to the next row of results, or undef
#          if the query is complete.

sub nextRow {
    my ($obj) = @_;
    
    my $result = $$obj{statement}->fetch();
    if (!defined($result)) {
    
        $$obj{statement}->finish();
	delete($$obj{statement});

    }
    return $result;
}
      

# Execute a non-Select command (see execute above)
#
#    $rows_affected = $ref->execute($sql);
#

sub oexecute {
    my ($obj, $sql) = @_;
    # Make any DB dependent modifications required.
    my $conn_info = $$obj{connection};
    $sql = fixQuery($sql, $conn_info->{Driver}->{Name});

    return $$obj{connection}->do($sql);
}


# Disconnect from the database.
sub disconnect {
    my ($obj) = @_;
    
    $$obj{connection}->disconnect;
    delete($$obj{connection});
}

# Indicate datatypes that should be treated as
# numeric
sub padOnLeft {
    my  ($type) = @_;
    return $dbPadding{$type};
}

# Return a string describing a particular type.
sub typeStr {

    if ($#_ > 0) {
        my @results;
	
	my $type;
    
        foreach $type (@_) {
	    push(@results, $dbTypes{$type});
	}
	return \@results;
	
    } else {
        return $dbTypes{$_[0]};
    }
       
}

package Browse::DBQ::Registry;

# This registry package maintains structures to allow
# the user to access the database without maintaining
# local copies of the actual DBQ objects.  The registry
# will create a new connection whenever required.

# The registry also maintains a list of 'translations'
# This allows the user to register a name for a database
# and provide the DBI connection parameters for that database.
#
# E.g., dbsrv.gsfc.nasa.gov might refer to 
#       (dbi:Sybase:SYBASE,hrcdba,passwd).
# At startup an initialization file can be used to configure
# the translations.

my %dbqConnections;		# Active Connections
my %translations;		# Strings to translate to connections.

my ($defaultID, $defaultDbi, $defaultUser, $defaultPwd);


# Initialize the registry.

sub init {

    # Drop connections and clear out translations.
    undef (%dbqConnections);
    undef (%translations);
    
    if (-e $_[0] && open(IN, $_[0])) {
        my @dbs = <IN>;
        close(IN);
	my $line;
	my $count=0;
	foreach $line(@dbs) {
	    chomp($line);
	    if ($line =~ /^\s*\#/  or  $line =~ /^\s*$/) {
	        next;
	    }
	    my @fields = split(" ", $line);
	    my $fld;
	    foreach $fld(@fields) {
	        $fld =~ /^\s*(.*)\s*$/;
		$fld = $1;
	    }
	    if ($count == 0) {
                $defaultID   = $fields[0];
	        $defaultDbi  = $fields[1];
		$defaultUser = $fields[2];
		$defaultPwd  = $fields[3];
                print "default db: @fields<br>\n" if $::debug;
	    }
	    addTranslation(@fields);
	    $count += 1;
	}
    } else {die "Database INIT error: $! $_[0]<br>\n";}
    return \%translations;
}

# Explicitly set the database default.
sub setDefault {       
    ($defaultDbi, $defaultUser, $defaultPwd) = @_;
}

# Get the default database.
sub getDefault {	
    return ($defaultID, $defaultDbi, $defaultUser, $defaultPwd);
}

sub addTranslation {	# Add a conversion from a name to connection parameters.
    my @fields = @_;
    my $name = shift(@fields);
    $translations{$name} = \@fields;
}

sub getTranslation {	# Translate a name to connection parameters.
    my ($name) = @_;
    return $translations{$name};
}

sub delTranslation {	# Delete a translation
    my ($name) = @_;
    delete($translations{$name});
}

sub getConnection {

    # This is called with 0, 1 or 3 parameters.
    # 0 -- Use default connection
    # 1 -- Need to translate a name
    # 3 -- Connection parameters are given explicitly.
    
    my @flds = getThreeFields(@_);

    my $join = join("|", @flds);
    if (defined($dbqConnections{$join})) {

        my $conn = $dbqConnections{$join};
        return $dbqConnections{$join};
	
    } else {
    
	my $dbq = Browse::DBQ->new(@flds);

	if (defined($dbq)) {
	    $dbqConnections{$join} = $dbq;
	}
	return $dbq;
    }
}

# Get the DBI database parameters for the users
# input.

sub getThreeFields {

    if ($#_ < 0 || length($_[0]) <= 0 || $#_ > 2) {
        return ($defaultDbi, $defaultUser, $defaultPwd);
    } elsif ($#_ == 0) {
        return @{getTranslation(@_)};
	
    } else {
        return @_;
    }
}

# Drop a connection.
sub delConnection {
    my $join = join("|", getThreeFields(@_));
    if (defined($dbqConnections{$join})) {
        $dbqConnections{$join}->disconnect();
    }
    delete($dbqConnections{$join});
}
1;
