# $Id: ExcelDisplay.pm,v 1.8 2007/10/09 14:10:50 lmm Exp $

# NOTE: doesn't work with mod_perl

use strict;

package Browse::ExcelDisplay;

use vars qw( @ISA );
use Spreadsheet::WriteExcel;
@ISA = qw( Browse::TextDisplay );
use Browse::TextDisplay;


# Minimize information at the top of the page.
sub pageHeader {

    my ($self, $navtrail) = @_;
    $self->setCoordFormat("decimal");
    $self->setDateFormat("mjd");
    print "Content-Type: application/vnd.ms-excel\n" .
	"Content-Disposition: attachment; filename=browse_results.xls\n\n";

    binmode(STDOUT);
    my $workbook  = Spreadsheet::WriteExcel->new(\*STDOUT);
    $workbook->set_tempdir($::tempdir);
    my $overview = $workbook->addworksheet("Overview");
    $$self{"excelobj"} = $workbook;
    $$self{"exceloverview"} = $overview;
    $$self{"intable"} =1;
}

sub tableHeader {
}

sub legend { }

sub tableFooter {
  my ($self,$html) = @_;
    return unless $html =~ /\S/;
    my $overview = $$self{"exceloverview"};
    my $xxx =$overview->write($$self{"overviewcount"},0,$html);
            if ($xxx == 0) {
        my $workbook = $$self{"excelobj"};
       $workbook->close();
       #     close(STDOUT);
    print "Content-Type: text/html\n\n";
    print "Error!";

            }
    $$self{"overviewcount"} += 1;
  
}
sub Directions {
}
sub showText {
}

sub closeTable {
    my ($self, $data) = @_;

    if ($$self{"intable"} and $$self{"meta"}{"table"}) {
        $$self{"intable"} = 0;

        my $workbook = $$self{"excelobj"};
        my $overview = $$self{"exceloverview"};
        my $sheetname = $self->chopTable($$self{"meta"}{"table"});
        $sheetname = substr($sheetname,0,30); # Excel limit
        my $worksheet = $workbook->addworksheet($sheetname);
        $worksheet->freeze_panes(1, 0); # freeze header row


        my $header = $workbook->addformat();
        $header->set_bold();
        $header->set_align('center');


        my $array = $$self{"dataarray"};
        my $names = $$self{"names"};
        my $types = $$self{"types"};
        my $table_info = $$self{"meta"}{"tableinfo"};

        my $nskip = 0;
        while ($nskip <= $#$names) {
            if (substr($$names[$nskip], 0, 3) !~ /w3_/i) {
                last;
            }
            $nskip += 1;
        }

        # Get rid of the unique key columns.
        splice(@$names, 0, $nskip);
        splice(@$types, 0, $nskip);
        my $namecnt=0;
        foreach my $colname (@$names) {
             $worksheet->write_url(0,$namecnt++, 
             $$table_info{"table_document_url"}."#".$colname, $colname);
        }

#        $worksheet->write_row(0,0,$names,$header); #plain text names

        my $row;
        my $i=1;  # 0 is the column header row
        # Loop over each row in the ouput.
        foreach $row (@$array) {
            splice(@$row, 0, $nskip);
            $worksheet->write_row($i,0,$row);
            $i++;
        }
        $i-=1;
        $overview->write($$self{"overviewcount"}++,0,"$i results found in $$self{meta}{table}"); 
    }
}


sub pageFooter {
    my ($self) = @_;
    my $workbook = $$self{"excelobj"};
    my $overview = $$self{"exceloverview"};
    $overview->write($$self{"overviewcount"}+2,0,"NOTE: Excel limits the number of columns to 254, and limits the number of rows to 65536. Results may be truncated.");
    $workbook->close() || Browse::Utils::Error("An error has occured while closing the spreadsheet file:  $! "); 
}

sub showEmpties {
# nothing for now, need to do something though.
}

sub YourSearchWas {
    my ($self,$positions,$qualifiers,$sorts,$dirs,$resmsg) = @_;

    use Browse::TableFormat;
    my $maxrows=$self->MaxRows;
    my ($coord_sys,$equinox) = $self->getCoordSys();
    my $workbook = $$self{"excelobj"};
    my $overview = $$self{"exceloverview"};
    $overview->hide_gridlines(2);
    my $row = $$self{"overviewcount"} || 0;

    #These are wrong if there are more than one sort param - fix
    my ($sortinfo,$sort);
    foreach $sort (@$sorts) {
        $sortinfo .= $sort;
    }
    my ($dirinfo,$dir);
    foreach $dir (@$dirs) {
        $dirinfo .= $dir;
    }

    my $radius;
    my($obj,$pos,@object_info);
    foreach $pos (@$positions){
       my ($coord1,$coord2,$rad,$name,$resolver) = split('\|',$pos);
       $radius = $rad;

       my $format = "decimal" if $coord_sys =~ /gal/i;
       ($coord1,$coord2)  = Browse::TableFormat::fromJ2000($coord1,$coord2,$coord_sys,$equinox,$format);
       $coord1 .= "&#176;" if $format =~ /decimal/;
       $coord2 .= "&#176;" if $format =~ /decimal/;
       $coord1  =~ s/\s+/ /g;
       $coord2  =~ s/\s+/ /g;

       if ($name =~ /[^\d\.\+\-\s\,]/) {
            $obj = "$name, resolved by $resolver " .
                  "to [ $coord1, $coord2 ]";
       }else{
            $obj = "$name";
       }
       push(@object_info, $obj);
    }

    my ($num2,$qual,$qual_info,@qual_info);
    foreach $qual (@$qualifiers){
       my ($field,$op,$min,$max) = split('\|',$qual);
       $min = DateTime::mjd2iso($min) if defined $min and $field =~ /metaparam_start_time/;
       $max = DateTime::mjd2iso($max) if defined $max and $field =~ /metaparam_start_time/;
       $field = "$1" if $field =~ /_metaparam_(.*)/;  # add formatting?
       $qual_info .= " AND<split>" if $num2;
       if ($op =~ /between/) {
          (my $not, $op) = $op =~ /(!?)(.*)$/;
          $not = "not" if $not;
          if (!$min or $min =~ /'\s?'/) {
             if (!$not) {
                 $qual_info .= "$field <= $max";
             } else {
                 $qual_info .= "$field > $max";
             }
          } elsif (!$max or $max =~ /'\s?'/) {
             if (!$not) {
                 $qual_info .= "$field >= $min";
             } else {
                 $qual_info .= "$field < $min";
             }
          } else {
             $qual_info .= "$field $not $op $min and $max";
          }
       } else {
          $qual_info .= "$field $op $min";
       }

       $qual_info  =~ s/\s+/ /g;
       @qual_info = split ('<split>',$qual_info);
       $num2++;

    }

    if ($radius =~ /\d/) {
        my $radunit = "deg";
        if ($radius < 1) {
           $radius *= 60;
           $radunit = "\'";
        }
        if ($radius < 1) {
           $radius *= 60;
           $radunit = "\"";
        }
        $radius = sprintf("%.0f", $radius) . " $radunit";
    }

    #print search criteria
    # need bold format
my $center  = $workbook->addformat(align => 'center');
my $left  = $workbook->addformat(align => 'left');
my $centerbold = $workbook->addformat(align => 'center', bold => 1);
my $bold = $workbook->addformat(bold => 1);
my $italic = $workbook->addformat(italic => 1);
my $directions = $workbook->addformat(color => 'red');
$overview->set_column(0,0,20);
$overview->set_column(1,1,300);

    $overview->write($row++,0,"Click on the tabs at the bottom of the screen to access the results for each table.",$directions);
    $row++;
    $overview->write($row++,0,"Search was based on:",$bold);

    if (scalar(@object_info)) {
      $overview->write($row,0,"Object/Coordinates:");
      foreach my $objinfo (@object_info) {
        $overview->write($row++,1,"$objinfo");
      }
    }

    my $cooline = "$coord_sys";
    $cooline .= ", equinox  $equinox" if ($coord_sys !~ /Galactic/i);
    $overview->write($row++,0,["Coord. System:",$cooline]);

    $maxrows = "(no limit)" unless $maxrows;
    $overview->write($row++,0,["Maximum Rows:","$maxrows"],$left);
    if (scalar(@qual_info)) {
      $overview->write($row,0,"Parameter Query:");
      foreach my $qualinfo (@qual_info) {
        $overview->write($row++,1,"$qualinfo");
      }
    }
    $overview->write($row++,0,["Search Radius:","$radius"],$left) if scalar(@object_info);

    $dirinfo = "" if $dirinfo =~ /down/ or !$dirinfo;
    $dirinfo = ", reverse order" if $dirinfo =~ /up/;
    $overview->write($row++,0,["Sorted by:","${sortinfo}$dirinfo"]) if $sortinfo;
    $row++;  # blank line
    $overview->write($row++,0,"Dates have been converted to MJD.");
    $overview->write($row++,0,"Coordinates have been converted to decimal degrees.");
    $$self{"overviewcount"} = $row + 5;   # several blank lines
} # end of YourSearchWas

sub resultsSummary {}
sub beforeNewTable {}
sub afterTable {}

1;
# $Log: ExcelDisplay.pm,v $
# Revision 1.8  2007/10/09 14:10:50  lmm
# Added empty methods beforeNewTable() and afterTable() .
#
# Revision 1.7  2005/11/22 16:10:41  lmm
# Added empty resultsSummary method
#
# Revision 1.6  2005/02/15 16:04:19  lmm
# Made changes to WriteExcel->new() call to work better with modperl
#    (code provided by Micah Johnson)
#
# Revision 1.5  2004/12/09 19:29:26  lmm
# Combined Content-type and Content-Disposition lines for use with modperl
#   (requested for Akbar)
#
# Revision 1.4  2003/07/31 19:10:38  cwh
# Overriding legend subroutine.
#
# Revision 1.3  2003/04/28 02:25:20  cwh
# Now calling chopTable as a method.
#
# Revision 1.2  2002/10/23 16:50:29  cwh
# Carried changes to YourSearchWas here.
#
# Revision 1.1  2002/05/14 18:19:00  cwh
# Excel display module.  Creates Browse output in Excel workbook, with each
# table's results in a separate worksheet.  The cover sheet is an overview
# with instructions and the YourSearchWas query summary.
#
