#$Id: HProdScriptGen.pm,v 1.21 2008/01/03 16:44:37 lmm Exp $
# Log of changes is at end of file

use strict;

use Browse::ProdMain;
use Browse::Log;
use Browse::DBQ;
use Browse::HDB;
use Browse::Utils;
use LWP::Simple;
use Sys::Hostname;

package Browse::HProdScriptGen;
# Inheritance.
use vars qw( @ISA );
@ISA = qw( Browse::ProdMain );

#----------------------------------------------------------------------------

sub doProd {
   my ($self,$val) = @_;
   my $url = $$val{'dp_url'};
   print "HProdScriptGen:looking at file $url...<br>" if $::debug;

   if ($url =~  /ftp\:\/\/[^\/]*(\/.*)/) {
         my $file = "/FTP$1";
         $file = $url unless -s $file;
         
         if ($self->fileFilter() !~ /\w/) {   
            #-- if no filter save file or dir, duplicate file check will apply
            $self->Request($file);
         } else {
            #--- if file filter do not save directories.  Apply filter to files
            #--- in directories
            my $filelist=();

            if ($$val{'dp_format'} =~/DIRECTORY/) {
               #--- descend directories if current product is a directory
             
               $filelist = $self->descendtree($url, $filelist);
               foreach my $fileFromDir (@$filelist) {
                 if ($self->applyFileFilter($fileFromDir)) {
                    $self->Request($fileFromDir);
                 }
               }
            } else {
              #--- data product is a file so apply filter directly
              print "HProdScriptGen::FILE adding $file<br>" if $::debug;
              if ($self->applyFileFilter($file)) {
                  $self->Request($file);
              }
         }
      }
   } else {
      if ($self->applyFileFilter($url)) {
           $self->Request($url);
      }
   }
}

sub finish {
    my ($self,$in) = @_;
    foreach (keys %$in) {
       if (/^File/) {
          foreach my $file (split("\0",$$in{$_})) {
            $self->addUrltoReq($file);
          }
       }
    }
    $self->startRequest($in);
}

sub pageHeader {
   my ($self,$in) = @_;
   Browse::Decor::Header("Data Products Download Commands",0,0,0,"dprodtable" );

   my $navtrail = $in->{'navtrail'};
   unless ($navtrail) {
      my $referrer = ($ENV{"HTTP_REFERER"} or "/".$::cgiurl."/w3browse.pl");
      $navtrail = "<a class=navpast href=\"$referrer\">  " .
	"Data Products Download Commands</a>";
   }
   print "$navtrail &gt; <span class=navcurrent>" .
	"Data Products Download Commands</span><br><p>\n";

}

sub pageFooter {
    my ($self) = @_;
    #--- future feature 
    ###$self->displayInstructions();
    Browse::Decor::Acknowledge();
    Browse::Decor::EmailFooter();
    Browse::Decor::EndTags();

}

#--- run special tar script to get file names
sub dotarscript {
    #my ($self,$file,$allfiles) = @_;
    my ($self,$file,%allhash) = @_;
    # need to stay on same host so I know where to look for tarfile
    my $host = Sys::Hostname::hostname();
    $file =~ s/heasarc|heasarcdev/$host/;

    if ($::debug ) {$file =~ s/cgi-bin\/W3Browse/cgi-bin\/w3bdev\/W3Browse/;}

    #my $filelist=join(',',@$allfiles);

    #--- use listonly parameter to get list of files
    my $lst=LWP::Simple::get($file."&listonly=1");

    #-- convert to hash to send to duplicate file check routine
    my @tarlist = split(/\n/,$lst);  
    my %tarhash = {};
    for (@tarlist) {
       #$_ =~ s/\/$//;
       #$tarhash{$_} = 1;
       my $hashname = $_;
       $hashname =~ s/\/$//;
       $tarhash{$hashname} = 1;
    }
    #my %allhash = {};
    #for (@$allfiles) {
       #$_ =~ s/\/$//;
       #$allhash{$_} = 1;
    #}

    print "HProdScriptGen::dotarscript:$file<br>" if $::debug;
    my $size;
    my $total=0;
    
    foreach my $ff (@tarlist) {
        #--- Before processing file check if it is already within 
        #--- a directory included in file list returned from the script
        #--- as well as from the original file list
        #print "tarscript $ff <br>" if $::debug;
        #--- Use http commands to follow xte links
        $ff =~ /^([^;\s]*)$/;   #--- make sure it is a file name
        $ff = $1;
        #print "---> altered tarscript $ff <br>" if $::debug;

        $size=0;
        if ($ff) {
           if ( (!Browse::Utils::isDuplicate($ff, \%tarhash) )  &&
                (!Browse::Utils::isDuplicate($ff, \%allhash) ) ) {
              my $cutDir = findCut($ff);
              $self->commandStart();
              print "wget -q -nH --cut-dirs=$cutDir  -r -l0 -c -N -np " .  
		"http://" . $::host . "$ff" ;
              $self->commandEnd();
           }
           $size= $self->fileSize($ff);
           print  "* script size=$size --> $ff\n" if $::debug;
     
        } 
        $total += $size;
        #print "* total after file check $total<br>" if $::debug;
    }
    

    return $total;
}


sub startRequest {

    my ($self,$in) = @_;
    #my $msg = "No data products found.<br><br>Please go back and make" .
	#" sure you have at least one table row and at least one data" .
	#" product type selected.  If so, the reason may be that we do" .
	#" not have any data products of the type selected for the table" .
	#" rows selected - for example, an observation you chose may be" .
	#" scheduled, or performed, but not yet in the archive.";
    unless ($self->Request()) {
         #Browse::Utils::Error("$msg");
         $self->noProductsMessage();
    }
    my @files = keys %{$self->Request()};
   
    unless (scalar(@files)) {
    #Browse::Utils::Error("$msg");
         $self->noProductsMessage();
    }
    print "HProdScriptGen:startRequest " .scalar(@files) . " files" if $::debug;
    $self->commandHeader($in);

    my $tflag = "c";
    my $size;
    my $total=0;
    
    #---- convert file list to hash for duplicate file checking
    #---- remove slash for duplicate checking but keep for listing in script
    my %tarhash = {};
    for (@files) {
       #$_ =~ s/\/$//;
       #$tarhash{$_} = 1;
       my $hashname = $_;
       $hashname =~ s/\/$//;
       $tarhash{$hashname} = 1;
    }
    # determine if file is local, remote or XTE and call appropriate 
    # subroutine
    foreach my $file(@files) {
       print "<br>------ new file $file<br>" if $::debug;
     
        if ($file =~ /[\!\$\;]/) {
            print "Invalid file: $file<br>\n";
	} elsif ($file =~ /xteTar\.pl/) {
            $size=$self->dotarscript($file,%tarhash);
            $total += $size;
	} elsif ($file =~ /\:|%3A/i) {
            #print " HProdScriptGen: remote file=$file<br> " if $::debug;
	    $self->doremotefile($file);
        } else {
            #print " HProdScriptGen: local file=$file<br> " if $::debug;
            #--- Before processing file check if it is already within 
            #--- a directory included in file list 

            #print "* local file size=" . $self->fileSize($file) .
		#"<br>" if $::debug;
            $total += $self->fileSize($file);
	    if (!Browse::Utils::isDuplicate($file,\%tarhash)) { 
               $size=$self->dolocalfile($file);
               #$total += $size;

            }
       }
    }
    $self->commandFooter($total);
    my $savetables = join(',',@{$self->Tables()}) || $$in{'savetables'} || 
		"unknown";
       Browse::Log::CommandsDisplayed($savetables,$total/1024);
}


#--- Create download commands for remote data products, some may be output 
#--- from CGI scripts
sub doremotefile {
     my ($self,$file) = @_;
     $file =~ /^([^;\s]*)$/;
     $file = $1;

     my $cutDir = findCut($file);
     if ($file =~ /http/) {
        #--- http and possibly cgi link (not ftp)
        $self->commandStart();
        print "wget --html-extension" .
	   "  '$file'\n";
        $self->commandEnd();
     } else {
        $self->commandStart();
        print "wget -q -nH --cut-dirs=$cutDir  -r -l0 -c -N -np -R 'index*' " .
	   "-erobots=off --retr-symlinks  " .  "$file";
        #print "wget -q -nH --cut-dirs=$cutDir  -r -l0 -c -N -np " .  "$file";
        #print "wget --passive-ftp -q -nH --cut-dirs=$cutDir  -r -l0 -c -N " .
	#"-np --retr-symlinks" .  "  $file";
        $self->commandEnd();
     }
     my $tflag = "c";
     return $tflag;
}

#--- Create download commands for local files
sub dolocalfile {
     my ($self, $file) = @_;
     $file =~ /^([^;\s]*)$/;
     $file = $1;

     my $size=0;
     if ($file) {
        my $cutDir = findCut($file);
        $self->commandStart();
        print "wget -q -nH --cut-dirs=$cutDir  -r -l0 -c -N -np -R 'index*' " .
		" -erobots=off --retr-symlinks " .  
		"http://" . $::host . "$file" ;
        #print "wget -q -nH --cut-dirs=$cutDir  -r -l0 -c -N -np " .  
		#"http://" . $::host . "$file" ;
        #print "wget --passive-ftp -q -nH --cut-dirs=$cutDir  -r -l0 -c -N " .
		#"-np --retr-symlinks ftp://" . $::ftphost . "$file" ;
        $self->commandEnd();
        #$size= $self->fileSize($file);
        #print " in local $file size = $size<br>";
     
     } 
     return $size;
     
}

#------------------------------------------------------------------------
#--- Cind "//" in data products path to indicate start directory of path 
#--- of downloaded files.   if there is more than one it picks the first
#--- if there is a host ("://") it finds the first occurrence after the
#--- host name
sub findCut {
   my ($file) = @_;
   my $slash = "/";
   my $slashCount=0;
   my ($start, $cut);
   my $hasHost=0;

   # finds first occurrence of //
   my $first = index($file, "//"); 

   # if // part of host name find next occurrence.  if not stick
   # with first index
   if (substr($file,$first-1,1) eq ":")  {
      $hasHost=1;
      $cut =index (substr($file,$first+2),"//")+$first+2;
   } else {
      $cut=$first;
   }

   if ($cut < 0 ) {return 2;}   #arbitrary choice if no // found??

   if ($hasHost) {
      #--- start counting directories after host name 
      $start =index (substr($file,$first+2,$cut),"/")+$first+2;
   } else {
      $start=0;
   }

   #--- extract path after host name (if it exists) and before //
   my $path = substr($file, $start, $cut-$start);

   #--- count directories in path
   my $pos = -1;
   while (($pos = index $path, $slash, $pos) >-1) {
     ++$slashCount;
     $pos++;
   }
   return $slashCount;
}

# Ctml for start of command line
sub commandStart {
   my ($self)=@_;
   print "<nobr><strong>";

}
# Ctml for end of command line
sub commandEnd {
   my ($self)=@_;
   print "</strong></nobr><br>\n";
}

# Text for bottom of html page
sub commandFooter {
   my ($self,$total)=@_;
   print "<p><p>Total size of data product files local to the " .
	"HEASARC system: " . 
   $self->optsize($total) .  "<br> File sizes of remote data products are
        not available";
}

# Text for top of html page
sub commandHeader {
    my($self,$in) =@_;
    # Some instructions
    print <<EOT;
    <strong>The following commands can be used to download selected data products:</strong> <p>
    Copy and paste lines to a local shell script or command line 
    or download commands to a file <form method='POST' name='w3hdprods' action='w3tar.pl'> <input type='submit' value='Download Commands To File' name='Download Script To File'>
    <p>

    Note: The network utility <i>wget</i> is included with most 
    systems.  To download <i>wget</i> or get more information visit the 
    <a href="http://www.gnu.org/software/wget/wget.html">GNU website</a>
    <p>
EOT


    print "<input type='hidden' name='savetables' value='".
	join(',',@{$self->Tables()})."'>\n";

    #--- Separate certain key/value pairs to format for processing
    foreach my $key (keys %$in) {
       if ($key =~ /^RetrieveSet|^File|^Target/) {
          foreach my $val (split("\0",$$in{$key})) {
             print "<input type='hidden' name='$key' value='" . $val . "'>\n";
          }
       } else {
          #print "<input type='hidden' name='$key' value='" . $in->{$key} . 
		#"'>\n";
          print <<EOT;
          <input type="hidden" name="$key" value="$$in{$key}">
EOT
       }
}

print <<EOT;
</form>
EOT

}

#
#future method
###sub displayInstructions {

   ###my ($self) =@_;
   ###my $tables = $self->Tables();
   ###my $sql;
###
   ###foreach my $table (@$tables) {
     ###$sql="select download_instructions from zzext where table_name = '$table'";
     ###my ($results) =Browse::DBQ::query($sql);
###
     ###my %tinfo = Browse::HDB::TableInfo($table);
###
     #####--- Log query
     ###my $loc = $tinfo{table_location};
     ###Browse::Log::SqlLogger($sql,$loc);
###
     ###foreach my $res (@$results) {
        ###my $filename = shift (@$res);
        ###my $filename="$::root/docs/integral/browse_data_download.html";
        ###print "<hr><p><b>More information for " .$tinfo{table_description} . 
		###" products:</b><br>";
      ### 
        ###print Browse::Utils::includefile($filename) if -s $filename;
     ###}
   ###}
###}



1;
#$Log: HProdScriptGen.pm,v $
#Revision 1.21  2008/01/03 16:44:37  lmm
#Added '/' to end of directories to avoid downloading unwanted
#  directories.
#
#Revision 1.20  2007/10/03 20:08:58  lmm
#Repositioned fileSize() commands to make sure files that are found by
#  isDuplicate() are counted
#
#Revision 1.19  2007/10/03 15:16:39  lmm
#Changes to call more efficient Utils::isDuplicate().
#
#Revision 1.18  2006/10/24 18:55:16  lmm
#Revised wget options for downloading directories as well as files.
#Replace "no products" error message with call to new class method
#  noProductsMessage().
#
#Revision 1.17  2006/07/11 13:45:18  lmm
#Fixed problem with quotes on hidden form input.
#
#Revision 1.16  2006/06/26 20:33:00  lmm
#Switched wget commands back to "ftp" commands for all missions other than
#  XTE due to problems with wget "http" commands  for other missions
#
#Revision 1.15  2006/06/01 15:04:42  lmm
#Made changes to wget to use http instead of ftp (suggested by ed sabol)
#
#Revision 1.14  2005/11/21 18:59:52  lmm
#Added user specified file filter check for remote files.
#
#Revision 1.13  2005/10/31 21:40:05  lmm
#Added --passive-ftp switch to wget commands to allow commands to work
#  from behind a firewall
#
#Revision 1.12  2005/09/19 17:40:46  lmm
#Removed debug statement
#
#Revision 1.11  2005/09/19 17:24:28  lmm
#Added call to descendtree() and applyFileFilter() to correct file
#  filter bug.
#Added timestamp option to wget command to allow for resumption of
#  interrupted sownloads.
#Table name and data size now logged in tar log file
#
#Revision 1.10  2005/06/23 19:37:53  lmm
#Better handling of duplicate file name display for xte listing
#
#Revision 1.9  2005/05/16 15:14:44  lmm
#Local files are not included in download command list if they are
#  located in a directory included in the data products file list
#
#Revision 1.8  2004/12/29 14:59:37  lmm
#Made a minor wording change
#
#Revision 1.7  2004/12/15 21:00:09  lmm
#Moved Download Script to File button to make it clear that it can be
#  used as an alternative to copy/paste
#Made commands boldfaced
#
#Revision 1.6  2004/12/06 16:48:46  lmm
#Small change to remove page extra line break after remote file command
#
#Revision 1.5  2004/12/06 16:36:18  lmm
#Broke apart some code cwand created new methods for overwriting in
#  inherited classes
#Added Download Script To File button
#
#Revision 1.4  2004/10/20 20:52:59  lmm
#Added --retr-symlinks option to wget in one other place
#
#Revision 1.3  2004/10/20 20:41:54  lmm
#Added wget option to follow symboloc links
#
#Revision 1.2  2004/10/08 14:40:36  lmm
#Added displayInstructions() which will include mission specific
#  instructions.  (for future use)
#Corrected call to method dolocalfile()
#Made some changes to general instructions for using commands
#
#Revision 1.1  2004/10/04 20:10:21  lmm
#Creates download commands for data products
#
