# $Id: Options.pm,v 1.51 2006/06/23 14:55:16 lmm Exp $
# $Log: Options.pm,v $
# Revision 1.51  2006/06/23 14:55:16  lmm
# Added Tabbed Display to output format options
#
# Revision 1.50  2006/05/24 14:34:53  lmm
# Changes made for new GRB name resolver
#
# Revision 1.49  2005/06/23 19:17:44  lmm
# Added Some HTML encoding
#
# Revision 1.48  2004/11/08 19:54:22  lmm
# Fixed little bug related to previous change
#
# Revision 1.47  2004/11/08 19:38:35  lmm
# Transferred hard-coded name of include file to Config.pm to allow
#   more flexibility for other users of Browse API
#
# Revision 1.46  2004/03/02 21:36:40  lmm
# Removed leading blank space in the mission name in the Javascript code
#    that builds the mission table menu for the Cross Correlation page.
#    The Safari browser does not ingore this leading space.
#
# Revision 1.45  2003/04/28 01:21:08  cwh
# Added Pure Text Display output format.
# Added new table style.
# Added labels for 508 compliance.
#
# Revision 1.44  2003/02/25 19:29:44  cwh
# Added default for Radius_unit, added it to the buildstate routine to remember
# previous setting.
#
# Revision 1.43  2003/01/24 18:18:29  cwh
# Added VOTable output option to xcorr.
# Fixed bug where the radius unit was mislabeled for xcorr forms.
#
# Revision 1.42  2002/12/16 18:18:32  cwh
# Added quotes where needed.
# Added labels, ids, and scopes for section 508 compliance.
# No longer encoding $info.
# Removed old radius form.
# Now generating 'select a mission' form element with javascript so that it
#   doesn't show up if javascript is turned off.
# Fixed bug where table name always reverted to the first table in the first
#   mission.
# Added noscript section which prints the old table select menu.
#
# Revision 1.41  2002/10/17 18:33:44  cwh
# Shortened arcminutes and arcseconds to arcmin and arcsec.
#
# Revision 1.40  2002/09/13 21:08:13  cwh
# Removed form start tag from Header.
# Changed the order of the default name resolvers.
# Changed the invocation of tickTable.
# Added Excel display option.
# Changed the radius input - now value and unit are separate.
# Added '.' to allowable chars in radius and time.
# New xcorr table selection interface.
#
# Revision 1.39  2002/04/10 22:07:58  cwh
# Changed sort keys for table (goes with change to Sort.pm).
# Moved finding out whether tables have data to here from w3table.pl.
# Set the max priority to null rather than def, so that all tables are queried.
#
# Revision 1.38  2002/04/03 15:39:43  cwh
# "All" checkbox is checked by default on table list page since all tables are
#   pre-checked now.
#
# Revision 1.37  2002/04/02 23:45:35  cwh
# Added code so that all options can be pre-specified in the URL.
# Fixed bug where the coord system was being overridden because the default
#   equinox was being set too early.
#
# Revision 1.36  2002/03/22 22:54:24  cwh
# Changed default # rows retrieved from 50 to 1000.
#
# Revision 1.35  2002/03/07 23:37:40  cwh
# Added additional flag for determining if a table is vizier for w3table list.
#
# Revision 1.34  2002/02/14 18:05:15  cwh
# HTML cleanup: added  type="text/javascript" to <script> elements, changed
#    <font> elements to <span> elements.
# Now default tables are pre-checked in table list.
#
# Revision 1.33  2001/12/12 18:40:03  cwh
# Fixed error where the Coordinate System setting wasn't being passed to the
# next form correctly.
#
# Revision 1.32  2001/11/20 15:51:04  cwh
# Priority can legally be 0, can't do $pri ||= 5.
#
# Revision 1.31  2001/11/08 20:20:39  cwh
# Added Browse:: to sub calls.
# Moved &HTML::Encode() to minimalencode() in this package, since here is the
#   only place that used it.
#
# Revision 1.30  2001/10/25 22:21:37  cwh
# Added Proposal to the table types on by default.
# Changed the labels for the sort arrows, and reversed their meaning.
#
# Revision 1.29  2001/10/25 19:09:18  cwh
# Fixed bug where if result limit set to 0 (no limit), it was overridden
#   on the next form.
# Fixed display bug if there is only one table in a mission.
# Added (commented out) hook for non-table display of results.
#
# Revision 1.28  2001/09/15 13:44:49  cwh
# Changed label for displaymode FITS from 'FITS' to 'FITS table'.
# Changed cross-correlation flag for radius from 'C' to 'R'.
#
# Revision 1.27  2001/08/28 19:43:39  cwh
# Removed extra 'print' from Decor::Header line.
# New OptionBar, just includes inc_summary.html and evals it with the states.
# in BuildState:
#  - simplified code in setting defaults
#  - removed TextDisplay from checkbox section
#  - removed GIFsize
# in TableHeadings:
#  - changed all hardcoded icon directories to use $::images
#  - changed Catalog to Description
#  - changed Short Form to Catalog
#  - changed 'tape' image to Data
#  - changed (') to Default Radius (')
# In NRSelection: fixed typo in setting the state of CheckCaches/SIMBAD/NED
#
# Revision 1.26  2001/08/08 20:53:50  cwh
# Added name for the form in case we want it some day.
# Changed Output Format section so that the state is preserved from
#    the prev page.
# Changed the labels in the Name Resolver section so 'Caches' is hidden
#    from users.
#
# Revision 1.25  2001/07/31 14:52:15  cwh
# Added HTML/text/FITS output selection.
# Added alt text to the 'tape2.gif' images.
#
# Revision 1.24  2001/07/23 16:49:52  cwh
# Added CheckCaches/NED/SIMBAD and CheckCaches/SIMBAD/NED to name
#   resolver options.
#
# Revision 1.23  2001/07/01 02:29:30  cwh
# Removed HEASARC-specific URLs.
# Added 'All parameters' option back in for non-parameter form pages.
#
# Revision 1.22  2001/07/01 01:41:19  cwh
# Changed cgi-bin to viz-bin. On some servers either will work, on others only the latter works.
#
# Revision 1.21  2001/06/20 22:57:33  cwh
# Got rid of $::writeHeader in calls to ::Error.
#
# Revision 1.20  2001/06/08 19:41:44  cwh
# Just added a few more selections for max rows and search radius.
#
# Revision 1.19  2001/05/17 17:51:01  cwh
# Slight tweaks to tablenames.
#
# Revision 1.18  2001/05/03 17:58:19  cwh
# Added link to help on wildcards, case sensitivity and spaces, shows up only on
#    parameter search form.
# Changed sort arrow key so it never shows up on parameter search form.
# Removed the checkbox for "All parameters".  Now done in w3query.pl.
#
# Revision 1.17  2001/04/25 17:05:17  cwh
# added Tom's VizieR flags, changed default #rows from 1000 to 50
#
# Revision 1.16  2001/04/03 16:31:27  cwh
# - added key to sort arrows in the left menu, and alt text to the sort arrows
# - added Action=Advanced Search to the url for the sort arrows
# - added priority listing (currently only used if debugging)
#
# Revision 1.15  2001/03/12 16:35:02  cwh
# TableHeadings has been changed so that it can set up sorts that can happen
# without requerying the database. Also adds a column for table priority if debug
# is on.
#
# DisplayTables has been changed so that if it gets a table without metadata, it
# gets the metadata.  Also it displays the table priorities if debug is on.
#
# Revision 1.14  2001/02/13 22:33:46  cwh
# Committing Ed Sabol's change to encode the table descriptions: <, >, and & didn't appear correctly in the table descriptions.  They are now HTML encoded.
#
# Revision 1.13  2001/01/02 23:32:52  cwh
# Reads $selected flag to see if we need to print the word 'selected' in the
# header of table search results.
#
# Changed Post to POST.
#
# Deleted GIFSelection().
#
# Added target=helpwin for all help links.
#
# Added tabletypes to output column headers.
#
# Revision 1.12  2000/07/26 18:17:25  saima
# Removed reference to $in->{'JavaPlot'}.
#
# Revision 1.11  2000/03/09 22:10:09  saima
# Default ResultMax rows displayed set to 1000 rows.
#
# Revision 1.10  2000/03/08 20:10:59  saima
# W3Browse to Browse name change.
#
# Revision 1.9  1999/12/20 17:54:20  saima
# Modified the default number of rows returned to 250 rows.
#
# Revision 1.8  1999/09/13 18:24:54  saima
# Added links to w3browse help files.
#
# Revision 1.7  1999/09/03 20:39:18  saima
# Fixed a bug in which 12 hrs was incorrectly set to '5"'. Also,
#   links are now relative.
#
# Revision 1.6  1999/08/26 19:33:28  saima
# Removed Java Plotting option from the options bar.
#
# Revision 1.5  1999/05/27 17:13:05  tam
# Made actions in for forms use relative addresses.
#   Fixed bug in sorting of table selections.
#   Added smaller values for table query limits.
#   Put in values in days for time offsets in cross-correlations
#    (this may not have been necessary).
#
# Revision 1.4  1999/02/22 18:55:11  saima
# "GIFsize" is now labelled "Max GIF size".
#
# Revision 1.3  1999/01/26 22:09:10  saima
# Added JavaScript to page so that if user clicks on 'All parameters'
#   option, all checkboxes under 'View' are selected.
#
# Revision 1.2  1998/12/23 20:16:04  saima
# Modified TableHeadings() routine.
#
# Revision 1.1  1998/12/23 16:40:23  saima
# Added several routines which help display the new interface. These
#   routines are used uniformly across all interfaces.
#
# Revision 1.4  1998/05/14 16:17:46  saima
# Added code to help debug for missing tables.
#
# Revision 1.3  1998/05/13 18:20:12  saima
# Minor cosmetic changes to SelectTable() routine.
#
# Revision 1.2  1998/05/11 19:06:23  saima
# Fixed parsing of 'start_time' and radius when displaying 'C' or 'T'.
#
# Revision 1.1  1998/05/07 20:37:33  saima
# Initial Revision.
#
#
#=======================================================================#
# High Energy Astrophysics Science Archive Research Center (HEASARC),
# NASA's Goddard Space Flight Center, Greenbelt, MD 20771, USA.
#
# You are free to use this code as you wish, but unless you change
# it significantly, please leave the above name & address intact.
#=======================================================================#
# Module Name:    Options.pm
# Module Author:  Saima Zobair, Raytheon STX
# Date:           April 1998
# Project:        Browse : HEASARC
#-----------------------------------------------------------------------#
# Module Description:
#
# The routines in this module contain code used to display different
# kinds of form options used in Browse.
#
#=======================================================================#

use strict;

package Browse::Options;

#--------------------------------------------------------------------------
sub Header {
   # displays the search page header

   my($missions, $selected, $other) = @_;
   my($html,$mlinks);

   $mlinks    = Browse::Decor::MissionLinks('missions' => $missions);
   my($title) = "Search&nbsp;of&nbsp;$selected&nbsp;$mlinks&nbsp;$other Catalog(s)&nbsp;"; 
   Browse::Decor::Header($title); #dont close table, cols=2

} #eosub--Header
#--------------------------------------------------------------------------
sub OptionBar {
    my ($in) = @_;
    my($state) = BuildState($in);
    
    #--- clean up for HTML display
    $in->{Entry} =~ s/\W/"&#".ord($&).";"/ge;
 
    if ($::inc_summary) {
       my $slurp = Browse::Utils::includefile($::inc_summary);
       #my $slurp = Browse::Utils::includefile("inc_summary.html");
       ($slurp) = $slurp =~ /(.*)/s;  #blindly untaint $slurp
       $slurp = "print <<EoF;\n$slurp\nEoF";
       eval $slurp; warn "eval: $@" if $@;
    }
}

sub OptionBar2 {
   # adds HTML to add query options on the left side of form
   my($in) = @_;

   my($state) = &BuildState($in);
   my($coord) = &CoordSelection($state);
   my($eq)    = &EquinoxSelection($state);
   my($rad)   = &RadiusSelection("Radius",$state);
   my($nr)    = &NRSelection($state);
   my($max)   = &ResultLimit($state);
   my($output) = &OutputFormat($state);

   my($paramsearch,$sortkey,$allparams);
   if ($$in{'Action'} =~ /Parameter/i) {
       $paramsearch = "<br><a target=helpwin href=\"$::w3bhelp#wildcards\"><b>Wildcards, Spaces, and Case Sensitivity</b></a> <br>";
   } else {
       $sortkey = " <p>\n<b>Sort columns:<br>\n<img src=\"$::images/up.gif\" width=15 height=15 alt=\"[UP]\"> in ascending order<br>\n<img src=\"$::images/down.gif\" width=15 height=15 alt=\"[DOWN]\"> in descending order</b><br>";
       $allparams = "<br><input type=\"checkbox\" name=\"Fields\" id=\"all\" value=\"All\" $state->{'Fields'} onClick=\"tickVars('varon')\"> <label for=\"all\"><b>All parameters</b></label>";
   }

   print <<EOT;
<tr><td align=left valign=TOP>
<table><tr>
<td align=left>
<p>
<br><a target=helpwin href="$::w3bhelp#other_options"><b>Coordinate System:</b></a>
<br>$coord<br>
<br><a target=helpwin href="$::w3bhelp#other_options"><b>Equinox:</b></a>
<br>$eq<br>
<br><a target=helpwin href="$::w3bhelp#other_options"><b>Search Radius:</b></a>
<br>$rad<br>
<br><a target=helpwin href="$::w3bhelp#resolver"><b>Name Resolver:</b></a>
<br>$nr<br>
<br><a target=helpwin href="$::w3bhelp#other_options"><b>Result Limit:</b></a>
<br>$max<br>
$paramsearch
<br>
<a target=helpwin href="$::w3bhelp#other_options"><b>Output Format:</b></a>
$output
<br>
$allparams
 <p>\n<br><b><img alt="archive data" src="$::images/tape2.gif"> : Archival data</b>
$sortkey
</table>
EOT
} # eosub--OptionBar
#--------------------------------------------------------------------------
sub BuildState {
  my($in) = @_;

  # Set form defaults.
  #$in->{'maxpriority'} ||= "def";  # not using the priority selection now
  $in->{'maxpriority'} ||= "";
  $in->{'NR'}          ||= 'CheckCaches/GRB/SIMBAD/NED';
  $in->{'Radius'}      ||= "Default";
  $in->{'Radius_unit'} ||= "arcminutes";
  # next one can be '0' legitimately
  $in->{'ResultMax'}   = "1000" unless defined $in->{'ResultMax'};
  $in->{'Fields'}      ||= "Standard";
  $in->{'Coordinates'} = 'Equatorial' if $in->{'Coordinates'} =~ /equ/i;
  $in->{'Coordinates'} = 'Galactic' if $in->{'Coordinates'} =~ /gal/i;
  $in->{'Coordinates'} ||= 'J2000';
  $in->{'Coordinates'} = "J2000" if $in->{'Equinox'} == 2000;
  $in->{'Coordinates'} = "B1950" if $in->{'Equinox'} == 1950;
  $in->{'Equinox'}     ||= 2000;
  $in->{'table_type'}  ||= "Observation\0Object\0Proposal";
  
  # Create state data structure.
  my($state) = {};

  # Checked items.
  $state->{'Fields'} = ($in->{'Fields'} =~ /all/i) ? 'checked' : '';
  $state->{'vizier_query'} = ($in->{'vizier_query'} =~ /on/i) ? 'checked' : '';
  my $type;
  foreach $type (split(/\0/,$in->{'table_type'})) {
     $state->{"table_type_$type"} = "checked";
  } 
  foreach my $mission (split(/\0/,$in->{'Observatory'})) {
     $state->{"Observatory_$mission"} = "checked";
  } 
  

  # Selected items.
  my($item);
  foreach $item (qw(Coordinates Equinox NR Radius_unit ResultMax displaymode)) {
      $state->{$item . '_' . Browse::CGI::Decode($in->{$item})} = 'selected';
  }

   
  return($state);
} # eosub--GetState
#--------------------------------------------------------------------------
sub TableHeadings {
  # Displays table headers for selection pages 

  my($tablespec, $tablehead, $catalog_page) = @_;

  if (! $tablehead ) {
    return;
  }

  my($ok_sort)  = ($catalog_page && defined($tablehead) && 
                   scalar(@ { $tablehead } ) > 1) ? 1 : 0;

  my($imgup)   = "<img alt=\"Reverse Sort (3,2,1)\" src=\"$::images/up.gif\" height=15 width=15 border=0>";  
  my($imgdown) = "<img alt=\"Sort (1,2,3)\" src=\"$::images/down.gif\" height=15 width=15 border=0>";

  my($th)=""; 
  if ( scalar(@ { $tablehead }) > 1 ) { 
     #document.write (" <br><label for='table'>All </label><input type='checkbox' checked id='table' name='table' onClick=tickTable('tablehead','table','')>");
   $th = qq|<th scope='col' class='othertable'><label for='tablehead'>Select:</label> <script type="text/javascript" language="JavaScript1.2">
<!--
     document.write (" <br><label for='table'>All </label><input type='checkbox' checked name='table' id='table' onClick=tickTable(this.form.tablehead,this.checked,'')>");
// -->
  </script> </th>|;
  }
  
  if ($ok_sort) {
    my $url = "w3table.pl?Action=Advanced+Search&$tablespec&sortby";
    print <<EOT;
  <table class='othertable'>
  <tr class='othertable'>
   $th<th scope="col" class='othertable'>Description<a href="$url=description">$imgdown</a><a href="$url=-description">$imgup</a></th>
   <th scope="col" class='othertable'>Catalog<a href="$url=name">$imgdown</a><a href="$url=-name">$imgup</a></th>
   <th scope="col" class='othertable'>Data<a href="$url=archive">$imgdown</a><a href="$url=-archive">$imgup</a></th>
   <th scope="col" class='othertable'>Default Radius (arcmin)<a href="$url=radius">$imgdown</a><a href="$url=-radius">$imgup</a></th>
   <th scope="col" class='othertable'>Mission<a href="$url=mission">$imgdown</a><a href="$url=-mission">$imgup</a></th>
   <th scope="col" class='othertable'>Table&nbsp;Type<a href="$url=tabletype">$imgdown</a><a href="$url=-tabletype">$imgup</a></th>
EOT
#'
   print "<th scope='col' class='othertable'>Priority<a href='$url=priority'>$imgdown</a><a href='$url=-priority'>$imgup</a></th>\n" if $::debug;
   print "</tr>\n";
  } else {
    print <<EOT;
  <table class='othertable'>
  <tr class='othertablehead'>
   $th<th scope="col" class='othertable'>Description</th>
   <th scope="col" class='othertable'>Catalog</th>
   <th scope="col" class='othertable'>Data</th>
   <th scope="col" class='othertable'>Default Radius (arcmin)</th>
   <th scope="col" class='othertable'>Mission</th>
   <th scope="col" class='othertable'>Table Type</th>
  </tr>
EOT
   #<th scope="col" class='othertable'>Priority</th>
  }
} # eosub--TableHeadings
#--------------------------------------------------------------------------
sub DisplayTables {

    #   'table_list' => a TableList data structure like one returned by
    #                     HDB::GetQueryTable(),
    my($table_list, $default_tables) = @_;
    if (! $table_list) { return; };
    my %default=[];

    foreach my $def ( @{ $default_tables } ) {
        $default{$def->{table_name}}=1;
        $default{$def->{name}}=1;
    }

    my ($havesets) = Browse::DBQ::query("select distinct table_name from zzdpsets");
    my %gotsets;
    foreach my $harr (@{$havesets}) { 
         $gotsets{$$harr[0]} =1; 
    }

    my($tableinfo) = {};
    foreach $tableinfo ( @{ $table_list } ) {
       #Browse::Utils::DumpHash($tableinfo) if $::debug;
       my($table) =  $tableinfo->{'name'} || $tableinfo->{'table_name'};
       my $check = "checked" if $default{$table};
       next unless $table;
       unless ($tableinfo->{'description'} || $tableinfo->{'table_description'}) {
          $tableinfo = {Browse::HDB::TableInfo($table)};
       }
       my $short =  $table;
       $short     =~ s/$::local_origin//i;
       $tableinfo->{'radius'} =  $tableinfo->{'default_search_radius'} 
                      if exists $tableinfo->{'default_search_radius'};   
       $tableinfo->{'radius'} =  "***" if($tableinfo->{'radius'} < 0);   
       $tableinfo->{'url'} ||= $tableinfo->{'table_document_url'};
       $tableinfo->{'mission'} ||= $tableinfo->{'observatory_name'};
       $tableinfo->{'tabletype'} ||= $tableinfo->{'table_type'};
       $tableinfo->{'description'} ||= $tableinfo->{'table_description'};
       $tableinfo->{'archive'}="Y" if ($gotsets{$table});
       $tableinfo->{'archive'} ||= 'N';
       $tableinfo->{'priority'} = '5' unless defined $tableinfo->{'priority'};
       # priority can be '0'


       my($info)  =  'name=' . $table .'&'.
                     'description=' . $tableinfo->{'description'}.'&'.
                     'url=' .         $tableinfo->{'url'}. '&' .
                     'archive=' .     $tableinfo->{'archive'}.'&'.
                     'radius=' .      $tableinfo->{'radius'} . '&' .
                     'mission=' .     $tableinfo->{'mission'} . '&' .
                     'priority=' .    $tableinfo->{'priority'} . '&' .
                     'tabletype=' .   $tableinfo->{'tabletype'};

       #$info = Browse::CGI::Encode($info);

       my $isViz = (($tableinfo->{'tabletype'} =~ /vizier/i) or ($table =~ /\//));

       my $xbg;
       if ($isViz) {
           $xbg = ' class="Vizier" ';
       } else {
           $xbg= " class='othertable' ";
       }
       print "<tr class='othertable'>\n";
       if (scalar(@ { $table_list }) > 1) {
         print "   <td $xbg><input type=\"checkbox\" name=\"tablehead\" id=\"tablehead\" $check value=\"$info\">".
               "</td>\n";
       }else{
         print "   <input type=\"hidden\" name=\"tablehead\" value=\"$info\">\n";
       }
    
       my $priority='';
       $priority = "HIGH" if int($tableinfo->{'priority'}) <= 3;
       $priority = "LOW" if int($tableinfo->{'priority'}) >= 6;
       $priority = "MED" if !$priority;

       if ($tableinfo->{'url'}) {
           print "   <td$xbg><a target=\"helpwin\" href=\"$tableinfo->{'url'}\">".
                 "$tableinfo->{'description'}</a></td>\n";
       } else {
           print "   <td$xbg>$tableinfo->{'description'}</td>\n";
       }
       print "   <td$xbg>$short</td><td$xbg>$tableinfo->{'archive'}</td>\n";
       print "   <td$xbg><center>$tableinfo->{'radius'}</center></td>\n";
       print "   <td$xbg>$tableinfo->{'mission'}</td>\n";
       print "   <td$xbg>";
       if ( $isViz ) {
           print "<a target=\"vizwin\" href='$::vizier/viz-bin/VizieR?-source=" .
            Browse::CGI::Encode($table)."'>";
       }
       print $tableinfo->{'tabletype'};
       if ($isViz) {
           print "</a>";
       }
       print "</td>\n";
       print "   <td$xbg>$priority ($tableinfo->{'priority'})</td>\n" if $::debug;
       #print "   <td>$priority</td>\n";# if $::debug;
       print "</tr>\n";
    }
    print "</table>\n";
} #eosub--DisplayTables
#--------------------------------------------------------------------------
sub ResultLimit {
  # This subroutine creates the Result limit selection box
  my($state) = @_;

  my($html) = <<EOT;
<select id="ResultMax" name="ResultMax">
      <option value="10"  $state->{'ResultMax_10'}> 10
      <option value="20"  $state->{'ResultMax_20'}> 20
      <option value="30"  $state->{'ResultMax_30'}> 30
      <option value="50"  $state->{'ResultMax_50'}> 50
      <option value="100"  $state->{'ResultMax_100'}> 100
      <option value="250"  $state->{'ResultMax_250'}> 250
      <option value="500"  $state->{'ResultMax_500'}> 500
      <option value="1000" $state->{'ResultMax_1000'}> 1000
      <option value="1500" $state->{'ResultMax_1500'}> 1500
      <option value="2000" $state->{'ResultMax_2000'}> 2000
      <option value="0"    $state->{'ResultMax_0'}> No Limit
</select>
EOT
  return($html);
}
#--------------------------------------------------------------------------
sub CoordSelection {
  # This subroutine creates the Coordinate Selection box
  my($state) = @_;

  my($html) = <<EOT;
<select name="Coordinates" id="Coordinates">
      <option $state->{'Coordinates_Equatorial'} value="Equatorial">Equatorial
      <option $state->{'Coordinates_Galactic'} value="Galactic">Galactic
</select>
EOT
  return($html);
} #eosub--CoordSelection
#--------------------------------------------------------------------------
sub EquinoxSelection {
  # This subroutine creates the Equinox Selection box
  my($state) = @_;

  my($html) = <<EOT;
<select name="Equinox" id="Equinox">
      <option $state->{'Equinox_2050'} > 2050
      <option $state->{'Equinox_2000'} > 2000
      <option $state->{'Equinox_1950'} > 1950
      <option $state->{'Equinox_1900'} > 1900
</select>
EOT
  return($html);
} #eosub--EquinoxSelection
#--------------------------------------------------------------------------
sub NRSelection {
  # This subroutine creates the Resolver box
  my($state) = @_;

  my($html) = <<EOT;
<select name="NR" id="NR" size="1"> 
    <option $state->{'NR_CheckCaches/GRB/SIMBAD/NED'} value='CheckCaches/GRB/SIMBAD/NED'>GRB, SIMBAD, NED
    <option $state->{'NR_CheckCaches/GRB/NED/SIMBAD'} value='CheckCaches/GRB/NED/SIMBAD'>GRB, NED, SIMBAD
    <option $state->{'NR_CheckCaches/SIMBAD/NED'} value='CheckCaches/GRB/SIMBAD/NED'>GRB, SIMBAD, NED
    <option $state->{'NR_CheckCaches/NED/SIMBAD'} value='CheckCaches/GRB/NED/SIMBAD'>GRB, NED, SIMBAD
    <option $state->{'NR_SIMBAD'} value='SIMBAD' > SIMBAD only
    <option $state->{'NR_NED'} value='NED' > NED only
    <option $state->{'NR_GRB'} value='GRB' > GRB only

</select> 
EOT
  #my($html) = <<EOT;
#<select name="NR" id="NR" size="1"> 
    #<option $state->{'NR_CheckCaches/SIMBAD/NED'} value='CheckCaches/SIMBAD/NED'>SIMBAD, NED
    #<option $state->{'NR_CheckCaches/NED/SIMBAD'} value='CheckCaches/NED/SIMBAD'>NED, SIMBAD
    #<option $state->{'NR_SIMBAD'} value='SIMBAD' > SIMBAD only
    #<option $state->{'NR_NED'} value='NED' > NED only
#
#</select> 
#EOT
  return($html);
} #eosub--ResolverSelection
#--------------------------------------------------------------------------
sub OutputFormat {
  my($state) = @_;
  my($html) = <<EOT;
<select name="displaymode" id="displaymode">
<option $state->{'displaymode_Display'} value='Display'> HTML Table
<option $state->{'displaymode_TabbedDisplay'} value='TabbedDisplay'> Tabbed Display
<!--<option $state->{'displaymode_NonTableDisplay'} value='NonTableDisplay'> Faster HTML Table-->
<option $state->{'displaymode_TextDisplay'} value='TextDisplay'> Text Table
<option $state->{'displaymode_FitsDisplay'} value='FitsDisplay'> FITS Table
<option $state->{'displaymode_XMLDisplay'} value='XMLDisplay'> astrores XML
<option $state->{'displaymode_VODisplay'} value='VODisplay'> VOTable XML
<option $state->{'displaymode_ExcelDisplay'} value='ExcelDisplay'>Excel-compatible
<option $state->{'displaymode_PureTextDisplay'} value='PureTextDisplay'>Pure Text
<!--<option $state->{'displaymode_ExcelDisplay2'} value='ExcelDisplay'>Excel-compatible (mod_perl test) -->
</select>
EOT
  return($html);
}
#--------------------------------------------------------------------------
sub TimeSelection {

  # This subroutine create the cross correlation time selection box

  my($html) = <<EOT;
<label for="XTime"><!-- Time Unit for Correlation --></label>
<select name = "XTime" id="XTime">
      <option value="Default">Default
      <option value=.041667>1 hr
      <option value=.25>6 hr
      <option value=.5>12 hr
      <option value=1>1 day
      <option value=7>1 week
      <option value=30.43>1 month
      <option value=365.25>1 year
</select>
EOT
  return($html);

} #eosub--TimeSelection
#--------------------------------------------------------------------------
sub RadiusSelection {
  # This subroutine creates the Radius Selection box

  my($form_radius,$state) = @_;

  my($html) = qq(
<label for="${form_radius}_unit"><!-- Radius Unit --></label>
<input type="text" name="$form_radius" id="CrossRadius" value="Default">
<select name="${form_radius}_unit" id="${form_radius}_unit">
<option>arcsec
<option selected>arcmin
<option>degrees
</select>
);

  return($html);
} #eosub--RadiusSelection
#--------------------------------------------------------------------------
sub SelectTable {
#return "";
    # Function  input: a hash with the following keys:
    #   'table_list' => a TableList data structure like one returned by
    #                     HDB::GetCrossTables(),
    #   'prompt'     => string to be used for the prompt
    #   'CGI_name'   => name of the CGI variable to populate
    #   'CGI_value'  => ref to array of info to include in the <option>'s
    #                     CGI value; possible array items are: 'table_name',
    #                     'table_description', and 'radius'; order is important!
    #   'CGI_valsep' => character to use for joining multiple values defined
    #                     by 'CGI_value' key
    #   'multiple'   => non-zero value means multiple tables can be selected
    #   'hint'       => non-zero value means to print a hint about hyperlinks
    #   'no_radius'  => non-zero value means don't care about default search
    #                     radii
    #   'rebuild'    => non-zero value means to select all tables
    #   'size'       => how many lines should the selection box display
    #   'max_size'   => the maximum number of lines the selection box should
    #                     display; 'size' is always <= 'max_size' if 'max_size'
    #                     is set
    #   'no_sort'    => set if the tables are already in the order they should
    #                     be displayed in.
    # Function output: HTML string that implements a selection box

    my(%args) = @_;

    Browse::Utils::Error("Invalid table list data structure.")
      if (!defined($args{'table_list'})
	  || !scalar(@{ $args{'table_list'}->{'list'} }));

    my($tables) = $args{'table_list'};
    $args{'prompt'} = 'Select Table:' unless (defined($args{'prompt'}));
    $args{'CGI_name'} = 'Table' unless (defined($args{'CGI_name'}));
    $args{'CGI_value'} = [ 'table_name' ] unless (defined($args{'CGI_value'})
						  && (ref($args{'CGI_value'})
						      eq 'ARRAY'));
    $args{'CGI_valsep'} = ' ' unless (defined($args{'CGI_valsep'}));
    $args{'multiple'} = 0 unless (defined($args{'multiple'}));
    $args{'hint'} = 1 unless (defined($args{'hint'}));
    $args{'no_radius'} = 0 unless (defined($args{'no_radius'}));
    $args{'rebuild'} = 0 unless (defined($args{'rebuild'}));
    $args{'max_size'} = 8 unless (defined($args{'size'})
				  || defined($args{'max_size'}));
    $args{'size'} = ((scalar(@{ $tables->{'list'} }) > $args{'max_size'})
		     ? $args{'max_size'} : scalar(@{ $tables->{'list'} }))
      unless (defined($args{'size'}));
    
    # Sort the tables in list.  We used to just sort the option
    # lines themselves, but that effectively sorts by table_name and not description
    if (!$args{no_sort}) {
        @{$$tables{list}} = sort {
            $$tables{descriptions}{$a} cmp $$tables{descriptions}{$b};
        } @{$$tables{list}};
    }
   
my %tables_by_mission= (); 

    my(@options) = ();
    my($item,$value,@values,%info);
    my($ct,$c,$t,$table_name,$table_radius,$mission,@columns);
    foreach $table_name (@{ $tables->{'list'} }) {
#print "table_name: $table_name<br>\n" if $::debug;
        my($code);
	$info{'table_description'} = $tables->{'descriptions'}{$table_name};
	$info{'table_name'} = $table_name;
        $info{'start_time'} = $tables->{'start_time'}{$table_name};
	$info{'radius'} = $table_radius = $tables->{'radii'}{$table_name};
	if ($table_radius =~ /^[0-9\.]+$/) {
	    $table_radius = "/ $table_radius'";
            $code = "R" if($args{'code'});
	}elsif($table_radius){
            $code = "R" if($args{'code'});
        } 
        if ($info{'start_time'} =~ /^\@([A-Z0-9a-z_\.]+)$/ && $args{'code'}) {
          $code .= "T";
        }
        my $tname = $info{'table_name'};
        $tname =~ s/^$::local_origin//i;
	$item = minimalEncode("$info{'table_description'} \(".
			      $tname. 
			      ($args{'no_radius'} ? " " : " $table_radius") .
			      "\)" . " $code ");
	@values = ();
	foreach $value (@{ $args{'CGI_value'} }) {
	    push(@values,$info{$value});
	}
	$value = join($args{'CGI_valsep'},@values);
	$tables_by_mission{$$tables{'mission'}{$table_name}} .= 
        minimalEncode($value)."||$item||<option value='" . minimalEncode($value) . "'" .
	     ($args{'rebuild'} ? " selected " : "") . ">$item\n".'||';
	#push(@options,"<option value=\"" . minimalEncode($value) . "\"" .
	#     ($args{'rebuild'} ? " selected " : "") . ">$item\n");
        print "SelectTable :: item : $item value: $value<br>\n" if ($::debug); 

        if($::debug){
           $c++ if ($code eq 'R'); 
           $ct++ if ($code eq 'RT'); 
           $t++ if ($code eq 'T'); 
        }
    }
    print "Number of R: $c <br>Number of T: $t<br>Number of RT: $ct<br>"
        if ($::debug); 
    # Build the form.
    my($codestuff)  = $args{'code'} ? " Correlation Flags " : " ";
    my($html) = "<strong>$args{'prompt'}</strong> " .
      "<br><span class=\"small\">[ <em>Key:</em> Descriptive Table Name " .
      "(Short Table Identifier" .
      ($args{'no_radius'} ? "" : " / Default Search Radius") .
      ")  $codestuff] </span><br>\n";
     
    # javascript form

$html .= qq|
<script type="text/javascript" language="JavaScript"><!--
function showlinks$args{CGI_name}() {
|;

    foreach my $mission (sort keys %tables_by_mission) {
       $html .= "\nif (mission$args{CGI_name} == '$mission') {\n";
       my @tablearr = split('\|\|',$tables_by_mission{$mission});
       while (scalar(@tablearr)) {
         my $tab = shift @tablearr;
         my $item = shift @tablearr;
         my $dummy = shift @tablearr;
         $html .= "opt('$tab',\"$item\",document.fullcross.$args{CGI_name});\n";
       }
       $html .= "}\n";
    }
$html .="}\n";

$html .= qq|
document.write('<label for="mission$args{CGI_name}"><B>Select a Mission:</B></label>  ');
document.write("<select name='mission$args{CGI_name}' id='mission$args{CGI_name}' onChange='reshow$args{CGI_name}(document.fullcross.mission$args{CGI_name},document.fullcross.$args{CGI_name})'>");
|;
    foreach my $mission (sort keys %tables_by_mission) {
        $html .= "document.write('<option>$mission');\n";
    } 
    $html .= qq|
document.write('</select> <br>');
var reloading = false;
var mission$args{CGI_name} = document.fullcross.mission$args{CGI_name}.options[document.fullcross.mission$args{CGI_name}.selectedIndex].text;

    document.write('<label for="$args{CGI_name}"><B>Select a Table:</B></label>  ');
    document.write('<select name="$args{CGI_name}" id="$args{CGI_name}">');
    showlinks$args{CGI_name}();
    document.write('<\/select>');

//--></script>

|;
    

    # non-javascript form    
    $html .="<noscript><label for=\"$args{'CGI_name'}\"><b>Select a Table:</b></label><br>\n";
    $html .= "<select name=\"$args{'CGI_name'}\" id=\"$args{'CGI_name'}\" size=$args{'size'}" .
      ($args{'multiple'} ? " multiple" : "") . ">\n" ;
    foreach my $mission (sort keys %tables_by_mission) {
       my @tablearr = split('\|\|',$tables_by_mission{$mission});
       while (scalar(@tablearr)) {
         my $tab = shift @tablearr;
         my $item = shift @tablearr;
         my $dummy = shift @tablearr;
         $html .= "<option value='$tab'>$item\n";
       }
       # $html .= $tables_by_mission{$mission};
    } 
    $html .="</select><br>\n";
    $html .="</noscript>\n";

    return($html);

} #eosub--SelectTable

sub minimalEncode {
    my($string) = shift(@_);

    $string =~ s/&/&amp;/g;
    $string =~ s/</&lt;/g;
    $string =~ s/>/&gt;/g;
    $string =~ s/\"/&quot;/g;

    return($string);
} #eosub--minimalEncode
#--------------------------------------------------------------------------
#--------------------------------------------------------------------------
#########
#  EOF  #
#########
1;

