# This class displays results in plain text.

use strict;

package Browse::PureTextDisplay;

use vars qw( @ISA );
@ISA = qw( Browse::TextDisplay );
use Browse::TextDisplay;
use Browse::Utils;
use CPAN::IO::String;

# Minimize information at the top of the page.
sub pageHeader {
  my ($self)=@_;
  # -- do not set up for web page display if results saved to file
  print "Content-type: text/plain; charset=us-ascii\n\n" 
	unless defined ($$self{"orgFilehandle"});
}
sub pageFooter {
    my ($self)=@_;
    #--- If filehandle has been changed then we assume that we need
    #--- to switch back to the original filehande (STDOUT most likely)
    #--- and the output string needs to be saved to a file 
    #if (defined($$self{"orgFilehandle"})) {
       #select ($$self{"orgFilehandle"});
       #my $targetFileName = $::savedTargetsFile .".$$.txt";
       #Browse::Utils::dataToFile($$self{"outString"}, $targetFileName);
    #}
}

#--- Set the filehandle to a string that will recieve the Pure text output
sub setAltOutput {
    my ($self)=@_;
    my $str_fh = CPAN::IO::String->new($$self{"outString"});
    $$self{"orgFilehandle"}= select($str_fh);
  
}
#--- adding to save Target File

# For this class, closeTable actually
# prints the results after 'neatening' the table.

sub closeTable {

    my ($self) = @_;
    
    if ($$self{"intable"}) {
	$$self{"intable"} = 0;
	
	my $array = $$self{"dataarray"};
	my $meta  = $$self{"meta"};
	my $names = $$self{"names"};
	my $types = $$self{"types"};
	
	my $nskip = 0;
	while ($nskip <= $#$names) {
	    if (substr($$names[$nskip], 0, 3) !~ /w3_/i) {
	        last;
	    }
	    $nskip += 1;
	}
        my $coordsys  = $$self{coordsys};
        my $tableinfo = $$meta{"tableinfo"};

        my $ra = $$tableinfo{"right_ascension"};
        $ra =~ s/^\@//;
        my $dec = $$tableinfo{"declination"};
        $dec =~ s/^\@//;

        #--- Change header for non-equatorial coordinates
        my ($oras, $odecs) = Browse::Display::otherRaDecs($tableinfo);

        foreach (@$names) {
           $_ = Browse::Display::renameRaDecs($_, $ra, $dec, $oras, 
	     $odecs, $coordsys );
        }

	
	Browse::TableFormat::neatenTable($array, $names, $types, undef);
	
	my $trans = $$self{"transform"};
	my $longs = $$trans{"longs"};
	my $delim = $$self{"delimiter"};
        $delim = "|" unless defined $delim;
	
	$self->displayTextLine($names, $nskip, $longs,$delim);
	
	my $line;
	foreach $line(@$array) {
	    $self->displayTextLine($line, $nskip, $longs,$delim);
	}
	
        my $s = "";
        if ($$self{"rowcount"} > 1) {
            $s = "s";
        }
        my $table = $$self{"meta"}{"table"};
        $table = $self->chopTable($table);
	
	$$self{rowcount} = 0;
	
    }
}

sub tableHeader {
    my ($self) = @_;
       
    my $textdisplay = $$self{"text"};
    my ($meta) = $$self{"meta"};

    my $table_info = $$meta{"tableinfo"};
    my $table      = $$meta{"table"};

    # Add coordinate system (to be used when parsing saved target file)
    my $coordsys  = $$self{coordsys};  

    $$self{"meta"} = $meta;
    $$self{"rowcount"} = 0;
    if ($$meta{"resource_name"} && $$meta{"resource_name"} ne 
            $$self{"last_resource"}) {
        my $url = $$meta{"query_url"};
        $url =~ s/asu-xml/VizieR/;
        print "VizieR resource: $$meta{resource_title} ($$meta{resource_name})\n";
        $$self{"last_resource"} = $$meta{"resource_name"};
    }
    #Browse::Results::PrintTableName($table, $table_info, '', $textdisplay);
    print "\n\nResults from $table: $$table_info{'table_description'}\n";
    print "Coordinate system:  $coordsys\n";

}

sub legend {}
sub resultsSummary {}
sub YourSearchWas {}
sub tableFooter {}
sub beforeNewTable {}
sub afterTable {}

sub showEmpties {
    my ($self, $empties, $errs) = @_;
    $self->closeTable();
    if ($#$empties >= 0) {

        print "\n-----------------------------------------------------------\n";
        print "No matches for:\n";
        my $niltab;
        foreach $niltab(@$empties) {
            my $stable = $$niltab[0];
            $stable =~ s/$::local_origin//g;
            print "     $$niltab[1] ($stable)\n";
        }
        print "\n";
    }
    my $lastErr;
    if ($#$errs >= 0) {
        print "------------------------------------------------------------\n";
        print "Search errors\n";
        my $errtab;
        foreach $errtab(@$errs) {
            my $stable = $$errtab[1];
            $stable =~ s/$::local_origin//g;
            if ($$errtab[0] ne $lastErr) {
                print "     $$errtab[0] ";
            }
            print "$$errtab[2] ($stable)\n";
        }
    }
}


## Successful load
1;
