# $Id: Resort.pm,v 1.29 2007/10/09 14:22:04 lmm Exp $
# $Log: Resort.pm,v $
# Revision 1.29  2007/10/09 14:22:04  lmm
# Made improvements using new Parameter class methods.
#
# Revision 1.28  2006/10/24 20:54:20  lmm
# Moved call to pageHeader() earlier in processing.
#
# Revision 1.27  2006/07/10 17:06:33  lmm
# Made changes to fix Excel output format.  Tabbed Display specific changes
#   were messing things up.
#
# Revision 1.26  2006/06/23 17:27:43  lmm
# Added changes for Tabbed Display and new Results class
#
# Revision 1.25  2006/01/31 20:48:04  lmm
# Added Save Query to File functionality for re-queries and re-displays.
# Cleaned up code indentation.
#
# Revision 1.24  2006/01/12 17:29:29  lmm
# Changed call to Display::resultsSummary() to reflect new argument
#
# Revision 1.23  2005/12/14 15:38:48  lmm
# Major changes to correct problems with requery including recheck of
#   empty tables.  This version also corrects previous incorrect CVS
#   checkin.
#
# Revision 1.22  2004/12/15 20:48:47  lmm
# Major changes to pass requery functionality to Parameter::Search()
#   rather than have duplicate code.
#
# Revision 1.21  2004/03/04 20:07:27  lmm
# Added definition of delimiter for collection of null table names from
#   previous query.
#
# Revision 1.20  2004/01/15 20:59:03  lmm
# Removed changes made in previous version for saving objects to file.
#   This is now handled with a subclass of PureTextDisplay
#
# Revision 1.19  2003/12/02 20:38:44  lmm
# Added redirect to string setup for Pure Text Display if Save All
#    Targets to File option is selected.
# Empty tables from previous query are reincorporated if Requery is selected.
# Bug fix - only one semi-colon was inserted between multiple targets.
#
# Revision 1.18  2003/09/03 21:08:27  lmm
# Table legend appears only once instead of before every table
#
# Revision 1.17  2003/08/28 21:36:53  lmm
# Define text for page header
#
# Revision 1.16  2003/07/31 18:49:01  cwh
# Added new call to displayer->legend(), which is broken out of tableHeader,
#   since not all display modes will want to display the legend.
#
# Revision 1.15  2003/07/31 13:54:52  cwh
# Added the ability to requery with modified Entry.
# Added the ability to requery with modified maxrows.
#
# Revision 1.14  2002/09/05 18:20:58  cwh
# Changed the input to YourSearchWas, since some input vars are display
# properties that YSW can get for itself.
#
# Revision 1.13  2002/05/14 18:12:50  cwh
# Changed the call to YourSearchWas to use display object.
# Added another display->closeTable, for the excel display method.
#
# Revision 1.12  2001/12/06 16:51:59  cwh
# No longer adding $::local_origin to table names, it is already there.
# Don't print YourSearchWas for cross-correlations - we don't have it set up
#    to display the input params correctly.
# Set displayer{offset_var} to $request->getOffsetVar()
#
# Revision 1.11  2001/11/07 23:34:19  cwh
# Moved everything except DBD/ into lib/Browse/, so as not to pollute the module
# namespace with our Sort, Search, CGI, etc.  This required putting Browse:: in
# front of every module call and package declaration.  Also any mod::new() had
# to be changed to mod->new().
#
# Revision 1.10  2001/10/25 22:22:19  cwh
# Reversed the meaning of the sort arrows.
#
# Revision 1.9  2001/09/17 19:24:28  cwh
# For mod-perl, must initialize $table to ''.
# For Vizier tables, where multiple tables are from the same query, don't
#   requery the exact same resource.
#
# Revision 1.8  2001/09/15 14:04:28  cwh
# Modified so that resort::display can handle the redisplay of results in a
#   different format: HTML, text, or FITS.
#
# Revision 1.7  2001/07/23 16:42:16  cwh
# Call setMaxRows for the display object with the max rows of the request obj.
# Create the search criteria summary box by calling Results::YourSearchWas(),
#   and print the box by calling showText(), then print Directions().
#
# Revision 1.6  2001/07/18 19:42:35  cwh
# Fixed logging line.
#
# Revision 1.5  2001/06/20 21:58:58  cwh
# Added use strict,
# Exchanged use Parameter for use Display.
# Removed refs to $mode, $textdisplay since irrelevant here.
# Fixed passing of $navtrail.
# Clarified definition of $dirs.
#
# Revision 1.4  2001/06/20 16:50:08  cwh
# Rewrote Resort to simply regenerate the serialized query, change the sort
# params, and reissue the query.
#
# Revision 1.3  2001/05/22 17:52:03  cwh
# Added Id and Log to files without them.
#

package Browse::Resort;

use strict;

use Browse::Display;
use Browse::SaveObjectsToFile;
use Browse::Log;
use Browse::Parameter;
use Browse::TabbedParameter;

use FreezeThaw qw(freeze thaw cmpStr safeFreeze cmpStrHard);

sub display {
   my ($key, $in) = @_;
   $key =~ s/\.[xy]$//;
   print "Resort: $key $$in{displaymode}<br>\n" if $::debug;

    
    my $crosscorr=0;
    my $table = '';
    my $sep = '';
    unless ($key =~ /Sort/i) {
       my $blah;
       my %seen = ();  # this is for Vizier - resources with multiple tables
       foreach $blah (keys %$in) {
          print "Resort: $blah->$$in{$blah}<br>\n" if $::debug;
          if ($blah =~ /^req_(.*)/ and !$seen{$$in{$blah}}) {
              print "Resort: req key->$blah <br>\n" if $::debug;
              $table .= $sep . $1;
              if ($1 =~ /\+x\+/) {$crosscorr = 1;}
              $sep = '|';
              $seen{$$in{$blah}}++;
          }
       }
    } 

    #-- Constructor and set up for Display 
    my $displaymode = $$in{displaymode};
    #--- don't use Tabbed display for cross corr
    if ($crosscorr && $displaymode =~ /Tabbed/i) {$displaymode = "Display";}
    my $displayer = Browse::Display->new($displaymode);
    my $parameterMode;
    if ($displaymode =~ /Tabbed/) {
       $parameterMode='TabbedParameter';
    } else { 
       $parameterMode='Parameter';
    }
    my $parameter = Browse::Parameter->new($parameterMode);
    my $batchflag;  #--- needed for saving query

    $displayer->setCoordSys($$in{"Coordinates"}, $$in{"Equinox"});
    $displayer->setPopupRef($$in{"popupFrom"});
    $displayer->pageHeader($$in{"navtrail"});
    #-- end Constructor and set up for Display 

   # If this is a requery add any tables originally requested but found empty.
   # Place table information in expected hash for later processing
   if ($key =~ /requery||redisplay/i) {
       my $ublah;
       my %useen = ();  
       foreach $ublah (keys %$in) {
          if ($ublah =~ /^unused_req_(.*)/ and !$useen{$$in{$ublah}}) {
              $table .= $sep . $1;
              $$in{"req_".$1} = $$in{"unused_req_".$1};
              $sep = '|';
              $useen{$$in{$ublah}}++;
          }
       }
   } 


   (undef, my $direction, $table, my $field) = split('\|', $key) 
	if $key =~ /Sort/;


   my ($firsttable) = 1;
   my $gotfirst=0;
   my $tableInfo;

   #--- Change delimiter to get table information for Summary table 
   my $infoTables;
   ($infoTables = Browse::CGI::Decode($table)) =~ s/\|/,/g;
   my @tablelist=split(/,/,$infoTables);

   
   if ($infoTables !~ /\,/) {
      $tableInfo = {};
      $$tableInfo{$infoTables} = {Browse::HDB::TableInfo($infoTables)};
   } else {
      $tableInfo = Browse::HDB::TableInfo($infoTables);
   }

   
   my $hiddenMissions="";
   if ($$in{displaymode} =~ /Tabbed/){
      #-- Look for selected missions to make sure tables belonging to multiple
      #-- missions are associated with a selected mission
      my @selectedmissions;
      foreach my $checkkey (keys %$in) {
       
        if ($checkkey=~ /^Observatory_|^mission/i ) {
          push @selectedmissions,   $in->{$checkkey} ;
          $hiddenMissions .= "<input  type='hidden' name='$checkkey' value='" .
	     $in->{$checkkey} ."'>\n";
        }
        if ($checkkey=~ /^displaymode/i ) {
          $hiddenMissions .= "<input  type='hidden' name='requerydisplay' " .
	     "value='" .  $in->{$checkkey} ."'>";
        }
  
      }
      $displayer->setMissionList($tableInfo, \@selectedmissions);  
   }
   my @nullTables;
   my @errTables;
   my $nullTablesHtml="";  #save unused tables as hidden params for requery
   my $allTablesCount=0;

my $msg;  #-- resolver message returned from resolving positions
foreach $table (split('\|', $table)) {
    my $info = Browse::CGI::Decode($in->{"req_$table"});
    print "Resort looking at $table: $info<br>" if $::debug;

    $table = Browse::CGI::Decode($table);

    my $tablename=$displayer->chopTable($$tableInfo{$table}{table_name});
    my $tabletitle=$tablename;
    if ($tabletitle =~ /\//) {
        #--- remove "out" suffix for title
        $tabletitle =~ s/\/out//;
    }

    my $request  = Browse::QueryRequest::regenerate($table,$info);
    Browse::Utils::DumpRef($request) if $::debug;

    if ($$in{Entry} =~ /\w/) {
       $$in{Entry} =~ s/\0/;/g; # multiple targets should be sepd by ';'

       delete $$in{"Position"};
       my $newpositions;
       #($newpositions, $msg) = Browse::Parameter::MakeConeQuery($in);
       ($newpositions, $msg) = $parameter->MakeConeQuery($in);
       $request->setPositions($newpositions);
    }


    if ($$in{maxrows} =~ /\d/) {
       $request->setMaxRows($$in{maxrows});
    }

    $displayer->MaxRows($request->getMaxRows());

    #--- These directives are processed for the first table only
    #--- or if query is Cross-Cor
    unless ($gotfirst or $table =~ / x /) {
       my $resmsg;  #undefined now, need to get this resolver message somewhere

       # Serialize and Encode current query
       my $saveQuery=FreezeThaw::freeze(\@tablelist, $request->getPositions(), 
	   $request->getQuals(), $in, $batchflag,$resmsg) ;  

       my($encSaveQuery)=Browse::CGI::Encode($saveQuery); 
       # pass on saved query to the displayer
       $displayer->setSavedQuery($encSaveQuery);
       #$displayer->pageHeader($$in{"navtrail"});


       $displayer->YourSearchWas($request->getPositions(),
                    $request->getQuals(), [$field],[$direction],
		    $msg,\@tablelist) ;
       $displayer->Directions();
       $displayer->resultsSummary($tableInfo,\@tablelist);
       $gotfirst=1;
       $parameter->beforeAllTables();

    }  #--- end processing if first table


    if ($direction) {
       my $dirs = ($direction =~ /up/) ? 1 : 0;
       $request->setSort([$field], [$dirs]);
    }

    my $block;
    my $nrow = 0;
    my $rowLimit=$request->getMaxRows();   # lmm
    $request->storeState();
    my $meta;

    while ($block = $request->getResult()) {
        if ($$block{"type"} eq "meta") {
            $displayer->processMeta($block);
            $meta=$block;

        } elsif (!$rowLimit || $nrow < $rowLimit) {   #lmm

            if ($nrow == 0) {
                # Display legend once before tables
                    $parameter->displayLegend($displayer,$firsttable);
                    $firsttable = 0;

                #if ($firsttable || $$in{displaymode} =~ /Tabbed/) {

                  #$displayer->legend($block);
                  #$firsttable=0;
                #} 
                $displayer->tableHeader($block);
                   
            }
            #--- earlier definition of offset_var was not sticking
            $$displayer{offset_var} = $request->getOffsetVar();
            $$displayer{"sortcolumn"} = $request->getSortString();

            $displayer->processData($block);
            $nrow += 1;
        }

     }   #--- end rows processing
  
           print "checking $table for null list<br>" if $::debug;
     print "parameter: table row count  $nrow<br>" if $::debug;

     $displayer->closeTable();
     $displayer->tableFooter();
     if ($nrow == 0 ) {
           #print "adding $table to null list<br>" if $::debug;
            $displayer->beforeNewTable($tablename,$tabletitle,
		$$tableInfo{$table}{table_description});
            $displayer->afterTable();

           push(@nullTables, [$table, 
		$$tableInfo{$table}{"table_description"}]);

            my $results = $displayer->results();
            if ($results) {
              $nullTablesHtml .= $results->DisplayUnusedLinks($meta)
		unless $meta == undef;
            }

        #} else {++$allTablesCount;}
        } else {$allTablesCount += $nrow;}


    }  #--- end foreach table

    #--- Finish up divs and tabs after last table
    $parameter->afterAllTables($displayer,$allTablesCount);

    $displayer->setNullTables($nullTablesHtml);
    $displayer->showEmpties(\@nullTables, \@errTables);

    
    if ($in->{displaymode} =~/Tabbed/) {
       if ($hiddenMissions) {print "$hiddenMissions\n";}

       if ($in->{Equinox}) {
          print "<input type='hidden' name='Equinox' value='".
	   $in->{Equinox}."'>\n";
       }
       if ($in->{Coordinates}) {
          print "<input type='hidden' name='Coordinates' value='".
	   $in->{Coordinates}."'>\n";
       }
       if ($in->{Time}) {
          print "<input type='hidden' name='Time' value='".
	   $in->{Time}."'>\n";
       }
       if ($in->{NR}) {
          print "<input type='hidden' name='NR' value='".
	   $in->{NR}."'>\n";
       }
       #print <<EOT;
          #<script type='text/javascript' language='JavaScript'>
          #setTab();
          #tpRem('tabcontrol','loading');
          #</script>
#EOT
       my $missionsDisplayed = $displayer->{missionlist};
       my  @m =keys %$missionsDisplayed;
       my $shortname = Browse::CGI::Decode($displayer->chopTable($table));
     
       if (scalar(@m) == 1 && $key =~ /Sort/) {
           my $mission= lc($m[0]);
           print <<EOT;
           <script type='text/javascript' language='JavaScript'>
           pShow('$mission','$shortname');
           </script>
EOT
       }
    
    }   ##-- end some Tabbed only code


    $displayer->pageFooter();

    Browse::Log::Query($0,$in,"Resort",[$table],"$field, $direction");

exit;
}

1;
