# $Id: TabbedResults.pm,v 1.1 2006/06/23 19:29:07 lmm Exp $
#=======================================================================#
# High Energy Astrophysics Science Archive Research Center (HEASARC),
# NASA's Goddard Space Flight Center, Greenbelt, MD 20771, USA.
#
# You are free to use this code as you wish, but unless you change
# it significantly, please leave the above name & address intact.
#=======================================================================#
# Module Name:    TabbedResults.pm
# Module Author:  Laura McDonald, SSAI 
# Date:           June, 2006
# Project:        Browse : HEASARC
#========================================================================
#--- Overrides Results methods for Tabbed Display
#========================================================================

use strict;

package Browse::TabbedResults;

use Browse::Results;

# Inheritance.
use vars qw( @ISA );
@ISA = qw( Browse::Results);

#------------------------------------------------------------------------
sub Header {

    my($op,$cols, $from) = @_;

    Browse::TabbedDecor::Header($from,$op,$cols); #dont close table, cols

    # This subroutine initiates the form tag on Results page
    print "\n<form method='POST' target='_blank' name='w3hdprods' action='/$::cgi/w3hdprods.pl'>\n";

} #eosub--Results::Header

#------------------------------------------------------------------------
#--- Further Actions section
sub DisplayTableOptions {
    my ($self, $table) = @_;
    (my $stable = $table) =~ s/$::local_origin//g;

    print "<span class='large'><b>Further Actions:</b></span><p>";
    print "Do you want to <input type='submit' name='Plot $table' value='Plot' onclick=\"this.form.target='_blank'\"> your $stable results? <a target='helpwin' href='$::w3bhelp#Java_Plot'>(help)</a><br>\n" unless ($table =~ / x / and $table =~ /\//);
    print "Do you want to <input type='submit' name='Correlate $table' value='Cross-correlate' onclick=\"this.form.target='_blank'\"> your $stable results with another catalog or table? <a target='helpwin' href='$::w3bhelp#cross_query'>(help)</a><br>\n" if $table !~ / x /; 
#print "<a href='http://heasarcdev.gsfc.nasa.gov/akbar/browsenotify/form.html?RPS_BEGIN=&Table+Name=$table&Entry=Cyg+X-1&Coordinate+System=J2000&Search+Radius=Default&Search+Radius+Unit=arcsec&RPS_END='>Notification Service $coords</a><p>";

    #print "Do you want to <input type='submit' name='Notify $table' value='Be Notified' onclick=\"this.form.target='_blank'\"> if your $stable results change? <a target='helpwin' href='$::w3bhelp#notification'>(help)</a><p>\n" if $table !~ / x /; 
} #-- end DisplayTableOptions


#------------------------------------------------------------------------
sub DisplayServiceOptions {
    my ($self, $table,$coords,$dp) = @_;
    my $stable = Browse::Display::chopTable(undef,$table);

    print "<div id='$stable" . "_serviceoptions' style='display:none'>";

my $service_list = Browse::Services::get_services_list($coords,$dp);
# List services unless there aren't any
unless ($table =~ / x / || ! scalar @$service_list ) {
   print "<table><tr><td valign='top'><label for='Services$table'>";
   print "<span style='font-size:80%';>Web-based services for selected rows</span><br>";
   print "</label></td></tr>";
   print "<tr><td><select multiple size='5' id='Services$table' name='Services2'>\n";

   foreach (@$service_list) {
     (my $s=$_)=~s/ /_/g;
     print "<option value='$table $s'> $_";
   }

   print "</select>\n";
   print "<input type='submit' name='GoService' value='GO' onclick=\"this.form.target='_blank'\"></td><tr></table><br>\n";
   print "<a target='helpwin' href='$::w3bhelp#Services'><span style='font-size:80%'>Web-based services help</span></a><p>";
   print "</div>";
}
}


sub DisplayUnusedLinks {

    my ($self, $meta) = @_;

    my($table, $req, $ra, $dec,$numstr) = $self->DisplayLinkSetup($meta);
    my $html;
    
    $html  = "<input type='hidden' name='unused_req_$table' value='$req'>\n";
    $html .= "<input type='hidden' name='ra_$table' value='$ra'>\n";
    $html .= "<input type='hidden' name='dec_$table' value='$dec'>\n";
    $html .= "<input type='hidden' name='numeric_$table' value='$numstr'>\n";
  
    return $html;

}

sub DisplayLinks {

    my ($self,$meta) = @_;
    my ($table, $req, $ra, $dec,$numstr) = $self->DisplayLinkSetup($meta);
    my $links=""; 
    $links .= "<input type='hidden' name='req_$table' value='$req'>";
    $links .= "<input type='hidden' name='ra_$table' value='$ra'>";
    $links .= "<input type='hidden' name='dec_$table' value='$dec'>";
    $links .= "<input type='hidden' name='numeric_$table' value='$numstr'>";
    return $links;

} #eosub--Results::DisplayLinks

#------------------------------------------------------------------------
#--- Selects/Deselects rows and adds/deletes rows on Data Product Retrieval 
#--- tab (overrides super)
sub selectAllRows {
   my ($self, $table) = @_;
   my $html = "<script type='text/javascript' language='JavaScript1.2'>" .
      "\n<!-- \ndocument.write (\"<input type='checkbox' " .
      "id='allobs_$table' name='allobs_$table' " .
      "onClick=tickTable(this.form.Target,this.checked,'$table');addDP(this);>" .
      "<label for='allobs_$table'>All</label>\"); " .
      "\n// --> \n" .
      "</script>\n ";
   return $html;
}

#------------------------------------------------------------------------
#------------------------------------------------------------------------
#--- Selects/Deselects data product sets and adds/deletes sets for Data 
#--- Product Retrieval tab (overrides super)

sub selectAllSets {
   my ($self,$table) = @_;
    my $html ="<script type='text/javascript' language='JavaScript1.2'>\n";
    $html .= "<!-- \n";
    $html .= "document.write (\"<input type='checkbox' checked name='checkall$table' id='checkall$table' onClick=tickTable(this.form.RetrieveSet_$table,this.checked,\\\'\\\');addDP(this)> <b><label for='checkall$table'>All</label><\/b><br>\");\n";
    $html .= "// -->\n";
    $html .= "</script>\n";
    return $html;
}
#------------------------------------------------------------------------
sub selectSet {
   my ($self,$table,$set, $setprint , $desc) = @_;

   print "    <input type='checkbox' checked name='RetrieveSet_$table' id='RetrieveSet_${table}_$setprint' value='$set' onClick='addDP()'><label for='RetrieveSet_${table}_$setprint'>$desc ($setprint)</label><br>\n";
}

#
#########
#  EOF  #
#########
1;

# $Log: TabbedResults.pm,v $
# Revision 1.1  2006/06/23 19:29:07  lmm
# initial commit
#
