# $Id: Vizier.pm,v 1.23 2002/09/13 20:56:16 cwh Exp $
# $Log: Vizier.pm,v $
# Revision 1.23  2002/09/13 20:56:16  cwh
# *** empty log message ***
#
# Revision 1.22  2002/04/06 23:47:16  cwh
# Missing priority was causing priority=5 to be assigned to individual
#   Vizier tables selected from main form.
#
# Revision 1.21  2002/02/14 17:44:09  cwh
# Keep track of params we've seen before, and only print them once on the
#   parameter search form.
#
# Revision 1.20  2002/01/22 22:35:44  cwh
# Added the -meta.max=9999 setting, to retrieve all tables, instead of the
#   default maximum of 500 (including journal tables, this is frequently
#   exceeded).
#
# Revision 1.19  2002/01/16 21:46:04  cwh
# Added the kludges - the degree symbol was causing an error.
#
# Revision 1.18  2001/12/11 19:51:53  cwh
# Some of the keywords have changed. Also added debug statement with link to
# vizier query.
#
# Revision 1.17  2001/11/07 23:34:21  cwh
# Moved everything except DBD/ into lib/Browse/, so as not to pollute the module
# namespace with our Sort, Search, CGI, etc.  This required putting Browse:: in
# front of every module call and package declaration.  Also any mod::new() had
# to be changed to mod->new().
#
# Revision 1.16  2001/08/28 19:14:26  cwh
# Fixed typo - $self didn't exist here.
#
# Revision 1.14  2001/08/20 16:49:47  cwh
# Added traps for XML parsing errors.
#
# Revision 1.13  2001/07/18 19:42:04  cwh
# Added more error info if debugging on.
#
# Revision 1.12  2001/07/01 01:41:19  cwh
# Changed cgi-bin to viz-bin. On some servers either will work, on others only the latter works.
#
# Revision 1.11  2001/06/20 22:57:34  cwh
# Got rid of $::writeHeader in calls to ::Error.
#
# Revision 1.10  2001/06/19 16:06:27  tam
# Make sure that added _RAJ2000/_DEJ2000 are 'default' columns
#
# Revision 1.9  2001/06/18 20:40:07  tam
# Add in missing line in previous commit.
#
# Revision 1.8  2001/06/18 20:38:25  tam
# Use only appropriate elements of Viziew table name for table_document_url
#
# Revision 1.7  2001/06/18 19:12:18  cwh
# These commits were lost in v1.6 and are added back in:
#    date: 2001/06/15 00:51:26;  author: tam;  state: Exp;  lines: +5 -4
#    Add table_document_url in returned table information.
#    Fixes bad links for table display format of VizieR tables.
#    ----------------------------
#    revision 1.5
#    date: 2001/06/14 21:30:04;  author: tam;  state: Exp;  lines: +81 -135
#    Several modifications to set up metadata for VizieR data consistently.
#
# Added metadata for 'All Vizier Tables' query (Z/0).
#
# Revision 1.4  2001/06/13 20:18:16  tam
# Ignore Obsolete catalogs
# More sophisticated translation of HEASARC missions to VizieR spec.
#
# Revision 1.3  2001/05/22 17:52:04  cwh
# Added Id and Log to files without them.
#

use strict;
use XML::Parser;
use LWP::UserAgent;
use HTTP::Request;

package Browse::Vizier;

#%Vizier::tableHash;
#%Vizier::paramHash;

my $textPointer;
my @textStack;
my $lastResource;
my %resourceHash;


# Get the VizieR tables associated with one or
# more HEASARC 'missions'. New tables are
# appended to an existing tables array which
# is passed by reference to the routine.

sub getTables  {
    my ($tables, $missions, $expand) = @_;
    
    my $mission;
    
    foreach $mission(@$missions) {
        processMission($mission, $tables, $expand);
    }
}


# Porcess a single mission.
sub processMission {

    my ($mission, $tables, $expand) = @_;
    my $vizKey;
    
    ($mission, $vizKey) = vizierKey($mission);
    
    if (defined($vizKey)) {
        my $resourceHash = vizierTableQuery($vizKey);
        pushTables($vizKey, $mission, $tables, $expand, $resourceHash);
    } 
}

sub vizierKey {

    my ($mission) = @_;
        
    
    my %trans = (
        "RADIO CATALOG" => "RADIO",
	"STAR CATALOG" => "STARS", 
	"GALAXY CATALOG" => "GALAXIES",
	"NEBULA CATALOG" => "NEBULAE",
	"PHYSICS DATA" => "ATOMIC_DATA"
    );
	
    my @vmiss = ("BeppoSAX", "CGRO",
                    "COBE", "Chandra", "Copernicus", "EUVE",
	            "EXOSAT", "Einstein", "FAUST", "GINGA",
                    "GRANAT", "HEAO", "HST", "Hipparcos", "IRAS",
                    "ISO", "IUE", "OAO-2", "ROSAT", "SMM",
                    "SOHO", "TD1", "UIT", "ULYSSES");
		    
    my @vregs  = ("Radio");
    
    my %vtypes = (
      "Stars" => 
                 ["Stars","Multiple_Stars","Parallaxes","Proper_Motions",
                  "Binaries:cataclysmic", "Binaries:eclipsing",
		  "Binaries:spectroscopic","Masses", "Orbits",
		  "Rotational_Velocities","Globular_Clusters","Open_Clusters",
		  "Stars:Emission","Stars:WR","Stars:early-type",
		  "Stars:late-type", "Stars:peculiar", "Stars:variable",
		  "Stars:white_dwarf", "Novae", "Positional_Data"],
      "Galaxies"=>
                 ["Galaxies", "AGN", "BL_Lac_objects", "Galaxies:Markarian",
		  "Galaxies:spectra", "QSOs", "Redshifts", 
		  "Seyfert_Galaxies", "Clusters_of_galaxies"],
      "Nebulae"=>
                 ["Nebulae", "HII_regions", "Nonstellar",
		  "Interstellar_Medium", "Planetary_Nebulae",
		  "SuperNovae_Remnants"],
      "Atomic_data" =>
                 ["Atomic_Data"]);
    
    if (defined($trans{$mission})) {
        $mission = $trans{$mission};
    }
    
    foreach (@vmiss) {
        if (uc($_) eq uc($mission)) {
	    return ($mission, "-kw.Mission=".Browse::CGI::Encode($_));
	}
    }
    
    foreach (@vregs) {
        if (uc($_) eq uc($mission)) {
	    return ($mission, "-kw.Wavelength=".Browse::CGI::Encode($_));
	}
    }
    
    foreach (keys(%vtypes)) {
        if (uc($_) eq uc($mission)) {
	    my $arr = $vtypes{$_};
	    my $amp;
	    my $crit;
	    my $ele;
	    foreach $ele(@$arr) {
	        $crit .= $amp."-kw.Astronomy=".Browse::CGI::Encode($ele);
	        $amp = '&';
	    }
	    return ($mission, $crit);
	}
    }
    return undef;
}

# Add tables that have been found into the existing array.
sub pushTables  {
    my ($field, $mission, $tables, $expand, $resourceHash) = @_;
    
    my $tab;
    my $jcnt = 0;
    foreach $tab(keys(%$resourceHash)) {
        my $priority = 11;
        if ($tab =~ /^J/) {
	    $priority = 12;
	}
	
        if (!$expand && $tab =~ /^J/) {
	    $jcnt += 1;
	} else {
            push(@$tables,
	        {"name"        => $tab,
		 "description" => $$resourceHash{$tab},
		 "priority"    => $priority,
		 "tabletype"   => "VizieR",
		 "radius"      => 10,
		 "mission"     => $mission,
		 "url"         => "$::vizier/viz-bin/Cat?".Browse::CGI::Encode($tab),
		 "archive"     => "N"});
	}
    }
    if ($jcnt) {
        ($field) = split("&", $field);
	$field =~ s/^\-kw\.//;
	$field =~ s/=.*//;
        push(@$tables,
	       {"name"         => "Z/$field/$mission",
	        "description"  => "$jcnt $mission VizieR journal tables",
		"priority"     => 12,
		"tabletype"    => "VizieR",
		"radius"       => 10,
		"right_ascension" => "ra",
		"declination"  => "dec",
		"mission"      => $mission,
		"url"          => undef,
		"archive"      => "N"});
    }
}

# Handlers for XML parser.  Note that these
# must be anonymous subroutines to avoid problems
# with closures.

my $obsolete;

my $startHandler = sub {

    my ($epat, $element, %attrib) = @_;
    
    $textPointer += 1;
    undef($textStack[$textPointer]);
    
    if ($element eq "RESOURCE") {
        $lastResource = $attrib{"ID"};
	$obsolete = 0;
    }
};

my $endHandler = sub {

    my ($epat, $element) = @_;
    if ($element eq "DESCRIPTION"  && defined($lastResource)) {
        $resourceHash{$lastResource} = $textStack[$textPointer];
    } elsif ($element eq "RESOURCE") {
        if ($obsolete) {
	    delete($resourceHash{$lastResource});
	}
        undef($lastResource);
    } elsif ($element eq "INFO") {
        my $text = $textStack[$textPointer];
	if ($text =~ /OBSOLETE/) {
	    $obsolete = 1;
	}
    }
    
    $textPointer -= 1;
};

my $charHandler = sub {
    my ($epat, $text) = @_;
    $textStack[$textPointer] .= $text;
};

my $parser = new XML::Parser(
         Handlers => {
	   Start => $startHandler,
	   End   => $endHandler,
	   Char  => $charHandler,
	 });

# Query VizieR to find tables which match
# a given criterion.

sub vizierTableQuery {
    $0 = 'Vizier.pm';  # required for MailMessage

    my ($qualifier) = @_;

    my $url = $::vizier;
    $url .= "/viz-bin/asu-xml?";
    $url .= "-meta.what";
    $url .= "\&$qualifier";
    $url .= "\&-meta.max=9999"; # want all tables, default max is 500
   
print "URL: <a href=\"$url\">$url</a><br>\n" if $::debug; 
    my $ua = new LWP::UserAgent;
    my $req = new HTTP::Request GET => $url;
		
    # Pass request to the user agent and get a response back
    my $res = $ua->request($req);
		    
    undef %resourceHash;
    
    # Check the outcome of the response
    if ($res->is_success) {
        eval {
	$parser->parse($res->content);
        };
        if ($@) {
         Browse::MailMessage::MailMessage("Vizier parsing error!\nVizier Request is: $url\n\n$@",$::addr);
         Browse::Utils::Error("Error:  Browse cannot parse XML outputs from VizieR query.  If you have done a query of all VizieR tables you may be able to query selected VizieR tables using the Mission selections buttons.<br>Notice has been sent to the maintainers.");
        }
    } else {
        Browse::Utils::Error("Unable to send VizieR table query");
    }
    return \%resourceHash;
    
 }

sub paramInfo {

    my ($table) = @_;
    
    if (!defined($Browse::Vizier::paramHash{$table})) {
        getMeta($table);
    }

    my $hash = $Browse::Vizier::paramHash{$table};
    my %thash = %$hash;
    
    my @keys = keys(%$hash);
    
    
#    $thash{"#all#"} = \@keys; 
    $thash{"#mjd#"} = [];
    return \%thash;
}
    
sub tableInfo {

    my ($table) = @_;
   
    if ($table =~ /^Z\/0$/i) {
         my $h = {};
         $$h{"table_description"}  = "All VizieR tables";
         $$h{"table_name"} = $table;
         $$h{"url"} = "$::vizier/viz-bin/vizHelp?cats/cats.htx";
	 $$h{"default_search_radius"} = 10;
         $$h{"tabletype"}          = "VizieR";
         $$h{"observatory_name"}   = " ** ";
         $$h{"priority"}   = "11";
         $$h{"right_ascension"} = "ra";
         $$h{"declination"}     = "dec";
         $$h{"equinox"} = 2000;
         $$h{"query_key"}  = "recno";
         $$h{"unique_key"} = "recno";
         $Browse::Vizier::tableHash{$table} = $h;
    }
    
    if (!$Browse::Vizier::tableHash{$table}) {
        getMeta($table);
    }
    my $h = $Browse::Vizier::tableHash{$table};
    
    return $Browse::Vizier::tableHash{$table};
    
}

sub getComboMeta {


    my ($table) = @_;
    my $h = {};
    my %params = ();
    
    $$h{"table_name"}            = $table;
    $$h{"table_location"}        = "vizier.u-strasbg.fr";
    $$h{"default_search_radius"} = 10;
    
    my ($z, $viztype, $mission) = split('/', $table);
    $$h{"table_description"}     = "VizieR $mission journal tables";
    $$h{"right_ascension"} = "ra";
    $$h{"declination"}     = "dec";
    $$h{"equinox"} = 2000;
    $$h{"query_key"}  = "recno";
    $$h{"unique_key"} = "recno";
    
    $Browse::Vizier::tableHash{$table} = $h;
    
    my @pars = ("name", "ra", "dec");
    
    my $par;
    foreach $par(@pars) {
    
        my $pp = {};
	
	$$pp{"table_name"}         =  $table;
	$$pp{"parameter_name"}     =    $par;
	$$pp{"parameter_is_index"} =       0;
	$$pp{"parameter_unit"}     =   undef;
	$$pp{"parameter_min"}      =   undef;
	$$pp{"parameter_max"}      =   undef;
	$$pp{"parameter_default"}  =       1;
	$$pp{"parameter_format"}   ="float8";
	$params{$par} = $pp;
    }
    
    $params{"#all#"} = [];
    $params{"#default#"} = [];
    $params{"#not_default#"} = [];
    $params{"#mjd#"} = [];
    
    $Browse::Vizier::paramHash{$table} = \%params;
}
    
sub getMeta {

    my ($table) = @_;
    
    
    if ($table =~ /^Z\/[^0]/) {
        getComboMeta($table);
	return;
    }
    my $url = "$::vizier/viz-bin/asu-xml?-meta.all&-source=".
           Browse::CGI::Encode($table)."&-out.all&-out.add=_RAJ2000,_DEJ2000";

    my $ua = new LWP::UserAgent;
    my $req = new HTTP::Request GET => $url;
    
    # Pass request to the user agent and get a response back
    my $res = $ua->request($req);

    # Check the outcome of the response
    if ($res->is_success) {
        my $data = $res->content;
        #$data =~ s/ display\=(0|1) / display="$1" /g;
	parseMeta($table, $data);
    } else {
        my $msg = $res->error_as_HTML;# if $::debug;
        Browse::Utils::Error("Error in VizieR meta query. $msg");
    }
}

sub parseMeta {
    my ($table, $text) = @_;
    
    my $desc;
    my %params;
    my $inRes;
    my @parOrder;
    my $startHandler = sub {

        my ($epat, $element, %attrib) = @_;
	
        $textPointer += 1;
        undef($textStack[$textPointer]);
	
	if ($element eq "FIELD") {
	    my $id = $attrib{"id"};
	    
	    if ($id eq "MaxRec" || $id eq "SkyLocation") {
	        return;
	    }
	    
	    $id = $attrib{"name"};
	    push(@parOrder, $id);
	    
	    if (defined($id)) {
	        $params{$id} = \%attrib;
	    }
	} elsif ($element eq "RESOURCE") {
	    $inRes = 1;
	}
    };

    my $endHandler = sub {

        my ($epat, $element) = @_;
        if ($element eq "DESCRIPTION"  && $inRes) {
	    $desc = $textStack[$textPointer];
	    $inRes = 0;
        }
    
        $textPointer -= 1;
    };

    my $parser = new XML::Parser(
         Handlers => {
	   Start => $startHandler,
	   End   => $endHandler,
	   Char  => $charHandler,
	 });

    # KludgeKludgeKludgeKludgeKludge
    $text =~ s/\<LINK \\[^\n]*//gm;
    $text =~ s// deg/gm;
    # KludgeKludgeKludgeKludgeKludge

    eval { 
    $parser->parse($text);
    };
    if ($@) {
      Browse::MailMessage::MailMessage("Vizier parsing error!\n\n$@",$::addr);
      Browse::Utils::Error("<pre>$text</pre>") if $::debug;
      Browse::Utils::Error("Error:  Browse cannot parse XML outputs from VizieR query.  If you have done a query of all VizieR tables you may be able to query selected VizieR tables using the Mission selections buttons.<br>Notice has been sent to the maintainers.");
    }
    my %h;
    
    my $key;
    my (@xkey) = keys(%params);
    
    $h{"table_name"} = $table;
    $h{"url"} = "$::vizier/viz-bin/Cat?".
     Browse::CGI::Encode($table);
    $h{"table_location"} = "vizier.u-strasbg.fr";
    foreach $key(@xkey) {
        #if ($key =~ /(ra)(J|B)?(2000|1950)/i ) {
        if ($key eq "_RAJ2000" ) {
	    $h{"default_search_radius"} = 10;
           $h{"equinox"}               = 2000;
           $h{"declination"}           = "_DEJ2000";
           $h{"right_ascension"}       = "_RAJ2000";
	}

    }
    
    chomp($desc);
    $desc =~ s/^\s*//;
    $desc =~ s/\s*$//;
    
    
    $h{"table_description"} = $desc;
    foreach $key(@xkey) {
        if ($key =~ /^(ra)(J|B)?(2000|1950)?$/i) {
	
	    # Use 2000 coordinates by preference, so
	    # if we've gotten those we skip anything further.
	    if ($h{"equinox"} == 2000) {
	        next;
	    }
	    
	    $h{"right_ascension"} = $key;
	    my $de;
	    if ($1 eq "RA") {
	        $de = "DE";
	    } else {
	        $de = "de";
	    }
	    $h{"declination"} = "$de$2$3";
            $h{"equinox"} = 2000;
	    last;
	}
    }
    
    $h{"tabletype"}          = "VizieR";
    $h{"observatory_name"}   = " ** ";
    $h{"priority"}   = "11";

    
    my $tdoc;
    my @flds = split('/', $table);
    if ($table =~ /^J/) {
	$tdoc = "$flds[0]/$flds[1]/$flds[2]/$flds[3]";
    } else {
        $tdoc = "$flds[0]/$flds[1]";
    }
	
    $h{"table_document_url"} = "$::vizier/viz-bin/Cat?$tdoc";

    $Browse::Vizier::tableHash{$table} = \%h;
    my $dft = 1;
    my @defaults;
    my @non_defaults;
    my %seen=();

    foreach $key(@parOrder) {

        next if $seen{$key};
        my $pars = $params{$key};

    
        if (substr($key, 0, 1) eq '-') {
	    delete($params{$key});
	    
	} else {
	
            my $h = $params{$key};
	
	    $$h{"table_name"}         = $table;
	    $$h{"parameter_name"}     = $key;
	    $$h{"parameter_is_index"} = 0;
	    $$h{"parameter_unit"}     = undef;
	    $$h{"parameter_min"}      = undef;
	    $$h{"parameter_max"}      = undef;
            if ($$pars{"display"}  || $key eq "_RAJ2000" ||
	                              $key eq "_DEJ2000" ) {
                $$h{"parameter_default"} = $dft;
                $dft += 1;
                push(@defaults, $key);
                $seen{$key}++;
            } else {
                $$h{"parameter_default"} = 0;
                push(@non_defaults, $key);
                $seen{$key}++;
            }

	    my $form = $$h{"datatype"};
           if ($key eq "_RAJ2000" || $key eq "_DEJ2000") {
               $$h{"parameter_format"} = "float8";
           } elsif ($form eq "A") {
	        $$h{"parameter_format"} = "char".$$h{"width"};
	    } elsif ($form eq "I") {
	        $$h{"parameter_format"} = "int4";
	    } elsif ($form eq "F") {
	        $$h{"parameter_format"} = "float8";
	    }
	}
    }
    
    $params{"#default#"} = \@defaults;
    $params{"#not_default#"} = \@non_defaults;
    my @all = @defaults;
    push(@all, @non_defaults);
    $params{"#all#"} = \@all;
    
    
    if ($params{"recno"}) {
        $h{"query_key"}  = "recno";
        $h{"unique_key"} = "recno";
	
    } else {
        if ($h{"table_name"} =~ /2mass/i) { 
            $h{"query_key"} = $h{"unique_key"} = "RAJ2000,DEJ2000";
        } else {
            foreach $key(@parOrder) {
	        
	        if ($key ne '_RAJ2000' &&
	            $key ne '_DEJ2000'  && $key !~ /^[\-\_]/) {
		
                    $h{"query_key"}  = $key;
                    $h{"unique_key"} = $key;
	            last;
	        }
	    }
        }
    }
		

    $Browse::Vizier::paramHash{$table} = \%params;
    
}
	 
1;
