#!/usr1/local/bin/perl5 -T
# $Id: w3hdprods.pl,v 1.48 2006/06/23 19:42:50 lmm Exp $
# $Log: w3hdprods.pl,v $
# Revision 1.48  2006/06/23 19:42:50  lmm
# Removed tabbed definition for data products
#
# Revision 1.47  2006/06/23 13:50:10  lmm
# Changes for Tabbed Display and future Notification Service
#
# Revision 1.46  2005/12/13 21:33:28  lmm
# Made slight change to redisplay/requery logic.
#
# Revision 1.45  2005/12/13 16:39:47  lmm
# Changed to ensure use of current display format upon requery.
#
# Revision 1.44  2005/09/19 18:12:34  lmm
# Replaced call to saveTofile with call to ProdHera.
# Added changes check for Continue Download for new choice
#   given to users when tar file is exceedingly large.
#
# Revision 1.43  2004/10/08 14:47:35  lmm
# Commented out Benchmark tar file testing (may be used in the future)
# Made a small change to the Data Products retrieval failure error message
#
# Revision 1.42  2004/10/04 21:25:01  lmm
# Added call to HProdScriptGen for download script display
# Added some debug code for ongoing data products download time tests
#
# Revision 1.41  2004/01/15 18:13:17  lmm
# Save Objects To File button now calls subclass of PureTextDisplay
#
# Revision 1.40  2003/12/02 22:15:31  lmm
# Added processing for Save All Targets to File and Save Query to File
#   buttons
#
# Revision 1.39  2003/08/28 20:28:49  lmm
# Added more meaningful text to headers on Data Product pages
#
# Revision 1.38  2003/07/31 15:31:43  lmm
# Added code for Printer-Friendly Button action
#
# Revision 1.37  2003/07/31 14:27:08  cwh
# *** empty log message ***
#
# Revision 1.36  2003/07/31 14:26:38  cwh
# Added requery action which goes to Resort.pm.
# Pull variables out of decoded string.
#
# Revision 1.35  2002/12/16 03:26:46  cwh
# No longer encoding Target, or searching for encoded targets, or creating
#   encoded targets.
#
# Revision 1.34  2002/09/13 18:33:54  cwh
# Now using new ProdMain/ProdGen/ProdDisplay modules.
#
# Revision 1.33  2002/02/14 17:18:54  cwh
# Added actions for Save to Hera and Services.
#
# Revision 1.32  2001/12/11 19:46:38  cwh
# Now encoding req_ et al.
#
# Revision 1.31  2001/12/05 23:41:37  cwh
# Changed rewriting of STScI-style requests to include the unique_key.
#
# Revision 1.30  2001/11/08 19:57:40  cwh
# Changed 'do w3config.pl' to 'use Config' and AlwaysRun().
# Added Browse:: to all sub calls.
#
# Revision 1.29  2001/10/25 22:30:25  cwh
# Changed the label of the View button to 'Preview and Retrieve'.
#
# Revision 1.28  2001/10/25 20:24:46  cwh
# Put Products::retrieve in an eval, mailing and reporting to user if fails.
#
# Revision 1.27  2001/09/15 02:43:26  cwh
# Added redisplay call as flag to go to Resort::display().
# Cleaned up code a teeny bit.
#
# Revision 1.26  2001/08/08 20:51:01  cwh
# Added section to translate STScI archive queries - they are still sending the
#   old form elements.
#
# Revision 1.25  2001/06/20 22:57:23  cwh
# Got rid of $::writeHeader in calls to ::Error.
#
# Revision 1.24  2001/06/08 19:46:31  cwh
# Now passing $::writeHeader to ::Error calls, so we don't get repeating headers.
#
# Revision 1.23  2001/05/24 16:40:44  cwh
# This mod_perl stuff is killing me.  Got rid of all the BEGIN blocks (those aren't
# called if they've been already done once).  Added the 'use lib "./lib";'s back in.
#
# Revision 1.22  2001/05/22 17:57:12  cwh
# Added Id, Log to those scripts that needed it.
#
#
#=======================================================================#
# High Energy Astrophysics Science Archive Research Center (HEASARC)
# NASA's Goddard Space Flight Center, Greenbelt, MD 20771, USA.
# You are free to use this code as you wish, but unless you change
# it significantly, please leave the above name & address intact.
#=======================================================================#

use strict;
use lib './lib';

use Browse::Config;
use Browse::ValidateInput;
use Browse::RowDisplay;
use Browse::Resort;
use Browse::ProdMain;
use Browse::Cross;
use Browse::Java;
use Browse::saveFile;
use Browse::ProdHera;
use Browse::Notification;
use Browse::Services;
use Browse::HDB;

Browse::Config::AlwaysRun();

$0 = 'w3hdprods.pl'; # Required for MailMessage. Do not remove.
#------------------------------------------------------------------------
$| = 1; # Make STDOUT unbuffered.
#$::debug=1;
print Browse::CGI::Header,"w3hdprods.pl: DEBUG<br>\n" if $::debug;
my($table_info) = {};

my($in) = Browse::CGI::ReadParse();

# this section is to translate STScI archive queries 
if ($$in{"Target"} =~ /table.*:|table.*%3A/) {
    #$$in{"Target"} = Browse::CGI::Encode($$in{"Target"}) if $$in{"Target"} =~ /:/;
    $$in{"Target"} = translate_target($$in{"Target"});
    my $setkey = '';
    foreach $setkey (%$in) {
         if ($setkey =~ /RetrieveSet/) {
            (my $tab) = $setkey =~ /RetrieveSet_(.*)/;
            my $save = $$in{$setkey};
            delete $$in{$setkey};
            $setkey = "RetrieveSet_".lc($tab);
            $$in{$setkey} = $save;
         }
    }
}

#--- pass along window title for popup info
if ($$in{"popupFrom"} eq "") {
    $$in{"popupFrom"} = 
	"Query Results from Main Search";
}

foreach my $key (keys %$in) {


    if ($key =~ /^imgDisp/) {
        Browse::RowDisplay::displaySingle($key, $in);

    } elsif ($key =~ /^imgSort|\bredisplay\b|\brequery\b/i) {
        #--- Ignore Redisplay setting if requery button is selected. 
        if ($key !~ /\bredisplay\b/i && ($$in{requerydisplay} =~ /\w/)) { 
           $$in{"displaymode"} = $$in{requerydisplay};
        }

        #-- Use Pure Text Display for Printer-Friendly and Save Targets formats
        if ($key =~ /redisplay/i && $$in{$key} =~ /Printer-Friendly/i) { 
              $$in{"displaymode"} = "PureTextDisplay";
        }

        if ($$in{$key} =~ /Save All Objects/i) {  
           $$in{"displaymode"} = "SaveObjectsToFile";
        } 

        Browse::Resort::display($key, $in);
        

    } elsif ($key =~ /^Notify (.*)/) {
       Browse::Notification::AccessService($1, $in);

    } elsif ($key =~ /^Correlate (.*)/) {
        Browse::Cross::CrossCorrelate($1, $in);

    } elsif ($key =~ /^Save to Hera/i) {

       $$in{"files"} = "analyze";
        

    eval {
      my $dp = new Browse::ProdMain($in);
      $dp->doProducts($in);
    };
    exit;

     } elsif ($key =~ /^GoService/) {
         Browse::Services::query_services($in);
        
    } elsif ($key =~ /^Plot (.*)/) {
        my $tab = Browse::CGI::Encode($1);
        my $request = $$in{"req_$tab"};
        my $ra      = $$in{"ra_$tab"};
        my $dec     = $$in{"dec_$tab"};
        my $numeric = $$in{"numeric_$tab"};
        Browse::Java::DoPlot($ra, $dec, $numeric, "Equatorial", 2000, $request, "applet");
    }
}

# need to keep the D/L/S/P flags for the STScI ROSAT and ASCA interfaces.
my $action = $$in{"files"};
print "w3hdprods.pl: Action is $action<br>" if $::debug;

# Get table name for data products page headers
my ($stable,$kval) = split('\|\|\|', $$in{"Target"}, 2);
my %tinfo = Browse::HDB::TableInfo($stable);

if ($action =~ /^Display|^P$/) {
    # Define page header title
    if ($action =~ /^P$/) {
       $$in{"popupFrom"} = "Parameter Display for single row in ". 
 		$tinfo{"table_description"};

    } elsif ($action =~ /^Display/) {
       $$in{"popupFrom"} = "Parameter Display for selected rows"; 
    }
    Browse::RowDisplay::displayMulti($$in{"Target"}, $in);
    exit;

} elsif ($action =~ /^Save Query To/i) { 
        # Save serialized query string to file for later use 
        Browse::Utils::dataToFile($$in{'savedQuery'},
		$::savedQueryFile . ".$$.data");
        exit;
} elsif ($action =~ /^Retrieve|^S$|Download Script|^Analyze|Continue Download|Add selected/) { 

    print " files=" .$$in{"files"} . "<br>" if $::debug;
    # Define page header title
    if ($action =~ /^Retrieve/) {
       $$in{"popupFrom"} = "Data Products Retrieval for selected rows"; 
       $$in{"files"} = "write";
 		
    } elsif ($action =~/Continue Download/) {
       $$in{createTarFile} = 1;
       $$in{"files"} = "write";
    } elsif ($action =~/Download Script To File/) {
       $$in{"files"} = "scriptfile";
    } elsif ($action =~/Download Script/) {
       $$in{"files"} = "script";
 		
    #} elsif ($action =~/Add selected rows/) {
       #$$in{"files"} = "tabbed";
    } elsif ($action =~/^Analyze/) {
       $$in{"files"} = "analyze";
    }

    #-------------------------------------------------------------
    #--- data products time tests
    #use Benchmark;
    #my $start_time = new Benchmark;
    #-------------------------------------------------------------
    eval {
      my $dp = new Browse::ProdMain($in);
      $dp->doProducts($in);
    };
    exit;

    #-------------------------------------------------------------
    #--- data products time tests
    #my $end_time = new Benchmark;   # benchmark
    #my $difference = timediff($end_time, $start_time); 	#benchmark
    #print "host:" .hostname();
    #print "<br>: took ", timestr($difference), "<br>" ;
    #-------------------------------------------------------------

    if ($@) {
      Browse::MailMessage::MailMessage(
	  "w3hdprods.pl's eval of ProdGen failed! $@",$::addr) and
      Browse::Utils::Error(
	  "Error providing data products.  Email has been sent to the maintainers.");
    }
    exit;
} elsif ($action =~/^Preview|^D$|^L$/) { 

    if ($action =~ /^Preview/ && $action =~/Retrieve/) {
       # Data products for all selected rows (button)
       $$in{"popupFrom"} = "Data Products for selected rows"; 
    } else {
       # Data products for one row
       $$in{"popupFrom"} = "Data Products for selected row in ". 
 		$tinfo{"table_description"};
    }


    $$in{"files"} = "view";
    my $dp = new Browse::ProdMain($in);
    $dp->doProducts($in);
    exit;
}

sub translate_target {
    (my $in_target) = @_;
     my $out_target = my $sep = my $target = '';
     my @targets = split('\0',$in_target);
     foreach $target (@targets) {
        my ($table,$key,$uniqid) = $target =~ /table%3D(.*)%26key%3D(.*)%3A.*%2F%2F([^%]*)/i;
        #my ($table,$key,$uniqid) = $target =~ /table=(.*)&key=(.*):.*\/\/([^\|]*)/i;
        
        print "table=$table, key=$key, uniqid=$uniqid<br>" if $::debug;
        $out_target .= "\0" if $out_target; # \0 is the separator
        #$out_target .= "$table%7C%7C%7C$key=$uniqid%7C%7C";
        $out_target .= "$table|||$key=$uniqid||";
     }
     return $out_target;
}

Browse::Utils::Error("No action specified.  Please try again.");
exit;

1;
