###############################################################################
#
# MODULE: $Id: HDBclass.pm,v 1.8 2002/01/23 20:04:16 sabol Exp $
#
#  AUTHOR: Edward J. Sabol <sabol@alderaan.gsfc.nasa.gov>
# CREATED: 1999/10
#
# HISTORY:
# $Log: HDBclass.pm,v $
# Revision 1.8  2002/01/23 20:04:16  sabol
# Minor changes to comments.
#
# Revision 1.7  2001/05/11 21:52:00  sabol
# Imported into HEASARC CVS repository.
#
# Revision 1.6  2001/04/04 13:20:47  sabol
# Removed 8224/"UNCLASSIFIED" hack. Changed 5500 from "BACKGROUND" to "X-RAY
#   BACKGROUND". Added 1890/"LUMINOUS BLUE VARIABLE (LBV)". Changed "SD" white
#   dwarf stars to "sD". Added "WITH COOLING FLOW" suffix to 50x1 classes.
#   Changed Wolf Rayet 20x9 classes from just spectral type 9 to spectral type
#   9 through 11. Added luminosity class VI throughout the STAR classes. Added
#   subclassifications to 8100 for each of the solar system's planets. Changed
#   8200 class from "THE SUN" to "SOLAR FEATURE" and added subclassifications
#   to it. Added subclassifications to 8300 and 8400 as well.
#
# Revision 1.5  2001/04/03 12:12:51  sabol
# num2name() now returns UNCLASSIFIED instead of ERROR for out-of-range input
#   values (for consistency with the old Browse and HDBfilter implementations).
#
# Revision 1.4  2001/03/14 14:29:34  sabol
# Changed the 2800 class from "STAR DME*" to "STAR M*e".
#
# Revision 1.3  2000/10/16 17:25:25  sabol
# Added num2name() routine.
#
# Revision 1.2  2000/07/14 17:08:34  sabol
# Added new classes (1810, 1820, 1830, 1840, 1850, 1860, 1870, 1880, 5100,
#   7800, 7900, 9200) and renamed classes 2900 and 3000.
#
# Revision 1.1  1999/11/04 17:59:50  sabol
# Initial release.
#
#=============================================================================#
# High Energy Astrophysics Science Archive Research Center (HEASARC),
# NASA's Goddard Space Flight Center, Greenbelt, MD 20771, USA.
#
# You are free to use this code as you wish, but unless you change
# it significantly, please leave the above name and address intact.
#=============================================================================#
# Module Description:
#   Perl hash data structure containing the HEASARC's class ID number to
#   class name schema.
#=============================================================================#

package HDBclass;

use strict;
use vars qw(%name);

%name =
(
    0 => 'UNCLASSIFIED',
    1 => 'UNCLASSIFIED',
    2 => 'UNCLASSIFIED',
    3 => 'UNCLASSIFIED',
    4 => 'UNCLASSIFIED',
    5 => 'UNCLASSIFIED',
    6 => 'UNCLASSIFIED',
    7 => 'UNCLASSIFIED',
    8 => 'UNCLASSIFIED',
    9 => 'UNCLASSIFIED',
    10 => 'UNCLASSIFIED',
    11 => 'UNCLASSIFIED',
    12 => 'UNCLASSIFIED',
    13 => 'UNCLASSIFIED',
    14 => 'UNCLASSIFIED',
    15 => 'UNCLASSIFIED',
    16 => 'UNCLASSIFIED',
    17 => 'UNCLASSIFIED',
    18 => 'UNCLASSIFIED',
    19 => 'UNCLASSIFIED',
    20 => 'UNCLASSIFIED',
    21 => 'UNCLASSIFIED',
    22 => 'UNCLASSIFIED',
    23 => 'UNCLASSIFIED',
    24 => 'UNCLASSIFIED',
    25 => 'UNCLASSIFIED',
    26 => 'UNCLASSIFIED',
    27 => 'UNCLASSIFIED',
    28 => 'UNCLASSIFIED',
    29 => 'UNCLASSIFIED',
    30 => 'UNCLASSIFIED',
    31 => 'UNCLASSIFIED',
    32 => 'UNCLASSIFIED',
    33 => 'UNCLASSIFIED',
    34 => 'UNCLASSIFIED',
    35 => 'UNCLASSIFIED',
    36 => 'UNCLASSIFIED',
    37 => 'UNCLASSIFIED',
    38 => 'UNCLASSIFIED',
    39 => 'UNCLASSIFIED',
    40 => 'UNCLASSIFIED',
    41 => 'UNCLASSIFIED',
    42 => 'UNCLASSIFIED',
    43 => 'UNCLASSIFIED',
    44 => 'UNCLASSIFIED',
    45 => 'UNCLASSIFIED',
    46 => 'UNCLASSIFIED',
    47 => 'UNCLASSIFIED',
    48 => 'UNCLASSIFIED',
    49 => 'UNCLASSIFIED',
    50 => 'UNCLASSIFIED',
    51 => 'UNCLASSIFIED',
    52 => 'UNCLASSIFIED',
    53 => 'UNCLASSIFIED',
    54 => 'UNCLASSIFIED',
    55 => 'UNCLASSIFIED',
    56 => 'UNCLASSIFIED',
    57 => 'UNCLASSIFIED',
    58 => 'UNCLASSIFIED',
    59 => 'UNCLASSIFIED',
    60 => 'UNCLASSIFIED',
    61 => 'UNCLASSIFIED',
    62 => 'UNCLASSIFIED',
    63 => 'UNCLASSIFIED',
    64 => 'UNCLASSIFIED',
    65 => 'UNCLASSIFIED',
    66 => 'UNCLASSIFIED',
    67 => 'UNCLASSIFIED',
    68 => 'UNCLASSIFIED',
    69 => 'UNCLASSIFIED',
    70 => 'UNCLASSIFIED',
    71 => 'UNCLASSIFIED',
    72 => 'UNCLASSIFIED',
    73 => 'UNCLASSIFIED',
    74 => 'UNCLASSIFIED',
    75 => 'UNCLASSIFIED',
    76 => 'UNCLASSIFIED',
    77 => 'UNCLASSIFIED',
    78 => 'UNCLASSIFIED',
    79 => 'UNCLASSIFIED',
    80 => 'UNCLASSIFIED',
    81 => 'UNCLASSIFIED',
    82 => 'UNCLASSIFIED',
    83 => 'UNCLASSIFIED',
    84 => 'UNCLASSIFIED',
    85 => 'UNCLASSIFIED',
    86 => 'UNCLASSIFIED',
    87 => 'UNCLASSIFIED',
    88 => 'UNCLASSIFIED',
    89 => 'UNCLASSIFIED',
    90 => 'UNCLASSIFIED',
    91 => 'UNCLASSIFIED',
    92 => 'UNCLASSIFIED',
    93 => 'UNCLASSIFIED',
    94 => 'UNCLASSIFIED',
    95 => 'UNCLASSIFIED',
    96 => 'UNCLASSIFIED',
    97 => 'UNCLASSIFIED',
    98 => 'UNCLASSIFIED',
    99 => 'UNCLASSIFIED',
    100 => 'UNCLASSIFIED',
    101 => 'UNCLASSIFIED',
    102 => 'UNCLASSIFIED',
    103 => 'UNCLASSIFIED',
    104 => 'UNCLASSIFIED',
    105 => 'UNCLASSIFIED',
    106 => 'UNCLASSIFIED',
    107 => 'UNCLASSIFIED',
    108 => 'UNCLASSIFIED',
    109 => 'UNCLASSIFIED',
    110 => 'UNCLASSIFIED',
    111 => 'UNCLASSIFIED',
    112 => 'UNCLASSIFIED',
    113 => 'UNCLASSIFIED',
    114 => 'UNCLASSIFIED',
    115 => 'UNCLASSIFIED',
    116 => 'UNCLASSIFIED',
    117 => 'UNCLASSIFIED',
    118 => 'UNCLASSIFIED',
    119 => 'UNCLASSIFIED',
    120 => 'UNCLASSIFIED',
    121 => 'UNCLASSIFIED',
    122 => 'UNCLASSIFIED',
    123 => 'UNCLASSIFIED',
    124 => 'UNCLASSIFIED',
    125 => 'UNCLASSIFIED',
    126 => 'UNCLASSIFIED',
    127 => 'UNCLASSIFIED',
    128 => 'UNCLASSIFIED',
    129 => 'UNCLASSIFIED',
    130 => 'UNCLASSIFIED',
    131 => 'UNCLASSIFIED',
    132 => 'UNCLASSIFIED',
    133 => 'UNCLASSIFIED',
    134 => 'UNCLASSIFIED',
    135 => 'UNCLASSIFIED',
    136 => 'UNCLASSIFIED',
    137 => 'UNCLASSIFIED',
    138 => 'UNCLASSIFIED',
    139 => 'UNCLASSIFIED',
    140 => 'UNCLASSIFIED',
    141 => 'UNCLASSIFIED',
    142 => 'UNCLASSIFIED',
    143 => 'UNCLASSIFIED',
    144 => 'UNCLASSIFIED',
    145 => 'UNCLASSIFIED',
    146 => 'UNCLASSIFIED',
    147 => 'UNCLASSIFIED',
    148 => 'UNCLASSIFIED',
    149 => 'UNCLASSIFIED',
    150 => 'UNCLASSIFIED',
    151 => 'UNCLASSIFIED',
    152 => 'UNCLASSIFIED',
    153 => 'UNCLASSIFIED',
    154 => 'UNCLASSIFIED',
    155 => 'UNCLASSIFIED',
    156 => 'UNCLASSIFIED',
    157 => 'UNCLASSIFIED',
    158 => 'UNCLASSIFIED',
    159 => 'UNCLASSIFIED',
    160 => 'UNCLASSIFIED',
    161 => 'UNCLASSIFIED',
    162 => 'UNCLASSIFIED',
    163 => 'UNCLASSIFIED',
    164 => 'UNCLASSIFIED',
    165 => 'UNCLASSIFIED',
    166 => 'UNCLASSIFIED',
    167 => 'UNCLASSIFIED',
    168 => 'UNCLASSIFIED',
    169 => 'UNCLASSIFIED',
    170 => 'UNCLASSIFIED',
    171 => 'UNCLASSIFIED',
    172 => 'UNCLASSIFIED',
    173 => 'UNCLASSIFIED',
    174 => 'UNCLASSIFIED',
    175 => 'UNCLASSIFIED',
    176 => 'UNCLASSIFIED',
    177 => 'UNCLASSIFIED',
    178 => 'UNCLASSIFIED',
    179 => 'UNCLASSIFIED',
    180 => 'UNCLASSIFIED',
    181 => 'UNCLASSIFIED',
    182 => 'UNCLASSIFIED',
    183 => 'UNCLASSIFIED',
    184 => 'UNCLASSIFIED',
    185 => 'UNCLASSIFIED',
    186 => 'UNCLASSIFIED',
    187 => 'UNCLASSIFIED',
    188 => 'UNCLASSIFIED',
    189 => 'UNCLASSIFIED',
    190 => 'UNCLASSIFIED',
    191 => 'UNCLASSIFIED',
    192 => 'UNCLASSIFIED',
    193 => 'UNCLASSIFIED',
    194 => 'UNCLASSIFIED',
    195 => 'UNCLASSIFIED',
    196 => 'UNCLASSIFIED',
    197 => 'UNCLASSIFIED',
    198 => 'UNCLASSIFIED',
    199 => 'UNCLASSIFIED',
    200 => 'UNCLASSIFIED',
    201 => 'UNCLASSIFIED',
    202 => 'UNCLASSIFIED',
    203 => 'UNCLASSIFIED',
    204 => 'UNCLASSIFIED',
    205 => 'UNCLASSIFIED',
    206 => 'UNCLASSIFIED',
    207 => 'UNCLASSIFIED',
    208 => 'UNCLASSIFIED',
    209 => 'UNCLASSIFIED',
    210 => 'UNCLASSIFIED',
    211 => 'UNCLASSIFIED',
    212 => 'UNCLASSIFIED',
    213 => 'UNCLASSIFIED',
    214 => 'UNCLASSIFIED',
    215 => 'UNCLASSIFIED',
    216 => 'UNCLASSIFIED',
    217 => 'UNCLASSIFIED',
    218 => 'UNCLASSIFIED',
    219 => 'UNCLASSIFIED',
    220 => 'UNCLASSIFIED',
    221 => 'UNCLASSIFIED',
    222 => 'UNCLASSIFIED',
    223 => 'UNCLASSIFIED',
    224 => 'UNCLASSIFIED',
    225 => 'UNCLASSIFIED',
    226 => 'UNCLASSIFIED',
    227 => 'UNCLASSIFIED',
    228 => 'UNCLASSIFIED',
    229 => 'UNCLASSIFIED',
    230 => 'UNCLASSIFIED',
    231 => 'UNCLASSIFIED',
    232 => 'UNCLASSIFIED',
    233 => 'UNCLASSIFIED',
    234 => 'UNCLASSIFIED',
    235 => 'UNCLASSIFIED',
    236 => 'UNCLASSIFIED',
    237 => 'UNCLASSIFIED',
    238 => 'UNCLASSIFIED',
    239 => 'UNCLASSIFIED',
    240 => 'UNCLASSIFIED',
    241 => 'UNCLASSIFIED',
    242 => 'UNCLASSIFIED',
    243 => 'UNCLASSIFIED',
    244 => 'UNCLASSIFIED',
    245 => 'UNCLASSIFIED',
    246 => 'UNCLASSIFIED',
    247 => 'UNCLASSIFIED',
    248 => 'UNCLASSIFIED',
    249 => 'UNCLASSIFIED',
    250 => 'UNCLASSIFIED',
    251 => 'UNCLASSIFIED',
    252 => 'UNCLASSIFIED',
    253 => 'UNCLASSIFIED',
    254 => 'UNCLASSIFIED',
    255 => 'UNCLASSIFIED',
    256 => 'UNCLASSIFIED',
    257 => 'UNCLASSIFIED',
    258 => 'UNCLASSIFIED',
    259 => 'UNCLASSIFIED',
    260 => 'UNCLASSIFIED',
    261 => 'UNCLASSIFIED',
    262 => 'UNCLASSIFIED',
    263 => 'UNCLASSIFIED',
    264 => 'UNCLASSIFIED',
    265 => 'UNCLASSIFIED',
    266 => 'UNCLASSIFIED',
    267 => 'UNCLASSIFIED',
    268 => 'UNCLASSIFIED',
    269 => 'UNCLASSIFIED',
    270 => 'UNCLASSIFIED',
    271 => 'UNCLASSIFIED',
    272 => 'UNCLASSIFIED',
    273 => 'UNCLASSIFIED',
    274 => 'UNCLASSIFIED',
    275 => 'UNCLASSIFIED',
    276 => 'UNCLASSIFIED',
    277 => 'UNCLASSIFIED',
    278 => 'UNCLASSIFIED',
    279 => 'UNCLASSIFIED',
    280 => 'UNCLASSIFIED',
    281 => 'UNCLASSIFIED',
    282 => 'UNCLASSIFIED',
    283 => 'UNCLASSIFIED',
    284 => 'UNCLASSIFIED',
    285 => 'UNCLASSIFIED',
    286 => 'UNCLASSIFIED',
    287 => 'UNCLASSIFIED',
    288 => 'UNCLASSIFIED',
    289 => 'UNCLASSIFIED',
    290 => 'UNCLASSIFIED',
    291 => 'UNCLASSIFIED',
    292 => 'UNCLASSIFIED',
    293 => 'UNCLASSIFIED',
    294 => 'UNCLASSIFIED',
    295 => 'UNCLASSIFIED',
    296 => 'UNCLASSIFIED',
    297 => 'UNCLASSIFIED',
    298 => 'UNCLASSIFIED',
    299 => 'UNCLASSIFIED',
    300 => 'UNCLASSIFIED',
    301 => 'UNCLASSIFIED',
    302 => 'UNCLASSIFIED',
    303 => 'UNCLASSIFIED',
    304 => 'UNCLASSIFIED',
    305 => 'UNCLASSIFIED',
    306 => 'UNCLASSIFIED',
    307 => 'UNCLASSIFIED',
    308 => 'UNCLASSIFIED',
    309 => 'UNCLASSIFIED',
    310 => 'UNCLASSIFIED',
    311 => 'UNCLASSIFIED',
    312 => 'UNCLASSIFIED',
    313 => 'UNCLASSIFIED',
    314 => 'UNCLASSIFIED',
    315 => 'UNCLASSIFIED',
    316 => 'UNCLASSIFIED',
    317 => 'UNCLASSIFIED',
    318 => 'UNCLASSIFIED',
    319 => 'UNCLASSIFIED',
    320 => 'UNCLASSIFIED',
    321 => 'UNCLASSIFIED',
    322 => 'UNCLASSIFIED',
    323 => 'UNCLASSIFIED',
    324 => 'UNCLASSIFIED',
    325 => 'UNCLASSIFIED',
    326 => 'UNCLASSIFIED',
    327 => 'UNCLASSIFIED',
    328 => 'UNCLASSIFIED',
    329 => 'UNCLASSIFIED',
    330 => 'UNCLASSIFIED',
    331 => 'UNCLASSIFIED',
    332 => 'UNCLASSIFIED',
    333 => 'UNCLASSIFIED',
    334 => 'UNCLASSIFIED',
    335 => 'UNCLASSIFIED',
    336 => 'UNCLASSIFIED',
    337 => 'UNCLASSIFIED',
    338 => 'UNCLASSIFIED',
    339 => 'UNCLASSIFIED',
    340 => 'UNCLASSIFIED',
    341 => 'UNCLASSIFIED',
    342 => 'UNCLASSIFIED',
    343 => 'UNCLASSIFIED',
    344 => 'UNCLASSIFIED',
    345 => 'UNCLASSIFIED',
    346 => 'UNCLASSIFIED',
    347 => 'UNCLASSIFIED',
    348 => 'UNCLASSIFIED',
    349 => 'UNCLASSIFIED',
    350 => 'UNCLASSIFIED',
    351 => 'UNCLASSIFIED',
    352 => 'UNCLASSIFIED',
    353 => 'UNCLASSIFIED',
    354 => 'UNCLASSIFIED',
    355 => 'UNCLASSIFIED',
    356 => 'UNCLASSIFIED',
    357 => 'UNCLASSIFIED',
    358 => 'UNCLASSIFIED',
    359 => 'UNCLASSIFIED',
    360 => 'UNCLASSIFIED',
    361 => 'UNCLASSIFIED',
    362 => 'UNCLASSIFIED',
    363 => 'UNCLASSIFIED',
    364 => 'UNCLASSIFIED',
    365 => 'UNCLASSIFIED',
    366 => 'UNCLASSIFIED',
    367 => 'UNCLASSIFIED',
    368 => 'UNCLASSIFIED',
    369 => 'UNCLASSIFIED',
    370 => 'UNCLASSIFIED',
    371 => 'UNCLASSIFIED',
    372 => 'UNCLASSIFIED',
    373 => 'UNCLASSIFIED',
    374 => 'UNCLASSIFIED',
    375 => 'UNCLASSIFIED',
    376 => 'UNCLASSIFIED',
    377 => 'UNCLASSIFIED',
    378 => 'UNCLASSIFIED',
    379 => 'UNCLASSIFIED',
    380 => 'UNCLASSIFIED',
    381 => 'UNCLASSIFIED',
    382 => 'UNCLASSIFIED',
    383 => 'UNCLASSIFIED',
    384 => 'UNCLASSIFIED',
    385 => 'UNCLASSIFIED',
    386 => 'UNCLASSIFIED',
    387 => 'UNCLASSIFIED',
    388 => 'UNCLASSIFIED',
    389 => 'UNCLASSIFIED',
    390 => 'UNCLASSIFIED',
    391 => 'UNCLASSIFIED',
    392 => 'UNCLASSIFIED',
    393 => 'UNCLASSIFIED',
    394 => 'UNCLASSIFIED',
    395 => 'UNCLASSIFIED',
    396 => 'UNCLASSIFIED',
    397 => 'UNCLASSIFIED',
    398 => 'UNCLASSIFIED',
    399 => 'UNCLASSIFIED',
    400 => 'UNCLASSIFIED',
    401 => 'UNCLASSIFIED',
    402 => 'UNCLASSIFIED',
    403 => 'UNCLASSIFIED',
    404 => 'UNCLASSIFIED',
    405 => 'UNCLASSIFIED',
    406 => 'UNCLASSIFIED',
    407 => 'UNCLASSIFIED',
    408 => 'UNCLASSIFIED',
    409 => 'UNCLASSIFIED',
    410 => 'UNCLASSIFIED',
    411 => 'UNCLASSIFIED',
    412 => 'UNCLASSIFIED',
    413 => 'UNCLASSIFIED',
    414 => 'UNCLASSIFIED',
    415 => 'UNCLASSIFIED',
    416 => 'UNCLASSIFIED',
    417 => 'UNCLASSIFIED',
    418 => 'UNCLASSIFIED',
    419 => 'UNCLASSIFIED',
    420 => 'UNCLASSIFIED',
    421 => 'UNCLASSIFIED',
    422 => 'UNCLASSIFIED',
    423 => 'UNCLASSIFIED',
    424 => 'UNCLASSIFIED',
    425 => 'UNCLASSIFIED',
    426 => 'UNCLASSIFIED',
    427 => 'UNCLASSIFIED',
    428 => 'UNCLASSIFIED',
    429 => 'UNCLASSIFIED',
    430 => 'UNCLASSIFIED',
    431 => 'UNCLASSIFIED',
    432 => 'UNCLASSIFIED',
    433 => 'UNCLASSIFIED',
    434 => 'UNCLASSIFIED',
    435 => 'UNCLASSIFIED',
    436 => 'UNCLASSIFIED',
    437 => 'UNCLASSIFIED',
    438 => 'UNCLASSIFIED',
    439 => 'UNCLASSIFIED',
    440 => 'UNCLASSIFIED',
    441 => 'UNCLASSIFIED',
    442 => 'UNCLASSIFIED',
    443 => 'UNCLASSIFIED',
    444 => 'UNCLASSIFIED',
    445 => 'UNCLASSIFIED',
    446 => 'UNCLASSIFIED',
    447 => 'UNCLASSIFIED',
    448 => 'UNCLASSIFIED',
    449 => 'UNCLASSIFIED',
    450 => 'UNCLASSIFIED',
    451 => 'UNCLASSIFIED',
    452 => 'UNCLASSIFIED',
    453 => 'UNCLASSIFIED',
    454 => 'UNCLASSIFIED',
    455 => 'UNCLASSIFIED',
    456 => 'UNCLASSIFIED',
    457 => 'UNCLASSIFIED',
    458 => 'UNCLASSIFIED',
    459 => 'UNCLASSIFIED',
    460 => 'UNCLASSIFIED',
    461 => 'UNCLASSIFIED',
    462 => 'UNCLASSIFIED',
    463 => 'UNCLASSIFIED',
    464 => 'UNCLASSIFIED',
    465 => 'UNCLASSIFIED',
    466 => 'UNCLASSIFIED',
    467 => 'UNCLASSIFIED',
    468 => 'UNCLASSIFIED',
    469 => 'UNCLASSIFIED',
    470 => 'UNCLASSIFIED',
    471 => 'UNCLASSIFIED',
    472 => 'UNCLASSIFIED',
    473 => 'UNCLASSIFIED',
    474 => 'UNCLASSIFIED',
    475 => 'UNCLASSIFIED',
    476 => 'UNCLASSIFIED',
    477 => 'UNCLASSIFIED',
    478 => 'UNCLASSIFIED',
    479 => 'UNCLASSIFIED',
    480 => 'UNCLASSIFIED',
    481 => 'UNCLASSIFIED',
    482 => 'UNCLASSIFIED',
    483 => 'UNCLASSIFIED',
    484 => 'UNCLASSIFIED',
    485 => 'UNCLASSIFIED',
    486 => 'UNCLASSIFIED',
    487 => 'UNCLASSIFIED',
    488 => 'UNCLASSIFIED',
    489 => 'UNCLASSIFIED',
    490 => 'UNCLASSIFIED',
    491 => 'UNCLASSIFIED',
    492 => 'UNCLASSIFIED',
    493 => 'UNCLASSIFIED',
    494 => 'UNCLASSIFIED',
    495 => 'UNCLASSIFIED',
    496 => 'UNCLASSIFIED',
    497 => 'UNCLASSIFIED',
    498 => 'UNCLASSIFIED',
    499 => 'UNCLASSIFIED',
    500 => 'UNCLASSIFIED',
    501 => 'UNCLASSIFIED',
    502 => 'UNCLASSIFIED',
    503 => 'UNCLASSIFIED',
    504 => 'UNCLASSIFIED',
    505 => 'UNCLASSIFIED',
    506 => 'UNCLASSIFIED',
    507 => 'UNCLASSIFIED',
    508 => 'UNCLASSIFIED',
    509 => 'UNCLASSIFIED',
    510 => 'UNCLASSIFIED',
    511 => 'UNCLASSIFIED',
    512 => 'UNCLASSIFIED',
    513 => 'UNCLASSIFIED',
    514 => 'UNCLASSIFIED',
    515 => 'UNCLASSIFIED',
    516 => 'UNCLASSIFIED',
    517 => 'UNCLASSIFIED',
    518 => 'UNCLASSIFIED',
    519 => 'UNCLASSIFIED',
    520 => 'UNCLASSIFIED',
    521 => 'UNCLASSIFIED',
    522 => 'UNCLASSIFIED',
    523 => 'UNCLASSIFIED',
    524 => 'UNCLASSIFIED',
    525 => 'UNCLASSIFIED',
    526 => 'UNCLASSIFIED',
    527 => 'UNCLASSIFIED',
    528 => 'UNCLASSIFIED',
    529 => 'UNCLASSIFIED',
    530 => 'UNCLASSIFIED',
    531 => 'UNCLASSIFIED',
    532 => 'UNCLASSIFIED',
    533 => 'UNCLASSIFIED',
    534 => 'UNCLASSIFIED',
    535 => 'UNCLASSIFIED',
    536 => 'UNCLASSIFIED',
    537 => 'UNCLASSIFIED',
    538 => 'UNCLASSIFIED',
    539 => 'UNCLASSIFIED',
    540 => 'UNCLASSIFIED',
    541 => 'UNCLASSIFIED',
    542 => 'UNCLASSIFIED',
    543 => 'UNCLASSIFIED',
    544 => 'UNCLASSIFIED',
    545 => 'UNCLASSIFIED',
    546 => 'UNCLASSIFIED',
    547 => 'UNCLASSIFIED',
    548 => 'UNCLASSIFIED',
    549 => 'UNCLASSIFIED',
    550 => 'UNCLASSIFIED',
    551 => 'UNCLASSIFIED',
    552 => 'UNCLASSIFIED',
    553 => 'UNCLASSIFIED',
    554 => 'UNCLASSIFIED',
    555 => 'UNCLASSIFIED',
    556 => 'UNCLASSIFIED',
    557 => 'UNCLASSIFIED',
    558 => 'UNCLASSIFIED',
    559 => 'UNCLASSIFIED',
    560 => 'UNCLASSIFIED',
    561 => 'UNCLASSIFIED',
    562 => 'UNCLASSIFIED',
    563 => 'UNCLASSIFIED',
    564 => 'UNCLASSIFIED',
    565 => 'UNCLASSIFIED',
    566 => 'UNCLASSIFIED',
    567 => 'UNCLASSIFIED',
    568 => 'UNCLASSIFIED',
    569 => 'UNCLASSIFIED',
    570 => 'UNCLASSIFIED',
    571 => 'UNCLASSIFIED',
    572 => 'UNCLASSIFIED',
    573 => 'UNCLASSIFIED',
    574 => 'UNCLASSIFIED',
    575 => 'UNCLASSIFIED',
    576 => 'UNCLASSIFIED',
    577 => 'UNCLASSIFIED',
    578 => 'UNCLASSIFIED',
    579 => 'UNCLASSIFIED',
    580 => 'UNCLASSIFIED',
    581 => 'UNCLASSIFIED',
    582 => 'UNCLASSIFIED',
    583 => 'UNCLASSIFIED',
    584 => 'UNCLASSIFIED',
    585 => 'UNCLASSIFIED',
    586 => 'UNCLASSIFIED',
    587 => 'UNCLASSIFIED',
    588 => 'UNCLASSIFIED',
    589 => 'UNCLASSIFIED',
    590 => 'UNCLASSIFIED',
    591 => 'UNCLASSIFIED',
    592 => 'UNCLASSIFIED',
    593 => 'UNCLASSIFIED',
    594 => 'UNCLASSIFIED',
    595 => 'UNCLASSIFIED',
    596 => 'UNCLASSIFIED',
    597 => 'UNCLASSIFIED',
    598 => 'UNCLASSIFIED',
    599 => 'UNCLASSIFIED',
    600 => 'UNCLASSIFIED',
    601 => 'UNCLASSIFIED',
    602 => 'UNCLASSIFIED',
    603 => 'UNCLASSIFIED',
    604 => 'UNCLASSIFIED',
    605 => 'UNCLASSIFIED',
    606 => 'UNCLASSIFIED',
    607 => 'UNCLASSIFIED',
    608 => 'UNCLASSIFIED',
    609 => 'UNCLASSIFIED',
    610 => 'UNCLASSIFIED',
    611 => 'UNCLASSIFIED',
    612 => 'UNCLASSIFIED',
    613 => 'UNCLASSIFIED',
    614 => 'UNCLASSIFIED',
    615 => 'UNCLASSIFIED',
    616 => 'UNCLASSIFIED',
    617 => 'UNCLASSIFIED',
    618 => 'UNCLASSIFIED',
    619 => 'UNCLASSIFIED',
    620 => 'UNCLASSIFIED',
    621 => 'UNCLASSIFIED',
    622 => 'UNCLASSIFIED',
    623 => 'UNCLASSIFIED',
    624 => 'UNCLASSIFIED',
    625 => 'UNCLASSIFIED',
    626 => 'UNCLASSIFIED',
    627 => 'UNCLASSIFIED',
    628 => 'UNCLASSIFIED',
    629 => 'UNCLASSIFIED',
    630 => 'UNCLASSIFIED',
    631 => 'UNCLASSIFIED',
    632 => 'UNCLASSIFIED',
    633 => 'UNCLASSIFIED',
    634 => 'UNCLASSIFIED',
    635 => 'UNCLASSIFIED',
    636 => 'UNCLASSIFIED',
    637 => 'UNCLASSIFIED',
    638 => 'UNCLASSIFIED',
    639 => 'UNCLASSIFIED',
    640 => 'UNCLASSIFIED',
    641 => 'UNCLASSIFIED',
    642 => 'UNCLASSIFIED',
    643 => 'UNCLASSIFIED',
    644 => 'UNCLASSIFIED',
    645 => 'UNCLASSIFIED',
    646 => 'UNCLASSIFIED',
    647 => 'UNCLASSIFIED',
    648 => 'UNCLASSIFIED',
    649 => 'UNCLASSIFIED',
    650 => 'UNCLASSIFIED',
    651 => 'UNCLASSIFIED',
    652 => 'UNCLASSIFIED',
    653 => 'UNCLASSIFIED',
    654 => 'UNCLASSIFIED',
    655 => 'UNCLASSIFIED',
    656 => 'UNCLASSIFIED',
    657 => 'UNCLASSIFIED',
    658 => 'UNCLASSIFIED',
    659 => 'UNCLASSIFIED',
    660 => 'UNCLASSIFIED',
    661 => 'UNCLASSIFIED',
    662 => 'UNCLASSIFIED',
    663 => 'UNCLASSIFIED',
    664 => 'UNCLASSIFIED',
    665 => 'UNCLASSIFIED',
    666 => 'UNCLASSIFIED',
    667 => 'UNCLASSIFIED',
    668 => 'UNCLASSIFIED',
    669 => 'UNCLASSIFIED',
    670 => 'UNCLASSIFIED',
    671 => 'UNCLASSIFIED',
    672 => 'UNCLASSIFIED',
    673 => 'UNCLASSIFIED',
    674 => 'UNCLASSIFIED',
    675 => 'UNCLASSIFIED',
    676 => 'UNCLASSIFIED',
    677 => 'UNCLASSIFIED',
    678 => 'UNCLASSIFIED',
    679 => 'UNCLASSIFIED',
    680 => 'UNCLASSIFIED',
    681 => 'UNCLASSIFIED',
    682 => 'UNCLASSIFIED',
    683 => 'UNCLASSIFIED',
    684 => 'UNCLASSIFIED',
    685 => 'UNCLASSIFIED',
    686 => 'UNCLASSIFIED',
    687 => 'UNCLASSIFIED',
    688 => 'UNCLASSIFIED',
    689 => 'UNCLASSIFIED',
    690 => 'UNCLASSIFIED',
    691 => 'UNCLASSIFIED',
    692 => 'UNCLASSIFIED',
    693 => 'UNCLASSIFIED',
    694 => 'UNCLASSIFIED',
    695 => 'UNCLASSIFIED',
    696 => 'UNCLASSIFIED',
    697 => 'UNCLASSIFIED',
    698 => 'UNCLASSIFIED',
    699 => 'UNCLASSIFIED',
    700 => 'UNCLASSIFIED',
    701 => 'UNCLASSIFIED',
    702 => 'UNCLASSIFIED',
    703 => 'UNCLASSIFIED',
    704 => 'UNCLASSIFIED',
    705 => 'UNCLASSIFIED',
    706 => 'UNCLASSIFIED',
    707 => 'UNCLASSIFIED',
    708 => 'UNCLASSIFIED',
    709 => 'UNCLASSIFIED',
    710 => 'UNCLASSIFIED',
    711 => 'UNCLASSIFIED',
    712 => 'UNCLASSIFIED',
    713 => 'UNCLASSIFIED',
    714 => 'UNCLASSIFIED',
    715 => 'UNCLASSIFIED',
    716 => 'UNCLASSIFIED',
    717 => 'UNCLASSIFIED',
    718 => 'UNCLASSIFIED',
    719 => 'UNCLASSIFIED',
    720 => 'UNCLASSIFIED',
    721 => 'UNCLASSIFIED',
    722 => 'UNCLASSIFIED',
    723 => 'UNCLASSIFIED',
    724 => 'UNCLASSIFIED',
    725 => 'UNCLASSIFIED',
    726 => 'UNCLASSIFIED',
    727 => 'UNCLASSIFIED',
    728 => 'UNCLASSIFIED',
    729 => 'UNCLASSIFIED',
    730 => 'UNCLASSIFIED',
    731 => 'UNCLASSIFIED',
    732 => 'UNCLASSIFIED',
    733 => 'UNCLASSIFIED',
    734 => 'UNCLASSIFIED',
    735 => 'UNCLASSIFIED',
    736 => 'UNCLASSIFIED',
    737 => 'UNCLASSIFIED',
    738 => 'UNCLASSIFIED',
    739 => 'UNCLASSIFIED',
    740 => 'UNCLASSIFIED',
    741 => 'UNCLASSIFIED',
    742 => 'UNCLASSIFIED',
    743 => 'UNCLASSIFIED',
    744 => 'UNCLASSIFIED',
    745 => 'UNCLASSIFIED',
    746 => 'UNCLASSIFIED',
    747 => 'UNCLASSIFIED',
    748 => 'UNCLASSIFIED',
    749 => 'UNCLASSIFIED',
    750 => 'UNCLASSIFIED',
    751 => 'UNCLASSIFIED',
    752 => 'UNCLASSIFIED',
    753 => 'UNCLASSIFIED',
    754 => 'UNCLASSIFIED',
    755 => 'UNCLASSIFIED',
    756 => 'UNCLASSIFIED',
    757 => 'UNCLASSIFIED',
    758 => 'UNCLASSIFIED',
    759 => 'UNCLASSIFIED',
    760 => 'UNCLASSIFIED',
    761 => 'UNCLASSIFIED',
    762 => 'UNCLASSIFIED',
    763 => 'UNCLASSIFIED',
    764 => 'UNCLASSIFIED',
    765 => 'UNCLASSIFIED',
    766 => 'UNCLASSIFIED',
    767 => 'UNCLASSIFIED',
    768 => 'UNCLASSIFIED',
    769 => 'UNCLASSIFIED',
    770 => 'UNCLASSIFIED',
    771 => 'UNCLASSIFIED',
    772 => 'UNCLASSIFIED',
    773 => 'UNCLASSIFIED',
    774 => 'UNCLASSIFIED',
    775 => 'UNCLASSIFIED',
    776 => 'UNCLASSIFIED',
    777 => 'UNCLASSIFIED',
    778 => 'UNCLASSIFIED',
    779 => 'UNCLASSIFIED',
    780 => 'UNCLASSIFIED',
    781 => 'UNCLASSIFIED',
    782 => 'UNCLASSIFIED',
    783 => 'UNCLASSIFIED',
    784 => 'UNCLASSIFIED',
    785 => 'UNCLASSIFIED',
    786 => 'UNCLASSIFIED',
    787 => 'UNCLASSIFIED',
    788 => 'UNCLASSIFIED',
    789 => 'UNCLASSIFIED',
    790 => 'UNCLASSIFIED',
    791 => 'UNCLASSIFIED',
    792 => 'UNCLASSIFIED',
    793 => 'UNCLASSIFIED',
    794 => 'UNCLASSIFIED',
    795 => 'UNCLASSIFIED',
    796 => 'UNCLASSIFIED',
    797 => 'UNCLASSIFIED',
    798 => 'UNCLASSIFIED',
    799 => 'UNCLASSIFIED',
    800 => 'UNCLASSIFIED',
    801 => 'UNCLASSIFIED',
    802 => 'UNCLASSIFIED',
    803 => 'UNCLASSIFIED',
    804 => 'UNCLASSIFIED',
    805 => 'UNCLASSIFIED',
    806 => 'UNCLASSIFIED',
    807 => 'UNCLASSIFIED',
    808 => 'UNCLASSIFIED',
    809 => 'UNCLASSIFIED',
    810 => 'UNCLASSIFIED',
    811 => 'UNCLASSIFIED',
    812 => 'UNCLASSIFIED',
    813 => 'UNCLASSIFIED',
    814 => 'UNCLASSIFIED',
    815 => 'UNCLASSIFIED',
    816 => 'UNCLASSIFIED',
    817 => 'UNCLASSIFIED',
    818 => 'UNCLASSIFIED',
    819 => 'UNCLASSIFIED',
    820 => 'UNCLASSIFIED',
    821 => 'UNCLASSIFIED',
    822 => 'UNCLASSIFIED',
    823 => 'UNCLASSIFIED',
    824 => 'UNCLASSIFIED',
    825 => 'UNCLASSIFIED',
    826 => 'UNCLASSIFIED',
    827 => 'UNCLASSIFIED',
    828 => 'UNCLASSIFIED',
    829 => 'UNCLASSIFIED',
    830 => 'UNCLASSIFIED',
    831 => 'UNCLASSIFIED',
    832 => 'UNCLASSIFIED',
    833 => 'UNCLASSIFIED',
    834 => 'UNCLASSIFIED',
    835 => 'UNCLASSIFIED',
    836 => 'UNCLASSIFIED',
    837 => 'UNCLASSIFIED',
    838 => 'UNCLASSIFIED',
    839 => 'UNCLASSIFIED',
    840 => 'UNCLASSIFIED',
    841 => 'UNCLASSIFIED',
    842 => 'UNCLASSIFIED',
    843 => 'UNCLASSIFIED',
    844 => 'UNCLASSIFIED',
    845 => 'UNCLASSIFIED',
    846 => 'UNCLASSIFIED',
    847 => 'UNCLASSIFIED',
    848 => 'UNCLASSIFIED',
    849 => 'UNCLASSIFIED',
    850 => 'UNCLASSIFIED',
    851 => 'UNCLASSIFIED',
    852 => 'UNCLASSIFIED',
    853 => 'UNCLASSIFIED',
    854 => 'UNCLASSIFIED',
    855 => 'UNCLASSIFIED',
    856 => 'UNCLASSIFIED',
    857 => 'UNCLASSIFIED',
    858 => 'UNCLASSIFIED',
    859 => 'UNCLASSIFIED',
    860 => 'UNCLASSIFIED',
    861 => 'UNCLASSIFIED',
    862 => 'UNCLASSIFIED',
    863 => 'UNCLASSIFIED',
    864 => 'UNCLASSIFIED',
    865 => 'UNCLASSIFIED',
    866 => 'UNCLASSIFIED',
    867 => 'UNCLASSIFIED',
    868 => 'UNCLASSIFIED',
    869 => 'UNCLASSIFIED',
    870 => 'UNCLASSIFIED',
    871 => 'UNCLASSIFIED',
    872 => 'UNCLASSIFIED',
    873 => 'UNCLASSIFIED',
    874 => 'UNCLASSIFIED',
    875 => 'UNCLASSIFIED',
    876 => 'UNCLASSIFIED',
    877 => 'UNCLASSIFIED',
    878 => 'UNCLASSIFIED',
    879 => 'UNCLASSIFIED',
    880 => 'UNCLASSIFIED',
    881 => 'UNCLASSIFIED',
    882 => 'UNCLASSIFIED',
    883 => 'UNCLASSIFIED',
    884 => 'UNCLASSIFIED',
    885 => 'UNCLASSIFIED',
    886 => 'UNCLASSIFIED',
    887 => 'UNCLASSIFIED',
    888 => 'UNCLASSIFIED',
    889 => 'UNCLASSIFIED',
    890 => 'UNCLASSIFIED',
    891 => 'UNCLASSIFIED',
    892 => 'UNCLASSIFIED',
    893 => 'UNCLASSIFIED',
    894 => 'UNCLASSIFIED',
    895 => 'UNCLASSIFIED',
    896 => 'UNCLASSIFIED',
    897 => 'UNCLASSIFIED',
    898 => 'UNCLASSIFIED',
    899 => 'UNCLASSIFIED',
    900 => 'UNCLASSIFIED',
    901 => 'UNCLASSIFIED',
    902 => 'UNCLASSIFIED',
    903 => 'UNCLASSIFIED',
    904 => 'UNCLASSIFIED',
    905 => 'UNCLASSIFIED',
    906 => 'UNCLASSIFIED',
    907 => 'UNCLASSIFIED',
    908 => 'UNCLASSIFIED',
    909 => 'UNCLASSIFIED',
    910 => 'UNCLASSIFIED',
    911 => 'UNCLASSIFIED',
    912 => 'UNCLASSIFIED',
    913 => 'UNCLASSIFIED',
    914 => 'UNCLASSIFIED',
    915 => 'UNCLASSIFIED',
    916 => 'UNCLASSIFIED',
    917 => 'UNCLASSIFIED',
    918 => 'UNCLASSIFIED',
    919 => 'UNCLASSIFIED',
    920 => 'UNCLASSIFIED',
    921 => 'UNCLASSIFIED',
    922 => 'UNCLASSIFIED',
    923 => 'UNCLASSIFIED',
    924 => 'UNCLASSIFIED',
    925 => 'UNCLASSIFIED',
    926 => 'UNCLASSIFIED',
    927 => 'UNCLASSIFIED',
    928 => 'UNCLASSIFIED',
    929 => 'UNCLASSIFIED',
    930 => 'UNCLASSIFIED',
    931 => 'UNCLASSIFIED',
    932 => 'UNCLASSIFIED',
    933 => 'UNCLASSIFIED',
    934 => 'UNCLASSIFIED',
    935 => 'UNCLASSIFIED',
    936 => 'UNCLASSIFIED',
    937 => 'UNCLASSIFIED',
    938 => 'UNCLASSIFIED',
    939 => 'UNCLASSIFIED',
    940 => 'UNCLASSIFIED',
    941 => 'UNCLASSIFIED',
    942 => 'UNCLASSIFIED',
    943 => 'UNCLASSIFIED',
    944 => 'UNCLASSIFIED',
    945 => 'UNCLASSIFIED',
    946 => 'UNCLASSIFIED',
    947 => 'UNCLASSIFIED',
    948 => 'UNCLASSIFIED',
    949 => 'UNCLASSIFIED',
    950 => 'UNCLASSIFIED',
    951 => 'UNCLASSIFIED',
    952 => 'UNCLASSIFIED',
    953 => 'UNCLASSIFIED',
    954 => 'UNCLASSIFIED',
    955 => 'UNCLASSIFIED',
    956 => 'UNCLASSIFIED',
    957 => 'UNCLASSIFIED',
    958 => 'UNCLASSIFIED',
    959 => 'UNCLASSIFIED',
    960 => 'UNCLASSIFIED',
    961 => 'UNCLASSIFIED',
    962 => 'UNCLASSIFIED',
    963 => 'UNCLASSIFIED',
    964 => 'UNCLASSIFIED',
    965 => 'UNCLASSIFIED',
    966 => 'UNCLASSIFIED',
    967 => 'UNCLASSIFIED',
    968 => 'UNCLASSIFIED',
    969 => 'UNCLASSIFIED',
    970 => 'UNCLASSIFIED',
    971 => 'UNCLASSIFIED',
    972 => 'UNCLASSIFIED',
    973 => 'UNCLASSIFIED',
    974 => 'UNCLASSIFIED',
    975 => 'UNCLASSIFIED',
    976 => 'UNCLASSIFIED',
    977 => 'UNCLASSIFIED',
    978 => 'UNCLASSIFIED',
    979 => 'UNCLASSIFIED',
    980 => 'UNCLASSIFIED',
    981 => 'UNCLASSIFIED',
    982 => 'UNCLASSIFIED',
    983 => 'UNCLASSIFIED',
    984 => 'UNCLASSIFIED',
    985 => 'UNCLASSIFIED',
    986 => 'UNCLASSIFIED',
    987 => 'UNCLASSIFIED',
    988 => 'UNCLASSIFIED',
    989 => 'UNCLASSIFIED',
    990 => 'UNCLASSIFIED',
    991 => 'UNCLASSIFIED',
    992 => 'UNCLASSIFIED',
    993 => 'UNCLASSIFIED',
    994 => 'UNCLASSIFIED',
    995 => 'UNCLASSIFIED',
    996 => 'UNCLASSIFIED',
    997 => 'UNCLASSIFIED',
    998 => 'UNCLASSIFIED',
    999 => 'UNCLASSIFIED',
    1000 => 'X-RAY BINARY',
    1001 => 'X-RAY BINARY FLARES',
    1002 => 'X-RAY BINARY JETS',
    1003 => 'X-RAY BINARY ECLIPSING',
    1004 => 'X-RAY BINARY ULTRA-SOFT TRANSIENT',
    1005 => 'X-RAY BINARY SOFT TRANSIENT',
    1006 => 'X-RAY BINARY HARD TRANSIENT',
    1007 => 'X-RAY BINARY ECLIPSING DIPPER',
    1008 => 'X-RAY BINARY ECLIPSING ADC',
    1009 => 'X-RAY BINARY DIPPER',
    1010 => 'X-RAY BINARY X-RAY PULSAR',
    1011 => 'X-RAY BINARY X-RAY PULSAR FLARES',
    1012 => 'X-RAY BINARY X-RAY PULSAR JETS',
    1013 => 'X-RAY BINARY X-RAY PULSAR ECLIPSING',
    1014 => 'X-RAY BINARY X-RAY PULSAR ULTRA-SOFT TRANSIENT',
    1015 => 'X-RAY BINARY X-RAY PULSAR SOFT TRANSIENT',
    1016 => 'X-RAY BINARY X-RAY PULSAR HARD TRANSIENT',
    1017 => 'X-RAY BINARY X-RAY PULSAR ECLIPSING DIPPER',
    1018 => 'X-RAY BINARY X-RAY PULSAR ECLIPSING ADC',
    1019 => 'X-RAY BINARY X-RAY PULSAR DIPPER',
    1020 => 'X-RAY BINARY BURSTER',
    1021 => 'X-RAY BINARY BURSTER FLARES',
    1022 => 'X-RAY BINARY BURSTER JETS',
    1023 => 'X-RAY BINARY BURSTER ECLIPSING',
    1024 => 'X-RAY BINARY BURSTER ULTRA-SOFT TRANSIENT',
    1025 => 'X-RAY BINARY BURSTER SOFT TRANSIENT',
    1026 => 'X-RAY BINARY BURSTER HARD TRANSIENT',
    1027 => 'X-RAY BINARY BURSTER ECLIPSING DIPPER',
    1028 => 'X-RAY BINARY BURSTER ECLIPSING ADC',
    1029 => 'X-RAY BINARY BURSTER DIPPER',
    1030 => 'X-RAY BINARY BLACK HOLE',
    1031 => 'X-RAY BINARY BLACK HOLE FLARES',
    1032 => 'X-RAY BINARY BLACK HOLE JETS',
    1033 => 'X-RAY BINARY BLACK HOLE ECLIPSING',
    1034 => 'X-RAY BINARY BLACK HOLE ULTRA-SOFT TRANSIENT',
    1035 => 'X-RAY BINARY BLACK HOLE SOFT TRANSIENT',
    1036 => 'X-RAY BINARY BLACK HOLE HARD TRANSIENT',
    1037 => 'X-RAY BINARY BLACK HOLE ECLIPSING DIPPER',
    1038 => 'X-RAY BINARY BLACK HOLE ECLIPSING ADC',
    1039 => 'X-RAY BINARY BLACK HOLE DIPPER',
    1040 => 'X-RAY BINARY QPO',
    1041 => 'X-RAY BINARY QPO FLARES',
    1042 => 'X-RAY BINARY QPO JETS',
    1043 => 'X-RAY BINARY QPO ECLIPSING',
    1044 => 'X-RAY BINARY QPO ULTRA-SOFT TRANSIENT',
    1045 => 'X-RAY BINARY QPO SOFT TRANSIENT',
    1046 => 'X-RAY BINARY QPO HARD TRANSIENT',
    1047 => 'X-RAY BINARY QPO ECLIPSING DIPPER',
    1048 => 'X-RAY BINARY QPO ECLIPSING ADC',
    1049 => 'X-RAY BINARY QPO DIPPER',
    1050 => 'X-RAY BINARY QPO AND BLACK HOLE',
    1051 => 'X-RAY BINARY QPO AND BLACK HOLE FLARES',
    1052 => 'X-RAY BINARY QPO AND BLACK HOLE JETS',
    1053 => 'X-RAY BINARY QPO AND BLACK HOLE ECLIPSING',
    1054 => 'X-RAY BINARY QPO AND BLACK HOLE ULTRA-SOFT TRANSIENT',
    1055 => 'X-RAY BINARY QPO AND BLACK HOLE SOFT TRANSIENT',
    1056 => 'X-RAY BINARY QPO AND BLACK HOLE HARD TRANSIENT',
    1057 => 'X-RAY BINARY QPO AND BLACK HOLE ECLIPSING DIPPER',
    1058 => 'X-RAY BINARY QPO AND BLACK HOLE ECLIPSING ADC',
    1059 => 'X-RAY BINARY QPO AND BLACK HOLE DIPPER',
    1060 => 'X-RAY BINARY QPO AND PULSAR',
    1061 => 'X-RAY BINARY QPO AND PULSAR FLARES',
    1062 => 'X-RAY BINARY QPO AND PULSAR JETS',
    1063 => 'X-RAY BINARY QPO AND PULSAR ECLIPSING',
    1064 => 'X-RAY BINARY QPO AND PULSAR ULTRA-SOFT TRANSIENT',
    1065 => 'X-RAY BINARY QPO AND PULSAR SOFT TRANSIENT',
    1066 => 'X-RAY BINARY QPO AND PULSAR HARD TRANSIENT',
    1067 => 'X-RAY BINARY QPO AND PULSAR ECLIPSING DIPPER',
    1068 => 'X-RAY BINARY QPO AND PULSAR ECLIPSING ADC',
    1069 => 'X-RAY BINARY QPO AND PULSAR DIPPER',
    1070 => 'X-RAY BINARY QPO AND BURSTS',
    1071 => 'X-RAY BINARY QPO AND BURSTS FLARES',
    1072 => 'X-RAY BINARY QPO AND BURSTS JETS',
    1073 => 'X-RAY BINARY QPO AND BURSTS ECLIPSING',
    1074 => 'X-RAY BINARY QPO AND BURSTS ULTRA-SOFT TRANSIENT',
    1075 => 'X-RAY BINARY QPO AND BURSTS SOFT TRANSIENT',
    1076 => 'X-RAY BINARY QPO AND BURSTS HARD TRANSIENT',
    1077 => 'X-RAY BINARY QPO AND BURSTS ECLIPSING DIPPER',
    1078 => 'X-RAY BINARY QPO AND BURSTS ECLIPSING ADC',
    1079 => 'X-RAY BINARY QPO AND BURSTS DIPPER',
    1080 => 'X-RAY BINARY QPO, PULSAR AND BURSTS',
    1081 => 'X-RAY BINARY QPO, PULSAR AND BURSTS FLARES',
    1082 => 'X-RAY BINARY QPO, PULSAR AND BURSTS JETS',
    1083 => 'X-RAY BINARY QPO, PULSAR AND BURSTS ECLIPSING',
    1084 => 'X-RAY BINARY QPO, PULSAR AND BURSTS ULTRA-SOFT TRANSIENT',
    1085 => 'X-RAY BINARY QPO, PULSAR AND BURSTS SOFT TRANSIENT',
    1086 => 'X-RAY BINARY QPO, PULSAR AND BURSTS HARD TRANSIENT',
    1087 => 'X-RAY BINARY QPO, PULSAR AND BURSTS ECLIPSING DIPPER',
    1088 => 'X-RAY BINARY QPO, PULSAR AND BURSTS ECLIPSING ADC',
    1089 => 'X-RAY BINARY QPO, PULSAR AND BURSTS DIPPER',
    1090 => 'X-RAY BINARY PULSAR AND BURSTS',
    1091 => 'X-RAY BINARY PULSAR AND BURSTS FLARES',
    1092 => 'X-RAY BINARY PULSAR AND BURSTS JETS',
    1093 => 'X-RAY BINARY PULSAR AND BURSTS ECLIPSING',
    1094 => 'X-RAY BINARY PULSAR AND BURSTS ULTRA-SOFT TRANSIENT',
    1095 => 'X-RAY BINARY PULSAR AND BURSTS SOFT TRANSIENT',
    1096 => 'X-RAY BINARY PULSAR AND BURSTS HARD TRANSIENT',
    1097 => 'X-RAY BINARY PULSAR AND BURSTS ECLIPSING DIPPER',
    1098 => 'X-RAY BINARY PULSAR AND BURSTS ECLIPSING ADC',
    1099 => 'X-RAY BINARY PULSAR AND BURSTS DIPPER',
    1100 => 'HMXRB',
    1101 => 'HMXRB FLARES',
    1102 => 'HMXRB JETS',
    1103 => 'HMXRB ECLIPSING',
    1104 => 'HMXRB ULTRA-SOFT TRANSIENT',
    1105 => 'HMXRB SOFT TRANSIENT',
    1106 => 'HMXRB HARD TRANSIENT',
    1107 => 'HMXRB ECLIPSING DIPPER',
    1108 => 'HMXRB ECLIPSING ADC',
    1109 => 'HMXRB DIPPER',
    1110 => 'HMXRB X-RAY PULSAR',
    1111 => 'HMXRB X-RAY PULSAR FLARES',
    1112 => 'HMXRB X-RAY PULSAR JETS',
    1113 => 'HMXRB X-RAY PULSAR ECLIPSING',
    1114 => 'HMXRB X-RAY PULSAR ULTRA-SOFT TRANSIENT',
    1115 => 'HMXRB X-RAY PULSAR SOFT TRANSIENT',
    1116 => 'HMXRB X-RAY PULSAR HARD TRANSIENT',
    1117 => 'HMXRB X-RAY PULSAR ECLIPSING DIPPER',
    1118 => 'HMXRB X-RAY PULSAR ECLIPSING ADC',
    1119 => 'HMXRB X-RAY PULSAR DIPPER',
    1120 => 'HMXRB BURSTER',
    1121 => 'HMXRB BURSTER FLARES',
    1122 => 'HMXRB BURSTER JETS',
    1123 => 'HMXRB BURSTER ECLIPSING',
    1124 => 'HMXRB BURSTER ULTRA-SOFT TRANSIENT',
    1125 => 'HMXRB BURSTER SOFT TRANSIENT',
    1126 => 'HMXRB BURSTER HARD TRANSIENT',
    1127 => 'HMXRB BURSTER ECLIPSING DIPPER',
    1128 => 'HMXRB BURSTER ECLIPSING ADC',
    1129 => 'HMXRB BURSTER DIPPER',
    1130 => 'HMXRB BLACK HOLE',
    1131 => 'HMXRB BLACK HOLE FLARES',
    1132 => 'HMXRB BLACK HOLE JETS',
    1133 => 'HMXRB BLACK HOLE ECLIPSING',
    1134 => 'HMXRB BLACK HOLE ULTRA-SOFT TRANSIENT',
    1135 => 'HMXRB BLACK HOLE SOFT TRANSIENT',
    1136 => 'HMXRB BLACK HOLE HARD TRANSIENT',
    1137 => 'HMXRB BLACK HOLE ECLIPSING DIPPER',
    1138 => 'HMXRB BLACK HOLE ECLIPSING ADC',
    1139 => 'HMXRB BLACK HOLE DIPPER',
    1140 => 'HMXRB QPO',
    1141 => 'HMXRB QPO FLARES',
    1142 => 'HMXRB QPO JETS',
    1143 => 'HMXRB QPO ECLIPSING',
    1144 => 'HMXRB QPO ULTRA-SOFT TRANSIENT',
    1145 => 'HMXRB QPO SOFT TRANSIENT',
    1146 => 'HMXRB QPO HARD TRANSIENT',
    1147 => 'HMXRB QPO ECLIPSING DIPPER',
    1148 => 'HMXRB QPO ECLIPSING ADC',
    1149 => 'HMXRB QPO DIPPER',
    1150 => 'HMXRB QPO AND BLACK HOLE',
    1151 => 'HMXRB QPO AND BLACK HOLE FLARES',
    1152 => 'HMXRB QPO AND BLACK HOLE JETS',
    1153 => 'HMXRB QPO AND BLACK HOLE ECLIPSING',
    1154 => 'HMXRB QPO AND BLACK HOLE ULTRA-SOFT TRANSIENT',
    1155 => 'HMXRB QPO AND BLACK HOLE SOFT TRANSIENT',
    1156 => 'HMXRB QPO AND BLACK HOLE HARD TRANSIENT',
    1157 => 'HMXRB QPO AND BLACK HOLE ECLIPSING DIPPER',
    1158 => 'HMXRB QPO AND BLACK HOLE ECLIPSING ADC',
    1159 => 'HMXRB QPO AND BLACK HOLE DIPPER',
    1160 => 'HMXRB QPO AND PULSAR',
    1161 => 'HMXRB QPO AND PULSAR FLARES',
    1162 => 'HMXRB QPO AND PULSAR JETS',
    1163 => 'HMXRB QPO AND PULSAR ECLIPSING',
    1164 => 'HMXRB QPO AND PULSAR ULTRA-SOFT TRANSIENT',
    1165 => 'HMXRB QPO AND PULSAR SOFT TRANSIENT',
    1166 => 'HMXRB QPO AND PULSAR HARD TRANSIENT',
    1167 => 'HMXRB QPO AND PULSAR ECLIPSING DIPPER',
    1168 => 'HMXRB QPO AND PULSAR ECLIPSING ADC',
    1169 => 'HMXRB QPO AND PULSAR DIPPER',
    1170 => 'HMXRB QPO AND BURSTS',
    1171 => 'HMXRB QPO AND BURSTS FLARES',
    1172 => 'HMXRB QPO AND BURSTS JETS',
    1173 => 'HMXRB QPO AND BURSTS ECLIPSING',
    1174 => 'HMXRB QPO AND BURSTS ULTRA-SOFT TRANSIENT',
    1175 => 'HMXRB QPO AND BURSTS SOFT TRANSIENT',
    1176 => 'HMXRB QPO AND BURSTS HARD TRANSIENT',
    1177 => 'HMXRB QPO AND BURSTS ECLIPSING DIPPER',
    1178 => 'HMXRB QPO AND BURSTS ECLIPSING ADC',
    1179 => 'HMXRB QPO AND BURSTS DIPPER',
    1180 => 'HMXRB QPO, PULSAR AND BURSTS',
    1181 => 'HMXRB QPO, PULSAR AND BURSTS FLARES',
    1182 => 'HMXRB QPO, PULSAR AND BURSTS JETS',
    1183 => 'HMXRB QPO, PULSAR AND BURSTS ECLIPSING',
    1184 => 'HMXRB QPO, PULSAR AND BURSTS ULTRA-SOFT TRANSIENT',
    1185 => 'HMXRB QPO, PULSAR AND BURSTS SOFT TRANSIENT',
    1186 => 'HMXRB QPO, PULSAR AND BURSTS HARD TRANSIENT',
    1187 => 'HMXRB QPO, PULSAR AND BURSTS ECLIPSING DIPPER',
    1188 => 'HMXRB QPO, PULSAR AND BURSTS ECLIPSING ADC',
    1189 => 'HMXRB QPO, PULSAR AND BURSTS DIPPER',
    1190 => 'HMXRB PULSAR AND BURSTS',
    1191 => 'HMXRB PULSAR AND BURSTS FLARES',
    1192 => 'HMXRB PULSAR AND BURSTS JETS',
    1193 => 'HMXRB PULSAR AND BURSTS ECLIPSING',
    1194 => 'HMXRB PULSAR AND BURSTS ULTRA-SOFT TRANSIENT',
    1195 => 'HMXRB PULSAR AND BURSTS SOFT TRANSIENT',
    1196 => 'HMXRB PULSAR AND BURSTS HARD TRANSIENT',
    1197 => 'HMXRB PULSAR AND BURSTS ECLIPSING DIPPER',
    1198 => 'HMXRB PULSAR AND BURSTS ECLIPSING ADC',
    1199 => 'HMXRB PULSAR AND BURSTS DIPPER',
    1200 => 'HMXRB SUPERGIANT',
    1201 => 'HMXRB SUPERGIANT FLARES',
    1202 => 'HMXRB SUPERGIANT JETS',
    1203 => 'HMXRB SUPERGIANT ECLIPSING',
    1204 => 'HMXRB SUPERGIANT ULTRA-SOFT TRANSIENT',
    1205 => 'HMXRB SUPERGIANT SOFT TRANSIENT',
    1206 => 'HMXRB SUPERGIANT HARD TRANSIENT',
    1207 => 'HMXRB SUPERGIANT ECLIPSING DIPPER',
    1208 => 'HMXRB SUPERGIANT ECLIPSING ADC',
    1209 => 'HMXRB SUPERGIANT DIPPER',
    1210 => 'HMXRB SUPERGIANT X-RAY PULSAR',
    1211 => 'HMXRB SUPERGIANT X-RAY PULSAR FLARES',
    1212 => 'HMXRB SUPERGIANT X-RAY PULSAR JETS',
    1213 => 'HMXRB SUPERGIANT X-RAY PULSAR ECLIPSING',
    1214 => 'HMXRB SUPERGIANT X-RAY PULSAR ULTRA-SOFT TRANSIENT',
    1215 => 'HMXRB SUPERGIANT X-RAY PULSAR SOFT TRANSIENT',
    1216 => 'HMXRB SUPERGIANT X-RAY PULSAR HARD TRANSIENT',
    1217 => 'HMXRB SUPERGIANT X-RAY PULSAR ECLIPSING DIPPER',
    1218 => 'HMXRB SUPERGIANT X-RAY PULSAR ECLIPSING ADC',
    1219 => 'HMXRB SUPERGIANT X-RAY PULSAR DIPPER',
    1220 => 'HMXRB SUPERGIANT BURSTER',
    1221 => 'HMXRB SUPERGIANT BURSTER FLARES',
    1222 => 'HMXRB SUPERGIANT BURSTER JETS',
    1223 => 'HMXRB SUPERGIANT BURSTER ECLIPSING',
    1224 => 'HMXRB SUPERGIANT BURSTER ULTRA-SOFT TRANSIENT',
    1225 => 'HMXRB SUPERGIANT BURSTER SOFT TRANSIENT',
    1226 => 'HMXRB SUPERGIANT BURSTER HARD TRANSIENT',
    1227 => 'HMXRB SUPERGIANT BURSTER ECLIPSING DIPPER',
    1228 => 'HMXRB SUPERGIANT BURSTER ECLIPSING ADC',
    1229 => 'HMXRB SUPERGIANT BURSTER DIPPER',
    1230 => 'HMXRB SUPERGIANT BLACK HOLE',
    1231 => 'HMXRB SUPERGIANT BLACK HOLE FLARES',
    1232 => 'HMXRB SUPERGIANT BLACK HOLE JETS',
    1233 => 'HMXRB SUPERGIANT BLACK HOLE ECLIPSING',
    1234 => 'HMXRB SUPERGIANT BLACK HOLE ULTRA-SOFT TRANSIENT',
    1235 => 'HMXRB SUPERGIANT BLACK HOLE SOFT TRANSIENT',
    1236 => 'HMXRB SUPERGIANT BLACK HOLE HARD TRANSIENT',
    1237 => 'HMXRB SUPERGIANT BLACK HOLE ECLIPSING DIPPER',
    1238 => 'HMXRB SUPERGIANT BLACK HOLE ECLIPSING ADC',
    1239 => 'HMXRB SUPERGIANT BLACK HOLE DIPPER',
    1240 => 'HMXRB SUPERGIANT QPO',
    1241 => 'HMXRB SUPERGIANT QPO FLARES',
    1242 => 'HMXRB SUPERGIANT QPO JETS',
    1243 => 'HMXRB SUPERGIANT QPO ECLIPSING',
    1244 => 'HMXRB SUPERGIANT QPO ULTRA-SOFT TRANSIENT',
    1245 => 'HMXRB SUPERGIANT QPO SOFT TRANSIENT',
    1246 => 'HMXRB SUPERGIANT QPO HARD TRANSIENT',
    1247 => 'HMXRB SUPERGIANT QPO ECLIPSING DIPPER',
    1248 => 'HMXRB SUPERGIANT QPO ECLIPSING ADC',
    1249 => 'HMXRB SUPERGIANT QPO DIPPER',
    1250 => 'HMXRB SUPERGIANT QPO AND BLACK HOLE',
    1251 => 'HMXRB SUPERGIANT QPO AND BLACK HOLE FLARES',
    1252 => 'HMXRB SUPERGIANT QPO AND BLACK HOLE JETS',
    1253 => 'HMXRB SUPERGIANT QPO AND BLACK HOLE ECLIPSING',
    1254 => 'HMXRB SUPERGIANT QPO AND BLACK HOLE ULTRA-SOFT TRANSIENT',
    1255 => 'HMXRB SUPERGIANT QPO AND BLACK HOLE SOFT TRANSIENT',
    1256 => 'HMXRB SUPERGIANT QPO AND BLACK HOLE HARD TRANSIENT',
    1257 => 'HMXRB SUPERGIANT QPO AND BLACK HOLE ECLIPSING DIPPER',
    1258 => 'HMXRB SUPERGIANT QPO AND BLACK HOLE ECLIPSING ADC',
    1259 => 'HMXRB SUPERGIANT QPO AND BLACK HOLE DIPPER',
    1260 => 'HMXRB SUPERGIANT QPO AND PULSAR',
    1261 => 'HMXRB SUPERGIANT QPO AND PULSAR FLARES',
    1262 => 'HMXRB SUPERGIANT QPO AND PULSAR JETS',
    1263 => 'HMXRB SUPERGIANT QPO AND PULSAR ECLIPSING',
    1264 => 'HMXRB SUPERGIANT QPO AND PULSAR ULTRA-SOFT TRANSIENT',
    1265 => 'HMXRB SUPERGIANT QPO AND PULSAR SOFT TRANSIENT',
    1266 => 'HMXRB SUPERGIANT QPO AND PULSAR HARD TRANSIENT',
    1267 => 'HMXRB SUPERGIANT QPO AND PULSAR ECLIPSING DIPPER',
    1268 => 'HMXRB SUPERGIANT QPO AND PULSAR ECLIPSING ADC',
    1269 => 'HMXRB SUPERGIANT QPO AND PULSAR DIPPER',
    1270 => 'HMXRB SUPERGIANT QPO AND BURSTS',
    1271 => 'HMXRB SUPERGIANT QPO AND BURSTS FLARES',
    1272 => 'HMXRB SUPERGIANT QPO AND BURSTS JETS',
    1273 => 'HMXRB SUPERGIANT QPO AND BURSTS ECLIPSING',
    1274 => 'HMXRB SUPERGIANT QPO AND BURSTS ULTRA-SOFT TRANSIENT',
    1275 => 'HMXRB SUPERGIANT QPO AND BURSTS SOFT TRANSIENT',
    1276 => 'HMXRB SUPERGIANT QPO AND BURSTS HARD TRANSIENT',
    1277 => 'HMXRB SUPERGIANT QPO AND BURSTS ECLIPSING DIPPER',
    1278 => 'HMXRB SUPERGIANT QPO AND BURSTS ECLIPSING ADC',
    1279 => 'HMXRB SUPERGIANT QPO AND BURSTS DIPPER',
    1280 => 'HMXRB SUPERGIANT QPO, PULSAR AND BURSTS',
    1281 => 'HMXRB SUPERGIANT QPO, PULSAR AND BURSTS FLARES',
    1282 => 'HMXRB SUPERGIANT QPO, PULSAR AND BURSTS JETS',
    1283 => 'HMXRB SUPERGIANT QPO, PULSAR AND BURSTS ECLIPSING',
    1284 => 'HMXRB SUPERGIANT QPO, PULSAR AND BURSTS ULTRA-SOFT TRANSIENT',
    1285 => 'HMXRB SUPERGIANT QPO, PULSAR AND BURSTS SOFT TRANSIENT',
    1286 => 'HMXRB SUPERGIANT QPO, PULSAR AND BURSTS HARD TRANSIENT',
    1287 => 'HMXRB SUPERGIANT QPO, PULSAR AND BURSTS ECLIPSING DIPPER',
    1288 => 'HMXRB SUPERGIANT QPO, PULSAR AND BURSTS ECLIPSING ADC',
    1289 => 'HMXRB SUPERGIANT QPO, PULSAR AND BURSTS DIPPER',
    1290 => 'HMXRB SUPERGIANT PULSAR AND BURSTS',
    1291 => 'HMXRB SUPERGIANT PULSAR AND BURSTS FLARES',
    1292 => 'HMXRB SUPERGIANT PULSAR AND BURSTS JETS',
    1293 => 'HMXRB SUPERGIANT PULSAR AND BURSTS ECLIPSING',
    1294 => 'HMXRB SUPERGIANT PULSAR AND BURSTS ULTRA-SOFT TRANSIENT',
    1295 => 'HMXRB SUPERGIANT PULSAR AND BURSTS SOFT TRANSIENT',
    1296 => 'HMXRB SUPERGIANT PULSAR AND BURSTS HARD TRANSIENT',
    1297 => 'HMXRB SUPERGIANT PULSAR AND BURSTS ECLIPSING DIPPER',
    1298 => 'HMXRB SUPERGIANT PULSAR AND BURSTS ECLIPSING ADC',
    1299 => 'HMXRB SUPERGIANT PULSAR AND BURSTS DIPPER',
    1300 => 'HMXRB BE STAR',
    1301 => 'HMXRB BE STAR FLARES',
    1302 => 'HMXRB BE STAR JETS',
    1303 => 'HMXRB BE STAR ECLIPSING',
    1304 => 'HMXRB BE STAR ULTRA-SOFT TRANSIENT',
    1305 => 'HMXRB BE STAR SOFT TRANSIENT',
    1306 => 'HMXRB BE STAR HARD TRANSIENT',
    1307 => 'HMXRB BE STAR ECLIPSING DIPPER',
    1308 => 'HMXRB BE STAR ECLIPSING ADC',
    1309 => 'HMXRB BE STAR DIPPER',
    1310 => 'HMXRB BE STAR X-RAY PULSAR',
    1311 => 'HMXRB BE STAR X-RAY PULSAR FLARES',
    1312 => 'HMXRB BE STAR X-RAY PULSAR JETS',
    1313 => 'HMXRB BE STAR X-RAY PULSAR ECLIPSING',
    1314 => 'HMXRB BE STAR X-RAY PULSAR ULTRA-SOFT TRANSIENT',
    1315 => 'HMXRB BE STAR X-RAY PULSAR SOFT TRANSIENT',
    1316 => 'HMXRB BE STAR X-RAY PULSAR HARD TRANSIENT',
    1317 => 'HMXRB BE STAR X-RAY PULSAR ECLIPSING DIPPER',
    1318 => 'HMXRB BE STAR X-RAY PULSAR ECLIPSING ADC',
    1319 => 'HMXRB BE STAR X-RAY PULSAR DIPPER',
    1320 => 'HMXRB BE STAR BURSTER',
    1321 => 'HMXRB BE STAR BURSTER FLARES',
    1322 => 'HMXRB BE STAR BURSTER JETS',
    1323 => 'HMXRB BE STAR BURSTER ECLIPSING',
    1324 => 'HMXRB BE STAR BURSTER ULTRA-SOFT TRANSIENT',
    1325 => 'HMXRB BE STAR BURSTER SOFT TRANSIENT',
    1326 => 'HMXRB BE STAR BURSTER HARD TRANSIENT',
    1327 => 'HMXRB BE STAR BURSTER ECLIPSING DIPPER',
    1328 => 'HMXRB BE STAR BURSTER ECLIPSING ADC',
    1329 => 'HMXRB BE STAR BURSTER DIPPER',
    1330 => 'HMXRB BE STAR BLACK HOLE',
    1331 => 'HMXRB BE STAR BLACK HOLE FLARES',
    1332 => 'HMXRB BE STAR BLACK HOLE JETS',
    1333 => 'HMXRB BE STAR BLACK HOLE ECLIPSING',
    1334 => 'HMXRB BE STAR BLACK HOLE ULTRA-SOFT TRANSIENT',
    1335 => 'HMXRB BE STAR BLACK HOLE SOFT TRANSIENT',
    1336 => 'HMXRB BE STAR BLACK HOLE HARD TRANSIENT',
    1337 => 'HMXRB BE STAR BLACK HOLE ECLIPSING DIPPER',
    1338 => 'HMXRB BE STAR BLACK HOLE ECLIPSING ADC',
    1339 => 'HMXRB BE STAR BLACK HOLE DIPPER',
    1340 => 'HMXRB BE STAR QPO',
    1341 => 'HMXRB BE STAR QPO FLARES',
    1342 => 'HMXRB BE STAR QPO JETS',
    1343 => 'HMXRB BE STAR QPO ECLIPSING',
    1344 => 'HMXRB BE STAR QPO ULTRA-SOFT TRANSIENT',
    1345 => 'HMXRB BE STAR QPO SOFT TRANSIENT',
    1346 => 'HMXRB BE STAR QPO HARD TRANSIENT',
    1347 => 'HMXRB BE STAR QPO ECLIPSING DIPPER',
    1348 => 'HMXRB BE STAR QPO ECLIPSING ADC',
    1349 => 'HMXRB BE STAR QPO DIPPER',
    1350 => 'HMXRB BE STAR QPO AND BLACK HOLE',
    1351 => 'HMXRB BE STAR QPO AND BLACK HOLE FLARES',
    1352 => 'HMXRB BE STAR QPO AND BLACK HOLE JETS',
    1353 => 'HMXRB BE STAR QPO AND BLACK HOLE ECLIPSING',
    1354 => 'HMXRB BE STAR QPO AND BLACK HOLE ULTRA-SOFT TRANSIENT',
    1355 => 'HMXRB BE STAR QPO AND BLACK HOLE SOFT TRANSIENT',
    1356 => 'HMXRB BE STAR QPO AND BLACK HOLE HARD TRANSIENT',
    1357 => 'HMXRB BE STAR QPO AND BLACK HOLE ECLIPSING DIPPER',
    1358 => 'HMXRB BE STAR QPO AND BLACK HOLE ECLIPSING ADC',
    1359 => 'HMXRB BE STAR QPO AND BLACK HOLE DIPPER',
    1360 => 'HMXRB BE STAR QPO AND PULSAR',
    1361 => 'HMXRB BE STAR QPO AND PULSAR FLARES',
    1362 => 'HMXRB BE STAR QPO AND PULSAR JETS',
    1363 => 'HMXRB BE STAR QPO AND PULSAR ECLIPSING',
    1364 => 'HMXRB BE STAR QPO AND PULSAR ULTRA-SOFT TRANSIENT',
    1365 => 'HMXRB BE STAR QPO AND PULSAR SOFT TRANSIENT',
    1366 => 'HMXRB BE STAR QPO AND PULSAR HARD TRANSIENT',
    1367 => 'HMXRB BE STAR QPO AND PULSAR ECLIPSING DIPPER',
    1368 => 'HMXRB BE STAR QPO AND PULSAR ECLIPSING ADC',
    1369 => 'HMXRB BE STAR QPO AND PULSAR DIPPER',
    1370 => 'HMXRB BE STAR QPO AND BURSTS',
    1371 => 'HMXRB BE STAR QPO AND BURSTS FLARES',
    1372 => 'HMXRB BE STAR QPO AND BURSTS JETS',
    1373 => 'HMXRB BE STAR QPO AND BURSTS ECLIPSING',
    1374 => 'HMXRB BE STAR QPO AND BURSTS ULTRA-SOFT TRANSIENT',
    1375 => 'HMXRB BE STAR QPO AND BURSTS SOFT TRANSIENT',
    1376 => 'HMXRB BE STAR QPO AND BURSTS HARD TRANSIENT',
    1377 => 'HMXRB BE STAR QPO AND BURSTS ECLIPSING DIPPER',
    1378 => 'HMXRB BE STAR QPO AND BURSTS ECLIPSING ADC',
    1379 => 'HMXRB BE STAR QPO AND BURSTS DIPPER',
    1380 => 'HMXRB BE STAR QPO, PULSAR AND BURSTS',
    1381 => 'HMXRB BE STAR QPO, PULSAR AND BURSTS FLARES',
    1382 => 'HMXRB BE STAR QPO, PULSAR AND BURSTS JETS',
    1383 => 'HMXRB BE STAR QPO, PULSAR AND BURSTS ECLIPSING',
    1384 => 'HMXRB BE STAR QPO, PULSAR AND BURSTS ULTRA-SOFT TRANSIENT',
    1385 => 'HMXRB BE STAR QPO, PULSAR AND BURSTS SOFT TRANSIENT',
    1386 => 'HMXRB BE STAR QPO, PULSAR AND BURSTS HARD TRANSIENT',
    1387 => 'HMXRB BE STAR QPO, PULSAR AND BURSTS ECLIPSING DIPPER',
    1388 => 'HMXRB BE STAR QPO, PULSAR AND BURSTS ECLIPSING ADC',
    1389 => 'HMXRB BE STAR QPO, PULSAR AND BURSTS DIPPER',
    1390 => 'HMXRB BE STAR PULSAR AND BURSTS',
    1391 => 'HMXRB BE STAR PULSAR AND BURSTS FLARES',
    1392 => 'HMXRB BE STAR PULSAR AND BURSTS JETS',
    1393 => 'HMXRB BE STAR PULSAR AND BURSTS ECLIPSING',
    1394 => 'HMXRB BE STAR PULSAR AND BURSTS ULTRA-SOFT TRANSIENT',
    1395 => 'HMXRB BE STAR PULSAR AND BURSTS SOFT TRANSIENT',
    1396 => 'HMXRB BE STAR PULSAR AND BURSTS HARD TRANSIENT',
    1397 => 'HMXRB BE STAR PULSAR AND BURSTS ECLIPSING DIPPER',
    1398 => 'HMXRB BE STAR PULSAR AND BURSTS ECLIPSING ADC',
    1399 => 'HMXRB BE STAR PULSAR AND BURSTS DIPPER',
    1400 => 'LMXRB',
    1401 => 'LMXRB FLARES',
    1402 => 'LMXRB JETS',
    1403 => 'LMXRB ECLIPSING',
    1404 => 'LMXRB ULTRA-SOFT TRANSIENT',
    1405 => 'LMXRB SOFT TRANSIENT',
    1406 => 'LMXRB HARD TRANSIENT',
    1407 => 'LMXRB ECLIPSING DIPPER',
    1408 => 'LMXRB ECLIPSING ADC',
    1409 => 'LMXRB DIPPER',
    1410 => 'LMXRB X-RAY PULSAR',
    1411 => 'LMXRB X-RAY PULSAR FLARES',
    1412 => 'LMXRB X-RAY PULSAR JETS',
    1413 => 'LMXRB X-RAY PULSAR ECLIPSING',
    1414 => 'LMXRB X-RAY PULSAR ULTRA-SOFT TRANSIENT',
    1415 => 'LMXRB X-RAY PULSAR SOFT TRANSIENT',
    1416 => 'LMXRB X-RAY PULSAR HARD TRANSIENT',
    1417 => 'LMXRB X-RAY PULSAR ECLIPSING DIPPER',
    1418 => 'LMXRB X-RAY PULSAR ECLIPSING ADC',
    1419 => 'LMXRB X-RAY PULSAR DIPPER',
    1420 => 'LMXRB BURSTER',
    1421 => 'LMXRB BURSTER FLARES',
    1422 => 'LMXRB BURSTER JETS',
    1423 => 'LMXRB BURSTER ECLIPSING',
    1424 => 'LMXRB BURSTER ULTRA-SOFT TRANSIENT',
    1425 => 'LMXRB BURSTER SOFT TRANSIENT',
    1426 => 'LMXRB BURSTER HARD TRANSIENT',
    1427 => 'LMXRB BURSTER ECLIPSING DIPPER',
    1428 => 'LMXRB BURSTER ECLIPSING ADC',
    1429 => 'LMXRB BURSTER DIPPER',
    1430 => 'LMXRB BLACK HOLE',
    1431 => 'LMXRB BLACK HOLE FLARES',
    1432 => 'LMXRB BLACK HOLE JETS',
    1433 => 'LMXRB BLACK HOLE ECLIPSING',
    1434 => 'LMXRB BLACK HOLE ULTRA-SOFT TRANSIENT',
    1435 => 'LMXRB BLACK HOLE SOFT TRANSIENT',
    1436 => 'LMXRB BLACK HOLE HARD TRANSIENT',
    1437 => 'LMXRB BLACK HOLE ECLIPSING DIPPER',
    1438 => 'LMXRB BLACK HOLE ECLIPSING ADC',
    1439 => 'LMXRB BLACK HOLE DIPPER',
    1440 => 'LMXRB QPO',
    1441 => 'LMXRB QPO FLARES',
    1442 => 'LMXRB QPO JETS',
    1443 => 'LMXRB QPO ECLIPSING',
    1444 => 'LMXRB QPO ULTRA-SOFT TRANSIENT',
    1445 => 'LMXRB QPO SOFT TRANSIENT',
    1446 => 'LMXRB QPO HARD TRANSIENT',
    1447 => 'LMXRB QPO ECLIPSING DIPPER',
    1448 => 'LMXRB QPO ECLIPSING ADC',
    1449 => 'LMXRB QPO DIPPER',
    1450 => 'LMXRB QPO AND BLACK HOLE',
    1451 => 'LMXRB QPO AND BLACK HOLE FLARES',
    1452 => 'LMXRB QPO AND BLACK HOLE JETS',
    1453 => 'LMXRB QPO AND BLACK HOLE ECLIPSING',
    1454 => 'LMXRB QPO AND BLACK HOLE ULTRA-SOFT TRANSIENT',
    1455 => 'LMXRB QPO AND BLACK HOLE SOFT TRANSIENT',
    1456 => 'LMXRB QPO AND BLACK HOLE HARD TRANSIENT',
    1457 => 'LMXRB QPO AND BLACK HOLE ECLIPSING DIPPER',
    1458 => 'LMXRB QPO AND BLACK HOLE ECLIPSING ADC',
    1459 => 'LMXRB QPO AND BLACK HOLE DIPPER',
    1460 => 'LMXRB QPO AND PULSAR',
    1461 => 'LMXRB QPO AND PULSAR FLARES',
    1462 => 'LMXRB QPO AND PULSAR JETS',
    1463 => 'LMXRB QPO AND PULSAR ECLIPSING',
    1464 => 'LMXRB QPO AND PULSAR ULTRA-SOFT TRANSIENT',
    1465 => 'LMXRB QPO AND PULSAR SOFT TRANSIENT',
    1466 => 'LMXRB QPO AND PULSAR HARD TRANSIENT',
    1467 => 'LMXRB QPO AND PULSAR ECLIPSING DIPPER',
    1468 => 'LMXRB QPO AND PULSAR ECLIPSING ADC',
    1469 => 'LMXRB QPO AND PULSAR DIPPER',
    1470 => 'LMXRB QPO AND BURSTS',
    1471 => 'LMXRB QPO AND BURSTS FLARES',
    1472 => 'LMXRB QPO AND BURSTS JETS',
    1473 => 'LMXRB QPO AND BURSTS ECLIPSING',
    1474 => 'LMXRB QPO AND BURSTS ULTRA-SOFT TRANSIENT',
    1475 => 'LMXRB QPO AND BURSTS SOFT TRANSIENT',
    1476 => 'LMXRB QPO AND BURSTS HARD TRANSIENT',
    1477 => 'LMXRB QPO AND BURSTS ECLIPSING DIPPER',
    1478 => 'LMXRB QPO AND BURSTS ECLIPSING ADC',
    1479 => 'LMXRB QPO AND BURSTS DIPPER',
    1480 => 'LMXRB QPO, PULSAR AND BURSTS',
    1481 => 'LMXRB QPO, PULSAR AND BURSTS FLARES',
    1482 => 'LMXRB QPO, PULSAR AND BURSTS JETS',
    1483 => 'LMXRB QPO, PULSAR AND BURSTS ECLIPSING',
    1484 => 'LMXRB QPO, PULSAR AND BURSTS ULTRA-SOFT TRANSIENT',
    1485 => 'LMXRB QPO, PULSAR AND BURSTS SOFT TRANSIENT',
    1486 => 'LMXRB QPO, PULSAR AND BURSTS HARD TRANSIENT',
    1487 => 'LMXRB QPO, PULSAR AND BURSTS ECLIPSING DIPPER',
    1488 => 'LMXRB QPO, PULSAR AND BURSTS ECLIPSING ADC',
    1489 => 'LMXRB QPO, PULSAR AND BURSTS DIPPER',
    1490 => 'LMXRB PULSAR AND BURSTS',
    1491 => 'LMXRB PULSAR AND BURSTS FLARES',
    1492 => 'LMXRB PULSAR AND BURSTS JETS',
    1493 => 'LMXRB PULSAR AND BURSTS ECLIPSING',
    1494 => 'LMXRB PULSAR AND BURSTS ULTRA-SOFT TRANSIENT',
    1495 => 'LMXRB PULSAR AND BURSTS SOFT TRANSIENT',
    1496 => 'LMXRB PULSAR AND BURSTS HARD TRANSIENT',
    1497 => 'LMXRB PULSAR AND BURSTS ECLIPSING DIPPER',
    1498 => 'LMXRB PULSAR AND BURSTS ECLIPSING ADC',
    1499 => 'LMXRB PULSAR AND BURSTS DIPPER',
    1500 => 'LMXRB GLOBULAR CLUSTER',
    1501 => 'LMXRB GLOBULAR CLUSTER FLARES',
    1502 => 'LMXRB GLOBULAR CLUSTER JETS',
    1503 => 'LMXRB GLOBULAR CLUSTER ECLIPSING',
    1504 => 'LMXRB GLOBULAR CLUSTER ULTRA-SOFT TRANSIENT',
    1505 => 'LMXRB GLOBULAR CLUSTER SOFT TRANSIENT',
    1506 => 'LMXRB GLOBULAR CLUSTER HARD TRANSIENT',
    1507 => 'LMXRB GLOBULAR CLUSTER ECLIPSING DIPPER',
    1508 => 'LMXRB GLOBULAR CLUSTER ECLIPSING ADC',
    1509 => 'LMXRB GLOBULAR CLUSTER DIPPER',
    1510 => 'LMXRB GLOBULAR CLUSTER X-RAY PULSAR',
    1511 => 'LMXRB GLOBULAR CLUSTER X-RAY PULSAR FLARES',
    1512 => 'LMXRB GLOBULAR CLUSTER X-RAY PULSAR JETS',
    1513 => 'LMXRB GLOBULAR CLUSTER X-RAY PULSAR ECLIPSING',
    1514 => 'LMXRB GLOBULAR CLUSTER X-RAY PULSAR ULTRA-SOFT TRANSIENT',
    1515 => 'LMXRB GLOBULAR CLUSTER X-RAY PULSAR SOFT TRANSIENT',
    1516 => 'LMXRB GLOBULAR CLUSTER X-RAY PULSAR HARD TRANSIENT',
    1517 => 'LMXRB GLOBULAR CLUSTER X-RAY PULSAR ECLIPSING DIPPER',
    1518 => 'LMXRB GLOBULAR CLUSTER X-RAY PULSAR ECLIPSING ADC',
    1519 => 'LMXRB GLOBULAR CLUSTER X-RAY PULSAR DIPPER',
    1520 => 'LMXRB GLOBULAR CLUSTER BURSTER',
    1521 => 'LMXRB GLOBULAR CLUSTER BURSTER FLARES',
    1522 => 'LMXRB GLOBULAR CLUSTER BURSTER JETS',
    1523 => 'LMXRB GLOBULAR CLUSTER BURSTER ECLIPSING',
    1524 => 'LMXRB GLOBULAR CLUSTER BURSTER ULTRA-SOFT TRANSIENT',
    1525 => 'LMXRB GLOBULAR CLUSTER BURSTER SOFT TRANSIENT',
    1526 => 'LMXRB GLOBULAR CLUSTER BURSTER HARD TRANSIENT',
    1527 => 'LMXRB GLOBULAR CLUSTER BURSTER ECLIPSING DIPPER',
    1528 => 'LMXRB GLOBULAR CLUSTER BURSTER ECLIPSING ADC',
    1529 => 'LMXRB GLOBULAR CLUSTER BURSTER DIPPER',
    1530 => 'LMXRB GLOBULAR CLUSTER BLACK HOLE',
    1531 => 'LMXRB GLOBULAR CLUSTER BLACK HOLE FLARES',
    1532 => 'LMXRB GLOBULAR CLUSTER BLACK HOLE JETS',
    1533 => 'LMXRB GLOBULAR CLUSTER BLACK HOLE ECLIPSING',
    1534 => 'LMXRB GLOBULAR CLUSTER BLACK HOLE ULTRA-SOFT TRANSIENT',
    1535 => 'LMXRB GLOBULAR CLUSTER BLACK HOLE SOFT TRANSIENT',
    1536 => 'LMXRB GLOBULAR CLUSTER BLACK HOLE HARD TRANSIENT',
    1537 => 'LMXRB GLOBULAR CLUSTER BLACK HOLE ECLIPSING DIPPER',
    1538 => 'LMXRB GLOBULAR CLUSTER BLACK HOLE ECLIPSING ADC',
    1539 => 'LMXRB GLOBULAR CLUSTER BLACK HOLE DIPPER',
    1540 => 'LMXRB GLOBULAR CLUSTER QPO',
    1541 => 'LMXRB GLOBULAR CLUSTER QPO FLARES',
    1542 => 'LMXRB GLOBULAR CLUSTER QPO JETS',
    1543 => 'LMXRB GLOBULAR CLUSTER QPO ECLIPSING',
    1544 => 'LMXRB GLOBULAR CLUSTER QPO ULTRA-SOFT TRANSIENT',
    1545 => 'LMXRB GLOBULAR CLUSTER QPO SOFT TRANSIENT',
    1546 => 'LMXRB GLOBULAR CLUSTER QPO HARD TRANSIENT',
    1547 => 'LMXRB GLOBULAR CLUSTER QPO ECLIPSING DIPPER',
    1548 => 'LMXRB GLOBULAR CLUSTER QPO ECLIPSING ADC',
    1549 => 'LMXRB GLOBULAR CLUSTER QPO DIPPER',
    1550 => 'LMXRB GLOBULAR CLUSTER QPO AND BLACK HOLE',
    1551 => 'LMXRB GLOBULAR CLUSTER QPO AND BLACK HOLE FLARES',
    1552 => 'LMXRB GLOBULAR CLUSTER QPO AND BLACK HOLE JETS',
    1553 => 'LMXRB GLOBULAR CLUSTER QPO AND BLACK HOLE ECLIPSING',
    1554 => 'LMXRB GLOBULAR CLUSTER QPO AND BLACK HOLE ULTRA-SOFT TRANSIENT',
    1555 => 'LMXRB GLOBULAR CLUSTER QPO AND BLACK HOLE SOFT TRANSIENT',
    1556 => 'LMXRB GLOBULAR CLUSTER QPO AND BLACK HOLE HARD TRANSIENT',
    1557 => 'LMXRB GLOBULAR CLUSTER QPO AND BLACK HOLE ECLIPSING DIPPER',
    1558 => 'LMXRB GLOBULAR CLUSTER QPO AND BLACK HOLE ECLIPSING ADC',
    1559 => 'LMXRB GLOBULAR CLUSTER QPO AND BLACK HOLE DIPPER',
    1560 => 'LMXRB GLOBULAR CLUSTER QPO AND PULSAR',
    1561 => 'LMXRB GLOBULAR CLUSTER QPO AND PULSAR FLARES',
    1562 => 'LMXRB GLOBULAR CLUSTER QPO AND PULSAR JETS',
    1563 => 'LMXRB GLOBULAR CLUSTER QPO AND PULSAR ECLIPSING',
    1564 => 'LMXRB GLOBULAR CLUSTER QPO AND PULSAR ULTRA-SOFT TRANSIENT',
    1565 => 'LMXRB GLOBULAR CLUSTER QPO AND PULSAR SOFT TRANSIENT',
    1566 => 'LMXRB GLOBULAR CLUSTER QPO AND PULSAR HARD TRANSIENT',
    1567 => 'LMXRB GLOBULAR CLUSTER QPO AND PULSAR ECLIPSING DIPPER',
    1568 => 'LMXRB GLOBULAR CLUSTER QPO AND PULSAR ECLIPSING ADC',
    1569 => 'LMXRB GLOBULAR CLUSTER QPO AND PULSAR DIPPER',
    1570 => 'LMXRB GLOBULAR CLUSTER QPO AND BURSTS',
    1571 => 'LMXRB GLOBULAR CLUSTER QPO AND BURSTS FLARES',
    1572 => 'LMXRB GLOBULAR CLUSTER QPO AND BURSTS JETS',
    1573 => 'LMXRB GLOBULAR CLUSTER QPO AND BURSTS ECLIPSING',
    1574 => 'LMXRB GLOBULAR CLUSTER QPO AND BURSTS ULTRA-SOFT TRANSIENT',
    1575 => 'LMXRB GLOBULAR CLUSTER QPO AND BURSTS SOFT TRANSIENT',
    1576 => 'LMXRB GLOBULAR CLUSTER QPO AND BURSTS HARD TRANSIENT',
    1577 => 'LMXRB GLOBULAR CLUSTER QPO AND BURSTS ECLIPSING DIPPER',
    1578 => 'LMXRB GLOBULAR CLUSTER QPO AND BURSTS ECLIPSING ADC',
    1579 => 'LMXRB GLOBULAR CLUSTER QPO AND BURSTS DIPPER',
    1580 => 'LMXRB GLOBULAR CLUSTER QPO, PULSAR AND BURSTS',
    1581 => 'LMXRB GLOBULAR CLUSTER QPO, PULSAR AND BURSTS FLARES',
    1582 => 'LMXRB GLOBULAR CLUSTER QPO, PULSAR AND BURSTS JETS',
    1583 => 'LMXRB GLOBULAR CLUSTER QPO, PULSAR AND BURSTS ECLIPSING',
    1584 => 'LMXRB GLOBULAR CLUSTER QPO, PULSAR AND BURSTS ULTRA-SOFT TRANSIENT',
    1585 => 'LMXRB GLOBULAR CLUSTER QPO, PULSAR AND BURSTS SOFT TRANSIENT',
    1586 => 'LMXRB GLOBULAR CLUSTER QPO, PULSAR AND BURSTS HARD TRANSIENT',
    1587 => 'LMXRB GLOBULAR CLUSTER QPO, PULSAR AND BURSTS ECLIPSING DIPPER',
    1588 => 'LMXRB GLOBULAR CLUSTER QPO, PULSAR AND BURSTS ECLIPSING ADC',
    1589 => 'LMXRB GLOBULAR CLUSTER QPO, PULSAR AND BURSTS DIPPER',
    1590 => 'LMXRB GLOBULAR CLUSTER PULSAR AND BURSTS',
    1591 => 'LMXRB GLOBULAR CLUSTER PULSAR AND BURSTS FLARES',
    1592 => 'LMXRB GLOBULAR CLUSTER PULSAR AND BURSTS JETS',
    1593 => 'LMXRB GLOBULAR CLUSTER PULSAR AND BURSTS ECLIPSING',
    1594 => 'LMXRB GLOBULAR CLUSTER PULSAR AND BURSTS ULTRA-SOFT TRANSIENT',
    1595 => 'LMXRB GLOBULAR CLUSTER PULSAR AND BURSTS SOFT TRANSIENT',
    1596 => 'LMXRB GLOBULAR CLUSTER PULSAR AND BURSTS HARD TRANSIENT',
    1597 => 'LMXRB GLOBULAR CLUSTER PULSAR AND BURSTS ECLIPSING DIPPER',
    1598 => 'LMXRB GLOBULAR CLUSTER PULSAR AND BURSTS ECLIPSING ADC',
    1599 => 'LMXRB GLOBULAR CLUSTER PULSAR AND BURSTS DIPPER',
    1600 => 'CV',
    1601 => 'CV OSCILLATIONS',
    1602 => 'CV COHERENT OSCILLATIONS',
    1603 => 'CV FAST',
    1604 => 'CV SLOW',
    1605 => 'CV ECLIPSING',
    1606 => 'CV',
    1607 => 'CV',
    1608 => 'CV',
    1609 => 'CV',
    1610 => 'CV CLASSICAL NOVA',
    1611 => 'CV CLASSICAL NOVA OSCILLATIONS',
    1612 => 'CV CLASSICAL NOVA COHERENT OSCILLATIONS',
    1613 => 'CV CLASSICAL NOVA FAST',
    1614 => 'CV CLASSICAL NOVA SLOW',
    1615 => 'CV CLASSICAL NOVA ECLIPSING',
    1616 => 'CV CLASSICAL NOVA',
    1617 => 'CV CLASSICAL NOVA',
    1618 => 'CV CLASSICAL NOVA',
    1619 => 'CV CLASSICAL NOVA',
    1620 => 'CV RECURRENT NOVA',
    1621 => 'CV RECURRENT NOVA OSCILLATIONS',
    1622 => 'CV RECURRENT NOVA COHERENT OSCILLATIONS',
    1623 => 'CV RECURRENT NOVA FAST',
    1624 => 'CV RECURRENT NOVA SLOW',
    1625 => 'CV RECURRENT NOVA ECLIPSING',
    1626 => 'CV RECURRENT NOVA',
    1627 => 'CV RECURRENT NOVA',
    1628 => 'CV RECURRENT NOVA',
    1629 => 'CV RECURRENT NOVA',
    1630 => 'CV AM HER (POLAR)',
    1631 => 'CV AM HER (POLAR) OSCILLATIONS',
    1632 => 'CV AM HER (POLAR) COHERENT OSCILLATIONS',
    1633 => 'CV AM HER (POLAR) FAST',
    1634 => 'CV AM HER (POLAR) SLOW',
    1635 => 'CV AM HER (POLAR) ECLIPSING',
    1636 => 'CV AM HER (POLAR)',
    1637 => 'CV AM HER (POLAR)',
    1638 => 'CV AM HER (POLAR)',
    1639 => 'CV AM HER (POLAR)',
    1640 => 'CV INTERMEDIATE POLAR',
    1641 => 'CV INTERMEDIATE POLAR OSCILLATIONS',
    1642 => 'CV INTERMEDIATE POLAR COHERENT OSCILLATIONS',
    1643 => 'CV INTERMEDIATE POLAR FAST',
    1644 => 'CV INTERMEDIATE POLAR SLOW',
    1645 => 'CV INTERMEDIATE POLAR ECLIPSING',
    1646 => 'CV INTERMEDIATE POLAR',
    1647 => 'CV INTERMEDIATE POLAR',
    1648 => 'CV INTERMEDIATE POLAR',
    1649 => 'CV INTERMEDIATE POLAR',
    1650 => 'CV DWARF NOVA',
    1651 => 'CV DWARF NOVA OSCILLATIONS',
    1652 => 'CV DWARF NOVA COHERENT OSCILLATIONS',
    1653 => 'CV DWARF NOVA FAST',
    1654 => 'CV DWARF NOVA SLOW',
    1655 => 'CV DWARF NOVA ECLIPSING',
    1656 => 'CV DWARF NOVA',
    1657 => 'CV DWARF NOVA',
    1658 => 'CV DWARF NOVA',
    1659 => 'CV DWARF NOVA',
    1660 => 'CV DWARF NOVA U GEM TYPE',
    1661 => 'CV DWARF NOVA U GEM TYPE OSCILLATIONS',
    1662 => 'CV DWARF NOVA U GEM TYPE COHERENT OSCILLATIONS',
    1663 => 'CV DWARF NOVA U GEM TYPE FAST',
    1664 => 'CV DWARF NOVA U GEM TYPE SLOW',
    1665 => 'CV DWARF NOVA U GEM TYPE ECLIPSING',
    1666 => 'CV DWARF NOVA U GEM TYPE',
    1667 => 'CV DWARF NOVA U GEM TYPE',
    1668 => 'CV DWARF NOVA U GEM TYPE',
    1669 => 'CV DWARF NOVA U GEM TYPE',
    1670 => 'CV DWARF NOVA Z CAM TYPE',
    1671 => 'CV DWARF NOVA Z CAM TYPE OSCILLATIONS',
    1672 => 'CV DWARF NOVA Z CAM TYPE COHERENT OSCILLATIONS',
    1673 => 'CV DWARF NOVA Z CAM TYPE FAST',
    1674 => 'CV DWARF NOVA Z CAM TYPE SLOW',
    1675 => 'CV DWARF NOVA Z CAM TYPE ECLIPSING',
    1676 => 'CV DWARF NOVA Z CAM TYPE',
    1677 => 'CV DWARF NOVA Z CAM TYPE',
    1678 => 'CV DWARF NOVA Z CAM TYPE',
    1679 => 'CV DWARF NOVA Z CAM TYPE',
    1680 => 'CV DWARF NOVA SU UMA TYPE',
    1681 => 'CV DWARF NOVA SU UMA TYPE OSCILLATIONS',
    1682 => 'CV DWARF NOVA SU UMA TYPE COHERENT OSCILLATIONS',
    1683 => 'CV DWARF NOVA SU UMA TYPE FAST',
    1684 => 'CV DWARF NOVA SU UMA TYPE SLOW',
    1685 => 'CV DWARF NOVA SU UMA TYPE ECLIPSING',
    1686 => 'CV DWARF NOVA SU UMA TYPE',
    1687 => 'CV DWARF NOVA SU UMA TYPE',
    1688 => 'CV DWARF NOVA SU UMA TYPE',
    1689 => 'CV DWARF NOVA SU UMA TYPE',
    1690 => 'CV NOVA LIKE',
    1691 => 'CV NOVA LIKE OSCILLATIONS',
    1692 => 'CV NOVA LIKE COHERENT OSCILLATIONS',
    1693 => 'CV NOVA LIKE FAST',
    1694 => 'CV NOVA LIKE SLOW',
    1695 => 'CV NOVA LIKE ECLIPSING',
    1696 => 'CV NOVA LIKE',
    1697 => 'CV NOVA LIKE',
    1698 => 'CV NOVA LIKE',
    1699 => 'CV NOVA LIKE',
    1700 => 'GAMMA RAY SOURCE',
    1701 => 'GAMMA RAY SOURCE PULSAR',
    1702 => 'GAMMA RAY SOURCE',
    1703 => 'GAMMA RAY SOURCE',
    1704 => 'GAMMA RAY SOURCE',
    1705 => 'GAMMA RAY SOURCE',
    1706 => 'GAMMA RAY SOURCE',
    1707 => 'GAMMA RAY SOURCE',
    1708 => 'GAMMA RAY SOURCE',
    1709 => 'GAMMA RAY SOURCE',
    1710 => 'GAMMA RAY BURSTS',
    1711 => 'GAMMA RAY BURSTS PULSAR',
    1712 => 'GAMMA RAY BURSTS',
    1713 => 'GAMMA RAY BURSTS',
    1714 => 'GAMMA RAY BURSTS',
    1715 => 'GAMMA RAY BURSTS',
    1716 => 'GAMMA RAY BURSTS',
    1717 => 'GAMMA RAY BURSTS',
    1718 => 'GAMMA RAY BURSTS',
    1719 => 'GAMMA RAY BURSTS',
    1720 => 'GAMMA RAY BURSTS SOFT REPEATER',
    1721 => 'GAMMA RAY BURSTS SOFT REPEATER PULSAR',
    1722 => 'GAMMA RAY BURSTS SOFT REPEATER',
    1723 => 'GAMMA RAY BURSTS SOFT REPEATER',
    1724 => 'GAMMA RAY BURSTS SOFT REPEATER',
    1725 => 'GAMMA RAY BURSTS SOFT REPEATER',
    1726 => 'GAMMA RAY BURSTS SOFT REPEATER',
    1727 => 'GAMMA RAY BURSTS SOFT REPEATER',
    1728 => 'GAMMA RAY BURSTS SOFT REPEATER',
    1729 => 'GAMMA RAY BURSTS SOFT REPEATER',
    1730 => 'GAMMA RAY',
    1731 => 'GAMMA RAY PULSAR',
    1732 => 'GAMMA RAY',
    1733 => 'GAMMA RAY',
    1734 => 'GAMMA RAY',
    1735 => 'GAMMA RAY',
    1736 => 'GAMMA RAY',
    1737 => 'GAMMA RAY',
    1738 => 'GAMMA RAY',
    1739 => 'GAMMA RAY',
    1740 => 'GAMMA RAY',
    1741 => 'GAMMA RAY PULSAR',
    1742 => 'GAMMA RAY',
    1743 => 'GAMMA RAY',
    1744 => 'GAMMA RAY',
    1745 => 'GAMMA RAY',
    1746 => 'GAMMA RAY',
    1747 => 'GAMMA RAY',
    1748 => 'GAMMA RAY',
    1749 => 'GAMMA RAY',
    1750 => 'GAMMA RAY',
    1751 => 'GAMMA RAY PULSAR',
    1752 => 'GAMMA RAY',
    1753 => 'GAMMA RAY',
    1754 => 'GAMMA RAY',
    1755 => 'GAMMA RAY',
    1756 => 'GAMMA RAY',
    1757 => 'GAMMA RAY',
    1758 => 'GAMMA RAY',
    1759 => 'GAMMA RAY',
    1760 => 'GAMMA RAY',
    1761 => 'GAMMA RAY PULSAR',
    1762 => 'GAMMA RAY',
    1763 => 'GAMMA RAY',
    1764 => 'GAMMA RAY',
    1765 => 'GAMMA RAY',
    1766 => 'GAMMA RAY',
    1767 => 'GAMMA RAY',
    1768 => 'GAMMA RAY',
    1769 => 'GAMMA RAY',
    1770 => 'GAMMA RAY',
    1771 => 'GAMMA RAY PULSAR',
    1772 => 'GAMMA RAY',
    1773 => 'GAMMA RAY',
    1774 => 'GAMMA RAY',
    1775 => 'GAMMA RAY',
    1776 => 'GAMMA RAY',
    1777 => 'GAMMA RAY',
    1778 => 'GAMMA RAY',
    1779 => 'GAMMA RAY',
    1780 => 'GAMMA RAY',
    1781 => 'GAMMA RAY PULSAR',
    1782 => 'GAMMA RAY',
    1783 => 'GAMMA RAY',
    1784 => 'GAMMA RAY',
    1785 => 'GAMMA RAY',
    1786 => 'GAMMA RAY',
    1787 => 'GAMMA RAY',
    1788 => 'GAMMA RAY',
    1789 => 'GAMMA RAY',
    1790 => 'GAMMA RAY',
    1791 => 'GAMMA RAY PULSAR',
    1792 => 'GAMMA RAY',
    1793 => 'GAMMA RAY',
    1794 => 'GAMMA RAY',
    1795 => 'GAMMA RAY',
    1796 => 'GAMMA RAY',
    1797 => 'GAMMA RAY',
    1798 => 'GAMMA RAY',
    1799 => 'GAMMA RAY',
    1800 => 'RADIO PULSAR',
    1801 => 'RADIO PULSAR',
    1802 => 'RADIO PULSAR',
    1803 => 'RADIO PULSAR',
    1804 => 'RADIO PULSAR',
    1805 => 'RADIO PULSAR',
    1806 => 'RADIO PULSAR',
    1807 => 'RADIO PULSAR',
    1808 => 'RADIO PULSAR',
    1809 => 'RADIO PULSAR',
    1810 => 'X-RAY PULSATOR',
    1811 => 'X-RAY PULSATOR',
    1812 => 'X-RAY PULSATOR',
    1813 => 'X-RAY PULSATOR',
    1814 => 'X-RAY PULSATOR',
    1815 => 'X-RAY PULSATOR',
    1816 => 'X-RAY PULSATOR',
    1817 => 'X-RAY PULSATOR',
    1818 => 'X-RAY PULSATOR',
    1819 => 'X-RAY PULSATOR',
    1820 => 'SUPERSOFT SOURCE',
    1821 => 'SUPERSOFT SOURCE',
    1822 => 'SUPERSOFT SOURCE',
    1823 => 'SUPERSOFT SOURCE',
    1824 => 'SUPERSOFT SOURCE',
    1825 => 'SUPERSOFT SOURCE',
    1826 => 'SUPERSOFT SOURCE',
    1827 => 'SUPERSOFT SOURCE',
    1828 => 'SUPERSOFT SOURCE',
    1829 => 'SUPERSOFT SOURCE',
    1830 => 'ISOLATED NEUTRON STAR',
    1831 => 'ISOLATED NEUTRON STAR',
    1832 => 'ISOLATED NEUTRON STAR',
    1833 => 'ISOLATED NEUTRON STAR',
    1834 => 'ISOLATED NEUTRON STAR',
    1835 => 'ISOLATED NEUTRON STAR',
    1836 => 'ISOLATED NEUTRON STAR',
    1837 => 'ISOLATED NEUTRON STAR',
    1838 => 'ISOLATED NEUTRON STAR',
    1839 => 'ISOLATED NEUTRON STAR',
    1840 => 'ANOMALOUS X-RAY PULSAR',
    1841 => 'ANOMALOUS X-RAY PULSAR',
    1842 => 'ANOMALOUS X-RAY PULSAR',
    1843 => 'ANOMALOUS X-RAY PULSAR',
    1844 => 'ANOMALOUS X-RAY PULSAR',
    1845 => 'ANOMALOUS X-RAY PULSAR',
    1846 => 'ANOMALOUS X-RAY PULSAR',
    1847 => 'ANOMALOUS X-RAY PULSAR',
    1848 => 'ANOMALOUS X-RAY PULSAR',
    1849 => 'ANOMALOUS X-RAY PULSAR',
    1850 => 'EXTRASOLAR PLANET',
    1851 => 'EXTRASOLAR PLANET',
    1852 => 'EXTRASOLAR PLANET',
    1853 => 'EXTRASOLAR PLANET',
    1854 => 'EXTRASOLAR PLANET',
    1855 => 'EXTRASOLAR PLANET',
    1856 => 'EXTRASOLAR PLANET',
    1857 => 'EXTRASOLAR PLANET',
    1858 => 'EXTRASOLAR PLANET',
    1859 => 'EXTRASOLAR PLANET',
    1860 => 'BROWN DWARF',
    1861 => 'BROWN DWARF',
    1862 => 'BROWN DWARF',
    1863 => 'BROWN DWARF',
    1864 => 'BROWN DWARF',
    1865 => 'BROWN DWARF',
    1866 => 'BROWN DWARF',
    1867 => 'BROWN DWARF',
    1868 => 'BROWN DWARF',
    1869 => 'BROWN DWARF',
    1870 => 'PROTOSTAR OF TYPE 0',
    1871 => 'PROTOSTAR OF TYPE 0',
    1872 => 'PROTOSTAR OF TYPE 0',
    1873 => 'PROTOSTAR OF TYPE 0',
    1874 => 'PROTOSTAR OF TYPE 0',
    1875 => 'PROTOSTAR OF TYPE 0',
    1876 => 'PROTOSTAR OF TYPE 0',
    1877 => 'PROTOSTAR OF TYPE 0',
    1878 => 'PROTOSTAR OF TYPE 0',
    1879 => 'PROTOSTAR OF TYPE 0',
    1880 => 'PROTOSTAR OF TYPE I',
    1881 => 'PROTOSTAR OF TYPE I',
    1882 => 'PROTOSTAR OF TYPE I',
    1883 => 'PROTOSTAR OF TYPE I',
    1884 => 'PROTOSTAR OF TYPE I',
    1885 => 'PROTOSTAR OF TYPE I',
    1886 => 'PROTOSTAR OF TYPE I',
    1887 => 'PROTOSTAR OF TYPE I',
    1888 => 'PROTOSTAR OF TYPE I',
    1889 => 'PROTOSTAR OF TYPE I',
    1890 => 'LUMINOUS BLUE VARIABLE (LBV)',
    1891 => 'LUMINOUS BLUE VARIABLE (LBV)',
    1892 => 'LUMINOUS BLUE VARIABLE (LBV)',
    1893 => 'LUMINOUS BLUE VARIABLE (LBV)',
    1894 => 'LUMINOUS BLUE VARIABLE (LBV)',
    1895 => 'LUMINOUS BLUE VARIABLE (LBV)',
    1896 => 'LUMINOUS BLUE VARIABLE (LBV)',
    1897 => 'LUMINOUS BLUE VARIABLE (LBV)',
    1898 => 'LUMINOUS BLUE VARIABLE (LBV)',
    1899 => 'LUMINOUS BLUE VARIABLE (LBV)',
    1900 => 'RS CVN STAR',
    1901 => 'RS CVN STAR',
    1902 => 'RS CVN STAR',
    1903 => 'RS CVN STAR',
    1904 => 'RS CVN STAR',
    1905 => 'RS CVN STAR',
    1906 => 'RS CVN STAR',
    1907 => 'RS CVN STAR',
    1908 => 'RS CVN STAR',
    1909 => 'RS CVN STAR',
    1910 => 'ALGOL STAR',
    1911 => 'ALGOL STAR',
    1912 => 'ALGOL STAR',
    1913 => 'ALGOL STAR',
    1914 => 'ALGOL STAR',
    1915 => 'ALGOL STAR',
    1916 => 'ALGOL STAR',
    1917 => 'ALGOL STAR',
    1918 => 'ALGOL STAR',
    1919 => 'ALGOL STAR',
    1920 => 'BETA LYR STAR',
    1921 => 'BETA LYR STAR',
    1922 => 'BETA LYR STAR',
    1923 => 'BETA LYR STAR',
    1924 => 'BETA LYR STAR',
    1925 => 'BETA LYR STAR',
    1926 => 'BETA LYR STAR',
    1927 => 'BETA LYR STAR',
    1928 => 'BETA LYR STAR',
    1929 => 'BETA LYR STAR',
    1930 => 'W UMA STAR',
    1931 => 'W UMA STAR',
    1932 => 'W UMA STAR',
    1933 => 'W UMA STAR',
    1934 => 'W UMA STAR',
    1935 => 'W UMA STAR',
    1936 => 'W UMA STAR',
    1937 => 'W UMA STAR',
    1938 => 'W UMA STAR',
    1939 => 'W UMA STAR',
    1940 => 'SYMBIOTIC STAR',
    1941 => 'SYMBIOTIC STAR',
    1942 => 'SYMBIOTIC STAR',
    1943 => 'SYMBIOTIC STAR',
    1944 => 'SYMBIOTIC STAR',
    1945 => 'SYMBIOTIC STAR',
    1946 => 'SYMBIOTIC STAR',
    1947 => 'SYMBIOTIC STAR',
    1948 => 'SYMBIOTIC STAR',
    1949 => 'SYMBIOTIC STAR',
    1950 => 'ZETA AURIGAE STAR',
    1951 => 'ZETA AURIGAE STAR',
    1952 => 'ZETA AURIGAE STAR',
    1953 => 'ZETA AURIGAE STAR',
    1954 => 'ZETA AURIGAE STAR',
    1955 => 'ZETA AURIGAE STAR',
    1956 => 'ZETA AURIGAE STAR',
    1957 => 'ZETA AURIGAE STAR',
    1958 => 'ZETA AURIGAE STAR',
    1959 => 'ZETA AURIGAE STAR',
    1960 => 'FK COMAE STAR',
    1961 => 'FK COMAE STAR',
    1962 => 'FK COMAE STAR',
    1963 => 'FK COMAE STAR',
    1964 => 'FK COMAE STAR',
    1965 => 'FK COMAE STAR',
    1966 => 'FK COMAE STAR',
    1967 => 'FK COMAE STAR',
    1968 => 'FK COMAE STAR',
    1969 => 'FK COMAE STAR',
    1970 => 'UV CETI TYPE',
    1971 => 'UV CETI TYPE',
    1972 => 'UV CETI TYPE',
    1973 => 'UV CETI TYPE',
    1974 => 'UV CETI TYPE',
    1975 => 'UV CETI TYPE',
    1976 => 'UV CETI TYPE',
    1977 => 'UV CETI TYPE',
    1978 => 'UV CETI TYPE',
    1979 => 'UV CETI TYPE',
    1980 => 'T TAURI STAR',
    1981 => 'T TAURI STAR (NAKED)',
    1982 => 'T TAURI STAR (POST)',
    1983 => 'T TAURI STAR',
    1984 => 'T TAURI STAR',
    1985 => 'T TAURI STAR',
    1986 => 'T TAURI STAR',
    1987 => 'T TAURI STAR',
    1988 => 'T TAURI STAR',
    1989 => 'T TAURI STAR',
    1990 => 'HERBIG AE/BE STAR',
    1991 => 'BE STAR',
    1992 => 'BE STAR',
    1993 => 'BE STAR',
    1994 => 'BE STAR',
    1995 => 'BE STAR',
    1996 => 'BE STAR',
    1997 => 'BE STAR',
    1998 => 'BE STAR',
    1999 => 'BE STAR',
    2000 => 'WOLF RAYET STAR',
    2001 => 'WOLF RAYET STAR 1',
    2002 => 'WOLF RAYET STAR 2',
    2003 => 'WOLF RAYET STAR 3',
    2004 => 'WOLF RAYET STAR 4',
    2005 => 'WOLF RAYET STAR 5',
    2006 => 'WOLF RAYET STAR 6',
    2007 => 'WOLF RAYET STAR 7',
    2008 => 'WOLF RAYET STAR 8',
    2009 => 'WOLF RAYET STAR 9-11',
    2010 => 'WOLF RAYET STAR WN',
    2011 => 'WOLF RAYET STAR WN1',
    2012 => 'WOLF RAYET STAR WN2',
    2013 => 'WOLF RAYET STAR WN3',
    2014 => 'WOLF RAYET STAR WN4',
    2015 => 'WOLF RAYET STAR WN5',
    2016 => 'WOLF RAYET STAR WN6',
    2017 => 'WOLF RAYET STAR WN7',
    2018 => 'WOLF RAYET STAR WN8',
    2019 => 'WOLF RAYET STAR WN9-11',
    2020 => 'WOLF RAYET STAR WC',
    2021 => 'WOLF RAYET STAR WC1',
    2022 => 'WOLF RAYET STAR WC2',
    2023 => 'WOLF RAYET STAR WC3',
    2024 => 'WOLF RAYET STAR WC4',
    2025 => 'WOLF RAYET STAR WC5',
    2026 => 'WOLF RAYET STAR WC6',
    2027 => 'WOLF RAYET STAR WC7',
    2028 => 'WOLF RAYET STAR WC8',
    2029 => 'WOLF RAYET STAR WC9-11',
    2030 => 'WOLF RAYET STAR WO',
    2031 => 'WOLF RAYET STAR WO1',
    2032 => 'WOLF RAYET STAR WO2',
    2033 => 'WOLF RAYET STAR WO3',
    2034 => 'WOLF RAYET STAR WO4',
    2035 => 'WOLF RAYET STAR WO5',
    2036 => 'WOLF RAYET STAR WO6',
    2037 => 'WOLF RAYET STAR WO7',
    2038 => 'WOLF RAYET STAR WO8',
    2039 => 'WOLF RAYET STAR WO9-11',
    2040 => 'WOLF RAYET STAR',
    2041 => 'WOLF RAYET STAR 1',
    2042 => 'WOLF RAYET STAR 2',
    2043 => 'WOLF RAYET STAR 3',
    2044 => 'WOLF RAYET STAR 4',
    2045 => 'WOLF RAYET STAR 5',
    2046 => 'WOLF RAYET STAR 6',
    2047 => 'WOLF RAYET STAR 7',
    2048 => 'WOLF RAYET STAR 8',
    2049 => 'WOLF RAYET STAR 9-11',
    2050 => 'WOLF RAYET STAR',
    2051 => 'WOLF RAYET STAR 1',
    2052 => 'WOLF RAYET STAR 2',
    2053 => 'WOLF RAYET STAR 3',
    2054 => 'WOLF RAYET STAR 4',
    2055 => 'WOLF RAYET STAR 5',
    2056 => 'WOLF RAYET STAR 6',
    2057 => 'WOLF RAYET STAR 7',
    2058 => 'WOLF RAYET STAR 8',
    2059 => 'WOLF RAYET STAR 9-11',
    2060 => 'WOLF RAYET STAR',
    2061 => 'WOLF RAYET STAR 1',
    2062 => 'WOLF RAYET STAR 2',
    2063 => 'WOLF RAYET STAR 3',
    2064 => 'WOLF RAYET STAR 4',
    2065 => 'WOLF RAYET STAR 5',
    2066 => 'WOLF RAYET STAR 6',
    2067 => 'WOLF RAYET STAR 7',
    2068 => 'WOLF RAYET STAR 8',
    2069 => 'WOLF RAYET STAR 9-11',
    2070 => 'WOLF RAYET STAR',
    2071 => 'WOLF RAYET STAR 1',
    2072 => 'WOLF RAYET STAR 2',
    2073 => 'WOLF RAYET STAR 3',
    2074 => 'WOLF RAYET STAR 4',
    2075 => 'WOLF RAYET STAR 5',
    2076 => 'WOLF RAYET STAR 6',
    2077 => 'WOLF RAYET STAR 7',
    2078 => 'WOLF RAYET STAR 8',
    2079 => 'WOLF RAYET STAR 9-11',
    2080 => 'WOLF RAYET STAR',
    2081 => 'WOLF RAYET STAR 1',
    2082 => 'WOLF RAYET STAR 2',
    2083 => 'WOLF RAYET STAR 3',
    2084 => 'WOLF RAYET STAR 4',
    2085 => 'WOLF RAYET STAR 5',
    2086 => 'WOLF RAYET STAR 6',
    2087 => 'WOLF RAYET STAR 7',
    2088 => 'WOLF RAYET STAR 8',
    2089 => 'WOLF RAYET STAR 9-11',
    2090 => 'WOLF RAYET STAR',
    2091 => 'WOLF RAYET STAR 1',
    2092 => 'WOLF RAYET STAR 2',
    2093 => 'WOLF RAYET STAR 3',
    2094 => 'WOLF RAYET STAR 4',
    2095 => 'WOLF RAYET STAR 5',
    2096 => 'WOLF RAYET STAR 6',
    2097 => 'WOLF RAYET STAR 7',
    2098 => 'WOLF RAYET STAR 8',
    2099 => 'WOLF RAYET STAR 9-11',
    2100 => 'STAR O0',
    2101 => 'STAR O0I',
    2102 => 'STAR O0II',
    2103 => 'STAR O0III',
    2104 => 'STAR O0IV',
    2105 => 'STAR O0V',
    2106 => 'STAR O0VI',
    2107 => 'STAR O0',
    2108 => 'STAR O0',
    2109 => 'STAR O0',
    2110 => 'STAR O1',
    2111 => 'STAR O1I',
    2112 => 'STAR O1II',
    2113 => 'STAR O1III',
    2114 => 'STAR O1IV',
    2115 => 'STAR O1V',
    2116 => 'STAR O1VI',
    2117 => 'STAR O1',
    2118 => 'STAR O1',
    2119 => 'STAR O1',
    2120 => 'STAR O2',
    2121 => 'STAR O2I',
    2122 => 'STAR O2II',
    2123 => 'STAR O2III',
    2124 => 'STAR O2IV',
    2125 => 'STAR O2V',
    2126 => 'STAR O2VI',
    2127 => 'STAR O2',
    2128 => 'STAR O2',
    2129 => 'STAR O2',
    2130 => 'STAR O3',
    2131 => 'STAR O3I',
    2132 => 'STAR O3II',
    2133 => 'STAR O3III',
    2134 => 'STAR O3IV',
    2135 => 'STAR O3V',
    2136 => 'STAR O3VI',
    2137 => 'STAR O3',
    2138 => 'STAR O3',
    2139 => 'STAR O3',
    2140 => 'STAR O4',
    2141 => 'STAR O4I',
    2142 => 'STAR O4II',
    2143 => 'STAR O4III',
    2144 => 'STAR O4IV',
    2145 => 'STAR O4V',
    2146 => 'STAR O4VI',
    2147 => 'STAR O4',
    2148 => 'STAR O4',
    2149 => 'STAR O4',
    2150 => 'STAR O5',
    2151 => 'STAR O5I',
    2152 => 'STAR O5II',
    2153 => 'STAR O5III',
    2154 => 'STAR O5IV',
    2155 => 'STAR O5V',
    2156 => 'STAR O5VI',
    2157 => 'STAR O5',
    2158 => 'STAR O5',
    2159 => 'STAR O5',
    2160 => 'STAR O6',
    2161 => 'STAR O6I',
    2162 => 'STAR O6II',
    2163 => 'STAR O6III',
    2164 => 'STAR O6IV',
    2165 => 'STAR O6V',
    2166 => 'STAR O6VI',
    2167 => 'STAR O6',
    2168 => 'STAR O6',
    2169 => 'STAR O6',
    2170 => 'STAR O7',
    2171 => 'STAR O7I',
    2172 => 'STAR O7II',
    2173 => 'STAR O7III',
    2174 => 'STAR O7IV',
    2175 => 'STAR O7V',
    2176 => 'STAR O7VI',
    2177 => 'STAR O7',
    2178 => 'STAR O7',
    2179 => 'STAR O7',
    2180 => 'STAR O8',
    2181 => 'STAR O8I',
    2182 => 'STAR O8II',
    2183 => 'STAR O8III',
    2184 => 'STAR O8IV',
    2185 => 'STAR O8V',
    2186 => 'STAR O8VI',
    2187 => 'STAR O8',
    2188 => 'STAR O8',
    2189 => 'STAR O8',
    2190 => 'STAR O9',
    2191 => 'STAR O9I',
    2192 => 'STAR O9II',
    2193 => 'STAR O9III',
    2194 => 'STAR O9IV',
    2195 => 'STAR O9V',
    2196 => 'STAR O9VI',
    2197 => 'STAR O9',
    2198 => 'STAR O9',
    2199 => 'STAR O9',
    2200 => 'STAR B0',
    2201 => 'STAR B0I',
    2202 => 'STAR B0II',
    2203 => 'STAR B0III',
    2204 => 'STAR B0IV',
    2205 => 'STAR B0V',
    2206 => 'STAR B0VI',
    2207 => 'STAR B0',
    2208 => 'STAR B0',
    2209 => 'STAR B0',
    2210 => 'STAR B1',
    2211 => 'STAR B1I',
    2212 => 'STAR B1II',
    2213 => 'STAR B1III',
    2214 => 'STAR B1IV',
    2215 => 'STAR B1V',
    2216 => 'STAR B1VI',
    2217 => 'STAR B1',
    2218 => 'STAR B1',
    2219 => 'STAR B1',
    2220 => 'STAR B2',
    2221 => 'STAR B2I',
    2222 => 'STAR B2II',
    2223 => 'STAR B2III',
    2224 => 'STAR B2IV',
    2225 => 'STAR B2V',
    2226 => 'STAR B2VI',
    2227 => 'STAR B2',
    2228 => 'STAR B2',
    2229 => 'STAR B2',
    2230 => 'STAR B3',
    2231 => 'STAR B3I',
    2232 => 'STAR B3II',
    2233 => 'STAR B3III',
    2234 => 'STAR B3IV',
    2235 => 'STAR B3V',
    2236 => 'STAR B3VI',
    2237 => 'STAR B3',
    2238 => 'STAR B3',
    2239 => 'STAR B3',
    2240 => 'STAR B4',
    2241 => 'STAR B4I',
    2242 => 'STAR B4II',
    2243 => 'STAR B4III',
    2244 => 'STAR B4IV',
    2245 => 'STAR B4V',
    2246 => 'STAR B4VI',
    2247 => 'STAR B4',
    2248 => 'STAR B4',
    2249 => 'STAR B4',
    2250 => 'STAR B5',
    2251 => 'STAR B5I',
    2252 => 'STAR B5II',
    2253 => 'STAR B5III',
    2254 => 'STAR B5IV',
    2255 => 'STAR B5V',
    2256 => 'STAR B5VI',
    2257 => 'STAR B5',
    2258 => 'STAR B5',
    2259 => 'STAR B5',
    2260 => 'STAR B6',
    2261 => 'STAR B6I',
    2262 => 'STAR B6II',
    2263 => 'STAR B6III',
    2264 => 'STAR B6IV',
    2265 => 'STAR B6V',
    2266 => 'STAR B6VI',
    2267 => 'STAR B6',
    2268 => 'STAR B6',
    2269 => 'STAR B6',
    2270 => 'STAR B7',
    2271 => 'STAR B7I',
    2272 => 'STAR B7II',
    2273 => 'STAR B7III',
    2274 => 'STAR B7IV',
    2275 => 'STAR B7V',
    2276 => 'STAR B7VI',
    2277 => 'STAR B7',
    2278 => 'STAR B7',
    2279 => 'STAR B7',
    2280 => 'STAR B8',
    2281 => 'STAR B8I',
    2282 => 'STAR B8II',
    2283 => 'STAR B8III',
    2284 => 'STAR B8IV',
    2285 => 'STAR B8V',
    2286 => 'STAR B8VI',
    2287 => 'STAR B8',
    2288 => 'STAR B8',
    2289 => 'STAR B8',
    2290 => 'STAR B9',
    2291 => 'STAR B9I',
    2292 => 'STAR B9II',
    2293 => 'STAR B9III',
    2294 => 'STAR B9IV',
    2295 => 'STAR B9V',
    2296 => 'STAR B9VI',
    2297 => 'STAR B9',
    2298 => 'STAR B9',
    2299 => 'STAR B9',
    2300 => 'STAR A0',
    2301 => 'STAR A0I',
    2302 => 'STAR A0II',
    2303 => 'STAR A0III',
    2304 => 'STAR A0IV',
    2305 => 'STAR A0V',
    2306 => 'STAR A0VI',
    2307 => 'STAR A0',
    2308 => 'STAR A0',
    2309 => 'STAR A0',
    2310 => 'STAR A1',
    2311 => 'STAR A1I',
    2312 => 'STAR A1II',
    2313 => 'STAR A1III',
    2314 => 'STAR A1IV',
    2315 => 'STAR A1V',
    2316 => 'STAR A1VI',
    2317 => 'STAR A1',
    2318 => 'STAR A1',
    2319 => 'STAR A1',
    2320 => 'STAR A2',
    2321 => 'STAR A2I',
    2322 => 'STAR A2II',
    2323 => 'STAR A2III',
    2324 => 'STAR A2IV',
    2325 => 'STAR A2V',
    2326 => 'STAR A2VI',
    2327 => 'STAR A2',
    2328 => 'STAR A2',
    2329 => 'STAR A2',
    2330 => 'STAR A3',
    2331 => 'STAR A3I',
    2332 => 'STAR A3II',
    2333 => 'STAR A3III',
    2334 => 'STAR A3IV',
    2335 => 'STAR A3V',
    2336 => 'STAR A3VI',
    2337 => 'STAR A3',
    2338 => 'STAR A3',
    2339 => 'STAR A3',
    2340 => 'STAR A4',
    2341 => 'STAR A4I',
    2342 => 'STAR A4II',
    2343 => 'STAR A4III',
    2344 => 'STAR A4IV',
    2345 => 'STAR A4V',
    2346 => 'STAR A4VI',
    2347 => 'STAR A4',
    2348 => 'STAR A4',
    2349 => 'STAR A4',
    2350 => 'STAR A5',
    2351 => 'STAR A5I',
    2352 => 'STAR A5II',
    2353 => 'STAR A5III',
    2354 => 'STAR A5IV',
    2355 => 'STAR A5V',
    2356 => 'STAR A5VI',
    2357 => 'STAR A5',
    2358 => 'STAR A5',
    2359 => 'STAR A5',
    2360 => 'STAR A6',
    2361 => 'STAR A6I',
    2362 => 'STAR A6II',
    2363 => 'STAR A6III',
    2364 => 'STAR A6IV',
    2365 => 'STAR A6V',
    2366 => 'STAR A6VI',
    2367 => 'STAR A6',
    2368 => 'STAR A6',
    2369 => 'STAR A6',
    2370 => 'STAR A7',
    2371 => 'STAR A7I',
    2372 => 'STAR A7II',
    2373 => 'STAR A7III',
    2374 => 'STAR A7IV',
    2375 => 'STAR A7V',
    2376 => 'STAR A7VI',
    2377 => 'STAR A7',
    2378 => 'STAR A7',
    2379 => 'STAR A7',
    2380 => 'STAR A8',
    2381 => 'STAR A8I',
    2382 => 'STAR A8II',
    2383 => 'STAR A8III',
    2384 => 'STAR A8IV',
    2385 => 'STAR A8V',
    2386 => 'STAR A8VI',
    2387 => 'STAR A8',
    2388 => 'STAR A8',
    2389 => 'STAR A8',
    2390 => 'STAR A9',
    2391 => 'STAR A9I',
    2392 => 'STAR A9II',
    2393 => 'STAR A9III',
    2394 => 'STAR A9IV',
    2395 => 'STAR A9V',
    2396 => 'STAR A9VI',
    2397 => 'STAR A9',
    2398 => 'STAR A9',
    2399 => 'STAR A9',
    2400 => 'STAR F0',
    2401 => 'STAR F0I',
    2402 => 'STAR F0II',
    2403 => 'STAR F0III',
    2404 => 'STAR F0IV',
    2405 => 'STAR F0V',
    2406 => 'STAR F0VI',
    2407 => 'STAR F0',
    2408 => 'STAR F0',
    2409 => 'STAR F0',
    2410 => 'STAR F1',
    2411 => 'STAR F1I',
    2412 => 'STAR F1II',
    2413 => 'STAR F1III',
    2414 => 'STAR F1IV',
    2415 => 'STAR F1V',
    2416 => 'STAR F1VI',
    2417 => 'STAR F1',
    2418 => 'STAR F1',
    2419 => 'STAR F1',
    2420 => 'STAR F2',
    2421 => 'STAR F2I',
    2422 => 'STAR F2II',
    2423 => 'STAR F2III',
    2424 => 'STAR F2IV',
    2425 => 'STAR F2V',
    2426 => 'STAR F2VI',
    2427 => 'STAR F2',
    2428 => 'STAR F2',
    2429 => 'STAR F2',
    2430 => 'STAR F3',
    2431 => 'STAR F3I',
    2432 => 'STAR F3II',
    2433 => 'STAR F3III',
    2434 => 'STAR F3IV',
    2435 => 'STAR F3V',
    2436 => 'STAR F3VI',
    2437 => 'STAR F3',
    2438 => 'STAR F3',
    2439 => 'STAR F3',
    2440 => 'STAR F4',
    2441 => 'STAR F4I',
    2442 => 'STAR F4II',
    2443 => 'STAR F4III',
    2444 => 'STAR F4IV',
    2445 => 'STAR F4V',
    2446 => 'STAR F4VI',
    2447 => 'STAR F4',
    2448 => 'STAR F4',
    2449 => 'STAR F4',
    2450 => 'STAR F5',
    2451 => 'STAR F5I',
    2452 => 'STAR F5II',
    2453 => 'STAR F5III',
    2454 => 'STAR F5IV',
    2455 => 'STAR F5V',
    2456 => 'STAR F5VI',
    2457 => 'STAR F5',
    2458 => 'STAR F5',
    2459 => 'STAR F5',
    2460 => 'STAR F6',
    2461 => 'STAR F6I',
    2462 => 'STAR F6II',
    2463 => 'STAR F6III',
    2464 => 'STAR F6IV',
    2465 => 'STAR F6V',
    2466 => 'STAR F6VI',
    2467 => 'STAR F6',
    2468 => 'STAR F6',
    2469 => 'STAR F6',
    2470 => 'STAR F7',
    2471 => 'STAR F7I',
    2472 => 'STAR F7II',
    2473 => 'STAR F7III',
    2474 => 'STAR F7IV',
    2475 => 'STAR F7V',
    2476 => 'STAR F7VI',
    2477 => 'STAR F7',
    2478 => 'STAR F7',
    2479 => 'STAR F7',
    2480 => 'STAR F8',
    2481 => 'STAR F8I',
    2482 => 'STAR F8II',
    2483 => 'STAR F8III',
    2484 => 'STAR F8IV',
    2485 => 'STAR F8V',
    2486 => 'STAR F8VI',
    2487 => 'STAR F8',
    2488 => 'STAR F8',
    2489 => 'STAR F8',
    2490 => 'STAR F9',
    2491 => 'STAR F9I',
    2492 => 'STAR F9II',
    2493 => 'STAR F9III',
    2494 => 'STAR F9IV',
    2495 => 'STAR F9V',
    2496 => 'STAR F9VI',
    2497 => 'STAR F9',
    2498 => 'STAR F9',
    2499 => 'STAR F9',
    2500 => 'STAR G0',
    2501 => 'STAR G0I',
    2502 => 'STAR G0II',
    2503 => 'STAR G0III',
    2504 => 'STAR G0IV',
    2505 => 'STAR G0V',
    2506 => 'STAR G0VI',
    2507 => 'STAR G0',
    2508 => 'STAR G0',
    2509 => 'STAR G0',
    2510 => 'STAR G1',
    2511 => 'STAR G1I',
    2512 => 'STAR G1II',
    2513 => 'STAR G1III',
    2514 => 'STAR G1IV',
    2515 => 'STAR G1V',
    2516 => 'STAR G1VI',
    2517 => 'STAR G1',
    2518 => 'STAR G1',
    2519 => 'STAR G1',
    2520 => 'STAR G2',
    2521 => 'STAR G2I',
    2522 => 'STAR G2II',
    2523 => 'STAR G2III',
    2524 => 'STAR G2IV',
    2525 => 'STAR G2V',
    2526 => 'STAR G2VI',
    2527 => 'STAR G2',
    2528 => 'STAR G2',
    2529 => 'STAR G2',
    2530 => 'STAR G3',
    2531 => 'STAR G3I',
    2532 => 'STAR G3II',
    2533 => 'STAR G3III',
    2534 => 'STAR G3IV',
    2535 => 'STAR G3V',
    2536 => 'STAR G3VI',
    2537 => 'STAR G3',
    2538 => 'STAR G3',
    2539 => 'STAR G3',
    2540 => 'STAR G4',
    2541 => 'STAR G4I',
    2542 => 'STAR G4II',
    2543 => 'STAR G4III',
    2544 => 'STAR G4IV',
    2545 => 'STAR G4V',
    2546 => 'STAR G4VI',
    2547 => 'STAR G4',
    2548 => 'STAR G4',
    2549 => 'STAR G4',
    2550 => 'STAR G5',
    2551 => 'STAR G5I',
    2552 => 'STAR G5II',
    2553 => 'STAR G5III',
    2554 => 'STAR G5IV',
    2555 => 'STAR G5V',
    2556 => 'STAR G5VI',
    2557 => 'STAR G5',
    2558 => 'STAR G5',
    2559 => 'STAR G5',
    2560 => 'STAR G6',
    2561 => 'STAR G6I',
    2562 => 'STAR G6II',
    2563 => 'STAR G6III',
    2564 => 'STAR G6IV',
    2565 => 'STAR G6V',
    2566 => 'STAR G6VI',
    2567 => 'STAR G6',
    2568 => 'STAR G6',
    2569 => 'STAR G6',
    2570 => 'STAR G7',
    2571 => 'STAR G7I',
    2572 => 'STAR G7II',
    2573 => 'STAR G7III',
    2574 => 'STAR G7IV',
    2575 => 'STAR G7V',
    2576 => 'STAR G7VI',
    2577 => 'STAR G7',
    2578 => 'STAR G7',
    2579 => 'STAR G7',
    2580 => 'STAR G8',
    2581 => 'STAR G8I',
    2582 => 'STAR G8II',
    2583 => 'STAR G8III',
    2584 => 'STAR G8IV',
    2585 => 'STAR G8V',
    2586 => 'STAR G8VI',
    2587 => 'STAR G8',
    2588 => 'STAR G8',
    2589 => 'STAR G8',
    2590 => 'STAR G9',
    2591 => 'STAR G9I',
    2592 => 'STAR G9II',
    2593 => 'STAR G9III',
    2594 => 'STAR G9IV',
    2595 => 'STAR G9V',
    2596 => 'STAR G9VI',
    2597 => 'STAR G9',
    2598 => 'STAR G9',
    2599 => 'STAR G9',
    2600 => 'STAR K0',
    2601 => 'STAR K0I',
    2602 => 'STAR K0II',
    2603 => 'STAR K0III',
    2604 => 'STAR K0IV',
    2605 => 'STAR K0V',
    2606 => 'STAR K0VI',
    2607 => 'STAR K0',
    2608 => 'STAR K0',
    2609 => 'STAR K0',
    2610 => 'STAR K1',
    2611 => 'STAR K1I',
    2612 => 'STAR K1II',
    2613 => 'STAR K1III',
    2614 => 'STAR K1IV',
    2615 => 'STAR K1V',
    2616 => 'STAR K1VI',
    2617 => 'STAR K1',
    2618 => 'STAR K1',
    2619 => 'STAR K1',
    2620 => 'STAR K2',
    2621 => 'STAR K2I',
    2622 => 'STAR K2II',
    2623 => 'STAR K2III',
    2624 => 'STAR K2IV',
    2625 => 'STAR K2V',
    2626 => 'STAR K2VI',
    2627 => 'STAR K2',
    2628 => 'STAR K2',
    2629 => 'STAR K2',
    2630 => 'STAR K3',
    2631 => 'STAR K3I',
    2632 => 'STAR K3II',
    2633 => 'STAR K3III',
    2634 => 'STAR K3IV',
    2635 => 'STAR K3V',
    2636 => 'STAR K3VI',
    2637 => 'STAR K3',
    2638 => 'STAR K3',
    2639 => 'STAR K3',
    2640 => 'STAR K4',
    2641 => 'STAR K4I',
    2642 => 'STAR K4II',
    2643 => 'STAR K4III',
    2644 => 'STAR K4IV',
    2645 => 'STAR K4V',
    2646 => 'STAR K4VI',
    2647 => 'STAR K4',
    2648 => 'STAR K4',
    2649 => 'STAR K4',
    2650 => 'STAR K5',
    2651 => 'STAR K5I',
    2652 => 'STAR K5II',
    2653 => 'STAR K5III',
    2654 => 'STAR K5IV',
    2655 => 'STAR K5V',
    2656 => 'STAR K5VI',
    2657 => 'STAR K5',
    2658 => 'STAR K5',
    2659 => 'STAR K5',
    2660 => 'STAR K6',
    2661 => 'STAR K6I',
    2662 => 'STAR K6II',
    2663 => 'STAR K6III',
    2664 => 'STAR K6IV',
    2665 => 'STAR K6V',
    2666 => 'STAR K6VI',
    2667 => 'STAR K6',
    2668 => 'STAR K6',
    2669 => 'STAR K6',
    2670 => 'STAR K7',
    2671 => 'STAR K7I',
    2672 => 'STAR K7II',
    2673 => 'STAR K7III',
    2674 => 'STAR K7IV',
    2675 => 'STAR K7V',
    2676 => 'STAR K7VI',
    2677 => 'STAR K7',
    2678 => 'STAR K7',
    2679 => 'STAR K7',
    2680 => 'STAR K8',
    2681 => 'STAR K8I',
    2682 => 'STAR K8II',
    2683 => 'STAR K8III',
    2684 => 'STAR K8IV',
    2685 => 'STAR K8V',
    2686 => 'STAR K8VI',
    2687 => 'STAR K8',
    2688 => 'STAR K8',
    2689 => 'STAR K8',
    2690 => 'STAR K9',
    2691 => 'STAR K9I',
    2692 => 'STAR K9II',
    2693 => 'STAR K9III',
    2694 => 'STAR K9IV',
    2695 => 'STAR K9V',
    2696 => 'STAR K9VI',
    2697 => 'STAR K9',
    2698 => 'STAR K9',
    2699 => 'STAR K9',
    2700 => 'STAR M0',
    2701 => 'STAR M0I',
    2702 => 'STAR M0II',
    2703 => 'STAR M0III',
    2704 => 'STAR M0IV',
    2705 => 'STAR M0V',
    2706 => 'STAR M0VI',
    2707 => 'STAR M0',
    2708 => 'STAR M0',
    2709 => 'STAR M0',
    2710 => 'STAR M1',
    2711 => 'STAR M1I',
    2712 => 'STAR M1II',
    2713 => 'STAR M1III',
    2714 => 'STAR M1IV',
    2715 => 'STAR M1V',
    2716 => 'STAR M1VI',
    2717 => 'STAR M1',
    2718 => 'STAR M1',
    2719 => 'STAR M1',
    2720 => 'STAR M2',
    2721 => 'STAR M2I',
    2722 => 'STAR M2II',
    2723 => 'STAR M2III',
    2724 => 'STAR M2IV',
    2725 => 'STAR M2V',
    2726 => 'STAR M2VI',
    2727 => 'STAR M2',
    2728 => 'STAR M2',
    2729 => 'STAR M2',
    2730 => 'STAR M3',
    2731 => 'STAR M3I',
    2732 => 'STAR M3II',
    2733 => 'STAR M3III',
    2734 => 'STAR M3IV',
    2735 => 'STAR M3V',
    2736 => 'STAR M3VI',
    2737 => 'STAR M3',
    2738 => 'STAR M3',
    2739 => 'STAR M3',
    2740 => 'STAR M4',
    2741 => 'STAR M4I',
    2742 => 'STAR M4II',
    2743 => 'STAR M4III',
    2744 => 'STAR M4IV',
    2745 => 'STAR M4V',
    2746 => 'STAR M4VI',
    2747 => 'STAR M4',
    2748 => 'STAR M4',
    2749 => 'STAR M4',
    2750 => 'STAR M5',
    2751 => 'STAR M5I',
    2752 => 'STAR M5II',
    2753 => 'STAR M5III',
    2754 => 'STAR M5IV',
    2755 => 'STAR M5V',
    2756 => 'STAR M5VI',
    2757 => 'STAR M5',
    2758 => 'STAR M5',
    2759 => 'STAR M5',
    2760 => 'STAR M6',
    2761 => 'STAR M6I',
    2762 => 'STAR M6II',
    2763 => 'STAR M6III',
    2764 => 'STAR M6IV',
    2765 => 'STAR M6V',
    2766 => 'STAR M6VI',
    2767 => 'STAR M6',
    2768 => 'STAR M6',
    2769 => 'STAR M6',
    2770 => 'STAR M7',
    2771 => 'STAR M7I',
    2772 => 'STAR M7II',
    2773 => 'STAR M7III',
    2774 => 'STAR M7IV',
    2775 => 'STAR M7V',
    2776 => 'STAR M7VI',
    2777 => 'STAR M7',
    2778 => 'STAR M7',
    2779 => 'STAR M7',
    2780 => 'STAR M8',
    2781 => 'STAR M8I',
    2782 => 'STAR M8II',
    2783 => 'STAR M8III',
    2784 => 'STAR M8IV',
    2785 => 'STAR M8V',
    2786 => 'STAR M8VI',
    2787 => 'STAR M8',
    2788 => 'STAR M8',
    2789 => 'STAR M8',
    2790 => 'STAR M9',
    2791 => 'STAR M9I',
    2792 => 'STAR M9II',
    2793 => 'STAR M9III',
    2794 => 'STAR M9IV',
    2795 => 'STAR M9V',
    2796 => 'STAR M9VI',
    2797 => 'STAR M9',
    2798 => 'STAR M9',
    2799 => 'STAR M9',
    2800 => 'STAR M0e',
    2801 => 'STAR M0Ie',
    2802 => 'STAR M0IIe',
    2803 => 'STAR M0IIIe',
    2804 => 'STAR M0IVe',
    2805 => 'STAR M0Ve',
    2806 => 'STAR M0VIe',
    2807 => 'STAR M0e',
    2808 => 'STAR M0e',
    2809 => 'STAR M0e',
    2810 => 'STAR M1e',
    2811 => 'STAR M1Ie',
    2812 => 'STAR M1IIe',
    2813 => 'STAR M1IIIe',
    2814 => 'STAR M1IVe',
    2815 => 'STAR M1Ve',
    2816 => 'STAR M1VIe',
    2817 => 'STAR M1e',
    2818 => 'STAR M1e',
    2819 => 'STAR M1e',
    2820 => 'STAR M2e',
    2821 => 'STAR M2Ie',
    2822 => 'STAR M2IIe',
    2823 => 'STAR M2IIIe',
    2824 => 'STAR M2IVe',
    2825 => 'STAR M2Ve',
    2826 => 'STAR M2VIe',
    2827 => 'STAR M2e',
    2828 => 'STAR M2e',
    2829 => 'STAR M2e',
    2830 => 'STAR M3e',
    2831 => 'STAR M3Ie',
    2832 => 'STAR M3IIe',
    2833 => 'STAR M3IIIe',
    2834 => 'STAR M3IVe',
    2835 => 'STAR M3Ve',
    2836 => 'STAR M3VIe',
    2837 => 'STAR M3e',
    2838 => 'STAR M3e',
    2839 => 'STAR M3e',
    2840 => 'STAR M4e',
    2841 => 'STAR M4Ie',
    2842 => 'STAR M4IIe',
    2843 => 'STAR M4IIIe',
    2844 => 'STAR M4IVe',
    2845 => 'STAR M4Ve',
    2846 => 'STAR M4VIe',
    2847 => 'STAR M4e',
    2848 => 'STAR M4e',
    2849 => 'STAR M4e',
    2850 => 'STAR M5e',
    2851 => 'STAR M5Ie',
    2852 => 'STAR M5IIe',
    2853 => 'STAR M5IIIe',
    2854 => 'STAR M5IVe',
    2855 => 'STAR M5Ve',
    2856 => 'STAR M5VIe',
    2857 => 'STAR M5e',
    2858 => 'STAR M5e',
    2859 => 'STAR M5e',
    2860 => 'STAR M6e',
    2861 => 'STAR M6Ie',
    2862 => 'STAR M6IIe',
    2863 => 'STAR M6IIIe',
    2864 => 'STAR M6IVe',
    2865 => 'STAR M6Ve',
    2866 => 'STAR M6VIe',
    2867 => 'STAR M6e',
    2868 => 'STAR M6e',
    2869 => 'STAR M6e',
    2870 => 'STAR M7e',
    2871 => 'STAR M7Ie',
    2872 => 'STAR M7IIe',
    2873 => 'STAR M7IIIe',
    2874 => 'STAR M7IVe',
    2875 => 'STAR M7Ve',
    2876 => 'STAR M7VIe',
    2877 => 'STAR M7e',
    2878 => 'STAR M7e',
    2879 => 'STAR M7e',
    2880 => 'STAR M8e',
    2881 => 'STAR M8Ie',
    2882 => 'STAR M8IIe',
    2883 => 'STAR M8IIIe',
    2884 => 'STAR M8IVe',
    2885 => 'STAR M8Ve',
    2886 => 'STAR M8VIe',
    2887 => 'STAR M8e',
    2888 => 'STAR M8e',
    2889 => 'STAR M8e',
    2890 => 'STAR M9e',
    2891 => 'STAR M9Ie',
    2892 => 'STAR M9IIe',
    2893 => 'STAR M9IIIe',
    2894 => 'STAR M9IVe',
    2895 => 'STAR M9Ve',
    2896 => 'STAR M9VIe',
    2897 => 'STAR M9e',
    2898 => 'STAR M9e',
    2899 => 'STAR M9e',
    2900 => 'STAR',
    2901 => 'STAR',
    2902 => 'STAR',
    2903 => 'STAR',
    2904 => 'STAR',
    2905 => 'STAR',
    2906 => 'STAR',
    2907 => 'STAR',
    2908 => 'STAR',
    2909 => 'STAR',
    2910 => 'STAR LUYTEN COLOR CLASS A',
    2911 => 'STAR LUYTEN COLOR CLASS A',
    2912 => 'STAR LUYTEN COLOR CLASS A',
    2913 => 'STAR LUYTEN COLOR CLASS A',
    2914 => 'STAR LUYTEN COLOR CLASS A',
    2915 => 'STAR LUYTEN COLOR CLASS A',
    2916 => 'STAR LUYTEN COLOR CLASS A',
    2917 => 'STAR LUYTEN COLOR CLASS A',
    2918 => 'STAR LUYTEN COLOR CLASS A',
    2919 => 'STAR LUYTEN COLOR CLASS A',
    2920 => 'STAR LUYTEN COLOR CLASS A-F',
    2921 => 'STAR LUYTEN COLOR CLASS A-F',
    2922 => 'STAR LUYTEN COLOR CLASS A-F',
    2923 => 'STAR LUYTEN COLOR CLASS A-F',
    2924 => 'STAR LUYTEN COLOR CLASS A-F',
    2925 => 'STAR LUYTEN COLOR CLASS A-F',
    2926 => 'STAR LUYTEN COLOR CLASS A-F',
    2927 => 'STAR LUYTEN COLOR CLASS A-F',
    2928 => 'STAR LUYTEN COLOR CLASS A-F',
    2929 => 'STAR LUYTEN COLOR CLASS A-F',
    2930 => 'STAR LUYTEN COLOR CLASS F',
    2931 => 'STAR LUYTEN COLOR CLASS F',
    2932 => 'STAR LUYTEN COLOR CLASS F',
    2933 => 'STAR LUYTEN COLOR CLASS F',
    2934 => 'STAR LUYTEN COLOR CLASS F',
    2935 => 'STAR LUYTEN COLOR CLASS F',
    2936 => 'STAR LUYTEN COLOR CLASS F',
    2937 => 'STAR LUYTEN COLOR CLASS F',
    2938 => 'STAR LUYTEN COLOR CLASS F',
    2939 => 'STAR LUYTEN COLOR CLASS F',
    2940 => 'STAR LUYTEN COLOR CLASS F-G',
    2941 => 'STAR LUYTEN COLOR CLASS F-G',
    2942 => 'STAR LUYTEN COLOR CLASS F-G',
    2943 => 'STAR LUYTEN COLOR CLASS F-G',
    2944 => 'STAR LUYTEN COLOR CLASS F-G',
    2945 => 'STAR LUYTEN COLOR CLASS F-G',
    2946 => 'STAR LUYTEN COLOR CLASS F-G',
    2947 => 'STAR LUYTEN COLOR CLASS F-G',
    2948 => 'STAR LUYTEN COLOR CLASS F-G',
    2949 => 'STAR LUYTEN COLOR CLASS F-G',
    2950 => 'STAR LUYTEN COLOR CLASS G',
    2951 => 'STAR LUYTEN COLOR CLASS G',
    2952 => 'STAR LUYTEN COLOR CLASS G',
    2953 => 'STAR LUYTEN COLOR CLASS G',
    2954 => 'STAR LUYTEN COLOR CLASS G',
    2955 => 'STAR LUYTEN COLOR CLASS G',
    2956 => 'STAR LUYTEN COLOR CLASS G',
    2957 => 'STAR LUYTEN COLOR CLASS G',
    2958 => 'STAR LUYTEN COLOR CLASS G',
    2959 => 'STAR LUYTEN COLOR CLASS G',
    2960 => 'STAR LUYTEN COLOR CLASS G-K',
    2961 => 'STAR LUYTEN COLOR CLASS G-K',
    2962 => 'STAR LUYTEN COLOR CLASS G-K',
    2963 => 'STAR LUYTEN COLOR CLASS G-K',
    2964 => 'STAR LUYTEN COLOR CLASS G-K',
    2965 => 'STAR LUYTEN COLOR CLASS G-K',
    2966 => 'STAR LUYTEN COLOR CLASS G-K',
    2967 => 'STAR LUYTEN COLOR CLASS G-K',
    2968 => 'STAR LUYTEN COLOR CLASS G-K',
    2969 => 'STAR LUYTEN COLOR CLASS G-K',
    2970 => 'STAR LUYTEN COLOR CLASS K',
    2971 => 'STAR LUYTEN COLOR CLASS K',
    2972 => 'STAR LUYTEN COLOR CLASS K',
    2973 => 'STAR LUYTEN COLOR CLASS K',
    2974 => 'STAR LUYTEN COLOR CLASS K',
    2975 => 'STAR LUYTEN COLOR CLASS K',
    2976 => 'STAR LUYTEN COLOR CLASS K',
    2977 => 'STAR LUYTEN COLOR CLASS K',
    2978 => 'STAR LUYTEN COLOR CLASS K',
    2979 => 'STAR LUYTEN COLOR CLASS K',
    2980 => 'STAR LUYTEN COLOR CLASS K-M',
    2981 => 'STAR LUYTEN COLOR CLASS K-M',
    2982 => 'STAR LUYTEN COLOR CLASS K-M',
    2983 => 'STAR LUYTEN COLOR CLASS K-M',
    2984 => 'STAR LUYTEN COLOR CLASS K-M',
    2985 => 'STAR LUYTEN COLOR CLASS K-M',
    2986 => 'STAR LUYTEN COLOR CLASS K-M',
    2987 => 'STAR LUYTEN COLOR CLASS K-M',
    2988 => 'STAR LUYTEN COLOR CLASS K-M',
    2989 => 'STAR LUYTEN COLOR CLASS K-M',
    2990 => 'STAR LUYTEN COLOR CLASS M',
    2991 => 'STAR LUYTEN COLOR CLASS M',
    2992 => 'STAR LUYTEN COLOR CLASS M',
    2993 => 'STAR LUYTEN COLOR CLASS M',
    2994 => 'STAR LUYTEN COLOR CLASS M',
    2995 => 'STAR LUYTEN COLOR CLASS M',
    2996 => 'STAR LUYTEN COLOR CLASS M',
    2997 => 'STAR LUYTEN COLOR CLASS M',
    2998 => 'STAR LUYTEN COLOR CLASS M',
    2999 => 'STAR LUYTEN COLOR CLASS M',
    3000 => 'EXTENDED GALACTIC OR EXTRAGALACTIC',
    3001 => 'EXTENDED GALACTIC OR EXTRAGALACTIC RADIO PULSAR',
    3002 => 'EXTENDED GALACTIC OR EXTRAGALACTIC TYPE I',
    3003 => 'EXTENDED GALACTIC OR EXTRAGALACTIC TYPE II',
    3004 => 'EXTENDED GALACTIC OR EXTRAGALACTIC',
    3005 => 'EXTENDED GALACTIC OR EXTRAGALACTIC',
    3006 => 'EXTENDED GALACTIC OR EXTRAGALACTIC',
    3007 => 'EXTENDED GALACTIC OR EXTRAGALACTIC',
    3008 => 'EXTENDED GALACTIC OR EXTRAGALACTIC',
    3009 => 'EXTENDED GALACTIC OR EXTRAGALACTIC',
    3010 => 'EXTENDED GALACTIC OR EXTRAGALACTIC SHELL',
    3011 => 'EXTENDED GALACTIC OR EXTRAGALACTIC SHELL RADIO PULSAR',
    3012 => 'EXTENDED GALACTIC OR EXTRAGALACTIC SHELL TYPE I',
    3013 => 'EXTENDED GALACTIC OR EXTRAGALACTIC SHELL TYPE II',
    3014 => 'EXTENDED GALACTIC OR EXTRAGALACTIC SHELL',
    3015 => 'EXTENDED GALACTIC OR EXTRAGALACTIC SHELL',
    3016 => 'EXTENDED GALACTIC OR EXTRAGALACTIC SHELL',
    3017 => 'EXTENDED GALACTIC OR EXTRAGALACTIC SHELL',
    3018 => 'EXTENDED GALACTIC OR EXTRAGALACTIC SHELL',
    3019 => 'EXTENDED GALACTIC OR EXTRAGALACTIC SHELL',
    3020 => 'EXTENDED GALACTIC OR EXTRAGALACTIC FILLED-CENTER',
    3021 => 'EXTENDED GALACTIC OR EXTRAGALACTIC FILLED-CENTER RADIO PULSAR',
    3022 => 'EXTENDED GALACTIC OR EXTRAGALACTIC FILLED-CENTER TYPE I',
    3023 => 'EXTENDED GALACTIC OR EXTRAGALACTIC FILLED-CENTER TYPE II',
    3024 => 'EXTENDED GALACTIC OR EXTRAGALACTIC FILLED-CENTER',
    3025 => 'EXTENDED GALACTIC OR EXTRAGALACTIC FILLED-CENTER',
    3026 => 'EXTENDED GALACTIC OR EXTRAGALACTIC FILLED-CENTER',
    3027 => 'EXTENDED GALACTIC OR EXTRAGALACTIC FILLED-CENTER',
    3028 => 'EXTENDED GALACTIC OR EXTRAGALACTIC FILLED-CENTER',
    3029 => 'EXTENDED GALACTIC OR EXTRAGALACTIC FILLED-CENTER',
    3030 => 'EXTENDED GALACTIC OR EXTRAGALACTIC COMPOSITE',
    3031 => 'EXTENDED GALACTIC OR EXTRAGALACTIC COMPOSITE RADIO PULSAR',
    3032 => 'EXTENDED GALACTIC OR EXTRAGALACTIC COMPOSITE TYPE I',
    3033 => 'EXTENDED GALACTIC OR EXTRAGALACTIC COMPOSITE TYPE II',
    3034 => 'EXTENDED GALACTIC OR EXTRAGALACTIC COMPOSITE',
    3035 => 'EXTENDED GALACTIC OR EXTRAGALACTIC COMPOSITE',
    3036 => 'EXTENDED GALACTIC OR EXTRAGALACTIC COMPOSITE',
    3037 => 'EXTENDED GALACTIC OR EXTRAGALACTIC COMPOSITE',
    3038 => 'EXTENDED GALACTIC OR EXTRAGALACTIC COMPOSITE',
    3039 => 'EXTENDED GALACTIC OR EXTRAGALACTIC COMPOSITE',
    3040 => 'DARK EXTENDED GALACTIC OR EXTRAGALACTIC',
    3041 => 'DARK EXTENDED GALACTIC OR EXTRAGALACTIC RADIO PULSAR',
    3042 => 'DARK EXTENDED GALACTIC OR EXTRAGALACTIC TYPE I',
    3043 => 'DARK EXTENDED GALACTIC OR EXTRAGALACTIC TYPE II',
    3044 => 'DARK EXTENDED GALACTIC OR EXTRAGALACTIC',
    3045 => 'DARK EXTENDED GALACTIC OR EXTRAGALACTIC',
    3046 => 'DARK EXTENDED GALACTIC OR EXTRAGALACTIC',
    3047 => 'DARK EXTENDED GALACTIC OR EXTRAGALACTIC',
    3048 => 'DARK EXTENDED GALACTIC OR EXTRAGALACTIC',
    3049 => 'DARK EXTENDED GALACTIC OR EXTRAGALACTIC',
    3050 => 'MOLECULAR EXTENDED GALACTIC OR EXTRAGALACTIC',
    3051 => 'MOLECULAR EXTENDED GALACTIC OR EXTRAGALACTIC RADIO PULSAR',
    3052 => 'MOLECULAR EXTENDED GALACTIC OR EXTRAGALACTIC TYPE I',
    3053 => 'MOLECULAR EXTENDED GALACTIC OR EXTRAGALACTIC TYPE II',
    3054 => 'MOLECULAR EXTENDED GALACTIC OR EXTRAGALACTIC',
    3055 => 'MOLECULAR EXTENDED GALACTIC OR EXTRAGALACTIC',
    3056 => 'MOLECULAR EXTENDED GALACTIC OR EXTRAGALACTIC',
    3057 => 'MOLECULAR EXTENDED GALACTIC OR EXTRAGALACTIC',
    3058 => 'MOLECULAR EXTENDED GALACTIC OR EXTRAGALACTIC',
    3059 => 'MOLECULAR EXTENDED GALACTIC OR EXTRAGALACTIC',
    3060 => 'PLANETARY EXTENDED GALACTIC OR EXTRAGALACTIC',
    3061 => 'PLANETARY EXTENDED GALACTIC OR EXTRAGALACTIC RADIO PULSAR',
    3062 => 'PLANETARY EXTENDED GALACTIC OR EXTRAGALACTIC TYPE I',
    3063 => 'PLANETARY EXTENDED GALACTIC OR EXTRAGALACTIC TYPE II',
    3064 => 'PLANETARY EXTENDED GALACTIC OR EXTRAGALACTIC',
    3065 => 'PLANETARY EXTENDED GALACTIC OR EXTRAGALACTIC',
    3066 => 'PLANETARY EXTENDED GALACTIC OR EXTRAGALACTIC',
    3067 => 'PLANETARY EXTENDED GALACTIC OR EXTRAGALACTIC',
    3068 => 'PLANETARY EXTENDED GALACTIC OR EXTRAGALACTIC',
    3069 => 'PLANETARY EXTENDED GALACTIC OR EXTRAGALACTIC',
    3070 => 'REFLECTION EXTENDED GALACTIC OR EXTRAGALACTIC',
    3071 => 'REFLECTION EXTENDED GALACTIC OR EXTRAGALACTIC RADIO PULSAR',
    3072 => 'REFLECTION EXTENDED GALACTIC OR EXTRAGALACTIC TYPE I',
    3073 => 'REFLECTION EXTENDED GALACTIC OR EXTRAGALACTIC TYPE II',
    3074 => 'REFLECTION EXTENDED GALACTIC OR EXTRAGALACTIC',
    3075 => 'REFLECTION EXTENDED GALACTIC OR EXTRAGALACTIC',
    3076 => 'REFLECTION EXTENDED GALACTIC OR EXTRAGALACTIC',
    3077 => 'REFLECTION EXTENDED GALACTIC OR EXTRAGALACTIC',
    3078 => 'REFLECTION EXTENDED GALACTIC OR EXTRAGALACTIC',
    3079 => 'REFLECTION EXTENDED GALACTIC OR EXTRAGALACTIC',
    3080 => 'GLOBULAR CLUSTER EXTENDED GALACTIC OR EXTRAGALACTIC',
    3081 => 'GLOBULAR CLUSTER EXTENDED GALACTIC OR EXTRAGALACTIC RADIO PULSAR',
    3082 => 'GLOBULAR CLUSTER EXTENDED GALACTIC OR EXTRAGALACTIC TYPE I',
    3083 => 'GLOBULAR CLUSTER EXTENDED GALACTIC OR EXTRAGALACTIC TYPE II',
    3084 => 'GLOBULAR CLUSTER EXTENDED GALACTIC OR EXTRAGALACTIC',
    3085 => 'GLOBULAR CLUSTER EXTENDED GALACTIC OR EXTRAGALACTIC',
    3086 => 'GLOBULAR CLUSTER EXTENDED GALACTIC OR EXTRAGALACTIC',
    3087 => 'GLOBULAR CLUSTER EXTENDED GALACTIC OR EXTRAGALACTIC',
    3088 => 'GLOBULAR CLUSTER EXTENDED GALACTIC OR EXTRAGALACTIC',
    3089 => 'GLOBULAR CLUSTER EXTENDED GALACTIC OR EXTRAGALACTIC',
    3090 => 'EXTENDED GALACTIC OR EXTRAGALACTIC',
    3091 => 'EXTENDED GALACTIC OR EXTRAGALACTIC RADIO PULSAR',
    3092 => 'EXTENDED GALACTIC OR EXTRAGALACTIC TYPE I',
    3093 => 'EXTENDED GALACTIC OR EXTRAGALACTIC TYPE II',
    3094 => 'EXTENDED GALACTIC OR EXTRAGALACTIC',
    3095 => 'EXTENDED GALACTIC OR EXTRAGALACTIC',
    3096 => 'EXTENDED GALACTIC OR EXTRAGALACTIC',
    3097 => 'EXTENDED GALACTIC OR EXTRAGALACTIC',
    3098 => 'EXTENDED GALACTIC OR EXTRAGALACTIC',
    3099 => 'EXTENDED GALACTIC OR EXTRAGALACTIC',
    3100 => 'SNR CRAB-LIKE',
    3101 => 'SNR CRAB-LIKE RADIO PULSAR',
    3102 => 'SNR CRAB-LIKE TYPE I',
    3103 => 'SNR CRAB-LIKE TYPE II',
    3104 => 'SNR CRAB-LIKE',
    3105 => 'SNR CRAB-LIKE',
    3106 => 'SNR CRAB-LIKE',
    3107 => 'SNR CRAB-LIKE',
    3108 => 'SNR CRAB-LIKE',
    3109 => 'SNR CRAB-LIKE',
    3110 => 'SNR CRAB-LIKE SHELL',
    3111 => 'SNR CRAB-LIKE SHELL RADIO PULSAR',
    3112 => 'SNR CRAB-LIKE SHELL TYPE I',
    3113 => 'SNR CRAB-LIKE SHELL TYPE II',
    3114 => 'SNR CRAB-LIKE SHELL',
    3115 => 'SNR CRAB-LIKE SHELL',
    3116 => 'SNR CRAB-LIKE SHELL',
    3117 => 'SNR CRAB-LIKE SHELL',
    3118 => 'SNR CRAB-LIKE SHELL',
    3119 => 'SNR CRAB-LIKE SHELL',
    3120 => 'SNR CRAB-LIKE FILLED-CENTER',
    3121 => 'SNR CRAB-LIKE FILLED-CENTER RADIO PULSAR',
    3122 => 'SNR CRAB-LIKE FILLED-CENTER TYPE I',
    3123 => 'SNR CRAB-LIKE FILLED-CENTER TYPE II',
    3124 => 'SNR CRAB-LIKE FILLED-CENTER',
    3125 => 'SNR CRAB-LIKE FILLED-CENTER',
    3126 => 'SNR CRAB-LIKE FILLED-CENTER',
    3127 => 'SNR CRAB-LIKE FILLED-CENTER',
    3128 => 'SNR CRAB-LIKE FILLED-CENTER',
    3129 => 'SNR CRAB-LIKE FILLED-CENTER',
    3130 => 'SNR CRAB-LIKE COMPOSITE',
    3131 => 'SNR CRAB-LIKE COMPOSITE RADIO PULSAR',
    3132 => 'SNR CRAB-LIKE COMPOSITE TYPE I',
    3133 => 'SNR CRAB-LIKE COMPOSITE TYPE II',
    3134 => 'SNR CRAB-LIKE COMPOSITE',
    3135 => 'SNR CRAB-LIKE COMPOSITE',
    3136 => 'SNR CRAB-LIKE COMPOSITE',
    3137 => 'SNR CRAB-LIKE COMPOSITE',
    3138 => 'SNR CRAB-LIKE COMPOSITE',
    3139 => 'SNR CRAB-LIKE COMPOSITE',
    3140 => 'DARK SNR CRAB-LIKE',
    3141 => 'DARK SNR CRAB-LIKE RADIO PULSAR',
    3142 => 'DARK SNR CRAB-LIKE TYPE I',
    3143 => 'DARK SNR CRAB-LIKE TYPE II',
    3144 => 'DARK SNR CRAB-LIKE',
    3145 => 'DARK SNR CRAB-LIKE',
    3146 => 'DARK SNR CRAB-LIKE',
    3147 => 'DARK SNR CRAB-LIKE',
    3148 => 'DARK SNR CRAB-LIKE',
    3149 => 'DARK SNR CRAB-LIKE',
    3150 => 'MOLECULAR SNR CRAB-LIKE',
    3151 => 'MOLECULAR SNR CRAB-LIKE RADIO PULSAR',
    3152 => 'MOLECULAR SNR CRAB-LIKE TYPE I',
    3153 => 'MOLECULAR SNR CRAB-LIKE TYPE II',
    3154 => 'MOLECULAR SNR CRAB-LIKE',
    3155 => 'MOLECULAR SNR CRAB-LIKE',
    3156 => 'MOLECULAR SNR CRAB-LIKE',
    3157 => 'MOLECULAR SNR CRAB-LIKE',
    3158 => 'MOLECULAR SNR CRAB-LIKE',
    3159 => 'MOLECULAR SNR CRAB-LIKE',
    3160 => 'PLANETARY SNR CRAB-LIKE',
    3161 => 'PLANETARY SNR CRAB-LIKE RADIO PULSAR',
    3162 => 'PLANETARY SNR CRAB-LIKE TYPE I',
    3163 => 'PLANETARY SNR CRAB-LIKE TYPE II',
    3164 => 'PLANETARY SNR CRAB-LIKE',
    3165 => 'PLANETARY SNR CRAB-LIKE',
    3166 => 'PLANETARY SNR CRAB-LIKE',
    3167 => 'PLANETARY SNR CRAB-LIKE',
    3168 => 'PLANETARY SNR CRAB-LIKE',
    3169 => 'PLANETARY SNR CRAB-LIKE',
    3170 => 'REFLECTION SNR CRAB-LIKE',
    3171 => 'REFLECTION SNR CRAB-LIKE RADIO PULSAR',
    3172 => 'REFLECTION SNR CRAB-LIKE TYPE I',
    3173 => 'REFLECTION SNR CRAB-LIKE TYPE II',
    3174 => 'REFLECTION SNR CRAB-LIKE',
    3175 => 'REFLECTION SNR CRAB-LIKE',
    3176 => 'REFLECTION SNR CRAB-LIKE',
    3177 => 'REFLECTION SNR CRAB-LIKE',
    3178 => 'REFLECTION SNR CRAB-LIKE',
    3179 => 'REFLECTION SNR CRAB-LIKE',
    3180 => 'GLOBULAR CLUSTER SNR CRAB-LIKE',
    3181 => 'GLOBULAR CLUSTER SNR CRAB-LIKE RADIO PULSAR',
    3182 => 'GLOBULAR CLUSTER SNR CRAB-LIKE TYPE I',
    3183 => 'GLOBULAR CLUSTER SNR CRAB-LIKE TYPE II',
    3184 => 'GLOBULAR CLUSTER SNR CRAB-LIKE',
    3185 => 'GLOBULAR CLUSTER SNR CRAB-LIKE',
    3186 => 'GLOBULAR CLUSTER SNR CRAB-LIKE',
    3187 => 'GLOBULAR CLUSTER SNR CRAB-LIKE',
    3188 => 'GLOBULAR CLUSTER SNR CRAB-LIKE',
    3189 => 'GLOBULAR CLUSTER SNR CRAB-LIKE',
    3190 => 'SNR CRAB-LIKE',
    3191 => 'SNR CRAB-LIKE RADIO PULSAR',
    3192 => 'SNR CRAB-LIKE TYPE I',
    3193 => 'SNR CRAB-LIKE TYPE II',
    3194 => 'SNR CRAB-LIKE',
    3195 => 'SNR CRAB-LIKE',
    3196 => 'SNR CRAB-LIKE',
    3197 => 'SNR CRAB-LIKE',
    3198 => 'SNR CRAB-LIKE',
    3199 => 'SNR CRAB-LIKE',
    3200 => 'SNR',
    3201 => 'SNR RADIO PULSAR',
    3202 => 'SNR TYPE I',
    3203 => 'SNR TYPE II',
    3204 => 'SNR',
    3205 => 'SNR',
    3206 => 'SNR',
    3207 => 'SNR',
    3208 => 'SNR',
    3209 => 'SNR',
    3210 => 'SNR SHELL',
    3211 => 'SNR SHELL RADIO PULSAR',
    3212 => 'SNR SHELL TYPE I',
    3213 => 'SNR SHELL TYPE II',
    3214 => 'SNR SHELL',
    3215 => 'SNR SHELL',
    3216 => 'SNR SHELL',
    3217 => 'SNR SHELL',
    3218 => 'SNR SHELL',
    3219 => 'SNR SHELL',
    3220 => 'SNR FILLED-CENTER',
    3221 => 'SNR FILLED-CENTER RADIO PULSAR',
    3222 => 'SNR FILLED-CENTER TYPE I',
    3223 => 'SNR FILLED-CENTER TYPE II',
    3224 => 'SNR FILLED-CENTER',
    3225 => 'SNR FILLED-CENTER',
    3226 => 'SNR FILLED-CENTER',
    3227 => 'SNR FILLED-CENTER',
    3228 => 'SNR FILLED-CENTER',
    3229 => 'SNR FILLED-CENTER',
    3230 => 'SNR COMPOSITE',
    3231 => 'SNR COMPOSITE RADIO PULSAR',
    3232 => 'SNR COMPOSITE TYPE I',
    3233 => 'SNR COMPOSITE TYPE II',
    3234 => 'SNR COMPOSITE',
    3235 => 'SNR COMPOSITE',
    3236 => 'SNR COMPOSITE',
    3237 => 'SNR COMPOSITE',
    3238 => 'SNR COMPOSITE',
    3239 => 'SNR COMPOSITE',
    3240 => 'DARK SNR',
    3241 => 'DARK SNR RADIO PULSAR',
    3242 => 'DARK SNR TYPE I',
    3243 => 'DARK SNR TYPE II',
    3244 => 'DARK SNR',
    3245 => 'DARK SNR',
    3246 => 'DARK SNR',
    3247 => 'DARK SNR',
    3248 => 'DARK SNR',
    3249 => 'DARK SNR',
    3250 => 'MOLECULAR SNR',
    3251 => 'MOLECULAR SNR RADIO PULSAR',
    3252 => 'MOLECULAR SNR TYPE I',
    3253 => 'MOLECULAR SNR TYPE II',
    3254 => 'MOLECULAR SNR',
    3255 => 'MOLECULAR SNR',
    3256 => 'MOLECULAR SNR',
    3257 => 'MOLECULAR SNR',
    3258 => 'MOLECULAR SNR',
    3259 => 'MOLECULAR SNR',
    3260 => 'PLANETARY SNR',
    3261 => 'PLANETARY SNR RADIO PULSAR',
    3262 => 'PLANETARY SNR TYPE I',
    3263 => 'PLANETARY SNR TYPE II',
    3264 => 'PLANETARY SNR',
    3265 => 'PLANETARY SNR',
    3266 => 'PLANETARY SNR',
    3267 => 'PLANETARY SNR',
    3268 => 'PLANETARY SNR',
    3269 => 'PLANETARY SNR',
    3270 => 'REFLECTION SNR',
    3271 => 'REFLECTION SNR RADIO PULSAR',
    3272 => 'REFLECTION SNR TYPE I',
    3273 => 'REFLECTION SNR TYPE II',
    3274 => 'REFLECTION SNR',
    3275 => 'REFLECTION SNR',
    3276 => 'REFLECTION SNR',
    3277 => 'REFLECTION SNR',
    3278 => 'REFLECTION SNR',
    3279 => 'REFLECTION SNR',
    3280 => 'GLOBULAR CLUSTER SNR',
    3281 => 'GLOBULAR CLUSTER SNR RADIO PULSAR',
    3282 => 'GLOBULAR CLUSTER SNR TYPE I',
    3283 => 'GLOBULAR CLUSTER SNR TYPE II',
    3284 => 'GLOBULAR CLUSTER SNR',
    3285 => 'GLOBULAR CLUSTER SNR',
    3286 => 'GLOBULAR CLUSTER SNR',
    3287 => 'GLOBULAR CLUSTER SNR',
    3288 => 'GLOBULAR CLUSTER SNR',
    3289 => 'GLOBULAR CLUSTER SNR',
    3290 => 'SNR',
    3291 => 'SNR RADIO PULSAR',
    3292 => 'SNR TYPE I',
    3293 => 'SNR TYPE II',
    3294 => 'SNR',
    3295 => 'SNR',
    3296 => 'SNR',
    3297 => 'SNR',
    3298 => 'SNR',
    3299 => 'SNR',
    3300 => 'STAR FORMING REGION',
    3301 => 'STAR FORMING REGION RADIO PULSAR',
    3302 => 'STAR FORMING REGION TYPE I',
    3303 => 'STAR FORMING REGION TYPE II',
    3304 => 'STAR FORMING REGION',
    3305 => 'STAR FORMING REGION',
    3306 => 'STAR FORMING REGION',
    3307 => 'STAR FORMING REGION',
    3308 => 'STAR FORMING REGION',
    3309 => 'STAR FORMING REGION',
    3310 => 'STAR FORMING REGION SHELL',
    3311 => 'STAR FORMING REGION SHELL RADIO PULSAR',
    3312 => 'STAR FORMING REGION SHELL TYPE I',
    3313 => 'STAR FORMING REGION SHELL TYPE II',
    3314 => 'STAR FORMING REGION SHELL',
    3315 => 'STAR FORMING REGION SHELL',
    3316 => 'STAR FORMING REGION SHELL',
    3317 => 'STAR FORMING REGION SHELL',
    3318 => 'STAR FORMING REGION SHELL',
    3319 => 'STAR FORMING REGION SHELL',
    3320 => 'STAR FORMING REGION FILLED-CENTER',
    3321 => 'STAR FORMING REGION FILLED-CENTER RADIO PULSAR',
    3322 => 'STAR FORMING REGION FILLED-CENTER TYPE I',
    3323 => 'STAR FORMING REGION FILLED-CENTER TYPE II',
    3324 => 'STAR FORMING REGION FILLED-CENTER',
    3325 => 'STAR FORMING REGION FILLED-CENTER',
    3326 => 'STAR FORMING REGION FILLED-CENTER',
    3327 => 'STAR FORMING REGION FILLED-CENTER',
    3328 => 'STAR FORMING REGION FILLED-CENTER',
    3329 => 'STAR FORMING REGION FILLED-CENTER',
    3330 => 'STAR FORMING REGION COMPOSITE',
    3331 => 'STAR FORMING REGION COMPOSITE RADIO PULSAR',
    3332 => 'STAR FORMING REGION COMPOSITE TYPE I',
    3333 => 'STAR FORMING REGION COMPOSITE TYPE II',
    3334 => 'STAR FORMING REGION COMPOSITE',
    3335 => 'STAR FORMING REGION COMPOSITE',
    3336 => 'STAR FORMING REGION COMPOSITE',
    3337 => 'STAR FORMING REGION COMPOSITE',
    3338 => 'STAR FORMING REGION COMPOSITE',
    3339 => 'STAR FORMING REGION COMPOSITE',
    3340 => 'DARK STAR FORMING REGION',
    3341 => 'DARK STAR FORMING REGION RADIO PULSAR',
    3342 => 'DARK STAR FORMING REGION TYPE I',
    3343 => 'DARK STAR FORMING REGION TYPE II',
    3344 => 'DARK STAR FORMING REGION',
    3345 => 'DARK STAR FORMING REGION',
    3346 => 'DARK STAR FORMING REGION',
    3347 => 'DARK STAR FORMING REGION',
    3348 => 'DARK STAR FORMING REGION',
    3349 => 'DARK STAR FORMING REGION',
    3350 => 'MOLECULAR STAR FORMING REGION',
    3351 => 'MOLECULAR STAR FORMING REGION RADIO PULSAR',
    3352 => 'MOLECULAR STAR FORMING REGION TYPE I',
    3353 => 'MOLECULAR STAR FORMING REGION TYPE II',
    3354 => 'MOLECULAR STAR FORMING REGION',
    3355 => 'MOLECULAR STAR FORMING REGION',
    3356 => 'MOLECULAR STAR FORMING REGION',
    3357 => 'MOLECULAR STAR FORMING REGION',
    3358 => 'MOLECULAR STAR FORMING REGION',
    3359 => 'MOLECULAR STAR FORMING REGION',
    3360 => 'PLANETARY STAR FORMING REGION',
    3361 => 'PLANETARY STAR FORMING REGION RADIO PULSAR',
    3362 => 'PLANETARY STAR FORMING REGION TYPE I',
    3363 => 'PLANETARY STAR FORMING REGION TYPE II',
    3364 => 'PLANETARY STAR FORMING REGION',
    3365 => 'PLANETARY STAR FORMING REGION',
    3366 => 'PLANETARY STAR FORMING REGION',
    3367 => 'PLANETARY STAR FORMING REGION',
    3368 => 'PLANETARY STAR FORMING REGION',
    3369 => 'PLANETARY STAR FORMING REGION',
    3370 => 'REFLECTION STAR FORMING REGION',
    3371 => 'REFLECTION STAR FORMING REGION RADIO PULSAR',
    3372 => 'REFLECTION STAR FORMING REGION TYPE I',
    3373 => 'REFLECTION STAR FORMING REGION TYPE II',
    3374 => 'REFLECTION STAR FORMING REGION',
    3375 => 'REFLECTION STAR FORMING REGION',
    3376 => 'REFLECTION STAR FORMING REGION',
    3377 => 'REFLECTION STAR FORMING REGION',
    3378 => 'REFLECTION STAR FORMING REGION',
    3379 => 'REFLECTION STAR FORMING REGION',
    3380 => 'GLOBULAR CLUSTER STAR FORMING REGION',
    3381 => 'GLOBULAR CLUSTER STAR FORMING REGION RADIO PULSAR',
    3382 => 'GLOBULAR CLUSTER STAR FORMING REGION TYPE I',
    3383 => 'GLOBULAR CLUSTER STAR FORMING REGION TYPE II',
    3384 => 'GLOBULAR CLUSTER STAR FORMING REGION',
    3385 => 'GLOBULAR CLUSTER STAR FORMING REGION',
    3386 => 'GLOBULAR CLUSTER STAR FORMING REGION',
    3387 => 'GLOBULAR CLUSTER STAR FORMING REGION',
    3388 => 'GLOBULAR CLUSTER STAR FORMING REGION',
    3389 => 'GLOBULAR CLUSTER STAR FORMING REGION',
    3390 => 'STAR FORMING REGION',
    3391 => 'STAR FORMING REGION RADIO PULSAR',
    3392 => 'STAR FORMING REGION TYPE I',
    3393 => 'STAR FORMING REGION TYPE II',
    3394 => 'STAR FORMING REGION',
    3395 => 'STAR FORMING REGION',
    3396 => 'STAR FORMING REGION',
    3397 => 'STAR FORMING REGION',
    3398 => 'STAR FORMING REGION',
    3399 => 'STAR FORMING REGION',
    3400 => 'CLOUD',
    3401 => 'CLOUD RADIO PULSAR',
    3402 => 'CLOUD TYPE I',
    3403 => 'CLOUD TYPE II',
    3404 => 'CLOUD',
    3405 => 'CLOUD',
    3406 => 'CLOUD',
    3407 => 'CLOUD',
    3408 => 'CLOUD',
    3409 => 'CLOUD',
    3410 => 'CLOUD SHELL',
    3411 => 'CLOUD SHELL RADIO PULSAR',
    3412 => 'CLOUD SHELL TYPE I',
    3413 => 'CLOUD SHELL TYPE II',
    3414 => 'CLOUD SHELL',
    3415 => 'CLOUD SHELL',
    3416 => 'CLOUD SHELL',
    3417 => 'CLOUD SHELL',
    3418 => 'CLOUD SHELL',
    3419 => 'CLOUD SHELL',
    3420 => 'CLOUD FILLED-CENTER',
    3421 => 'CLOUD FILLED-CENTER RADIO PULSAR',
    3422 => 'CLOUD FILLED-CENTER TYPE I',
    3423 => 'CLOUD FILLED-CENTER TYPE II',
    3424 => 'CLOUD FILLED-CENTER',
    3425 => 'CLOUD FILLED-CENTER',
    3426 => 'CLOUD FILLED-CENTER',
    3427 => 'CLOUD FILLED-CENTER',
    3428 => 'CLOUD FILLED-CENTER',
    3429 => 'CLOUD FILLED-CENTER',
    3430 => 'CLOUD COMPOSITE',
    3431 => 'CLOUD COMPOSITE RADIO PULSAR',
    3432 => 'CLOUD COMPOSITE TYPE I',
    3433 => 'CLOUD COMPOSITE TYPE II',
    3434 => 'CLOUD COMPOSITE',
    3435 => 'CLOUD COMPOSITE',
    3436 => 'CLOUD COMPOSITE',
    3437 => 'CLOUD COMPOSITE',
    3438 => 'CLOUD COMPOSITE',
    3439 => 'CLOUD COMPOSITE',
    3440 => 'DARK CLOUD',
    3441 => 'DARK CLOUD RADIO PULSAR',
    3442 => 'DARK CLOUD TYPE I',
    3443 => 'DARK CLOUD TYPE II',
    3444 => 'DARK CLOUD',
    3445 => 'DARK CLOUD',
    3446 => 'DARK CLOUD',
    3447 => 'DARK CLOUD',
    3448 => 'DARK CLOUD',
    3449 => 'DARK CLOUD',
    3450 => 'MOLECULAR CLOUD',
    3451 => 'MOLECULAR CLOUD RADIO PULSAR',
    3452 => 'MOLECULAR CLOUD TYPE I',
    3453 => 'MOLECULAR CLOUD TYPE II',
    3454 => 'MOLECULAR CLOUD',
    3455 => 'MOLECULAR CLOUD',
    3456 => 'MOLECULAR CLOUD',
    3457 => 'MOLECULAR CLOUD',
    3458 => 'MOLECULAR CLOUD',
    3459 => 'MOLECULAR CLOUD',
    3460 => 'PLANETARY CLOUD',
    3461 => 'PLANETARY CLOUD RADIO PULSAR',
    3462 => 'PLANETARY CLOUD TYPE I',
    3463 => 'PLANETARY CLOUD TYPE II',
    3464 => 'PLANETARY CLOUD',
    3465 => 'PLANETARY CLOUD',
    3466 => 'PLANETARY CLOUD',
    3467 => 'PLANETARY CLOUD',
    3468 => 'PLANETARY CLOUD',
    3469 => 'PLANETARY CLOUD',
    3470 => 'REFLECTION CLOUD',
    3471 => 'REFLECTION CLOUD RADIO PULSAR',
    3472 => 'REFLECTION CLOUD TYPE I',
    3473 => 'REFLECTION CLOUD TYPE II',
    3474 => 'REFLECTION CLOUD',
    3475 => 'REFLECTION CLOUD',
    3476 => 'REFLECTION CLOUD',
    3477 => 'REFLECTION CLOUD',
    3478 => 'REFLECTION CLOUD',
    3479 => 'REFLECTION CLOUD',
    3480 => 'GLOBULAR CLUSTER CLOUD',
    3481 => 'GLOBULAR CLUSTER CLOUD RADIO PULSAR',
    3482 => 'GLOBULAR CLUSTER CLOUD TYPE I',
    3483 => 'GLOBULAR CLUSTER CLOUD TYPE II',
    3484 => 'GLOBULAR CLUSTER CLOUD',
    3485 => 'GLOBULAR CLUSTER CLOUD',
    3486 => 'GLOBULAR CLUSTER CLOUD',
    3487 => 'GLOBULAR CLUSTER CLOUD',
    3488 => 'GLOBULAR CLUSTER CLOUD',
    3489 => 'GLOBULAR CLUSTER CLOUD',
    3490 => 'CLOUD',
    3491 => 'CLOUD RADIO PULSAR',
    3492 => 'CLOUD TYPE I',
    3493 => 'CLOUD TYPE II',
    3494 => 'CLOUD',
    3495 => 'CLOUD',
    3496 => 'CLOUD',
    3497 => 'CLOUD',
    3498 => 'CLOUD',
    3499 => 'CLOUD',
    3500 => 'NEBULA',
    3501 => 'NEBULA RADIO PULSAR',
    3502 => 'NEBULA TYPE I',
    3503 => 'NEBULA TYPE II',
    3504 => 'NEBULA',
    3505 => 'NEBULA',
    3506 => 'NEBULA',
    3507 => 'NEBULA',
    3508 => 'NEBULA',
    3509 => 'NEBULA',
    3510 => 'NEBULA SHELL',
    3511 => 'NEBULA SHELL RADIO PULSAR',
    3512 => 'NEBULA SHELL TYPE I',
    3513 => 'NEBULA SHELL TYPE II',
    3514 => 'NEBULA SHELL',
    3515 => 'NEBULA SHELL',
    3516 => 'NEBULA SHELL',
    3517 => 'NEBULA SHELL',
    3518 => 'NEBULA SHELL',
    3519 => 'NEBULA SHELL',
    3520 => 'NEBULA FILLED-CENTER',
    3521 => 'NEBULA FILLED-CENTER RADIO PULSAR',
    3522 => 'NEBULA FILLED-CENTER TYPE I',
    3523 => 'NEBULA FILLED-CENTER TYPE II',
    3524 => 'NEBULA FILLED-CENTER',
    3525 => 'NEBULA FILLED-CENTER',
    3526 => 'NEBULA FILLED-CENTER',
    3527 => 'NEBULA FILLED-CENTER',
    3528 => 'NEBULA FILLED-CENTER',
    3529 => 'NEBULA FILLED-CENTER',
    3530 => 'NEBULA COMPOSITE',
    3531 => 'NEBULA COMPOSITE RADIO PULSAR',
    3532 => 'NEBULA COMPOSITE TYPE I',
    3533 => 'NEBULA COMPOSITE TYPE II',
    3534 => 'NEBULA COMPOSITE',
    3535 => 'NEBULA COMPOSITE',
    3536 => 'NEBULA COMPOSITE',
    3537 => 'NEBULA COMPOSITE',
    3538 => 'NEBULA COMPOSITE',
    3539 => 'NEBULA COMPOSITE',
    3540 => 'DARK NEBULA',
    3541 => 'DARK NEBULA RADIO PULSAR',
    3542 => 'DARK NEBULA TYPE I',
    3543 => 'DARK NEBULA TYPE II',
    3544 => 'DARK NEBULA',
    3545 => 'DARK NEBULA',
    3546 => 'DARK NEBULA',
    3547 => 'DARK NEBULA',
    3548 => 'DARK NEBULA',
    3549 => 'DARK NEBULA',
    3550 => 'MOLECULAR NEBULA',
    3551 => 'MOLECULAR NEBULA RADIO PULSAR',
    3552 => 'MOLECULAR NEBULA TYPE I',
    3553 => 'MOLECULAR NEBULA TYPE II',
    3554 => 'MOLECULAR NEBULA',
    3555 => 'MOLECULAR NEBULA',
    3556 => 'MOLECULAR NEBULA',
    3557 => 'MOLECULAR NEBULA',
    3558 => 'MOLECULAR NEBULA',
    3559 => 'MOLECULAR NEBULA',
    3560 => 'PLANETARY NEBULA',
    3561 => 'PLANETARY NEBULA RADIO PULSAR',
    3562 => 'PLANETARY NEBULA TYPE I',
    3563 => 'PLANETARY NEBULA TYPE II',
    3564 => 'PLANETARY NEBULA',
    3565 => 'PLANETARY NEBULA',
    3566 => 'PLANETARY NEBULA',
    3567 => 'PLANETARY NEBULA',
    3568 => 'PLANETARY NEBULA',
    3569 => 'PLANETARY NEBULA',
    3570 => 'REFLECTION NEBULA',
    3571 => 'REFLECTION NEBULA RADIO PULSAR',
    3572 => 'REFLECTION NEBULA TYPE I',
    3573 => 'REFLECTION NEBULA TYPE II',
    3574 => 'REFLECTION NEBULA',
    3575 => 'REFLECTION NEBULA',
    3576 => 'REFLECTION NEBULA',
    3577 => 'REFLECTION NEBULA',
    3578 => 'REFLECTION NEBULA',
    3579 => 'REFLECTION NEBULA',
    3580 => 'GLOBULAR CLUSTER NEBULA',
    3581 => 'GLOBULAR CLUSTER NEBULA RADIO PULSAR',
    3582 => 'GLOBULAR CLUSTER NEBULA TYPE I',
    3583 => 'GLOBULAR CLUSTER NEBULA TYPE II',
    3584 => 'GLOBULAR CLUSTER NEBULA',
    3585 => 'GLOBULAR CLUSTER NEBULA',
    3586 => 'GLOBULAR CLUSTER NEBULA',
    3587 => 'GLOBULAR CLUSTER NEBULA',
    3588 => 'GLOBULAR CLUSTER NEBULA',
    3589 => 'GLOBULAR CLUSTER NEBULA',
    3590 => 'NEBULA',
    3591 => 'NEBULA RADIO PULSAR',
    3592 => 'NEBULA TYPE I',
    3593 => 'NEBULA TYPE II',
    3594 => 'NEBULA',
    3595 => 'NEBULA',
    3596 => 'NEBULA',
    3597 => 'NEBULA',
    3598 => 'NEBULA',
    3599 => 'NEBULA',
    3600 => 'OPEN STAR CLUSTER',
    3601 => 'OPEN STAR CLUSTER RADIO PULSAR',
    3602 => 'OPEN STAR CLUSTER TYPE I',
    3603 => 'OPEN STAR CLUSTER TYPE II',
    3604 => 'OPEN STAR CLUSTER',
    3605 => 'OPEN STAR CLUSTER',
    3606 => 'OPEN STAR CLUSTER',
    3607 => 'OPEN STAR CLUSTER',
    3608 => 'OPEN STAR CLUSTER',
    3609 => 'OPEN STAR CLUSTER',
    3610 => 'OPEN STAR CLUSTER SHELL',
    3611 => 'OPEN STAR CLUSTER SHELL RADIO PULSAR',
    3612 => 'OPEN STAR CLUSTER SHELL TYPE I',
    3613 => 'OPEN STAR CLUSTER SHELL TYPE II',
    3614 => 'OPEN STAR CLUSTER SHELL',
    3615 => 'OPEN STAR CLUSTER SHELL',
    3616 => 'OPEN STAR CLUSTER SHELL',
    3617 => 'OPEN STAR CLUSTER SHELL',
    3618 => 'OPEN STAR CLUSTER SHELL',
    3619 => 'OPEN STAR CLUSTER SHELL',
    3620 => 'OPEN STAR CLUSTER FILLED-CENTER',
    3621 => 'OPEN STAR CLUSTER FILLED-CENTER RADIO PULSAR',
    3622 => 'OPEN STAR CLUSTER FILLED-CENTER TYPE I',
    3623 => 'OPEN STAR CLUSTER FILLED-CENTER TYPE II',
    3624 => 'OPEN STAR CLUSTER FILLED-CENTER',
    3625 => 'OPEN STAR CLUSTER FILLED-CENTER',
    3626 => 'OPEN STAR CLUSTER FILLED-CENTER',
    3627 => 'OPEN STAR CLUSTER FILLED-CENTER',
    3628 => 'OPEN STAR CLUSTER FILLED-CENTER',
    3629 => 'OPEN STAR CLUSTER FILLED-CENTER',
    3630 => 'OPEN STAR CLUSTER COMPOSITE',
    3631 => 'OPEN STAR CLUSTER COMPOSITE RADIO PULSAR',
    3632 => 'OPEN STAR CLUSTER COMPOSITE TYPE I',
    3633 => 'OPEN STAR CLUSTER COMPOSITE TYPE II',
    3634 => 'OPEN STAR CLUSTER COMPOSITE',
    3635 => 'OPEN STAR CLUSTER COMPOSITE',
    3636 => 'OPEN STAR CLUSTER COMPOSITE',
    3637 => 'OPEN STAR CLUSTER COMPOSITE',
    3638 => 'OPEN STAR CLUSTER COMPOSITE',
    3639 => 'OPEN STAR CLUSTER COMPOSITE',
    3640 => 'DARK OPEN STAR CLUSTER',
    3641 => 'DARK OPEN STAR CLUSTER RADIO PULSAR',
    3642 => 'DARK OPEN STAR CLUSTER TYPE I',
    3643 => 'DARK OPEN STAR CLUSTER TYPE II',
    3644 => 'DARK OPEN STAR CLUSTER',
    3645 => 'DARK OPEN STAR CLUSTER',
    3646 => 'DARK OPEN STAR CLUSTER',
    3647 => 'DARK OPEN STAR CLUSTER',
    3648 => 'DARK OPEN STAR CLUSTER',
    3649 => 'DARK OPEN STAR CLUSTER',
    3650 => 'MOLECULAR OPEN STAR CLUSTER',
    3651 => 'MOLECULAR OPEN STAR CLUSTER RADIO PULSAR',
    3652 => 'MOLECULAR OPEN STAR CLUSTER TYPE I',
    3653 => 'MOLECULAR OPEN STAR CLUSTER TYPE II',
    3654 => 'MOLECULAR OPEN STAR CLUSTER',
    3655 => 'MOLECULAR OPEN STAR CLUSTER',
    3656 => 'MOLECULAR OPEN STAR CLUSTER',
    3657 => 'MOLECULAR OPEN STAR CLUSTER',
    3658 => 'MOLECULAR OPEN STAR CLUSTER',
    3659 => 'MOLECULAR OPEN STAR CLUSTER',
    3660 => 'PLANETARY OPEN STAR CLUSTER',
    3661 => 'PLANETARY OPEN STAR CLUSTER RADIO PULSAR',
    3662 => 'PLANETARY OPEN STAR CLUSTER TYPE I',
    3663 => 'PLANETARY OPEN STAR CLUSTER TYPE II',
    3664 => 'PLANETARY OPEN STAR CLUSTER',
    3665 => 'PLANETARY OPEN STAR CLUSTER',
    3666 => 'PLANETARY OPEN STAR CLUSTER',
    3667 => 'PLANETARY OPEN STAR CLUSTER',
    3668 => 'PLANETARY OPEN STAR CLUSTER',
    3669 => 'PLANETARY OPEN STAR CLUSTER',
    3670 => 'REFLECTION OPEN STAR CLUSTER',
    3671 => 'REFLECTION OPEN STAR CLUSTER RADIO PULSAR',
    3672 => 'REFLECTION OPEN STAR CLUSTER TYPE I',
    3673 => 'REFLECTION OPEN STAR CLUSTER TYPE II',
    3674 => 'REFLECTION OPEN STAR CLUSTER',
    3675 => 'REFLECTION OPEN STAR CLUSTER',
    3676 => 'REFLECTION OPEN STAR CLUSTER',
    3677 => 'REFLECTION OPEN STAR CLUSTER',
    3678 => 'REFLECTION OPEN STAR CLUSTER',
    3679 => 'REFLECTION OPEN STAR CLUSTER',
    3680 => 'GLOBULAR CLUSTER OPEN STAR CLUSTER',
    3681 => 'GLOBULAR CLUSTER OPEN STAR CLUSTER RADIO PULSAR',
    3682 => 'GLOBULAR CLUSTER OPEN STAR CLUSTER TYPE I',
    3683 => 'GLOBULAR CLUSTER OPEN STAR CLUSTER TYPE II',
    3684 => 'GLOBULAR CLUSTER OPEN STAR CLUSTER',
    3685 => 'GLOBULAR CLUSTER OPEN STAR CLUSTER',
    3686 => 'GLOBULAR CLUSTER OPEN STAR CLUSTER',
    3687 => 'GLOBULAR CLUSTER OPEN STAR CLUSTER',
    3688 => 'GLOBULAR CLUSTER OPEN STAR CLUSTER',
    3689 => 'GLOBULAR CLUSTER OPEN STAR CLUSTER',
    3690 => 'OPEN STAR CLUSTER',
    3691 => 'OPEN STAR CLUSTER RADIO PULSAR',
    3692 => 'OPEN STAR CLUSTER TYPE I',
    3693 => 'OPEN STAR CLUSTER TYPE II',
    3694 => 'OPEN STAR CLUSTER',
    3695 => 'OPEN STAR CLUSTER',
    3696 => 'OPEN STAR CLUSTER',
    3697 => 'OPEN STAR CLUSTER',
    3698 => 'OPEN STAR CLUSTER',
    3699 => 'OPEN STAR CLUSTER',
    3700 => 'OB ASSOCIATION/HII REGION',
    3701 => 'OB ASSOCIATION/HII REGION RADIO PULSAR',
    3702 => 'OB ASSOCIATION/HII REGION TYPE I',
    3703 => 'OB ASSOCIATION/HII REGION TYPE II',
    3704 => 'OB ASSOCIATION/HII REGION',
    3705 => 'OB ASSOCIATION/HII REGION',
    3706 => 'OB ASSOCIATION/HII REGION',
    3707 => 'OB ASSOCIATION/HII REGION',
    3708 => 'OB ASSOCIATION/HII REGION',
    3709 => 'OB ASSOCIATION/HII REGION',
    3710 => 'OB ASSOCIATION/HII REGION SHELL',
    3711 => 'OB ASSOCIATION/HII REGION SHELL RADIO PULSAR',
    3712 => 'OB ASSOCIATION/HII REGION SHELL TYPE I',
    3713 => 'OB ASSOCIATION/HII REGION SHELL TYPE II',
    3714 => 'OB ASSOCIATION/HII REGION SHELL',
    3715 => 'OB ASSOCIATION/HII REGION SHELL',
    3716 => 'OB ASSOCIATION/HII REGION SHELL',
    3717 => 'OB ASSOCIATION/HII REGION SHELL',
    3718 => 'OB ASSOCIATION/HII REGION SHELL',
    3719 => 'OB ASSOCIATION/HII REGION SHELL',
    3720 => 'OB ASSOCIATION/HII REGION FILLED-CENTER',
    3721 => 'OB ASSOCIATION/HII REGION FILLED-CENTER RADIO PULSAR',
    3722 => 'OB ASSOCIATION/HII REGION FILLED-CENTER TYPE I',
    3723 => 'OB ASSOCIATION/HII REGION FILLED-CENTER TYPE II',
    3724 => 'OB ASSOCIATION/HII REGION FILLED-CENTER',
    3725 => 'OB ASSOCIATION/HII REGION FILLED-CENTER',
    3726 => 'OB ASSOCIATION/HII REGION FILLED-CENTER',
    3727 => 'OB ASSOCIATION/HII REGION FILLED-CENTER',
    3728 => 'OB ASSOCIATION/HII REGION FILLED-CENTER',
    3729 => 'OB ASSOCIATION/HII REGION FILLED-CENTER',
    3730 => 'OB ASSOCIATION/HII REGION COMPOSITE',
    3731 => 'OB ASSOCIATION/HII REGION COMPOSITE RADIO PULSAR',
    3732 => 'OB ASSOCIATION/HII REGION COMPOSITE TYPE I',
    3733 => 'OB ASSOCIATION/HII REGION COMPOSITE TYPE II',
    3734 => 'OB ASSOCIATION/HII REGION COMPOSITE',
    3735 => 'OB ASSOCIATION/HII REGION COMPOSITE',
    3736 => 'OB ASSOCIATION/HII REGION COMPOSITE',
    3737 => 'OB ASSOCIATION/HII REGION COMPOSITE',
    3738 => 'OB ASSOCIATION/HII REGION COMPOSITE',
    3739 => 'OB ASSOCIATION/HII REGION COMPOSITE',
    3740 => 'DARK OB ASSOCIATION/HII REGION',
    3741 => 'DARK OB ASSOCIATION/HII REGION RADIO PULSAR',
    3742 => 'DARK OB ASSOCIATION/HII REGION TYPE I',
    3743 => 'DARK OB ASSOCIATION/HII REGION TYPE II',
    3744 => 'DARK OB ASSOCIATION/HII REGION',
    3745 => 'DARK OB ASSOCIATION/HII REGION',
    3746 => 'DARK OB ASSOCIATION/HII REGION',
    3747 => 'DARK OB ASSOCIATION/HII REGION',
    3748 => 'DARK OB ASSOCIATION/HII REGION',
    3749 => 'DARK OB ASSOCIATION/HII REGION',
    3750 => 'MOLECULAR OB ASSOCIATION/HII REGION',
    3751 => 'MOLECULAR OB ASSOCIATION/HII REGION RADIO PULSAR',
    3752 => 'MOLECULAR OB ASSOCIATION/HII REGION TYPE I',
    3753 => 'MOLECULAR OB ASSOCIATION/HII REGION TYPE II',
    3754 => 'MOLECULAR OB ASSOCIATION/HII REGION',
    3755 => 'MOLECULAR OB ASSOCIATION/HII REGION',
    3756 => 'MOLECULAR OB ASSOCIATION/HII REGION',
    3757 => 'MOLECULAR OB ASSOCIATION/HII REGION',
    3758 => 'MOLECULAR OB ASSOCIATION/HII REGION',
    3759 => 'MOLECULAR OB ASSOCIATION/HII REGION',
    3760 => 'PLANETARY OB ASSOCIATION/HII REGION',
    3761 => 'PLANETARY OB ASSOCIATION/HII REGION RADIO PULSAR',
    3762 => 'PLANETARY OB ASSOCIATION/HII REGION TYPE I',
    3763 => 'PLANETARY OB ASSOCIATION/HII REGION TYPE II',
    3764 => 'PLANETARY OB ASSOCIATION/HII REGION',
    3765 => 'PLANETARY OB ASSOCIATION/HII REGION',
    3766 => 'PLANETARY OB ASSOCIATION/HII REGION',
    3767 => 'PLANETARY OB ASSOCIATION/HII REGION',
    3768 => 'PLANETARY OB ASSOCIATION/HII REGION',
    3769 => 'PLANETARY OB ASSOCIATION/HII REGION',
    3770 => 'REFLECTION OB ASSOCIATION/HII REGION',
    3771 => 'REFLECTION OB ASSOCIATION/HII REGION RADIO PULSAR',
    3772 => 'REFLECTION OB ASSOCIATION/HII REGION TYPE I',
    3773 => 'REFLECTION OB ASSOCIATION/HII REGION TYPE II',
    3774 => 'REFLECTION OB ASSOCIATION/HII REGION',
    3775 => 'REFLECTION OB ASSOCIATION/HII REGION',
    3776 => 'REFLECTION OB ASSOCIATION/HII REGION',
    3777 => 'REFLECTION OB ASSOCIATION/HII REGION',
    3778 => 'REFLECTION OB ASSOCIATION/HII REGION',
    3779 => 'REFLECTION OB ASSOCIATION/HII REGION',
    3780 => 'GLOBULAR CLUSTER OB ASSOCIATION/HII REGION',
    3781 => 'GLOBULAR CLUSTER OB ASSOCIATION/HII REGION RADIO PULSAR',
    3782 => 'GLOBULAR CLUSTER OB ASSOCIATION/HII REGION TYPE I',
    3783 => 'GLOBULAR CLUSTER OB ASSOCIATION/HII REGION TYPE II',
    3784 => 'GLOBULAR CLUSTER OB ASSOCIATION/HII REGION',
    3785 => 'GLOBULAR CLUSTER OB ASSOCIATION/HII REGION',
    3786 => 'GLOBULAR CLUSTER OB ASSOCIATION/HII REGION',
    3787 => 'GLOBULAR CLUSTER OB ASSOCIATION/HII REGION',
    3788 => 'GLOBULAR CLUSTER OB ASSOCIATION/HII REGION',
    3789 => 'GLOBULAR CLUSTER OB ASSOCIATION/HII REGION',
    3790 => 'OB ASSOCIATION/HII REGION',
    3791 => 'OB ASSOCIATION/HII REGION RADIO PULSAR',
    3792 => 'OB ASSOCIATION/HII REGION TYPE I',
    3793 => 'OB ASSOCIATION/HII REGION TYPE II',
    3794 => 'OB ASSOCIATION/HII REGION',
    3795 => 'OB ASSOCIATION/HII REGION',
    3796 => 'OB ASSOCIATION/HII REGION',
    3797 => 'OB ASSOCIATION/HII REGION',
    3798 => 'OB ASSOCIATION/HII REGION',
    3799 => 'OB ASSOCIATION/HII REGION',
    3800 => 'HERBIG-HARO OBJECT',
    3801 => 'HERBIG-HARO OBJECT RADIO PULSAR',
    3802 => 'HERBIG-HARO OBJECT TYPE I',
    3803 => 'HERBIG-HARO OBJECT TYPE II',
    3804 => 'HERBIG-HARO OBJECT',
    3805 => 'HERBIG-HARO OBJECT',
    3806 => 'HERBIG-HARO OBJECT',
    3807 => 'HERBIG-HARO OBJECT',
    3808 => 'HERBIG-HARO OBJECT',
    3809 => 'HERBIG-HARO OBJECT',
    3810 => 'HERBIG-HARO OBJECT SHELL',
    3811 => 'HERBIG-HARO OBJECT SHELL RADIO PULSAR',
    3812 => 'HERBIG-HARO OBJECT SHELL TYPE I',
    3813 => 'HERBIG-HARO OBJECT SHELL TYPE II',
    3814 => 'HERBIG-HARO OBJECT SHELL',
    3815 => 'HERBIG-HARO OBJECT SHELL',
    3816 => 'HERBIG-HARO OBJECT SHELL',
    3817 => 'HERBIG-HARO OBJECT SHELL',
    3818 => 'HERBIG-HARO OBJECT SHELL',
    3819 => 'HERBIG-HARO OBJECT SHELL',
    3820 => 'HERBIG-HARO OBJECT FILLED-CENTER',
    3821 => 'HERBIG-HARO OBJECT FILLED-CENTER RADIO PULSAR',
    3822 => 'HERBIG-HARO OBJECT FILLED-CENTER TYPE I',
    3823 => 'HERBIG-HARO OBJECT FILLED-CENTER TYPE II',
    3824 => 'HERBIG-HARO OBJECT FILLED-CENTER',
    3825 => 'HERBIG-HARO OBJECT FILLED-CENTER',
    3826 => 'HERBIG-HARO OBJECT FILLED-CENTER',
    3827 => 'HERBIG-HARO OBJECT FILLED-CENTER',
    3828 => 'HERBIG-HARO OBJECT FILLED-CENTER',
    3829 => 'HERBIG-HARO OBJECT FILLED-CENTER',
    3830 => 'HERBIG-HARO OBJECT COMPOSITE',
    3831 => 'HERBIG-HARO OBJECT COMPOSITE RADIO PULSAR',
    3832 => 'HERBIG-HARO OBJECT COMPOSITE TYPE I',
    3833 => 'HERBIG-HARO OBJECT COMPOSITE TYPE II',
    3834 => 'HERBIG-HARO OBJECT COMPOSITE',
    3835 => 'HERBIG-HARO OBJECT COMPOSITE',
    3836 => 'HERBIG-HARO OBJECT COMPOSITE',
    3837 => 'HERBIG-HARO OBJECT COMPOSITE',
    3838 => 'HERBIG-HARO OBJECT COMPOSITE',
    3839 => 'HERBIG-HARO OBJECT COMPOSITE',
    3840 => 'DARK HERBIG-HARO OBJECT',
    3841 => 'DARK HERBIG-HARO OBJECT RADIO PULSAR',
    3842 => 'DARK HERBIG-HARO OBJECT TYPE I',
    3843 => 'DARK HERBIG-HARO OBJECT TYPE II',
    3844 => 'DARK HERBIG-HARO OBJECT',
    3845 => 'DARK HERBIG-HARO OBJECT',
    3846 => 'DARK HERBIG-HARO OBJECT',
    3847 => 'DARK HERBIG-HARO OBJECT',
    3848 => 'DARK HERBIG-HARO OBJECT',
    3849 => 'DARK HERBIG-HARO OBJECT',
    3850 => 'MOLECULAR HERBIG-HARO OBJECT',
    3851 => 'MOLECULAR HERBIG-HARO OBJECT RADIO PULSAR',
    3852 => 'MOLECULAR HERBIG-HARO OBJECT TYPE I',
    3853 => 'MOLECULAR HERBIG-HARO OBJECT TYPE II',
    3854 => 'MOLECULAR HERBIG-HARO OBJECT',
    3855 => 'MOLECULAR HERBIG-HARO OBJECT',
    3856 => 'MOLECULAR HERBIG-HARO OBJECT',
    3857 => 'MOLECULAR HERBIG-HARO OBJECT',
    3858 => 'MOLECULAR HERBIG-HARO OBJECT',
    3859 => 'MOLECULAR HERBIG-HARO OBJECT',
    3860 => 'PLANETARY HERBIG-HARO OBJECT',
    3861 => 'PLANETARY HERBIG-HARO OBJECT RADIO PULSAR',
    3862 => 'PLANETARY HERBIG-HARO OBJECT TYPE I',
    3863 => 'PLANETARY HERBIG-HARO OBJECT TYPE II',
    3864 => 'PLANETARY HERBIG-HARO OBJECT',
    3865 => 'PLANETARY HERBIG-HARO OBJECT',
    3866 => 'PLANETARY HERBIG-HARO OBJECT',
    3867 => 'PLANETARY HERBIG-HARO OBJECT',
    3868 => 'PLANETARY HERBIG-HARO OBJECT',
    3869 => 'PLANETARY HERBIG-HARO OBJECT',
    3870 => 'REFLECTION HERBIG-HARO OBJECT',
    3871 => 'REFLECTION HERBIG-HARO OBJECT RADIO PULSAR',
    3872 => 'REFLECTION HERBIG-HARO OBJECT TYPE I',
    3873 => 'REFLECTION HERBIG-HARO OBJECT TYPE II',
    3874 => 'REFLECTION HERBIG-HARO OBJECT',
    3875 => 'REFLECTION HERBIG-HARO OBJECT',
    3876 => 'REFLECTION HERBIG-HARO OBJECT',
    3877 => 'REFLECTION HERBIG-HARO OBJECT',
    3878 => 'REFLECTION HERBIG-HARO OBJECT',
    3879 => 'REFLECTION HERBIG-HARO OBJECT',
    3880 => 'GLOBULAR CLUSTER HERBIG-HARO OBJECT',
    3881 => 'GLOBULAR CLUSTER HERBIG-HARO OBJECT RADIO PULSAR',
    3882 => 'GLOBULAR CLUSTER HERBIG-HARO OBJECT TYPE I',
    3883 => 'GLOBULAR CLUSTER HERBIG-HARO OBJECT TYPE II',
    3884 => 'GLOBULAR CLUSTER HERBIG-HARO OBJECT',
    3885 => 'GLOBULAR CLUSTER HERBIG-HARO OBJECT',
    3886 => 'GLOBULAR CLUSTER HERBIG-HARO OBJECT',
    3887 => 'GLOBULAR CLUSTER HERBIG-HARO OBJECT',
    3888 => 'GLOBULAR CLUSTER HERBIG-HARO OBJECT',
    3889 => 'GLOBULAR CLUSTER HERBIG-HARO OBJECT',
    3890 => 'HERBIG-HARO OBJECT',
    3891 => 'HERBIG-HARO OBJECT RADIO PULSAR',
    3892 => 'HERBIG-HARO OBJECT TYPE I',
    3893 => 'HERBIG-HARO OBJECT TYPE II',
    3894 => 'HERBIG-HARO OBJECT',
    3895 => 'HERBIG-HARO OBJECT',
    3896 => 'HERBIG-HARO OBJECT',
    3897 => 'HERBIG-HARO OBJECT',
    3898 => 'HERBIG-HARO OBJECT',
    3899 => 'HERBIG-HARO OBJECT',
    3900 => 'DIFFUSE X-RAY EMISSION',
    3901 => 'DIFFUSE X-RAY EMISSION RADIO PULSAR',
    3902 => 'DIFFUSE X-RAY EMISSION TYPE I',
    3903 => 'DIFFUSE X-RAY EMISSION TYPE II',
    3904 => 'DIFFUSE X-RAY EMISSION',
    3905 => 'DIFFUSE X-RAY EMISSION',
    3906 => 'DIFFUSE X-RAY EMISSION',
    3907 => 'DIFFUSE X-RAY EMISSION',
    3908 => 'DIFFUSE X-RAY EMISSION',
    3909 => 'DIFFUSE X-RAY EMISSION',
    3910 => 'DIFFUSE X-RAY EMISSION SHELL',
    3911 => 'DIFFUSE X-RAY EMISSION SHELL RADIO PULSAR',
    3912 => 'DIFFUSE X-RAY EMISSION SHELL TYPE I',
    3913 => 'DIFFUSE X-RAY EMISSION SHELL TYPE II',
    3914 => 'DIFFUSE X-RAY EMISSION SHELL',
    3915 => 'DIFFUSE X-RAY EMISSION SHELL',
    3916 => 'DIFFUSE X-RAY EMISSION SHELL',
    3917 => 'DIFFUSE X-RAY EMISSION SHELL',
    3918 => 'DIFFUSE X-RAY EMISSION SHELL',
    3919 => 'DIFFUSE X-RAY EMISSION SHELL',
    3920 => 'DIFFUSE X-RAY EMISSION FILLED-CENTER',
    3921 => 'DIFFUSE X-RAY EMISSION FILLED-CENTER RADIO PULSAR',
    3922 => 'DIFFUSE X-RAY EMISSION FILLED-CENTER TYPE I',
    3923 => 'DIFFUSE X-RAY EMISSION FILLED-CENTER TYPE II',
    3924 => 'DIFFUSE X-RAY EMISSION FILLED-CENTER',
    3925 => 'DIFFUSE X-RAY EMISSION FILLED-CENTER',
    3926 => 'DIFFUSE X-RAY EMISSION FILLED-CENTER',
    3927 => 'DIFFUSE X-RAY EMISSION FILLED-CENTER',
    3928 => 'DIFFUSE X-RAY EMISSION FILLED-CENTER',
    3929 => 'DIFFUSE X-RAY EMISSION FILLED-CENTER',
    3930 => 'DIFFUSE X-RAY EMISSION COMPOSITE',
    3931 => 'DIFFUSE X-RAY EMISSION COMPOSITE RADIO PULSAR',
    3932 => 'DIFFUSE X-RAY EMISSION COMPOSITE TYPE I',
    3933 => 'DIFFUSE X-RAY EMISSION COMPOSITE TYPE II',
    3934 => 'DIFFUSE X-RAY EMISSION COMPOSITE',
    3935 => 'DIFFUSE X-RAY EMISSION COMPOSITE',
    3936 => 'DIFFUSE X-RAY EMISSION COMPOSITE',
    3937 => 'DIFFUSE X-RAY EMISSION COMPOSITE',
    3938 => 'DIFFUSE X-RAY EMISSION COMPOSITE',
    3939 => 'DIFFUSE X-RAY EMISSION COMPOSITE',
    3940 => 'DARK DIFFUSE X-RAY EMISSION',
    3941 => 'DARK DIFFUSE X-RAY EMISSION RADIO PULSAR',
    3942 => 'DARK DIFFUSE X-RAY EMISSION TYPE I',
    3943 => 'DARK DIFFUSE X-RAY EMISSION TYPE II',
    3944 => 'DARK DIFFUSE X-RAY EMISSION',
    3945 => 'DARK DIFFUSE X-RAY EMISSION',
    3946 => 'DARK DIFFUSE X-RAY EMISSION',
    3947 => 'DARK DIFFUSE X-RAY EMISSION',
    3948 => 'DARK DIFFUSE X-RAY EMISSION',
    3949 => 'DARK DIFFUSE X-RAY EMISSION',
    3950 => 'MOLECULAR DIFFUSE X-RAY EMISSION',
    3951 => 'MOLECULAR DIFFUSE X-RAY EMISSION RADIO PULSAR',
    3952 => 'MOLECULAR DIFFUSE X-RAY EMISSION TYPE I',
    3953 => 'MOLECULAR DIFFUSE X-RAY EMISSION TYPE II',
    3954 => 'MOLECULAR DIFFUSE X-RAY EMISSION',
    3955 => 'MOLECULAR DIFFUSE X-RAY EMISSION',
    3956 => 'MOLECULAR DIFFUSE X-RAY EMISSION',
    3957 => 'MOLECULAR DIFFUSE X-RAY EMISSION',
    3958 => 'MOLECULAR DIFFUSE X-RAY EMISSION',
    3959 => 'MOLECULAR DIFFUSE X-RAY EMISSION',
    3960 => 'PLANETARY DIFFUSE X-RAY EMISSION',
    3961 => 'PLANETARY DIFFUSE X-RAY EMISSION RADIO PULSAR',
    3962 => 'PLANETARY DIFFUSE X-RAY EMISSION TYPE I',
    3963 => 'PLANETARY DIFFUSE X-RAY EMISSION TYPE II',
    3964 => 'PLANETARY DIFFUSE X-RAY EMISSION',
    3965 => 'PLANETARY DIFFUSE X-RAY EMISSION',
    3966 => 'PLANETARY DIFFUSE X-RAY EMISSION',
    3967 => 'PLANETARY DIFFUSE X-RAY EMISSION',
    3968 => 'PLANETARY DIFFUSE X-RAY EMISSION',
    3969 => 'PLANETARY DIFFUSE X-RAY EMISSION',
    3970 => 'REFLECTION DIFFUSE X-RAY EMISSION',
    3971 => 'REFLECTION DIFFUSE X-RAY EMISSION RADIO PULSAR',
    3972 => 'REFLECTION DIFFUSE X-RAY EMISSION TYPE I',
    3973 => 'REFLECTION DIFFUSE X-RAY EMISSION TYPE II',
    3974 => 'REFLECTION DIFFUSE X-RAY EMISSION',
    3975 => 'REFLECTION DIFFUSE X-RAY EMISSION',
    3976 => 'REFLECTION DIFFUSE X-RAY EMISSION',
    3977 => 'REFLECTION DIFFUSE X-RAY EMISSION',
    3978 => 'REFLECTION DIFFUSE X-RAY EMISSION',
    3979 => 'REFLECTION DIFFUSE X-RAY EMISSION',
    3980 => 'GLOBULAR CLUSTER DIFFUSE X-RAY EMISSION',
    3981 => 'GLOBULAR CLUSTER DIFFUSE X-RAY EMISSION RADIO PULSAR',
    3982 => 'GLOBULAR CLUSTER DIFFUSE X-RAY EMISSION TYPE I',
    3983 => 'GLOBULAR CLUSTER DIFFUSE X-RAY EMISSION TYPE II',
    3984 => 'GLOBULAR CLUSTER DIFFUSE X-RAY EMISSION',
    3985 => 'GLOBULAR CLUSTER DIFFUSE X-RAY EMISSION',
    3986 => 'GLOBULAR CLUSTER DIFFUSE X-RAY EMISSION',
    3987 => 'GLOBULAR CLUSTER DIFFUSE X-RAY EMISSION',
    3988 => 'GLOBULAR CLUSTER DIFFUSE X-RAY EMISSION',
    3989 => 'GLOBULAR CLUSTER DIFFUSE X-RAY EMISSION',
    3990 => 'DIFFUSE X-RAY EMISSION',
    3991 => 'DIFFUSE X-RAY EMISSION RADIO PULSAR',
    3992 => 'DIFFUSE X-RAY EMISSION TYPE I',
    3993 => 'DIFFUSE X-RAY EMISSION TYPE II',
    3994 => 'DIFFUSE X-RAY EMISSION',
    3995 => 'DIFFUSE X-RAY EMISSION',
    3996 => 'DIFFUSE X-RAY EMISSION',
    3997 => 'DIFFUSE X-RAY EMISSION',
    3998 => 'DIFFUSE X-RAY EMISSION',
    3999 => 'DIFFUSE X-RAY EMISSION',
    4000 => 'WHITE DWARF',
    4001 => 'WHITE DWARF PNN',
    4002 => 'WHITE DWARF PNN DETACHED BINARY',
    4003 => 'WHITE DWARF DETACHED BINARY',
    4004 => 'WHITE DWARF O',
    4005 => 'WHITE DWARF Z',
    4006 => 'WHITE DWARF Q',
    4007 => 'WHITE DWARF X',
    4008 => 'WHITE DWARF P',
    4009 => 'WHITE DWARF H',
    4010 => 'WHITE DWARF A',
    4011 => 'WHITE DWARF A PNN',
    4012 => 'WHITE DWARF A PNN DETACHED BINARY',
    4013 => 'WHITE DWARF A DETACHED BINARY',
    4014 => 'WHITE DWARF A O',
    4015 => 'WHITE DWARF A Z',
    4016 => 'WHITE DWARF A Q',
    4017 => 'WHITE DWARF A X',
    4018 => 'WHITE DWARF A P',
    4019 => 'WHITE DWARF A H',
    4020 => 'WHITE DWARF B',
    4021 => 'WHITE DWARF B PNN',
    4022 => 'WHITE DWARF B PNN DETACHED BINARY',
    4023 => 'WHITE DWARF B DETACHED BINARY',
    4024 => 'WHITE DWARF B O',
    4025 => 'WHITE DWARF B Z',
    4026 => 'WHITE DWARF B Q',
    4027 => 'WHITE DWARF B X',
    4028 => 'WHITE DWARF B P',
    4029 => 'WHITE DWARF B H',
    4030 => 'WHITE DWARF C',
    4031 => 'WHITE DWARF C PNN',
    4032 => 'WHITE DWARF C PNN DETACHED BINARY',
    4033 => 'WHITE DWARF C DETACHED BINARY',
    4034 => 'WHITE DWARF C O',
    4035 => 'WHITE DWARF C Z',
    4036 => 'WHITE DWARF C Q',
    4037 => 'WHITE DWARF C X',
    4038 => 'WHITE DWARF C P',
    4039 => 'WHITE DWARF C H',
    4040 => 'WHITE DWARF O',
    4041 => 'WHITE DWARF O PNN',
    4042 => 'WHITE DWARF O PNN DETACHED BINARY',
    4043 => 'WHITE DWARF O DETACHED BINARY',
    4044 => 'WHITE DWARF O O',
    4045 => 'WHITE DWARF O Z',
    4046 => 'WHITE DWARF O Q',
    4047 => 'WHITE DWARF O X',
    4048 => 'WHITE DWARF O P',
    4049 => 'WHITE DWARF O H',
    4050 => 'WHITE DWARF Z',
    4051 => 'WHITE DWARF Z PNN',
    4052 => 'WHITE DWARF Z PNN DETACHED BINARY',
    4053 => 'WHITE DWARF Z DETACHED BINARY',
    4054 => 'WHITE DWARF Z O',
    4055 => 'WHITE DWARF Z Z',
    4056 => 'WHITE DWARF Z Q',
    4057 => 'WHITE DWARF Z X',
    4058 => 'WHITE DWARF Z P',
    4059 => 'WHITE DWARF Z H',
    4060 => 'WHITE DWARF Q',
    4061 => 'WHITE DWARF Q PNN',
    4062 => 'WHITE DWARF Q PNN DETACHED BINARY',
    4063 => 'WHITE DWARF Q DETACHED BINARY',
    4064 => 'WHITE DWARF Q O',
    4065 => 'WHITE DWARF Q Z',
    4066 => 'WHITE DWARF Q Q',
    4067 => 'WHITE DWARF Q X',
    4068 => 'WHITE DWARF Q P',
    4069 => 'WHITE DWARF Q H',
    4070 => 'WHITE DWARF X',
    4071 => 'WHITE DWARF X PNN',
    4072 => 'WHITE DWARF X PNN DETACHED BINARY',
    4073 => 'WHITE DWARF X DETACHED BINARY',
    4074 => 'WHITE DWARF X O',
    4075 => 'WHITE DWARF X Z',
    4076 => 'WHITE DWARF X Q',
    4077 => 'WHITE DWARF X X',
    4078 => 'WHITE DWARF X P',
    4079 => 'WHITE DWARF X H',
    4080 => 'WHITE DWARF V',
    4081 => 'WHITE DWARF V PNN',
    4082 => 'WHITE DWARF V PNN DETACHED BINARY',
    4083 => 'WHITE DWARF V DETACHED BINARY',
    4084 => 'WHITE DWARF V O',
    4085 => 'WHITE DWARF V Z',
    4086 => 'WHITE DWARF V Q',
    4087 => 'WHITE DWARF V X',
    4088 => 'WHITE DWARF V P',
    4089 => 'WHITE DWARF V H',
    4090 => 'WHITE DWARF P',
    4091 => 'WHITE DWARF P PNN',
    4092 => 'WHITE DWARF P PNN DETACHED BINARY',
    4093 => 'WHITE DWARF P DETACHED BINARY',
    4094 => 'WHITE DWARF P O',
    4095 => 'WHITE DWARF P Z',
    4096 => 'WHITE DWARF P Q',
    4097 => 'WHITE DWARF P X',
    4098 => 'WHITE DWARF P P',
    4099 => 'WHITE DWARF P H',
    4100 => 'WHITE DWARF D A',
    4101 => 'WHITE DWARF D A PNN',
    4102 => 'WHITE DWARF D A PNN DETACHED BINARY',
    4103 => 'WHITE DWARF D A DETACHED BINARY',
    4104 => 'WHITE DWARF D A O',
    4105 => 'WHITE DWARF D A Z',
    4106 => 'WHITE DWARF D A Q',
    4107 => 'WHITE DWARF D A X',
    4108 => 'WHITE DWARF D A P',
    4109 => 'WHITE DWARF D A H',
    4110 => 'WHITE DWARF D A A',
    4111 => 'WHITE DWARF D A A PNN',
    4112 => 'WHITE DWARF D A A PNN DETACHED BINARY',
    4113 => 'WHITE DWARF D A A DETACHED BINARY',
    4114 => 'WHITE DWARF D A A O',
    4115 => 'WHITE DWARF D A A Z',
    4116 => 'WHITE DWARF D A A Q',
    4117 => 'WHITE DWARF D A A X',
    4118 => 'WHITE DWARF D A A P',
    4119 => 'WHITE DWARF D A A H',
    4120 => 'WHITE DWARF D A B',
    4121 => 'WHITE DWARF D A B PNN',
    4122 => 'WHITE DWARF D A B PNN DETACHED BINARY',
    4123 => 'WHITE DWARF D A B DETACHED BINARY',
    4124 => 'WHITE DWARF D A B O',
    4125 => 'WHITE DWARF D A B Z',
    4126 => 'WHITE DWARF D A B Q',
    4127 => 'WHITE DWARF D A B X',
    4128 => 'WHITE DWARF D A B P',
    4129 => 'WHITE DWARF D A B H',
    4130 => 'WHITE DWARF D A C',
    4131 => 'WHITE DWARF D A C PNN',
    4132 => 'WHITE DWARF D A C PNN DETACHED BINARY',
    4133 => 'WHITE DWARF D A C DETACHED BINARY',
    4134 => 'WHITE DWARF D A C O',
    4135 => 'WHITE DWARF D A C Z',
    4136 => 'WHITE DWARF D A C Q',
    4137 => 'WHITE DWARF D A C X',
    4138 => 'WHITE DWARF D A C P',
    4139 => 'WHITE DWARF D A C H',
    4140 => 'WHITE DWARF D A O',
    4141 => 'WHITE DWARF D A O PNN',
    4142 => 'WHITE DWARF D A O PNN DETACHED BINARY',
    4143 => 'WHITE DWARF D A O DETACHED BINARY',
    4144 => 'WHITE DWARF D A O O',
    4145 => 'WHITE DWARF D A O Z',
    4146 => 'WHITE DWARF D A O Q',
    4147 => 'WHITE DWARF D A O X',
    4148 => 'WHITE DWARF D A O P',
    4149 => 'WHITE DWARF D A O H',
    4150 => 'WHITE DWARF D A Z',
    4151 => 'WHITE DWARF D A Z PNN',
    4152 => 'WHITE DWARF D A Z PNN DETACHED BINARY',
    4153 => 'WHITE DWARF D A Z DETACHED BINARY',
    4154 => 'WHITE DWARF D A Z O',
    4155 => 'WHITE DWARF D A Z Z',
    4156 => 'WHITE DWARF D A Z Q',
    4157 => 'WHITE DWARF D A Z X',
    4158 => 'WHITE DWARF D A Z P',
    4159 => 'WHITE DWARF D A Z H',
    4160 => 'WHITE DWARF D A Q',
    4161 => 'WHITE DWARF D A Q PNN',
    4162 => 'WHITE DWARF D A Q PNN DETACHED BINARY',
    4163 => 'WHITE DWARF D A Q DETACHED BINARY',
    4164 => 'WHITE DWARF D A Q O',
    4165 => 'WHITE DWARF D A Q Z',
    4166 => 'WHITE DWARF D A Q Q',
    4167 => 'WHITE DWARF D A Q X',
    4168 => 'WHITE DWARF D A Q P',
    4169 => 'WHITE DWARF D A Q H',
    4170 => 'WHITE DWARF D A X',
    4171 => 'WHITE DWARF D A X PNN',
    4172 => 'WHITE DWARF D A X PNN DETACHED BINARY',
    4173 => 'WHITE DWARF D A X DETACHED BINARY',
    4174 => 'WHITE DWARF D A X O',
    4175 => 'WHITE DWARF D A X Z',
    4176 => 'WHITE DWARF D A X Q',
    4177 => 'WHITE DWARF D A X X',
    4178 => 'WHITE DWARF D A X P',
    4179 => 'WHITE DWARF D A X H',
    4180 => 'WHITE DWARF D A V',
    4181 => 'WHITE DWARF D A V PNN',
    4182 => 'WHITE DWARF D A V PNN DETACHED BINARY',
    4183 => 'WHITE DWARF D A V DETACHED BINARY',
    4184 => 'WHITE DWARF D A V O',
    4185 => 'WHITE DWARF D A V Z',
    4186 => 'WHITE DWARF D A V Q',
    4187 => 'WHITE DWARF D A V X',
    4188 => 'WHITE DWARF D A V P',
    4189 => 'WHITE DWARF D A V H',
    4190 => 'WHITE DWARF D A P',
    4191 => 'WHITE DWARF D A P PNN',
    4192 => 'WHITE DWARF D A P PNN DETACHED BINARY',
    4193 => 'WHITE DWARF D A P DETACHED BINARY',
    4194 => 'WHITE DWARF D A P O',
    4195 => 'WHITE DWARF D A P Z',
    4196 => 'WHITE DWARF D A P Q',
    4197 => 'WHITE DWARF D A P X',
    4198 => 'WHITE DWARF D A P P',
    4199 => 'WHITE DWARF D A P H',
    4200 => 'WHITE DWARF D B',
    4201 => 'WHITE DWARF D B PNN',
    4202 => 'WHITE DWARF D B PNN DETACHED BINARY',
    4203 => 'WHITE DWARF D B DETACHED BINARY',
    4204 => 'WHITE DWARF D B O',
    4205 => 'WHITE DWARF D B Z',
    4206 => 'WHITE DWARF D B Q',
    4207 => 'WHITE DWARF D B X',
    4208 => 'WHITE DWARF D B P',
    4209 => 'WHITE DWARF D B H',
    4210 => 'WHITE DWARF D B A',
    4211 => 'WHITE DWARF D B A PNN',
    4212 => 'WHITE DWARF D B A PNN DETACHED BINARY',
    4213 => 'WHITE DWARF D B A DETACHED BINARY',
    4214 => 'WHITE DWARF D B A O',
    4215 => 'WHITE DWARF D B A Z',
    4216 => 'WHITE DWARF D B A Q',
    4217 => 'WHITE DWARF D B A X',
    4218 => 'WHITE DWARF D B A P',
    4219 => 'WHITE DWARF D B A H',
    4220 => 'WHITE DWARF D B B',
    4221 => 'WHITE DWARF D B B PNN',
    4222 => 'WHITE DWARF D B B PNN DETACHED BINARY',
    4223 => 'WHITE DWARF D B B DETACHED BINARY',
    4224 => 'WHITE DWARF D B B O',
    4225 => 'WHITE DWARF D B B Z',
    4226 => 'WHITE DWARF D B B Q',
    4227 => 'WHITE DWARF D B B X',
    4228 => 'WHITE DWARF D B B P',
    4229 => 'WHITE DWARF D B B H',
    4230 => 'WHITE DWARF D B C',
    4231 => 'WHITE DWARF D B C PNN',
    4232 => 'WHITE DWARF D B C PNN DETACHED BINARY',
    4233 => 'WHITE DWARF D B C DETACHED BINARY',
    4234 => 'WHITE DWARF D B C O',
    4235 => 'WHITE DWARF D B C Z',
    4236 => 'WHITE DWARF D B C Q',
    4237 => 'WHITE DWARF D B C X',
    4238 => 'WHITE DWARF D B C P',
    4239 => 'WHITE DWARF D B C H',
    4240 => 'WHITE DWARF D B O',
    4241 => 'WHITE DWARF D B O PNN',
    4242 => 'WHITE DWARF D B O PNN DETACHED BINARY',
    4243 => 'WHITE DWARF D B O DETACHED BINARY',
    4244 => 'WHITE DWARF D B O O',
    4245 => 'WHITE DWARF D B O Z',
    4246 => 'WHITE DWARF D B O Q',
    4247 => 'WHITE DWARF D B O X',
    4248 => 'WHITE DWARF D B O P',
    4249 => 'WHITE DWARF D B O H',
    4250 => 'WHITE DWARF D B Z',
    4251 => 'WHITE DWARF D B Z PNN',
    4252 => 'WHITE DWARF D B Z PNN DETACHED BINARY',
    4253 => 'WHITE DWARF D B Z DETACHED BINARY',
    4254 => 'WHITE DWARF D B Z O',
    4255 => 'WHITE DWARF D B Z Z',
    4256 => 'WHITE DWARF D B Z Q',
    4257 => 'WHITE DWARF D B Z X',
    4258 => 'WHITE DWARF D B Z P',
    4259 => 'WHITE DWARF D B Z H',
    4260 => 'WHITE DWARF D B Q',
    4261 => 'WHITE DWARF D B Q PNN',
    4262 => 'WHITE DWARF D B Q PNN DETACHED BINARY',
    4263 => 'WHITE DWARF D B Q DETACHED BINARY',
    4264 => 'WHITE DWARF D B Q O',
    4265 => 'WHITE DWARF D B Q Z',
    4266 => 'WHITE DWARF D B Q Q',
    4267 => 'WHITE DWARF D B Q X',
    4268 => 'WHITE DWARF D B Q P',
    4269 => 'WHITE DWARF D B Q H',
    4270 => 'WHITE DWARF D B X',
    4271 => 'WHITE DWARF D B X PNN',
    4272 => 'WHITE DWARF D B X PNN DETACHED BINARY',
    4273 => 'WHITE DWARF D B X DETACHED BINARY',
    4274 => 'WHITE DWARF D B X O',
    4275 => 'WHITE DWARF D B X Z',
    4276 => 'WHITE DWARF D B X Q',
    4277 => 'WHITE DWARF D B X X',
    4278 => 'WHITE DWARF D B X P',
    4279 => 'WHITE DWARF D B X H',
    4280 => 'WHITE DWARF D B V',
    4281 => 'WHITE DWARF D B V PNN',
    4282 => 'WHITE DWARF D B V PNN DETACHED BINARY',
    4283 => 'WHITE DWARF D B V DETACHED BINARY',
    4284 => 'WHITE DWARF D B V O',
    4285 => 'WHITE DWARF D B V Z',
    4286 => 'WHITE DWARF D B V Q',
    4287 => 'WHITE DWARF D B V X',
    4288 => 'WHITE DWARF D B V P',
    4289 => 'WHITE DWARF D B V H',
    4290 => 'WHITE DWARF D B P',
    4291 => 'WHITE DWARF D B P PNN',
    4292 => 'WHITE DWARF D B P PNN DETACHED BINARY',
    4293 => 'WHITE DWARF D B P DETACHED BINARY',
    4294 => 'WHITE DWARF D B P O',
    4295 => 'WHITE DWARF D B P Z',
    4296 => 'WHITE DWARF D B P Q',
    4297 => 'WHITE DWARF D B P X',
    4298 => 'WHITE DWARF D B P P',
    4299 => 'WHITE DWARF D B P H',
    4300 => 'WHITE DWARF D C',
    4301 => 'WHITE DWARF D C PNN',
    4302 => 'WHITE DWARF D C PNN DETACHED BINARY',
    4303 => 'WHITE DWARF D C DETACHED BINARY',
    4304 => 'WHITE DWARF D C O',
    4305 => 'WHITE DWARF D C Z',
    4306 => 'WHITE DWARF D C Q',
    4307 => 'WHITE DWARF D C X',
    4308 => 'WHITE DWARF D C P',
    4309 => 'WHITE DWARF D C H',
    4310 => 'WHITE DWARF D C A',
    4311 => 'WHITE DWARF D C A PNN',
    4312 => 'WHITE DWARF D C A PNN DETACHED BINARY',
    4313 => 'WHITE DWARF D C A DETACHED BINARY',
    4314 => 'WHITE DWARF D C A O',
    4315 => 'WHITE DWARF D C A Z',
    4316 => 'WHITE DWARF D C A Q',
    4317 => 'WHITE DWARF D C A X',
    4318 => 'WHITE DWARF D C A P',
    4319 => 'WHITE DWARF D C A H',
    4320 => 'WHITE DWARF D C B',
    4321 => 'WHITE DWARF D C B PNN',
    4322 => 'WHITE DWARF D C B PNN DETACHED BINARY',
    4323 => 'WHITE DWARF D C B DETACHED BINARY',
    4324 => 'WHITE DWARF D C B O',
    4325 => 'WHITE DWARF D C B Z',
    4326 => 'WHITE DWARF D C B Q',
    4327 => 'WHITE DWARF D C B X',
    4328 => 'WHITE DWARF D C B P',
    4329 => 'WHITE DWARF D C B H',
    4330 => 'WHITE DWARF D C C',
    4331 => 'WHITE DWARF D C C PNN',
    4332 => 'WHITE DWARF D C C PNN DETACHED BINARY',
    4333 => 'WHITE DWARF D C C DETACHED BINARY',
    4334 => 'WHITE DWARF D C C O',
    4335 => 'WHITE DWARF D C C Z',
    4336 => 'WHITE DWARF D C C Q',
    4337 => 'WHITE DWARF D C C X',
    4338 => 'WHITE DWARF D C C P',
    4339 => 'WHITE DWARF D C C H',
    4340 => 'WHITE DWARF D C O',
    4341 => 'WHITE DWARF D C O PNN',
    4342 => 'WHITE DWARF D C O PNN DETACHED BINARY',
    4343 => 'WHITE DWARF D C O DETACHED BINARY',
    4344 => 'WHITE DWARF D C O O',
    4345 => 'WHITE DWARF D C O Z',
    4346 => 'WHITE DWARF D C O Q',
    4347 => 'WHITE DWARF D C O X',
    4348 => 'WHITE DWARF D C O P',
    4349 => 'WHITE DWARF D C O H',
    4350 => 'WHITE DWARF D C Z',
    4351 => 'WHITE DWARF D C Z PNN',
    4352 => 'WHITE DWARF D C Z PNN DETACHED BINARY',
    4353 => 'WHITE DWARF D C Z DETACHED BINARY',
    4354 => 'WHITE DWARF D C Z O',
    4355 => 'WHITE DWARF D C Z Z',
    4356 => 'WHITE DWARF D C Z Q',
    4357 => 'WHITE DWARF D C Z X',
    4358 => 'WHITE DWARF D C Z P',
    4359 => 'WHITE DWARF D C Z H',
    4360 => 'WHITE DWARF D C Q',
    4361 => 'WHITE DWARF D C Q PNN',
    4362 => 'WHITE DWARF D C Q PNN DETACHED BINARY',
    4363 => 'WHITE DWARF D C Q DETACHED BINARY',
    4364 => 'WHITE DWARF D C Q O',
    4365 => 'WHITE DWARF D C Q Z',
    4366 => 'WHITE DWARF D C Q Q',
    4367 => 'WHITE DWARF D C Q X',
    4368 => 'WHITE DWARF D C Q P',
    4369 => 'WHITE DWARF D C Q H',
    4370 => 'WHITE DWARF D C X',
    4371 => 'WHITE DWARF D C X PNN',
    4372 => 'WHITE DWARF D C X PNN DETACHED BINARY',
    4373 => 'WHITE DWARF D C X DETACHED BINARY',
    4374 => 'WHITE DWARF D C X O',
    4375 => 'WHITE DWARF D C X Z',
    4376 => 'WHITE DWARF D C X Q',
    4377 => 'WHITE DWARF D C X X',
    4378 => 'WHITE DWARF D C X P',
    4379 => 'WHITE DWARF D C X H',
    4380 => 'WHITE DWARF D C V',
    4381 => 'WHITE DWARF D C V PNN',
    4382 => 'WHITE DWARF D C V PNN DETACHED BINARY',
    4383 => 'WHITE DWARF D C V DETACHED BINARY',
    4384 => 'WHITE DWARF D C V O',
    4385 => 'WHITE DWARF D C V Z',
    4386 => 'WHITE DWARF D C V Q',
    4387 => 'WHITE DWARF D C V X',
    4388 => 'WHITE DWARF D C V P',
    4389 => 'WHITE DWARF D C V H',
    4390 => 'WHITE DWARF D C P',
    4391 => 'WHITE DWARF D C P PNN',
    4392 => 'WHITE DWARF D C P PNN DETACHED BINARY',
    4393 => 'WHITE DWARF D C P DETACHED BINARY',
    4394 => 'WHITE DWARF D C P O',
    4395 => 'WHITE DWARF D C P Z',
    4396 => 'WHITE DWARF D C P Q',
    4397 => 'WHITE DWARF D C P X',
    4398 => 'WHITE DWARF D C P P',
    4399 => 'WHITE DWARF D C P H',
    4400 => 'WHITE DWARF D O',
    4401 => 'WHITE DWARF D O PNN',
    4402 => 'WHITE DWARF D O PNN DETACHED BINARY',
    4403 => 'WHITE DWARF D O DETACHED BINARY',
    4404 => 'WHITE DWARF D O O',
    4405 => 'WHITE DWARF D O Z',
    4406 => 'WHITE DWARF D O Q',
    4407 => 'WHITE DWARF D O X',
    4408 => 'WHITE DWARF D O P',
    4409 => 'WHITE DWARF D O H',
    4410 => 'WHITE DWARF D O A',
    4411 => 'WHITE DWARF D O A PNN',
    4412 => 'WHITE DWARF D O A PNN DETACHED BINARY',
    4413 => 'WHITE DWARF D O A DETACHED BINARY',
    4414 => 'WHITE DWARF D O A O',
    4415 => 'WHITE DWARF D O A Z',
    4416 => 'WHITE DWARF D O A Q',
    4417 => 'WHITE DWARF D O A X',
    4418 => 'WHITE DWARF D O A P',
    4419 => 'WHITE DWARF D O A H',
    4420 => 'WHITE DWARF D O B',
    4421 => 'WHITE DWARF D O B PNN',
    4422 => 'WHITE DWARF D O B PNN DETACHED BINARY',
    4423 => 'WHITE DWARF D O B DETACHED BINARY',
    4424 => 'WHITE DWARF D O B O',
    4425 => 'WHITE DWARF D O B Z',
    4426 => 'WHITE DWARF D O B Q',
    4427 => 'WHITE DWARF D O B X',
    4428 => 'WHITE DWARF D O B P',
    4429 => 'WHITE DWARF D O B H',
    4430 => 'WHITE DWARF D O C',
    4431 => 'WHITE DWARF D O C PNN',
    4432 => 'WHITE DWARF D O C PNN DETACHED BINARY',
    4433 => 'WHITE DWARF D O C DETACHED BINARY',
    4434 => 'WHITE DWARF D O C O',
    4435 => 'WHITE DWARF D O C Z',
    4436 => 'WHITE DWARF D O C Q',
    4437 => 'WHITE DWARF D O C X',
    4438 => 'WHITE DWARF D O C P',
    4439 => 'WHITE DWARF D O C H',
    4440 => 'WHITE DWARF D O O',
    4441 => 'WHITE DWARF D O O PNN',
    4442 => 'WHITE DWARF D O O PNN DETACHED BINARY',
    4443 => 'WHITE DWARF D O O DETACHED BINARY',
    4444 => 'WHITE DWARF D O O O',
    4445 => 'WHITE DWARF D O O Z',
    4446 => 'WHITE DWARF D O O Q',
    4447 => 'WHITE DWARF D O O X',
    4448 => 'WHITE DWARF D O O P',
    4449 => 'WHITE DWARF D O O H',
    4450 => 'WHITE DWARF D O Z',
    4451 => 'WHITE DWARF D O Z PNN',
    4452 => 'WHITE DWARF D O Z PNN DETACHED BINARY',
    4453 => 'WHITE DWARF D O Z DETACHED BINARY',
    4454 => 'WHITE DWARF D O Z O',
    4455 => 'WHITE DWARF D O Z Z',
    4456 => 'WHITE DWARF D O Z Q',
    4457 => 'WHITE DWARF D O Z X',
    4458 => 'WHITE DWARF D O Z P',
    4459 => 'WHITE DWARF D O Z H',
    4460 => 'WHITE DWARF D O Q',
    4461 => 'WHITE DWARF D O Q PNN',
    4462 => 'WHITE DWARF D O Q PNN DETACHED BINARY',
    4463 => 'WHITE DWARF D O Q DETACHED BINARY',
    4464 => 'WHITE DWARF D O Q O',
    4465 => 'WHITE DWARF D O Q Z',
    4466 => 'WHITE DWARF D O Q Q',
    4467 => 'WHITE DWARF D O Q X',
    4468 => 'WHITE DWARF D O Q P',
    4469 => 'WHITE DWARF D O Q H',
    4470 => 'WHITE DWARF D O X',
    4471 => 'WHITE DWARF D O X PNN',
    4472 => 'WHITE DWARF D O X PNN DETACHED BINARY',
    4473 => 'WHITE DWARF D O X DETACHED BINARY',
    4474 => 'WHITE DWARF D O X O',
    4475 => 'WHITE DWARF D O X Z',
    4476 => 'WHITE DWARF D O X Q',
    4477 => 'WHITE DWARF D O X X',
    4478 => 'WHITE DWARF D O X P',
    4479 => 'WHITE DWARF D O X H',
    4480 => 'WHITE DWARF D O V',
    4481 => 'WHITE DWARF D O V PNN',
    4482 => 'WHITE DWARF D O V PNN DETACHED BINARY',
    4483 => 'WHITE DWARF D O V DETACHED BINARY',
    4484 => 'WHITE DWARF D O V O',
    4485 => 'WHITE DWARF D O V Z',
    4486 => 'WHITE DWARF D O V Q',
    4487 => 'WHITE DWARF D O V X',
    4488 => 'WHITE DWARF D O V P',
    4489 => 'WHITE DWARF D O V H',
    4490 => 'WHITE DWARF D O P',
    4491 => 'WHITE DWARF D O P PNN',
    4492 => 'WHITE DWARF D O P PNN DETACHED BINARY',
    4493 => 'WHITE DWARF D O P DETACHED BINARY',
    4494 => 'WHITE DWARF D O P O',
    4495 => 'WHITE DWARF D O P Z',
    4496 => 'WHITE DWARF D O P Q',
    4497 => 'WHITE DWARF D O P X',
    4498 => 'WHITE DWARF D O P P',
    4499 => 'WHITE DWARF D O P H',
    4500 => 'WHITE DWARF D Z',
    4501 => 'WHITE DWARF D Z PNN',
    4502 => 'WHITE DWARF D Z PNN DETACHED BINARY',
    4503 => 'WHITE DWARF D Z DETACHED BINARY',
    4504 => 'WHITE DWARF D Z O',
    4505 => 'WHITE DWARF D Z Z',
    4506 => 'WHITE DWARF D Z Q',
    4507 => 'WHITE DWARF D Z X',
    4508 => 'WHITE DWARF D Z P',
    4509 => 'WHITE DWARF D Z H',
    4510 => 'WHITE DWARF D Z A',
    4511 => 'WHITE DWARF D Z A PNN',
    4512 => 'WHITE DWARF D Z A PNN DETACHED BINARY',
    4513 => 'WHITE DWARF D Z A DETACHED BINARY',
    4514 => 'WHITE DWARF D Z A O',
    4515 => 'WHITE DWARF D Z A Z',
    4516 => 'WHITE DWARF D Z A Q',
    4517 => 'WHITE DWARF D Z A X',
    4518 => 'WHITE DWARF D Z A P',
    4519 => 'WHITE DWARF D Z A H',
    4520 => 'WHITE DWARF D Z B',
    4521 => 'WHITE DWARF D Z B PNN',
    4522 => 'WHITE DWARF D Z B PNN DETACHED BINARY',
    4523 => 'WHITE DWARF D Z B DETACHED BINARY',
    4524 => 'WHITE DWARF D Z B O',
    4525 => 'WHITE DWARF D Z B Z',
    4526 => 'WHITE DWARF D Z B Q',
    4527 => 'WHITE DWARF D Z B X',
    4528 => 'WHITE DWARF D Z B P',
    4529 => 'WHITE DWARF D Z B H',
    4530 => 'WHITE DWARF D Z C',
    4531 => 'WHITE DWARF D Z C PNN',
    4532 => 'WHITE DWARF D Z C PNN DETACHED BINARY',
    4533 => 'WHITE DWARF D Z C DETACHED BINARY',
    4534 => 'WHITE DWARF D Z C O',
    4535 => 'WHITE DWARF D Z C Z',
    4536 => 'WHITE DWARF D Z C Q',
    4537 => 'WHITE DWARF D Z C X',
    4538 => 'WHITE DWARF D Z C P',
    4539 => 'WHITE DWARF D Z C H',
    4540 => 'WHITE DWARF D Z O',
    4541 => 'WHITE DWARF D Z O PNN',
    4542 => 'WHITE DWARF D Z O PNN DETACHED BINARY',
    4543 => 'WHITE DWARF D Z O DETACHED BINARY',
    4544 => 'WHITE DWARF D Z O O',
    4545 => 'WHITE DWARF D Z O Z',
    4546 => 'WHITE DWARF D Z O Q',
    4547 => 'WHITE DWARF D Z O X',
    4548 => 'WHITE DWARF D Z O P',
    4549 => 'WHITE DWARF D Z O H',
    4550 => 'WHITE DWARF D Z Z',
    4551 => 'WHITE DWARF D Z Z PNN',
    4552 => 'WHITE DWARF D Z Z PNN DETACHED BINARY',
    4553 => 'WHITE DWARF D Z Z DETACHED BINARY',
    4554 => 'WHITE DWARF D Z Z O',
    4555 => 'WHITE DWARF D Z Z Z',
    4556 => 'WHITE DWARF D Z Z Q',
    4557 => 'WHITE DWARF D Z Z X',
    4558 => 'WHITE DWARF D Z Z P',
    4559 => 'WHITE DWARF D Z Z H',
    4560 => 'WHITE DWARF D Z Q',
    4561 => 'WHITE DWARF D Z Q PNN',
    4562 => 'WHITE DWARF D Z Q PNN DETACHED BINARY',
    4563 => 'WHITE DWARF D Z Q DETACHED BINARY',
    4564 => 'WHITE DWARF D Z Q O',
    4565 => 'WHITE DWARF D Z Q Z',
    4566 => 'WHITE DWARF D Z Q Q',
    4567 => 'WHITE DWARF D Z Q X',
    4568 => 'WHITE DWARF D Z Q P',
    4569 => 'WHITE DWARF D Z Q H',
    4570 => 'WHITE DWARF D Z X',
    4571 => 'WHITE DWARF D Z X PNN',
    4572 => 'WHITE DWARF D Z X PNN DETACHED BINARY',
    4573 => 'WHITE DWARF D Z X DETACHED BINARY',
    4574 => 'WHITE DWARF D Z X O',
    4575 => 'WHITE DWARF D Z X Z',
    4576 => 'WHITE DWARF D Z X Q',
    4577 => 'WHITE DWARF D Z X X',
    4578 => 'WHITE DWARF D Z X P',
    4579 => 'WHITE DWARF D Z X H',
    4580 => 'WHITE DWARF D Z V',
    4581 => 'WHITE DWARF D Z V PNN',
    4582 => 'WHITE DWARF D Z V PNN DETACHED BINARY',
    4583 => 'WHITE DWARF D Z V DETACHED BINARY',
    4584 => 'WHITE DWARF D Z V O',
    4585 => 'WHITE DWARF D Z V Z',
    4586 => 'WHITE DWARF D Z V Q',
    4587 => 'WHITE DWARF D Z V X',
    4588 => 'WHITE DWARF D Z V P',
    4589 => 'WHITE DWARF D Z V H',
    4590 => 'WHITE DWARF D Z P',
    4591 => 'WHITE DWARF D Z P PNN',
    4592 => 'WHITE DWARF D Z P PNN DETACHED BINARY',
    4593 => 'WHITE DWARF D Z P DETACHED BINARY',
    4594 => 'WHITE DWARF D Z P O',
    4595 => 'WHITE DWARF D Z P Z',
    4596 => 'WHITE DWARF D Z P Q',
    4597 => 'WHITE DWARF D Z P X',
    4598 => 'WHITE DWARF D Z P P',
    4599 => 'WHITE DWARF D Z P H',
    4600 => 'WHITE DWARF D Q',
    4601 => 'WHITE DWARF D Q PNN',
    4602 => 'WHITE DWARF D Q PNN DETACHED BINARY',
    4603 => 'WHITE DWARF D Q DETACHED BINARY',
    4604 => 'WHITE DWARF D Q O',
    4605 => 'WHITE DWARF D Q Z',
    4606 => 'WHITE DWARF D Q Q',
    4607 => 'WHITE DWARF D Q X',
    4608 => 'WHITE DWARF D Q P',
    4609 => 'WHITE DWARF D Q H',
    4610 => 'WHITE DWARF D Q A',
    4611 => 'WHITE DWARF D Q A PNN',
    4612 => 'WHITE DWARF D Q A PNN DETACHED BINARY',
    4613 => 'WHITE DWARF D Q A DETACHED BINARY',
    4614 => 'WHITE DWARF D Q A O',
    4615 => 'WHITE DWARF D Q A Z',
    4616 => 'WHITE DWARF D Q A Q',
    4617 => 'WHITE DWARF D Q A X',
    4618 => 'WHITE DWARF D Q A P',
    4619 => 'WHITE DWARF D Q A H',
    4620 => 'WHITE DWARF D Q B',
    4621 => 'WHITE DWARF D Q B PNN',
    4622 => 'WHITE DWARF D Q B PNN DETACHED BINARY',
    4623 => 'WHITE DWARF D Q B DETACHED BINARY',
    4624 => 'WHITE DWARF D Q B O',
    4625 => 'WHITE DWARF D Q B Z',
    4626 => 'WHITE DWARF D Q B Q',
    4627 => 'WHITE DWARF D Q B X',
    4628 => 'WHITE DWARF D Q B P',
    4629 => 'WHITE DWARF D Q B H',
    4630 => 'WHITE DWARF D Q C',
    4631 => 'WHITE DWARF D Q C PNN',
    4632 => 'WHITE DWARF D Q C PNN DETACHED BINARY',
    4633 => 'WHITE DWARF D Q C DETACHED BINARY',
    4634 => 'WHITE DWARF D Q C O',
    4635 => 'WHITE DWARF D Q C Z',
    4636 => 'WHITE DWARF D Q C Q',
    4637 => 'WHITE DWARF D Q C X',
    4638 => 'WHITE DWARF D Q C P',
    4639 => 'WHITE DWARF D Q C H',
    4640 => 'WHITE DWARF D Q O',
    4641 => 'WHITE DWARF D Q O PNN',
    4642 => 'WHITE DWARF D Q O PNN DETACHED BINARY',
    4643 => 'WHITE DWARF D Q O DETACHED BINARY',
    4644 => 'WHITE DWARF D Q O O',
    4645 => 'WHITE DWARF D Q O Z',
    4646 => 'WHITE DWARF D Q O Q',
    4647 => 'WHITE DWARF D Q O X',
    4648 => 'WHITE DWARF D Q O P',
    4649 => 'WHITE DWARF D Q O H',
    4650 => 'WHITE DWARF D Q Z',
    4651 => 'WHITE DWARF D Q Z PNN',
    4652 => 'WHITE DWARF D Q Z PNN DETACHED BINARY',
    4653 => 'WHITE DWARF D Q Z DETACHED BINARY',
    4654 => 'WHITE DWARF D Q Z O',
    4655 => 'WHITE DWARF D Q Z Z',
    4656 => 'WHITE DWARF D Q Z Q',
    4657 => 'WHITE DWARF D Q Z X',
    4658 => 'WHITE DWARF D Q Z P',
    4659 => 'WHITE DWARF D Q Z H',
    4660 => 'WHITE DWARF D Q Q',
    4661 => 'WHITE DWARF D Q Q PNN',
    4662 => 'WHITE DWARF D Q Q PNN DETACHED BINARY',
    4663 => 'WHITE DWARF D Q Q DETACHED BINARY',
    4664 => 'WHITE DWARF D Q Q O',
    4665 => 'WHITE DWARF D Q Q Z',
    4666 => 'WHITE DWARF D Q Q Q',
    4667 => 'WHITE DWARF D Q Q X',
    4668 => 'WHITE DWARF D Q Q P',
    4669 => 'WHITE DWARF D Q Q H',
    4670 => 'WHITE DWARF D Q X',
    4671 => 'WHITE DWARF D Q X PNN',
    4672 => 'WHITE DWARF D Q X PNN DETACHED BINARY',
    4673 => 'WHITE DWARF D Q X DETACHED BINARY',
    4674 => 'WHITE DWARF D Q X O',
    4675 => 'WHITE DWARF D Q X Z',
    4676 => 'WHITE DWARF D Q X Q',
    4677 => 'WHITE DWARF D Q X X',
    4678 => 'WHITE DWARF D Q X P',
    4679 => 'WHITE DWARF D Q X H',
    4680 => 'WHITE DWARF D Q V',
    4681 => 'WHITE DWARF D Q V PNN',
    4682 => 'WHITE DWARF D Q V PNN DETACHED BINARY',
    4683 => 'WHITE DWARF D Q V DETACHED BINARY',
    4684 => 'WHITE DWARF D Q V O',
    4685 => 'WHITE DWARF D Q V Z',
    4686 => 'WHITE DWARF D Q V Q',
    4687 => 'WHITE DWARF D Q V X',
    4688 => 'WHITE DWARF D Q V P',
    4689 => 'WHITE DWARF D Q V H',
    4690 => 'WHITE DWARF D Q P',
    4691 => 'WHITE DWARF D Q P PNN',
    4692 => 'WHITE DWARF D Q P PNN DETACHED BINARY',
    4693 => 'WHITE DWARF D Q P DETACHED BINARY',
    4694 => 'WHITE DWARF D Q P O',
    4695 => 'WHITE DWARF D Q P Z',
    4696 => 'WHITE DWARF D Q P Q',
    4697 => 'WHITE DWARF D Q P X',
    4698 => 'WHITE DWARF D Q P P',
    4699 => 'WHITE DWARF D Q P H',
    4700 => 'WHITE DWARF D X',
    4701 => 'WHITE DWARF D X PNN',
    4702 => 'WHITE DWARF D X PNN DETACHED BINARY',
    4703 => 'WHITE DWARF D X DETACHED BINARY',
    4704 => 'WHITE DWARF D X O',
    4705 => 'WHITE DWARF D X Z',
    4706 => 'WHITE DWARF D X Q',
    4707 => 'WHITE DWARF D X X',
    4708 => 'WHITE DWARF D X P',
    4709 => 'WHITE DWARF D X H',
    4710 => 'WHITE DWARF D X A',
    4711 => 'WHITE DWARF D X A PNN',
    4712 => 'WHITE DWARF D X A PNN DETACHED BINARY',
    4713 => 'WHITE DWARF D X A DETACHED BINARY',
    4714 => 'WHITE DWARF D X A O',
    4715 => 'WHITE DWARF D X A Z',
    4716 => 'WHITE DWARF D X A Q',
    4717 => 'WHITE DWARF D X A X',
    4718 => 'WHITE DWARF D X A P',
    4719 => 'WHITE DWARF D X A H',
    4720 => 'WHITE DWARF D X B',
    4721 => 'WHITE DWARF D X B PNN',
    4722 => 'WHITE DWARF D X B PNN DETACHED BINARY',
    4723 => 'WHITE DWARF D X B DETACHED BINARY',
    4724 => 'WHITE DWARF D X B O',
    4725 => 'WHITE DWARF D X B Z',
    4726 => 'WHITE DWARF D X B Q',
    4727 => 'WHITE DWARF D X B X',
    4728 => 'WHITE DWARF D X B P',
    4729 => 'WHITE DWARF D X B H',
    4730 => 'WHITE DWARF D X C',
    4731 => 'WHITE DWARF D X C PNN',
    4732 => 'WHITE DWARF D X C PNN DETACHED BINARY',
    4733 => 'WHITE DWARF D X C DETACHED BINARY',
    4734 => 'WHITE DWARF D X C O',
    4735 => 'WHITE DWARF D X C Z',
    4736 => 'WHITE DWARF D X C Q',
    4737 => 'WHITE DWARF D X C X',
    4738 => 'WHITE DWARF D X C P',
    4739 => 'WHITE DWARF D X C H',
    4740 => 'WHITE DWARF D X O',
    4741 => 'WHITE DWARF D X O PNN',
    4742 => 'WHITE DWARF D X O PNN DETACHED BINARY',
    4743 => 'WHITE DWARF D X O DETACHED BINARY',
    4744 => 'WHITE DWARF D X O O',
    4745 => 'WHITE DWARF D X O Z',
    4746 => 'WHITE DWARF D X O Q',
    4747 => 'WHITE DWARF D X O X',
    4748 => 'WHITE DWARF D X O P',
    4749 => 'WHITE DWARF D X O H',
    4750 => 'WHITE DWARF D X Z',
    4751 => 'WHITE DWARF D X Z PNN',
    4752 => 'WHITE DWARF D X Z PNN DETACHED BINARY',
    4753 => 'WHITE DWARF D X Z DETACHED BINARY',
    4754 => 'WHITE DWARF D X Z O',
    4755 => 'WHITE DWARF D X Z Z',
    4756 => 'WHITE DWARF D X Z Q',
    4757 => 'WHITE DWARF D X Z X',
    4758 => 'WHITE DWARF D X Z P',
    4759 => 'WHITE DWARF D X Z H',
    4760 => 'WHITE DWARF D X Q',
    4761 => 'WHITE DWARF D X Q PNN',
    4762 => 'WHITE DWARF D X Q PNN DETACHED BINARY',
    4763 => 'WHITE DWARF D X Q DETACHED BINARY',
    4764 => 'WHITE DWARF D X Q O',
    4765 => 'WHITE DWARF D X Q Z',
    4766 => 'WHITE DWARF D X Q Q',
    4767 => 'WHITE DWARF D X Q X',
    4768 => 'WHITE DWARF D X Q P',
    4769 => 'WHITE DWARF D X Q H',
    4770 => 'WHITE DWARF D X X',
    4771 => 'WHITE DWARF D X X PNN',
    4772 => 'WHITE DWARF D X X PNN DETACHED BINARY',
    4773 => 'WHITE DWARF D X X DETACHED BINARY',
    4774 => 'WHITE DWARF D X X O',
    4775 => 'WHITE DWARF D X X Z',
    4776 => 'WHITE DWARF D X X Q',
    4777 => 'WHITE DWARF D X X X',
    4778 => 'WHITE DWARF D X X P',
    4779 => 'WHITE DWARF D X X H',
    4780 => 'WHITE DWARF D X V',
    4781 => 'WHITE DWARF D X V PNN',
    4782 => 'WHITE DWARF D X V PNN DETACHED BINARY',
    4783 => 'WHITE DWARF D X V DETACHED BINARY',
    4784 => 'WHITE DWARF D X V O',
    4785 => 'WHITE DWARF D X V Z',
    4786 => 'WHITE DWARF D X V Q',
    4787 => 'WHITE DWARF D X V X',
    4788 => 'WHITE DWARF D X V P',
    4789 => 'WHITE DWARF D X V H',
    4790 => 'WHITE DWARF D X P',
    4791 => 'WHITE DWARF D X P PNN',
    4792 => 'WHITE DWARF D X P PNN DETACHED BINARY',
    4793 => 'WHITE DWARF D X P DETACHED BINARY',
    4794 => 'WHITE DWARF D X P O',
    4795 => 'WHITE DWARF D X P Z',
    4796 => 'WHITE DWARF D X P Q',
    4797 => 'WHITE DWARF D X P X',
    4798 => 'WHITE DWARF D X P P',
    4799 => 'WHITE DWARF D X P H',
    4800 => 'WHITE DWARF sD',
    4801 => 'WHITE DWARF sD PNN',
    4802 => 'WHITE DWARF sD PNN DETACHED BINARY',
    4803 => 'WHITE DWARF sD DETACHED BINARY',
    4804 => 'WHITE DWARF sD O',
    4805 => 'WHITE DWARF sD Z',
    4806 => 'WHITE DWARF sD Q',
    4807 => 'WHITE DWARF sD X',
    4808 => 'WHITE DWARF sD P',
    4809 => 'WHITE DWARF sD H',
    4810 => 'WHITE DWARF sD A',
    4811 => 'WHITE DWARF sD A PNN',
    4812 => 'WHITE DWARF sD A PNN DETACHED BINARY',
    4813 => 'WHITE DWARF sD A DETACHED BINARY',
    4814 => 'WHITE DWARF sD A O',
    4815 => 'WHITE DWARF sD A Z',
    4816 => 'WHITE DWARF sD A Q',
    4817 => 'WHITE DWARF sD A X',
    4818 => 'WHITE DWARF sD A P',
    4819 => 'WHITE DWARF sD A H',
    4820 => 'WHITE DWARF sD B',
    4821 => 'WHITE DWARF sD B PNN',
    4822 => 'WHITE DWARF sD B PNN DETACHED BINARY',
    4823 => 'WHITE DWARF sD B DETACHED BINARY',
    4824 => 'WHITE DWARF sD B O',
    4825 => 'WHITE DWARF sD B Z',
    4826 => 'WHITE DWARF sD B Q',
    4827 => 'WHITE DWARF sD B X',
    4828 => 'WHITE DWARF sD B P',
    4829 => 'WHITE DWARF sD B H',
    4830 => 'WHITE DWARF sD C',
    4831 => 'WHITE DWARF sD C PNN',
    4832 => 'WHITE DWARF sD C PNN DETACHED BINARY',
    4833 => 'WHITE DWARF sD C DETACHED BINARY',
    4834 => 'WHITE DWARF sD C O',
    4835 => 'WHITE DWARF sD C Z',
    4836 => 'WHITE DWARF sD C Q',
    4837 => 'WHITE DWARF sD C X',
    4838 => 'WHITE DWARF sD C P',
    4839 => 'WHITE DWARF sD C H',
    4840 => 'WHITE DWARF sD O',
    4841 => 'WHITE DWARF sD O PNN',
    4842 => 'WHITE DWARF sD O PNN DETACHED BINARY',
    4843 => 'WHITE DWARF sD O DETACHED BINARY',
    4844 => 'WHITE DWARF sD O O',
    4845 => 'WHITE DWARF sD O Z',
    4846 => 'WHITE DWARF sD O Q',
    4847 => 'WHITE DWARF sD O X',
    4848 => 'WHITE DWARF sD O P',
    4849 => 'WHITE DWARF sD O H',
    4850 => 'WHITE DWARF sD Z',
    4851 => 'WHITE DWARF sD Z PNN',
    4852 => 'WHITE DWARF sD Z PNN DETACHED BINARY',
    4853 => 'WHITE DWARF sD Z DETACHED BINARY',
    4854 => 'WHITE DWARF sD Z O',
    4855 => 'WHITE DWARF sD Z Z',
    4856 => 'WHITE DWARF sD Z Q',
    4857 => 'WHITE DWARF sD Z X',
    4858 => 'WHITE DWARF sD Z P',
    4859 => 'WHITE DWARF sD Z H',
    4860 => 'WHITE DWARF sD Q',
    4861 => 'WHITE DWARF sD Q PNN',
    4862 => 'WHITE DWARF sD Q PNN DETACHED BINARY',
    4863 => 'WHITE DWARF sD Q DETACHED BINARY',
    4864 => 'WHITE DWARF sD Q O',
    4865 => 'WHITE DWARF sD Q Z',
    4866 => 'WHITE DWARF sD Q Q',
    4867 => 'WHITE DWARF sD Q X',
    4868 => 'WHITE DWARF sD Q P',
    4869 => 'WHITE DWARF sD Q H',
    4870 => 'WHITE DWARF sD X',
    4871 => 'WHITE DWARF sD X PNN',
    4872 => 'WHITE DWARF sD X PNN DETACHED BINARY',
    4873 => 'WHITE DWARF sD X DETACHED BINARY',
    4874 => 'WHITE DWARF sD X O',
    4875 => 'WHITE DWARF sD X Z',
    4876 => 'WHITE DWARF sD X Q',
    4877 => 'WHITE DWARF sD X X',
    4878 => 'WHITE DWARF sD X P',
    4879 => 'WHITE DWARF sD X H',
    4880 => 'WHITE DWARF sD V',
    4881 => 'WHITE DWARF sD V PNN',
    4882 => 'WHITE DWARF sD V PNN DETACHED BINARY',
    4883 => 'WHITE DWARF sD V DETACHED BINARY',
    4884 => 'WHITE DWARF sD V O',
    4885 => 'WHITE DWARF sD V Z',
    4886 => 'WHITE DWARF sD V Q',
    4887 => 'WHITE DWARF sD V X',
    4888 => 'WHITE DWARF sD V P',
    4889 => 'WHITE DWARF sD V H',
    4890 => 'WHITE DWARF sD P',
    4891 => 'WHITE DWARF sD P PNN',
    4892 => 'WHITE DWARF sD P PNN DETACHED BINARY',
    4893 => 'WHITE DWARF sD P DETACHED BINARY',
    4894 => 'WHITE DWARF sD P O',
    4895 => 'WHITE DWARF sD P Z',
    4896 => 'WHITE DWARF sD P Q',
    4897 => 'WHITE DWARF sD P X',
    4898 => 'WHITE DWARF sD P P',
    4899 => 'WHITE DWARF sD P H',
    4900 => 'WHITE DWARF PG1159-TYPE',
    4901 => 'WHITE DWARF PG1159-TYPE',
    4902 => 'WHITE DWARF PG1159-TYPE',
    4903 => 'WHITE DWARF PG1159-TYPE',
    4904 => 'WHITE DWARF PG1159-TYPE',
    4905 => 'WHITE DWARF PG1159-TYPE',
    4906 => 'WHITE DWARF PG1159-TYPE',
    4907 => 'WHITE DWARF PG1159-TYPE',
    4908 => 'WHITE DWARF PG1159-TYPE',
    4909 => 'WHITE DWARF PG1159-TYPE',
    4910 => 'WHITE DWARF PG1159-TYPE',
    4911 => 'WHITE DWARF PG1159-TYPE',
    4912 => 'WHITE DWARF PG1159-TYPE',
    4913 => 'WHITE DWARF PG1159-TYPE',
    4914 => 'WHITE DWARF PG1159-TYPE',
    4915 => 'WHITE DWARF PG1159-TYPE',
    4916 => 'WHITE DWARF PG1159-TYPE',
    4917 => 'WHITE DWARF PG1159-TYPE',
    4918 => 'WHITE DWARF PG1159-TYPE',
    4919 => 'WHITE DWARF PG1159-TYPE',
    4920 => 'WHITE DWARF PG1159-TYPE',
    4921 => 'WHITE DWARF PG1159-TYPE',
    4922 => 'WHITE DWARF PG1159-TYPE',
    4923 => 'WHITE DWARF PG1159-TYPE',
    4924 => 'WHITE DWARF PG1159-TYPE',
    4925 => 'WHITE DWARF PG1159-TYPE',
    4926 => 'WHITE DWARF PG1159-TYPE',
    4927 => 'WHITE DWARF PG1159-TYPE',
    4928 => 'WHITE DWARF PG1159-TYPE',
    4929 => 'WHITE DWARF PG1159-TYPE',
    4930 => 'WHITE DWARF PG1159-TYPE',
    4931 => 'WHITE DWARF PG1159-TYPE',
    4932 => 'WHITE DWARF PG1159-TYPE',
    4933 => 'WHITE DWARF PG1159-TYPE',
    4934 => 'WHITE DWARF PG1159-TYPE',
    4935 => 'WHITE DWARF PG1159-TYPE',
    4936 => 'WHITE DWARF PG1159-TYPE',
    4937 => 'WHITE DWARF PG1159-TYPE',
    4938 => 'WHITE DWARF PG1159-TYPE',
    4939 => 'WHITE DWARF PG1159-TYPE',
    4940 => 'WHITE DWARF PG1159-TYPE',
    4941 => 'WHITE DWARF PG1159-TYPE',
    4942 => 'WHITE DWARF PG1159-TYPE',
    4943 => 'WHITE DWARF PG1159-TYPE',
    4944 => 'WHITE DWARF PG1159-TYPE',
    4945 => 'WHITE DWARF PG1159-TYPE',
    4946 => 'WHITE DWARF PG1159-TYPE',
    4947 => 'WHITE DWARF PG1159-TYPE',
    4948 => 'WHITE DWARF PG1159-TYPE',
    4949 => 'WHITE DWARF PG1159-TYPE',
    4950 => 'WHITE DWARF PG1159-TYPE',
    4951 => 'WHITE DWARF PG1159-TYPE',
    4952 => 'WHITE DWARF PG1159-TYPE',
    4953 => 'WHITE DWARF PG1159-TYPE',
    4954 => 'WHITE DWARF PG1159-TYPE',
    4955 => 'WHITE DWARF PG1159-TYPE',
    4956 => 'WHITE DWARF PG1159-TYPE',
    4957 => 'WHITE DWARF PG1159-TYPE',
    4958 => 'WHITE DWARF PG1159-TYPE',
    4959 => 'WHITE DWARF PG1159-TYPE',
    4960 => 'WHITE DWARF PG1159-TYPE',
    4961 => 'WHITE DWARF PG1159-TYPE',
    4962 => 'WHITE DWARF PG1159-TYPE',
    4963 => 'WHITE DWARF PG1159-TYPE',
    4964 => 'WHITE DWARF PG1159-TYPE',
    4965 => 'WHITE DWARF PG1159-TYPE',
    4966 => 'WHITE DWARF PG1159-TYPE',
    4967 => 'WHITE DWARF PG1159-TYPE',
    4968 => 'WHITE DWARF PG1159-TYPE',
    4969 => 'WHITE DWARF PG1159-TYPE',
    4970 => 'WHITE DWARF PG1159-TYPE',
    4971 => 'WHITE DWARF PG1159-TYPE',
    4972 => 'WHITE DWARF PG1159-TYPE',
    4973 => 'WHITE DWARF PG1159-TYPE',
    4974 => 'WHITE DWARF PG1159-TYPE',
    4975 => 'WHITE DWARF PG1159-TYPE',
    4976 => 'WHITE DWARF PG1159-TYPE',
    4977 => 'WHITE DWARF PG1159-TYPE',
    4978 => 'WHITE DWARF PG1159-TYPE',
    4979 => 'WHITE DWARF PG1159-TYPE',
    4980 => 'WHITE DWARF PG1159-TYPE',
    4981 => 'WHITE DWARF PG1159-TYPE',
    4982 => 'WHITE DWARF PG1159-TYPE',
    4983 => 'WHITE DWARF PG1159-TYPE',
    4984 => 'WHITE DWARF PG1159-TYPE',
    4985 => 'WHITE DWARF PG1159-TYPE',
    4986 => 'WHITE DWARF PG1159-TYPE',
    4987 => 'WHITE DWARF PG1159-TYPE',
    4988 => 'WHITE DWARF PG1159-TYPE',
    4989 => 'WHITE DWARF PG1159-TYPE',
    4990 => 'WHITE DWARF PG1159-TYPE',
    4991 => 'WHITE DWARF PG1159-TYPE',
    4992 => 'WHITE DWARF PG1159-TYPE',
    4993 => 'WHITE DWARF PG1159-TYPE',
    4994 => 'WHITE DWARF PG1159-TYPE',
    4995 => 'WHITE DWARF PG1159-TYPE',
    4996 => 'WHITE DWARF PG1159-TYPE',
    4997 => 'WHITE DWARF PG1159-TYPE',
    4998 => 'WHITE DWARF PG1159-TYPE',
    4999 => 'WHITE DWARF PG1159-TYPE',
    5000 => 'CLUSTER OF GALAXIES',
    5001 => 'CLUSTER OF GALAXIES WITH COOLING FLOW',
    5002 => 'CLUSTER OF GALAXIES',
    5003 => 'CLUSTER OF GALAXIES',
    5004 => 'CLUSTER OF GALAXIES',
    5005 => 'CLUSTER OF GALAXIES',
    5006 => 'CLUSTER OF GALAXIES',
    5007 => 'CLUSTER OF GALAXIES',
    5008 => 'CLUSTER OF GALAXIES',
    5009 => 'CLUSTER OF GALAXIES',
    5010 => 'CLUSTER OF GALAXIES ABELL CLASS <0',
    5011 => 'CLUSTER OF GALAXIES ABELL CLASS <0 WITH COOLING FLOW',
    5012 => 'CLUSTER OF GALAXIES ABELL CLASS <0',
    5013 => 'CLUSTER OF GALAXIES ABELL CLASS <0',
    5014 => 'CLUSTER OF GALAXIES ABELL CLASS <0',
    5015 => 'CLUSTER OF GALAXIES ABELL CLASS <0',
    5016 => 'CLUSTER OF GALAXIES ABELL CLASS <0',
    5017 => 'CLUSTER OF GALAXIES ABELL CLASS <0',
    5018 => 'CLUSTER OF GALAXIES ABELL CLASS <0',
    5019 => 'CLUSTER OF GALAXIES ABELL CLASS <0',
    5020 => 'CLUSTER OF GALAXIES ABELL CLASS 0',
    5021 => 'CLUSTER OF GALAXIES ABELL CLASS 0 WITH COOLING FLOW',
    5022 => 'CLUSTER OF GALAXIES ABELL CLASS 0',
    5023 => 'CLUSTER OF GALAXIES ABELL CLASS 0',
    5024 => 'CLUSTER OF GALAXIES ABELL CLASS 0',
    5025 => 'CLUSTER OF GALAXIES ABELL CLASS 0',
    5026 => 'CLUSTER OF GALAXIES ABELL CLASS 0',
    5027 => 'CLUSTER OF GALAXIES ABELL CLASS 0',
    5028 => 'CLUSTER OF GALAXIES ABELL CLASS 0',
    5029 => 'CLUSTER OF GALAXIES ABELL CLASS 0',
    5030 => 'CLUSTER OF GALAXIES ABELL CLASS 1',
    5031 => 'CLUSTER OF GALAXIES ABELL CLASS 1 WITH COOLING FLOW',
    5032 => 'CLUSTER OF GALAXIES ABELL CLASS 1',
    5033 => 'CLUSTER OF GALAXIES ABELL CLASS 1',
    5034 => 'CLUSTER OF GALAXIES ABELL CLASS 1',
    5035 => 'CLUSTER OF GALAXIES ABELL CLASS 1',
    5036 => 'CLUSTER OF GALAXIES ABELL CLASS 1',
    5037 => 'CLUSTER OF GALAXIES ABELL CLASS 1',
    5038 => 'CLUSTER OF GALAXIES ABELL CLASS 1',
    5039 => 'CLUSTER OF GALAXIES ABELL CLASS 1',
    5040 => 'CLUSTER OF GALAXIES ABELL CLASS 2',
    5041 => 'CLUSTER OF GALAXIES ABELL CLASS 2 WITH COOLING FLOW',
    5042 => 'CLUSTER OF GALAXIES ABELL CLASS 2',
    5043 => 'CLUSTER OF GALAXIES ABELL CLASS 2',
    5044 => 'CLUSTER OF GALAXIES ABELL CLASS 2',
    5045 => 'CLUSTER OF GALAXIES ABELL CLASS 2',
    5046 => 'CLUSTER OF GALAXIES ABELL CLASS 2',
    5047 => 'CLUSTER OF GALAXIES ABELL CLASS 2',
    5048 => 'CLUSTER OF GALAXIES ABELL CLASS 2',
    5049 => 'CLUSTER OF GALAXIES ABELL CLASS 2',
    5050 => 'CLUSTER OF GALAXIES ABELL CLASS >2',
    5051 => 'CLUSTER OF GALAXIES ABELL CLASS >2 WITH COOLING FLOW',
    5052 => 'CLUSTER OF GALAXIES ABELL CLASS >2',
    5053 => 'CLUSTER OF GALAXIES ABELL CLASS >2',
    5054 => 'CLUSTER OF GALAXIES ABELL CLASS >2',
    5055 => 'CLUSTER OF GALAXIES ABELL CLASS >2',
    5056 => 'CLUSTER OF GALAXIES ABELL CLASS >2',
    5057 => 'CLUSTER OF GALAXIES ABELL CLASS >2',
    5058 => 'CLUSTER OF GALAXIES ABELL CLASS >2',
    5059 => 'CLUSTER OF GALAXIES ABELL CLASS >2',
    5060 => 'CLUSTER OF GALAXIES (NON-ABELL)',
    5061 => 'CLUSTER OF GALAXIES (NON-ABELL) WITH COOLING FLOW',
    5062 => 'CLUSTER OF GALAXIES (NON-ABELL)',
    5063 => 'CLUSTER OF GALAXIES (NON-ABELL)',
    5064 => 'CLUSTER OF GALAXIES (NON-ABELL)',
    5065 => 'CLUSTER OF GALAXIES (NON-ABELL)',
    5066 => 'CLUSTER OF GALAXIES (NON-ABELL)',
    5067 => 'CLUSTER OF GALAXIES (NON-ABELL)',
    5068 => 'CLUSTER OF GALAXIES (NON-ABELL)',
    5069 => 'CLUSTER OF GALAXIES (NON-ABELL)',
    5070 => 'CLUSTER OF GALAXIES (NON-ABELL)',
    5071 => 'CLUSTER OF GALAXIES (NON-ABELL) WITH COOLING FLOW',
    5072 => 'CLUSTER OF GALAXIES (NON-ABELL)',
    5073 => 'CLUSTER OF GALAXIES (NON-ABELL)',
    5074 => 'CLUSTER OF GALAXIES (NON-ABELL)',
    5075 => 'CLUSTER OF GALAXIES (NON-ABELL)',
    5076 => 'CLUSTER OF GALAXIES (NON-ABELL)',
    5077 => 'CLUSTER OF GALAXIES (NON-ABELL)',
    5078 => 'CLUSTER OF GALAXIES (NON-ABELL)',
    5079 => 'CLUSTER OF GALAXIES (NON-ABELL)',
    5080 => 'CLUSTER OF GALAXIES (NON-ABELL)',
    5081 => 'CLUSTER OF GALAXIES (NON-ABELL) WITH COOLING FLOW',
    5082 => 'CLUSTER OF GALAXIES (NON-ABELL)',
    5083 => 'CLUSTER OF GALAXIES (NON-ABELL)',
    5084 => 'CLUSTER OF GALAXIES (NON-ABELL)',
    5085 => 'CLUSTER OF GALAXIES (NON-ABELL)',
    5086 => 'CLUSTER OF GALAXIES (NON-ABELL)',
    5087 => 'CLUSTER OF GALAXIES (NON-ABELL)',
    5088 => 'CLUSTER OF GALAXIES (NON-ABELL)',
    5089 => 'CLUSTER OF GALAXIES (NON-ABELL)',
    5090 => 'CLUSTER OF GALAXIES (NON-ABELL)',
    5091 => 'CLUSTER OF GALAXIES (NON-ABELL) WITH COOLING FLOW',
    5092 => 'CLUSTER OF GALAXIES (NON-ABELL)',
    5093 => 'CLUSTER OF GALAXIES (NON-ABELL)',
    5094 => 'CLUSTER OF GALAXIES (NON-ABELL)',
    5095 => 'CLUSTER OF GALAXIES (NON-ABELL)',
    5096 => 'CLUSTER OF GALAXIES (NON-ABELL)',
    5097 => 'CLUSTER OF GALAXIES (NON-ABELL)',
    5098 => 'CLUSTER OF GALAXIES (NON-ABELL)',
    5099 => 'CLUSTER OF GALAXIES (NON-ABELL)',
    5100 => 'COMPACT GROUP OF GALAXIES',
    5101 => 'COMPACT GROUP OF GALAXIES',
    5102 => 'COMPACT GROUP OF GALAXIES',
    5103 => 'COMPACT GROUP OF GALAXIES',
    5104 => 'COMPACT GROUP OF GALAXIES',
    5105 => 'COMPACT GROUP OF GALAXIES',
    5106 => 'COMPACT GROUP OF GALAXIES',
    5107 => 'COMPACT GROUP OF GALAXIES',
    5108 => 'COMPACT GROUP OF GALAXIES',
    5109 => 'COMPACT GROUP OF GALAXIES',
    5110 => 'COMPACT GROUP OF GALAXIES',
    5111 => 'COMPACT GROUP OF GALAXIES',
    5112 => 'COMPACT GROUP OF GALAXIES',
    5113 => 'COMPACT GROUP OF GALAXIES',
    5114 => 'COMPACT GROUP OF GALAXIES',
    5115 => 'COMPACT GROUP OF GALAXIES',
    5116 => 'COMPACT GROUP OF GALAXIES',
    5117 => 'COMPACT GROUP OF GALAXIES',
    5118 => 'COMPACT GROUP OF GALAXIES',
    5119 => 'COMPACT GROUP OF GALAXIES',
    5120 => 'COMPACT GROUP OF GALAXIES',
    5121 => 'COMPACT GROUP OF GALAXIES',
    5122 => 'COMPACT GROUP OF GALAXIES',
    5123 => 'COMPACT GROUP OF GALAXIES',
    5124 => 'COMPACT GROUP OF GALAXIES',
    5125 => 'COMPACT GROUP OF GALAXIES',
    5126 => 'COMPACT GROUP OF GALAXIES',
    5127 => 'COMPACT GROUP OF GALAXIES',
    5128 => 'COMPACT GROUP OF GALAXIES',
    5129 => 'COMPACT GROUP OF GALAXIES',
    5130 => 'COMPACT GROUP OF GALAXIES',
    5131 => 'COMPACT GROUP OF GALAXIES',
    5132 => 'COMPACT GROUP OF GALAXIES',
    5133 => 'COMPACT GROUP OF GALAXIES',
    5134 => 'COMPACT GROUP OF GALAXIES',
    5135 => 'COMPACT GROUP OF GALAXIES',
    5136 => 'COMPACT GROUP OF GALAXIES',
    5137 => 'COMPACT GROUP OF GALAXIES',
    5138 => 'COMPACT GROUP OF GALAXIES',
    5139 => 'COMPACT GROUP OF GALAXIES',
    5140 => 'COMPACT GROUP OF GALAXIES',
    5141 => 'COMPACT GROUP OF GALAXIES',
    5142 => 'COMPACT GROUP OF GALAXIES',
    5143 => 'COMPACT GROUP OF GALAXIES',
    5144 => 'COMPACT GROUP OF GALAXIES',
    5145 => 'COMPACT GROUP OF GALAXIES',
    5146 => 'COMPACT GROUP OF GALAXIES',
    5147 => 'COMPACT GROUP OF GALAXIES',
    5148 => 'COMPACT GROUP OF GALAXIES',
    5149 => 'COMPACT GROUP OF GALAXIES',
    5150 => 'COMPACT GROUP OF GALAXIES',
    5151 => 'COMPACT GROUP OF GALAXIES',
    5152 => 'COMPACT GROUP OF GALAXIES',
    5153 => 'COMPACT GROUP OF GALAXIES',
    5154 => 'COMPACT GROUP OF GALAXIES',
    5155 => 'COMPACT GROUP OF GALAXIES',
    5156 => 'COMPACT GROUP OF GALAXIES',
    5157 => 'COMPACT GROUP OF GALAXIES',
    5158 => 'COMPACT GROUP OF GALAXIES',
    5159 => 'COMPACT GROUP OF GALAXIES',
    5160 => 'COMPACT GROUP OF GALAXIES',
    5161 => 'COMPACT GROUP OF GALAXIES',
    5162 => 'COMPACT GROUP OF GALAXIES',
    5163 => 'COMPACT GROUP OF GALAXIES',
    5164 => 'COMPACT GROUP OF GALAXIES',
    5165 => 'COMPACT GROUP OF GALAXIES',
    5166 => 'COMPACT GROUP OF GALAXIES',
    5167 => 'COMPACT GROUP OF GALAXIES',
    5168 => 'COMPACT GROUP OF GALAXIES',
    5169 => 'COMPACT GROUP OF GALAXIES',
    5170 => 'COMPACT GROUP OF GALAXIES',
    5171 => 'COMPACT GROUP OF GALAXIES',
    5172 => 'COMPACT GROUP OF GALAXIES',
    5173 => 'COMPACT GROUP OF GALAXIES',
    5174 => 'COMPACT GROUP OF GALAXIES',
    5175 => 'COMPACT GROUP OF GALAXIES',
    5176 => 'COMPACT GROUP OF GALAXIES',
    5177 => 'COMPACT GROUP OF GALAXIES',
    5178 => 'COMPACT GROUP OF GALAXIES',
    5179 => 'COMPACT GROUP OF GALAXIES',
    5180 => 'COMPACT GROUP OF GALAXIES',
    5181 => 'COMPACT GROUP OF GALAXIES',
    5182 => 'COMPACT GROUP OF GALAXIES',
    5183 => 'COMPACT GROUP OF GALAXIES',
    5184 => 'COMPACT GROUP OF GALAXIES',
    5185 => 'COMPACT GROUP OF GALAXIES',
    5186 => 'COMPACT GROUP OF GALAXIES',
    5187 => 'COMPACT GROUP OF GALAXIES',
    5188 => 'COMPACT GROUP OF GALAXIES',
    5189 => 'COMPACT GROUP OF GALAXIES',
    5190 => 'COMPACT GROUP OF GALAXIES',
    5191 => 'COMPACT GROUP OF GALAXIES',
    5192 => 'COMPACT GROUP OF GALAXIES',
    5193 => 'COMPACT GROUP OF GALAXIES',
    5194 => 'COMPACT GROUP OF GALAXIES',
    5195 => 'COMPACT GROUP OF GALAXIES',
    5196 => 'COMPACT GROUP OF GALAXIES',
    5197 => 'COMPACT GROUP OF GALAXIES',
    5198 => 'COMPACT GROUP OF GALAXIES',
    5199 => 'COMPACT GROUP OF GALAXIES',
    5200 => 'UNCLASSIFIED',
    5201 => 'UNCLASSIFIED',
    5202 => 'UNCLASSIFIED',
    5203 => 'UNCLASSIFIED',
    5204 => 'UNCLASSIFIED',
    5205 => 'UNCLASSIFIED',
    5206 => 'UNCLASSIFIED',
    5207 => 'UNCLASSIFIED',
    5208 => 'UNCLASSIFIED',
    5209 => 'UNCLASSIFIED',
    5210 => 'UNCLASSIFIED',
    5211 => 'UNCLASSIFIED',
    5212 => 'UNCLASSIFIED',
    5213 => 'UNCLASSIFIED',
    5214 => 'UNCLASSIFIED',
    5215 => 'UNCLASSIFIED',
    5216 => 'UNCLASSIFIED',
    5217 => 'UNCLASSIFIED',
    5218 => 'UNCLASSIFIED',
    5219 => 'UNCLASSIFIED',
    5220 => 'UNCLASSIFIED',
    5221 => 'UNCLASSIFIED',
    5222 => 'UNCLASSIFIED',
    5223 => 'UNCLASSIFIED',
    5224 => 'UNCLASSIFIED',
    5225 => 'UNCLASSIFIED',
    5226 => 'UNCLASSIFIED',
    5227 => 'UNCLASSIFIED',
    5228 => 'UNCLASSIFIED',
    5229 => 'UNCLASSIFIED',
    5230 => 'UNCLASSIFIED',
    5231 => 'UNCLASSIFIED',
    5232 => 'UNCLASSIFIED',
    5233 => 'UNCLASSIFIED',
    5234 => 'UNCLASSIFIED',
    5235 => 'UNCLASSIFIED',
    5236 => 'UNCLASSIFIED',
    5237 => 'UNCLASSIFIED',
    5238 => 'UNCLASSIFIED',
    5239 => 'UNCLASSIFIED',
    5240 => 'UNCLASSIFIED',
    5241 => 'UNCLASSIFIED',
    5242 => 'UNCLASSIFIED',
    5243 => 'UNCLASSIFIED',
    5244 => 'UNCLASSIFIED',
    5245 => 'UNCLASSIFIED',
    5246 => 'UNCLASSIFIED',
    5247 => 'UNCLASSIFIED',
    5248 => 'UNCLASSIFIED',
    5249 => 'UNCLASSIFIED',
    5250 => 'UNCLASSIFIED',
    5251 => 'UNCLASSIFIED',
    5252 => 'UNCLASSIFIED',
    5253 => 'UNCLASSIFIED',
    5254 => 'UNCLASSIFIED',
    5255 => 'UNCLASSIFIED',
    5256 => 'UNCLASSIFIED',
    5257 => 'UNCLASSIFIED',
    5258 => 'UNCLASSIFIED',
    5259 => 'UNCLASSIFIED',
    5260 => 'UNCLASSIFIED',
    5261 => 'UNCLASSIFIED',
    5262 => 'UNCLASSIFIED',
    5263 => 'UNCLASSIFIED',
    5264 => 'UNCLASSIFIED',
    5265 => 'UNCLASSIFIED',
    5266 => 'UNCLASSIFIED',
    5267 => 'UNCLASSIFIED',
    5268 => 'UNCLASSIFIED',
    5269 => 'UNCLASSIFIED',
    5270 => 'UNCLASSIFIED',
    5271 => 'UNCLASSIFIED',
    5272 => 'UNCLASSIFIED',
    5273 => 'UNCLASSIFIED',
    5274 => 'UNCLASSIFIED',
    5275 => 'UNCLASSIFIED',
    5276 => 'UNCLASSIFIED',
    5277 => 'UNCLASSIFIED',
    5278 => 'UNCLASSIFIED',
    5279 => 'UNCLASSIFIED',
    5280 => 'UNCLASSIFIED',
    5281 => 'UNCLASSIFIED',
    5282 => 'UNCLASSIFIED',
    5283 => 'UNCLASSIFIED',
    5284 => 'UNCLASSIFIED',
    5285 => 'UNCLASSIFIED',
    5286 => 'UNCLASSIFIED',
    5287 => 'UNCLASSIFIED',
    5288 => 'UNCLASSIFIED',
    5289 => 'UNCLASSIFIED',
    5290 => 'UNCLASSIFIED',
    5291 => 'UNCLASSIFIED',
    5292 => 'UNCLASSIFIED',
    5293 => 'UNCLASSIFIED',
    5294 => 'UNCLASSIFIED',
    5295 => 'UNCLASSIFIED',
    5296 => 'UNCLASSIFIED',
    5297 => 'UNCLASSIFIED',
    5298 => 'UNCLASSIFIED',
    5299 => 'UNCLASSIFIED',
    5300 => 'UNCLASSIFIED',
    5301 => 'UNCLASSIFIED',
    5302 => 'UNCLASSIFIED',
    5303 => 'UNCLASSIFIED',
    5304 => 'UNCLASSIFIED',
    5305 => 'UNCLASSIFIED',
    5306 => 'UNCLASSIFIED',
    5307 => 'UNCLASSIFIED',
    5308 => 'UNCLASSIFIED',
    5309 => 'UNCLASSIFIED',
    5310 => 'UNCLASSIFIED',
    5311 => 'UNCLASSIFIED',
    5312 => 'UNCLASSIFIED',
    5313 => 'UNCLASSIFIED',
    5314 => 'UNCLASSIFIED',
    5315 => 'UNCLASSIFIED',
    5316 => 'UNCLASSIFIED',
    5317 => 'UNCLASSIFIED',
    5318 => 'UNCLASSIFIED',
    5319 => 'UNCLASSIFIED',
    5320 => 'UNCLASSIFIED',
    5321 => 'UNCLASSIFIED',
    5322 => 'UNCLASSIFIED',
    5323 => 'UNCLASSIFIED',
    5324 => 'UNCLASSIFIED',
    5325 => 'UNCLASSIFIED',
    5326 => 'UNCLASSIFIED',
    5327 => 'UNCLASSIFIED',
    5328 => 'UNCLASSIFIED',
    5329 => 'UNCLASSIFIED',
    5330 => 'UNCLASSIFIED',
    5331 => 'UNCLASSIFIED',
    5332 => 'UNCLASSIFIED',
    5333 => 'UNCLASSIFIED',
    5334 => 'UNCLASSIFIED',
    5335 => 'UNCLASSIFIED',
    5336 => 'UNCLASSIFIED',
    5337 => 'UNCLASSIFIED',
    5338 => 'UNCLASSIFIED',
    5339 => 'UNCLASSIFIED',
    5340 => 'UNCLASSIFIED',
    5341 => 'UNCLASSIFIED',
    5342 => 'UNCLASSIFIED',
    5343 => 'UNCLASSIFIED',
    5344 => 'UNCLASSIFIED',
    5345 => 'UNCLASSIFIED',
    5346 => 'UNCLASSIFIED',
    5347 => 'UNCLASSIFIED',
    5348 => 'UNCLASSIFIED',
    5349 => 'UNCLASSIFIED',
    5350 => 'UNCLASSIFIED',
    5351 => 'UNCLASSIFIED',
    5352 => 'UNCLASSIFIED',
    5353 => 'UNCLASSIFIED',
    5354 => 'UNCLASSIFIED',
    5355 => 'UNCLASSIFIED',
    5356 => 'UNCLASSIFIED',
    5357 => 'UNCLASSIFIED',
    5358 => 'UNCLASSIFIED',
    5359 => 'UNCLASSIFIED',
    5360 => 'UNCLASSIFIED',
    5361 => 'UNCLASSIFIED',
    5362 => 'UNCLASSIFIED',
    5363 => 'UNCLASSIFIED',
    5364 => 'UNCLASSIFIED',
    5365 => 'UNCLASSIFIED',
    5366 => 'UNCLASSIFIED',
    5367 => 'UNCLASSIFIED',
    5368 => 'UNCLASSIFIED',
    5369 => 'UNCLASSIFIED',
    5370 => 'UNCLASSIFIED',
    5371 => 'UNCLASSIFIED',
    5372 => 'UNCLASSIFIED',
    5373 => 'UNCLASSIFIED',
    5374 => 'UNCLASSIFIED',
    5375 => 'UNCLASSIFIED',
    5376 => 'UNCLASSIFIED',
    5377 => 'UNCLASSIFIED',
    5378 => 'UNCLASSIFIED',
    5379 => 'UNCLASSIFIED',
    5380 => 'UNCLASSIFIED',
    5381 => 'UNCLASSIFIED',
    5382 => 'UNCLASSIFIED',
    5383 => 'UNCLASSIFIED',
    5384 => 'UNCLASSIFIED',
    5385 => 'UNCLASSIFIED',
    5386 => 'UNCLASSIFIED',
    5387 => 'UNCLASSIFIED',
    5388 => 'UNCLASSIFIED',
    5389 => 'UNCLASSIFIED',
    5390 => 'UNCLASSIFIED',
    5391 => 'UNCLASSIFIED',
    5392 => 'UNCLASSIFIED',
    5393 => 'UNCLASSIFIED',
    5394 => 'UNCLASSIFIED',
    5395 => 'UNCLASSIFIED',
    5396 => 'UNCLASSIFIED',
    5397 => 'UNCLASSIFIED',
    5398 => 'UNCLASSIFIED',
    5399 => 'UNCLASSIFIED',
    5400 => 'UNCLASSIFIED',
    5401 => 'UNCLASSIFIED',
    5402 => 'UNCLASSIFIED',
    5403 => 'UNCLASSIFIED',
    5404 => 'UNCLASSIFIED',
    5405 => 'UNCLASSIFIED',
    5406 => 'UNCLASSIFIED',
    5407 => 'UNCLASSIFIED',
    5408 => 'UNCLASSIFIED',
    5409 => 'UNCLASSIFIED',
    5410 => 'UNCLASSIFIED',
    5411 => 'UNCLASSIFIED',
    5412 => 'UNCLASSIFIED',
    5413 => 'UNCLASSIFIED',
    5414 => 'UNCLASSIFIED',
    5415 => 'UNCLASSIFIED',
    5416 => 'UNCLASSIFIED',
    5417 => 'UNCLASSIFIED',
    5418 => 'UNCLASSIFIED',
    5419 => 'UNCLASSIFIED',
    5420 => 'UNCLASSIFIED',
    5421 => 'UNCLASSIFIED',
    5422 => 'UNCLASSIFIED',
    5423 => 'UNCLASSIFIED',
    5424 => 'UNCLASSIFIED',
    5425 => 'UNCLASSIFIED',
    5426 => 'UNCLASSIFIED',
    5427 => 'UNCLASSIFIED',
    5428 => 'UNCLASSIFIED',
    5429 => 'UNCLASSIFIED',
    5430 => 'UNCLASSIFIED',
    5431 => 'UNCLASSIFIED',
    5432 => 'UNCLASSIFIED',
    5433 => 'UNCLASSIFIED',
    5434 => 'UNCLASSIFIED',
    5435 => 'UNCLASSIFIED',
    5436 => 'UNCLASSIFIED',
    5437 => 'UNCLASSIFIED',
    5438 => 'UNCLASSIFIED',
    5439 => 'UNCLASSIFIED',
    5440 => 'UNCLASSIFIED',
    5441 => 'UNCLASSIFIED',
    5442 => 'UNCLASSIFIED',
    5443 => 'UNCLASSIFIED',
    5444 => 'UNCLASSIFIED',
    5445 => 'UNCLASSIFIED',
    5446 => 'UNCLASSIFIED',
    5447 => 'UNCLASSIFIED',
    5448 => 'UNCLASSIFIED',
    5449 => 'UNCLASSIFIED',
    5450 => 'UNCLASSIFIED',
    5451 => 'UNCLASSIFIED',
    5452 => 'UNCLASSIFIED',
    5453 => 'UNCLASSIFIED',
    5454 => 'UNCLASSIFIED',
    5455 => 'UNCLASSIFIED',
    5456 => 'UNCLASSIFIED',
    5457 => 'UNCLASSIFIED',
    5458 => 'UNCLASSIFIED',
    5459 => 'UNCLASSIFIED',
    5460 => 'UNCLASSIFIED',
    5461 => 'UNCLASSIFIED',
    5462 => 'UNCLASSIFIED',
    5463 => 'UNCLASSIFIED',
    5464 => 'UNCLASSIFIED',
    5465 => 'UNCLASSIFIED',
    5466 => 'UNCLASSIFIED',
    5467 => 'UNCLASSIFIED',
    5468 => 'UNCLASSIFIED',
    5469 => 'UNCLASSIFIED',
    5470 => 'UNCLASSIFIED',
    5471 => 'UNCLASSIFIED',
    5472 => 'UNCLASSIFIED',
    5473 => 'UNCLASSIFIED',
    5474 => 'UNCLASSIFIED',
    5475 => 'UNCLASSIFIED',
    5476 => 'UNCLASSIFIED',
    5477 => 'UNCLASSIFIED',
    5478 => 'UNCLASSIFIED',
    5479 => 'UNCLASSIFIED',
    5480 => 'UNCLASSIFIED',
    5481 => 'UNCLASSIFIED',
    5482 => 'UNCLASSIFIED',
    5483 => 'UNCLASSIFIED',
    5484 => 'UNCLASSIFIED',
    5485 => 'UNCLASSIFIED',
    5486 => 'UNCLASSIFIED',
    5487 => 'UNCLASSIFIED',
    5488 => 'UNCLASSIFIED',
    5489 => 'UNCLASSIFIED',
    5490 => 'UNCLASSIFIED',
    5491 => 'UNCLASSIFIED',
    5492 => 'UNCLASSIFIED',
    5493 => 'UNCLASSIFIED',
    5494 => 'UNCLASSIFIED',
    5495 => 'UNCLASSIFIED',
    5496 => 'UNCLASSIFIED',
    5497 => 'UNCLASSIFIED',
    5498 => 'UNCLASSIFIED',
    5499 => 'UNCLASSIFIED',
    5500 => 'X-RAY BACKGROUND',
    5501 => 'X-RAY BACKGROUND',
    5502 => 'X-RAY BACKGROUND',
    5503 => 'X-RAY BACKGROUND',
    5504 => 'X-RAY BACKGROUND',
    5505 => 'X-RAY BACKGROUND',
    5506 => 'X-RAY BACKGROUND',
    5507 => 'X-RAY BACKGROUND',
    5508 => 'X-RAY BACKGROUND',
    5509 => 'X-RAY BACKGROUND',
    5510 => 'X-RAY BACKGROUND',
    5511 => 'X-RAY BACKGROUND',
    5512 => 'X-RAY BACKGROUND',
    5513 => 'X-RAY BACKGROUND',
    5514 => 'X-RAY BACKGROUND',
    5515 => 'X-RAY BACKGROUND',
    5516 => 'X-RAY BACKGROUND',
    5517 => 'X-RAY BACKGROUND',
    5518 => 'X-RAY BACKGROUND',
    5519 => 'X-RAY BACKGROUND',
    5520 => 'X-RAY BACKGROUND',
    5521 => 'X-RAY BACKGROUND',
    5522 => 'X-RAY BACKGROUND',
    5523 => 'X-RAY BACKGROUND',
    5524 => 'X-RAY BACKGROUND',
    5525 => 'X-RAY BACKGROUND',
    5526 => 'X-RAY BACKGROUND',
    5527 => 'X-RAY BACKGROUND',
    5528 => 'X-RAY BACKGROUND',
    5529 => 'X-RAY BACKGROUND',
    5530 => 'X-RAY BACKGROUND',
    5531 => 'X-RAY BACKGROUND',
    5532 => 'X-RAY BACKGROUND',
    5533 => 'X-RAY BACKGROUND',
    5534 => 'X-RAY BACKGROUND',
    5535 => 'X-RAY BACKGROUND',
    5536 => 'X-RAY BACKGROUND',
    5537 => 'X-RAY BACKGROUND',
    5538 => 'X-RAY BACKGROUND',
    5539 => 'X-RAY BACKGROUND',
    5540 => 'X-RAY BACKGROUND',
    5541 => 'X-RAY BACKGROUND',
    5542 => 'X-RAY BACKGROUND',
    5543 => 'X-RAY BACKGROUND',
    5544 => 'X-RAY BACKGROUND',
    5545 => 'X-RAY BACKGROUND',
    5546 => 'X-RAY BACKGROUND',
    5547 => 'X-RAY BACKGROUND',
    5548 => 'X-RAY BACKGROUND',
    5549 => 'X-RAY BACKGROUND',
    5550 => 'X-RAY BACKGROUND',
    5551 => 'X-RAY BACKGROUND',
    5552 => 'X-RAY BACKGROUND',
    5553 => 'X-RAY BACKGROUND',
    5554 => 'X-RAY BACKGROUND',
    5555 => 'X-RAY BACKGROUND',
    5556 => 'X-RAY BACKGROUND',
    5557 => 'X-RAY BACKGROUND',
    5558 => 'X-RAY BACKGROUND',
    5559 => 'X-RAY BACKGROUND',
    5560 => 'X-RAY BACKGROUND',
    5561 => 'X-RAY BACKGROUND',
    5562 => 'X-RAY BACKGROUND',
    5563 => 'X-RAY BACKGROUND',
    5564 => 'X-RAY BACKGROUND',
    5565 => 'X-RAY BACKGROUND',
    5566 => 'X-RAY BACKGROUND',
    5567 => 'X-RAY BACKGROUND',
    5568 => 'X-RAY BACKGROUND',
    5569 => 'X-RAY BACKGROUND',
    5570 => 'X-RAY BACKGROUND',
    5571 => 'X-RAY BACKGROUND',
    5572 => 'X-RAY BACKGROUND',
    5573 => 'X-RAY BACKGROUND',
    5574 => 'X-RAY BACKGROUND',
    5575 => 'X-RAY BACKGROUND',
    5576 => 'X-RAY BACKGROUND',
    5577 => 'X-RAY BACKGROUND',
    5578 => 'X-RAY BACKGROUND',
    5579 => 'X-RAY BACKGROUND',
    5580 => 'X-RAY BACKGROUND',
    5581 => 'X-RAY BACKGROUND',
    5582 => 'X-RAY BACKGROUND',
    5583 => 'X-RAY BACKGROUND',
    5584 => 'X-RAY BACKGROUND',
    5585 => 'X-RAY BACKGROUND',
    5586 => 'X-RAY BACKGROUND',
    5587 => 'X-RAY BACKGROUND',
    5588 => 'X-RAY BACKGROUND',
    5589 => 'X-RAY BACKGROUND',
    5590 => 'X-RAY BACKGROUND',
    5591 => 'X-RAY BACKGROUND',
    5592 => 'X-RAY BACKGROUND',
    5593 => 'X-RAY BACKGROUND',
    5594 => 'X-RAY BACKGROUND',
    5595 => 'X-RAY BACKGROUND',
    5596 => 'X-RAY BACKGROUND',
    5597 => 'X-RAY BACKGROUND',
    5598 => 'X-RAY BACKGROUND',
    5599 => 'X-RAY BACKGROUND',
    5600 => 'UNCLASSIFIED',
    5601 => 'UNCLASSIFIED',
    5602 => 'UNCLASSIFIED',
    5603 => 'UNCLASSIFIED',
    5604 => 'UNCLASSIFIED',
    5605 => 'UNCLASSIFIED',
    5606 => 'UNCLASSIFIED',
    5607 => 'UNCLASSIFIED',
    5608 => 'UNCLASSIFIED',
    5609 => 'UNCLASSIFIED',
    5610 => 'UNCLASSIFIED',
    5611 => 'UNCLASSIFIED',
    5612 => 'UNCLASSIFIED',
    5613 => 'UNCLASSIFIED',
    5614 => 'UNCLASSIFIED',
    5615 => 'UNCLASSIFIED',
    5616 => 'UNCLASSIFIED',
    5617 => 'UNCLASSIFIED',
    5618 => 'UNCLASSIFIED',
    5619 => 'UNCLASSIFIED',
    5620 => 'UNCLASSIFIED',
    5621 => 'UNCLASSIFIED',
    5622 => 'UNCLASSIFIED',
    5623 => 'UNCLASSIFIED',
    5624 => 'UNCLASSIFIED',
    5625 => 'UNCLASSIFIED',
    5626 => 'UNCLASSIFIED',
    5627 => 'UNCLASSIFIED',
    5628 => 'UNCLASSIFIED',
    5629 => 'UNCLASSIFIED',
    5630 => 'UNCLASSIFIED',
    5631 => 'UNCLASSIFIED',
    5632 => 'UNCLASSIFIED',
    5633 => 'UNCLASSIFIED',
    5634 => 'UNCLASSIFIED',
    5635 => 'UNCLASSIFIED',
    5636 => 'UNCLASSIFIED',
    5637 => 'UNCLASSIFIED',
    5638 => 'UNCLASSIFIED',
    5639 => 'UNCLASSIFIED',
    5640 => 'UNCLASSIFIED',
    5641 => 'UNCLASSIFIED',
    5642 => 'UNCLASSIFIED',
    5643 => 'UNCLASSIFIED',
    5644 => 'UNCLASSIFIED',
    5645 => 'UNCLASSIFIED',
    5646 => 'UNCLASSIFIED',
    5647 => 'UNCLASSIFIED',
    5648 => 'UNCLASSIFIED',
    5649 => 'UNCLASSIFIED',
    5650 => 'UNCLASSIFIED',
    5651 => 'UNCLASSIFIED',
    5652 => 'UNCLASSIFIED',
    5653 => 'UNCLASSIFIED',
    5654 => 'UNCLASSIFIED',
    5655 => 'UNCLASSIFIED',
    5656 => 'UNCLASSIFIED',
    5657 => 'UNCLASSIFIED',
    5658 => 'UNCLASSIFIED',
    5659 => 'UNCLASSIFIED',
    5660 => 'UNCLASSIFIED',
    5661 => 'UNCLASSIFIED',
    5662 => 'UNCLASSIFIED',
    5663 => 'UNCLASSIFIED',
    5664 => 'UNCLASSIFIED',
    5665 => 'UNCLASSIFIED',
    5666 => 'UNCLASSIFIED',
    5667 => 'UNCLASSIFIED',
    5668 => 'UNCLASSIFIED',
    5669 => 'UNCLASSIFIED',
    5670 => 'UNCLASSIFIED',
    5671 => 'UNCLASSIFIED',
    5672 => 'UNCLASSIFIED',
    5673 => 'UNCLASSIFIED',
    5674 => 'UNCLASSIFIED',
    5675 => 'UNCLASSIFIED',
    5676 => 'UNCLASSIFIED',
    5677 => 'UNCLASSIFIED',
    5678 => 'UNCLASSIFIED',
    5679 => 'UNCLASSIFIED',
    5680 => 'UNCLASSIFIED',
    5681 => 'UNCLASSIFIED',
    5682 => 'UNCLASSIFIED',
    5683 => 'UNCLASSIFIED',
    5684 => 'UNCLASSIFIED',
    5685 => 'UNCLASSIFIED',
    5686 => 'UNCLASSIFIED',
    5687 => 'UNCLASSIFIED',
    5688 => 'UNCLASSIFIED',
    5689 => 'UNCLASSIFIED',
    5690 => 'UNCLASSIFIED',
    5691 => 'UNCLASSIFIED',
    5692 => 'UNCLASSIFIED',
    5693 => 'UNCLASSIFIED',
    5694 => 'UNCLASSIFIED',
    5695 => 'UNCLASSIFIED',
    5696 => 'UNCLASSIFIED',
    5697 => 'UNCLASSIFIED',
    5698 => 'UNCLASSIFIED',
    5699 => 'UNCLASSIFIED',
    5700 => 'UNCLASSIFIED',
    5701 => 'UNCLASSIFIED',
    5702 => 'UNCLASSIFIED',
    5703 => 'UNCLASSIFIED',
    5704 => 'UNCLASSIFIED',
    5705 => 'UNCLASSIFIED',
    5706 => 'UNCLASSIFIED',
    5707 => 'UNCLASSIFIED',
    5708 => 'UNCLASSIFIED',
    5709 => 'UNCLASSIFIED',
    5710 => 'UNCLASSIFIED',
    5711 => 'UNCLASSIFIED',
    5712 => 'UNCLASSIFIED',
    5713 => 'UNCLASSIFIED',
    5714 => 'UNCLASSIFIED',
    5715 => 'UNCLASSIFIED',
    5716 => 'UNCLASSIFIED',
    5717 => 'UNCLASSIFIED',
    5718 => 'UNCLASSIFIED',
    5719 => 'UNCLASSIFIED',
    5720 => 'UNCLASSIFIED',
    5721 => 'UNCLASSIFIED',
    5722 => 'UNCLASSIFIED',
    5723 => 'UNCLASSIFIED',
    5724 => 'UNCLASSIFIED',
    5725 => 'UNCLASSIFIED',
    5726 => 'UNCLASSIFIED',
    5727 => 'UNCLASSIFIED',
    5728 => 'UNCLASSIFIED',
    5729 => 'UNCLASSIFIED',
    5730 => 'UNCLASSIFIED',
    5731 => 'UNCLASSIFIED',
    5732 => 'UNCLASSIFIED',
    5733 => 'UNCLASSIFIED',
    5734 => 'UNCLASSIFIED',
    5735 => 'UNCLASSIFIED',
    5736 => 'UNCLASSIFIED',
    5737 => 'UNCLASSIFIED',
    5738 => 'UNCLASSIFIED',
    5739 => 'UNCLASSIFIED',
    5740 => 'UNCLASSIFIED',
    5741 => 'UNCLASSIFIED',
    5742 => 'UNCLASSIFIED',
    5743 => 'UNCLASSIFIED',
    5744 => 'UNCLASSIFIED',
    5745 => 'UNCLASSIFIED',
    5746 => 'UNCLASSIFIED',
    5747 => 'UNCLASSIFIED',
    5748 => 'UNCLASSIFIED',
    5749 => 'UNCLASSIFIED',
    5750 => 'UNCLASSIFIED',
    5751 => 'UNCLASSIFIED',
    5752 => 'UNCLASSIFIED',
    5753 => 'UNCLASSIFIED',
    5754 => 'UNCLASSIFIED',
    5755 => 'UNCLASSIFIED',
    5756 => 'UNCLASSIFIED',
    5757 => 'UNCLASSIFIED',
    5758 => 'UNCLASSIFIED',
    5759 => 'UNCLASSIFIED',
    5760 => 'UNCLASSIFIED',
    5761 => 'UNCLASSIFIED',
    5762 => 'UNCLASSIFIED',
    5763 => 'UNCLASSIFIED',
    5764 => 'UNCLASSIFIED',
    5765 => 'UNCLASSIFIED',
    5766 => 'UNCLASSIFIED',
    5767 => 'UNCLASSIFIED',
    5768 => 'UNCLASSIFIED',
    5769 => 'UNCLASSIFIED',
    5770 => 'UNCLASSIFIED',
    5771 => 'UNCLASSIFIED',
    5772 => 'UNCLASSIFIED',
    5773 => 'UNCLASSIFIED',
    5774 => 'UNCLASSIFIED',
    5775 => 'UNCLASSIFIED',
    5776 => 'UNCLASSIFIED',
    5777 => 'UNCLASSIFIED',
    5778 => 'UNCLASSIFIED',
    5779 => 'UNCLASSIFIED',
    5780 => 'UNCLASSIFIED',
    5781 => 'UNCLASSIFIED',
    5782 => 'UNCLASSIFIED',
    5783 => 'UNCLASSIFIED',
    5784 => 'UNCLASSIFIED',
    5785 => 'UNCLASSIFIED',
    5786 => 'UNCLASSIFIED',
    5787 => 'UNCLASSIFIED',
    5788 => 'UNCLASSIFIED',
    5789 => 'UNCLASSIFIED',
    5790 => 'UNCLASSIFIED',
    5791 => 'UNCLASSIFIED',
    5792 => 'UNCLASSIFIED',
    5793 => 'UNCLASSIFIED',
    5794 => 'UNCLASSIFIED',
    5795 => 'UNCLASSIFIED',
    5796 => 'UNCLASSIFIED',
    5797 => 'UNCLASSIFIED',
    5798 => 'UNCLASSIFIED',
    5799 => 'UNCLASSIFIED',
    5800 => 'UNCLASSIFIED',
    5801 => 'UNCLASSIFIED',
    5802 => 'UNCLASSIFIED',
    5803 => 'UNCLASSIFIED',
    5804 => 'UNCLASSIFIED',
    5805 => 'UNCLASSIFIED',
    5806 => 'UNCLASSIFIED',
    5807 => 'UNCLASSIFIED',
    5808 => 'UNCLASSIFIED',
    5809 => 'UNCLASSIFIED',
    5810 => 'UNCLASSIFIED',
    5811 => 'UNCLASSIFIED',
    5812 => 'UNCLASSIFIED',
    5813 => 'UNCLASSIFIED',
    5814 => 'UNCLASSIFIED',
    5815 => 'UNCLASSIFIED',
    5816 => 'UNCLASSIFIED',
    5817 => 'UNCLASSIFIED',
    5818 => 'UNCLASSIFIED',
    5819 => 'UNCLASSIFIED',
    5820 => 'UNCLASSIFIED',
    5821 => 'UNCLASSIFIED',
    5822 => 'UNCLASSIFIED',
    5823 => 'UNCLASSIFIED',
    5824 => 'UNCLASSIFIED',
    5825 => 'UNCLASSIFIED',
    5826 => 'UNCLASSIFIED',
    5827 => 'UNCLASSIFIED',
    5828 => 'UNCLASSIFIED',
    5829 => 'UNCLASSIFIED',
    5830 => 'UNCLASSIFIED',
    5831 => 'UNCLASSIFIED',
    5832 => 'UNCLASSIFIED',
    5833 => 'UNCLASSIFIED',
    5834 => 'UNCLASSIFIED',
    5835 => 'UNCLASSIFIED',
    5836 => 'UNCLASSIFIED',
    5837 => 'UNCLASSIFIED',
    5838 => 'UNCLASSIFIED',
    5839 => 'UNCLASSIFIED',
    5840 => 'UNCLASSIFIED',
    5841 => 'UNCLASSIFIED',
    5842 => 'UNCLASSIFIED',
    5843 => 'UNCLASSIFIED',
    5844 => 'UNCLASSIFIED',
    5845 => 'UNCLASSIFIED',
    5846 => 'UNCLASSIFIED',
    5847 => 'UNCLASSIFIED',
    5848 => 'UNCLASSIFIED',
    5849 => 'UNCLASSIFIED',
    5850 => 'UNCLASSIFIED',
    5851 => 'UNCLASSIFIED',
    5852 => 'UNCLASSIFIED',
    5853 => 'UNCLASSIFIED',
    5854 => 'UNCLASSIFIED',
    5855 => 'UNCLASSIFIED',
    5856 => 'UNCLASSIFIED',
    5857 => 'UNCLASSIFIED',
    5858 => 'UNCLASSIFIED',
    5859 => 'UNCLASSIFIED',
    5860 => 'UNCLASSIFIED',
    5861 => 'UNCLASSIFIED',
    5862 => 'UNCLASSIFIED',
    5863 => 'UNCLASSIFIED',
    5864 => 'UNCLASSIFIED',
    5865 => 'UNCLASSIFIED',
    5866 => 'UNCLASSIFIED',
    5867 => 'UNCLASSIFIED',
    5868 => 'UNCLASSIFIED',
    5869 => 'UNCLASSIFIED',
    5870 => 'UNCLASSIFIED',
    5871 => 'UNCLASSIFIED',
    5872 => 'UNCLASSIFIED',
    5873 => 'UNCLASSIFIED',
    5874 => 'UNCLASSIFIED',
    5875 => 'UNCLASSIFIED',
    5876 => 'UNCLASSIFIED',
    5877 => 'UNCLASSIFIED',
    5878 => 'UNCLASSIFIED',
    5879 => 'UNCLASSIFIED',
    5880 => 'UNCLASSIFIED',
    5881 => 'UNCLASSIFIED',
    5882 => 'UNCLASSIFIED',
    5883 => 'UNCLASSIFIED',
    5884 => 'UNCLASSIFIED',
    5885 => 'UNCLASSIFIED',
    5886 => 'UNCLASSIFIED',
    5887 => 'UNCLASSIFIED',
    5888 => 'UNCLASSIFIED',
    5889 => 'UNCLASSIFIED',
    5890 => 'UNCLASSIFIED',
    5891 => 'UNCLASSIFIED',
    5892 => 'UNCLASSIFIED',
    5893 => 'UNCLASSIFIED',
    5894 => 'UNCLASSIFIED',
    5895 => 'UNCLASSIFIED',
    5896 => 'UNCLASSIFIED',
    5897 => 'UNCLASSIFIED',
    5898 => 'UNCLASSIFIED',
    5899 => 'UNCLASSIFIED',
    5900 => 'UNCLASSIFIED',
    5901 => 'UNCLASSIFIED',
    5902 => 'UNCLASSIFIED',
    5903 => 'UNCLASSIFIED',
    5904 => 'UNCLASSIFIED',
    5905 => 'UNCLASSIFIED',
    5906 => 'UNCLASSIFIED',
    5907 => 'UNCLASSIFIED',
    5908 => 'UNCLASSIFIED',
    5909 => 'UNCLASSIFIED',
    5910 => 'UNCLASSIFIED',
    5911 => 'UNCLASSIFIED',
    5912 => 'UNCLASSIFIED',
    5913 => 'UNCLASSIFIED',
    5914 => 'UNCLASSIFIED',
    5915 => 'UNCLASSIFIED',
    5916 => 'UNCLASSIFIED',
    5917 => 'UNCLASSIFIED',
    5918 => 'UNCLASSIFIED',
    5919 => 'UNCLASSIFIED',
    5920 => 'UNCLASSIFIED',
    5921 => 'UNCLASSIFIED',
    5922 => 'UNCLASSIFIED',
    5923 => 'UNCLASSIFIED',
    5924 => 'UNCLASSIFIED',
    5925 => 'UNCLASSIFIED',
    5926 => 'UNCLASSIFIED',
    5927 => 'UNCLASSIFIED',
    5928 => 'UNCLASSIFIED',
    5929 => 'UNCLASSIFIED',
    5930 => 'UNCLASSIFIED',
    5931 => 'UNCLASSIFIED',
    5932 => 'UNCLASSIFIED',
    5933 => 'UNCLASSIFIED',
    5934 => 'UNCLASSIFIED',
    5935 => 'UNCLASSIFIED',
    5936 => 'UNCLASSIFIED',
    5937 => 'UNCLASSIFIED',
    5938 => 'UNCLASSIFIED',
    5939 => 'UNCLASSIFIED',
    5940 => 'UNCLASSIFIED',
    5941 => 'UNCLASSIFIED',
    5942 => 'UNCLASSIFIED',
    5943 => 'UNCLASSIFIED',
    5944 => 'UNCLASSIFIED',
    5945 => 'UNCLASSIFIED',
    5946 => 'UNCLASSIFIED',
    5947 => 'UNCLASSIFIED',
    5948 => 'UNCLASSIFIED',
    5949 => 'UNCLASSIFIED',
    5950 => 'UNCLASSIFIED',
    5951 => 'UNCLASSIFIED',
    5952 => 'UNCLASSIFIED',
    5953 => 'UNCLASSIFIED',
    5954 => 'UNCLASSIFIED',
    5955 => 'UNCLASSIFIED',
    5956 => 'UNCLASSIFIED',
    5957 => 'UNCLASSIFIED',
    5958 => 'UNCLASSIFIED',
    5959 => 'UNCLASSIFIED',
    5960 => 'UNCLASSIFIED',
    5961 => 'UNCLASSIFIED',
    5962 => 'UNCLASSIFIED',
    5963 => 'UNCLASSIFIED',
    5964 => 'UNCLASSIFIED',
    5965 => 'UNCLASSIFIED',
    5966 => 'UNCLASSIFIED',
    5967 => 'UNCLASSIFIED',
    5968 => 'UNCLASSIFIED',
    5969 => 'UNCLASSIFIED',
    5970 => 'UNCLASSIFIED',
    5971 => 'UNCLASSIFIED',
    5972 => 'UNCLASSIFIED',
    5973 => 'UNCLASSIFIED',
    5974 => 'UNCLASSIFIED',
    5975 => 'UNCLASSIFIED',
    5976 => 'UNCLASSIFIED',
    5977 => 'UNCLASSIFIED',
    5978 => 'UNCLASSIFIED',
    5979 => 'UNCLASSIFIED',
    5980 => 'UNCLASSIFIED',
    5981 => 'UNCLASSIFIED',
    5982 => 'UNCLASSIFIED',
    5983 => 'UNCLASSIFIED',
    5984 => 'UNCLASSIFIED',
    5985 => 'UNCLASSIFIED',
    5986 => 'UNCLASSIFIED',
    5987 => 'UNCLASSIFIED',
    5988 => 'UNCLASSIFIED',
    5989 => 'UNCLASSIFIED',
    5990 => 'UNCLASSIFIED',
    5991 => 'UNCLASSIFIED',
    5992 => 'UNCLASSIFIED',
    5993 => 'UNCLASSIFIED',
    5994 => 'UNCLASSIFIED',
    5995 => 'UNCLASSIFIED',
    5996 => 'UNCLASSIFIED',
    5997 => 'UNCLASSIFIED',
    5998 => 'UNCLASSIFIED',
    5999 => 'UNCLASSIFIED',
    6000 => 'NON-ACTIVE GALAXY',
    6001 => 'NON-ACTIVE GALAXY FLAT RADIO SP',
    6002 => 'NON-ACTIVE GALAXY STEEP RADIO SP',
    6003 => 'NON-ACTIVE GALAXY INVERTED RADIO SP',
    6004 => 'NON-ACTIVE GALAXY',
    6005 => 'NON-ACTIVE GALAXY',
    6006 => 'NON-ACTIVE GALAXY',
    6007 => 'NON-ACTIVE GALAXY',
    6008 => 'NON-ACTIVE GALAXY',
    6009 => 'NON-ACTIVE GALAXY',
    6010 => 'NON-ACTIVE GALAXY RADIO LOUD',
    6011 => 'NON-ACTIVE GALAXY RADIO LOUD FLAT RADIO SP',
    6012 => 'NON-ACTIVE GALAXY RADIO LOUD STEEP RADIO SP',
    6013 => 'NON-ACTIVE GALAXY RADIO LOUD INVERTED RADIO SP',
    6014 => 'NON-ACTIVE GALAXY RADIO LOUD',
    6015 => 'NON-ACTIVE GALAXY RADIO LOUD',
    6016 => 'NON-ACTIVE GALAXY RADIO LOUD',
    6017 => 'NON-ACTIVE GALAXY RADIO LOUD',
    6018 => 'NON-ACTIVE GALAXY RADIO LOUD',
    6019 => 'NON-ACTIVE GALAXY RADIO LOUD',
    6020 => 'NON-ACTIVE GALAXY HII REGION',
    6021 => 'NON-ACTIVE GALAXY HII REGION FLAT RADIO SP',
    6022 => 'NON-ACTIVE GALAXY HII REGION STEEP RADIO SP',
    6023 => 'NON-ACTIVE GALAXY HII REGION INVERTED RADIO SP',
    6024 => 'NON-ACTIVE GALAXY HII REGION',
    6025 => 'NON-ACTIVE GALAXY HII REGION',
    6026 => 'NON-ACTIVE GALAXY HII REGION',
    6027 => 'NON-ACTIVE GALAXY HII REGION',
    6028 => 'NON-ACTIVE GALAXY HII REGION',
    6029 => 'NON-ACTIVE GALAXY HII REGION',
    6030 => 'NON-ACTIVE GALAXY MULTIPLE NUCLEI',
    6031 => 'NON-ACTIVE GALAXY MULTIPLE NUCLEI FLAT RADIO SP',
    6032 => 'NON-ACTIVE GALAXY MULTIPLE NUCLEI STEEP RADIO SP',
    6033 => 'NON-ACTIVE GALAXY MULTIPLE NUCLEI INVERTED RADIO SP',
    6034 => 'NON-ACTIVE GALAXY MULTIPLE NUCLEI',
    6035 => 'NON-ACTIVE GALAXY MULTIPLE NUCLEI',
    6036 => 'NON-ACTIVE GALAXY MULTIPLE NUCLEI',
    6037 => 'NON-ACTIVE GALAXY MULTIPLE NUCLEI',
    6038 => 'NON-ACTIVE GALAXY MULTIPLE NUCLEI',
    6039 => 'NON-ACTIVE GALAXY MULTIPLE NUCLEI',
    6040 => 'NON-ACTIVE GALAXY BARRED',
    6041 => 'NON-ACTIVE GALAXY BARRED FLAT RADIO SP',
    6042 => 'NON-ACTIVE GALAXY BARRED STEEP RADIO SP',
    6043 => 'NON-ACTIVE GALAXY BARRED INVERTED RADIO SP',
    6044 => 'NON-ACTIVE GALAXY BARRED',
    6045 => 'NON-ACTIVE GALAXY BARRED',
    6046 => 'NON-ACTIVE GALAXY BARRED',
    6047 => 'NON-ACTIVE GALAXY BARRED',
    6048 => 'NON-ACTIVE GALAXY BARRED',
    6049 => 'NON-ACTIVE GALAXY BARRED',
    6050 => 'NON-ACTIVE GALAXY UNBARRED',
    6051 => 'NON-ACTIVE GALAXY UNBARRED FLAT RADIO SP',
    6052 => 'NON-ACTIVE GALAXY UNBARRED STEEP RADIO SP',
    6053 => 'NON-ACTIVE GALAXY UNBARRED INVERTED RADIO SP',
    6054 => 'NON-ACTIVE GALAXY UNBARRED',
    6055 => 'NON-ACTIVE GALAXY UNBARRED',
    6056 => 'NON-ACTIVE GALAXY UNBARRED',
    6057 => 'NON-ACTIVE GALAXY UNBARRED',
    6058 => 'NON-ACTIVE GALAXY UNBARRED',
    6059 => 'NON-ACTIVE GALAXY UNBARRED',
    6060 => 'NON-ACTIVE GALAXY MIXED',
    6061 => 'NON-ACTIVE GALAXY MIXED FLAT RADIO SP',
    6062 => 'NON-ACTIVE GALAXY MIXED STEEP RADIO SP',
    6063 => 'NON-ACTIVE GALAXY MIXED INVERTED RADIO SP',
    6064 => 'NON-ACTIVE GALAXY MIXED',
    6065 => 'NON-ACTIVE GALAXY MIXED',
    6066 => 'NON-ACTIVE GALAXY MIXED',
    6067 => 'NON-ACTIVE GALAXY MIXED',
    6068 => 'NON-ACTIVE GALAXY MIXED',
    6069 => 'NON-ACTIVE GALAXY MIXED',
    6070 => 'NON-ACTIVE GALAXY NEBULOUS REGION',
    6071 => 'NON-ACTIVE GALAXY NEBULOUS REGION FLAT RADIO SP',
    6072 => 'NON-ACTIVE GALAXY NEBULOUS REGION STEEP RADIO SP',
    6073 => 'NON-ACTIVE GALAXY NEBULOUS REGION INVERTED RADIO SP',
    6074 => 'NON-ACTIVE GALAXY NEBULOUS REGION',
    6075 => 'NON-ACTIVE GALAXY NEBULOUS REGION',
    6076 => 'NON-ACTIVE GALAXY NEBULOUS REGION',
    6077 => 'NON-ACTIVE GALAXY NEBULOUS REGION',
    6078 => 'NON-ACTIVE GALAXY NEBULOUS REGION',
    6079 => 'NON-ACTIVE GALAXY NEBULOUS REGION',
    6080 => 'NON-ACTIVE GALAXY',
    6081 => 'NON-ACTIVE GALAXY FLAT RADIO SP',
    6082 => 'NON-ACTIVE GALAXY STEEP RADIO SP',
    6083 => 'NON-ACTIVE GALAXY INVERTED RADIO SP',
    6084 => 'NON-ACTIVE GALAXY',
    6085 => 'NON-ACTIVE GALAXY',
    6086 => 'NON-ACTIVE GALAXY',
    6087 => 'NON-ACTIVE GALAXY',
    6088 => 'NON-ACTIVE GALAXY',
    6089 => 'NON-ACTIVE GALAXY',
    6090 => 'NON-ACTIVE GALAXY',
    6091 => 'NON-ACTIVE GALAXY FLAT RADIO SP',
    6092 => 'NON-ACTIVE GALAXY STEEP RADIO SP',
    6093 => 'NON-ACTIVE GALAXY INVERTED RADIO SP',
    6094 => 'NON-ACTIVE GALAXY',
    6095 => 'NON-ACTIVE GALAXY',
    6096 => 'NON-ACTIVE GALAXY',
    6097 => 'NON-ACTIVE GALAXY',
    6098 => 'NON-ACTIVE GALAXY',
    6099 => 'NON-ACTIVE GALAXY',
    6100 => 'DWARF GALAXY',
    6101 => 'DWARF GALAXY FLAT RADIO SP',
    6102 => 'DWARF GALAXY STEEP RADIO SP',
    6103 => 'DWARF GALAXY INVERTED RADIO SP',
    6104 => 'DWARF GALAXY',
    6105 => 'DWARF GALAXY',
    6106 => 'DWARF GALAXY',
    6107 => 'DWARF GALAXY',
    6108 => 'DWARF GALAXY',
    6109 => 'DWARF GALAXY',
    6110 => 'DWARF GALAXY RADIO LOUD',
    6111 => 'DWARF GALAXY RADIO LOUD FLAT RADIO SP',
    6112 => 'DWARF GALAXY RADIO LOUD STEEP RADIO SP',
    6113 => 'DWARF GALAXY RADIO LOUD INVERTED RADIO SP',
    6114 => 'DWARF GALAXY RADIO LOUD',
    6115 => 'DWARF GALAXY RADIO LOUD',
    6116 => 'DWARF GALAXY RADIO LOUD',
    6117 => 'DWARF GALAXY RADIO LOUD',
    6118 => 'DWARF GALAXY RADIO LOUD',
    6119 => 'DWARF GALAXY RADIO LOUD',
    6120 => 'DWARF GALAXY HII REGION',
    6121 => 'DWARF GALAXY HII REGION FLAT RADIO SP',
    6122 => 'DWARF GALAXY HII REGION STEEP RADIO SP',
    6123 => 'DWARF GALAXY HII REGION INVERTED RADIO SP',
    6124 => 'DWARF GALAXY HII REGION',
    6125 => 'DWARF GALAXY HII REGION',
    6126 => 'DWARF GALAXY HII REGION',
    6127 => 'DWARF GALAXY HII REGION',
    6128 => 'DWARF GALAXY HII REGION',
    6129 => 'DWARF GALAXY HII REGION',
    6130 => 'DWARF GALAXY MULTIPLE NUCLEI',
    6131 => 'DWARF GALAXY MULTIPLE NUCLEI FLAT RADIO SP',
    6132 => 'DWARF GALAXY MULTIPLE NUCLEI STEEP RADIO SP',
    6133 => 'DWARF GALAXY MULTIPLE NUCLEI INVERTED RADIO SP',
    6134 => 'DWARF GALAXY MULTIPLE NUCLEI',
    6135 => 'DWARF GALAXY MULTIPLE NUCLEI',
    6136 => 'DWARF GALAXY MULTIPLE NUCLEI',
    6137 => 'DWARF GALAXY MULTIPLE NUCLEI',
    6138 => 'DWARF GALAXY MULTIPLE NUCLEI',
    6139 => 'DWARF GALAXY MULTIPLE NUCLEI',
    6140 => 'DWARF GALAXY BARRED',
    6141 => 'DWARF GALAXY BARRED FLAT RADIO SP',
    6142 => 'DWARF GALAXY BARRED STEEP RADIO SP',
    6143 => 'DWARF GALAXY BARRED INVERTED RADIO SP',
    6144 => 'DWARF GALAXY BARRED',
    6145 => 'DWARF GALAXY BARRED',
    6146 => 'DWARF GALAXY BARRED',
    6147 => 'DWARF GALAXY BARRED',
    6148 => 'DWARF GALAXY BARRED',
    6149 => 'DWARF GALAXY BARRED',
    6150 => 'DWARF GALAXY UNBARRED',
    6151 => 'DWARF GALAXY UNBARRED FLAT RADIO SP',
    6152 => 'DWARF GALAXY UNBARRED STEEP RADIO SP',
    6153 => 'DWARF GALAXY UNBARRED INVERTED RADIO SP',
    6154 => 'DWARF GALAXY UNBARRED',
    6155 => 'DWARF GALAXY UNBARRED',
    6156 => 'DWARF GALAXY UNBARRED',
    6157 => 'DWARF GALAXY UNBARRED',
    6158 => 'DWARF GALAXY UNBARRED',
    6159 => 'DWARF GALAXY UNBARRED',
    6160 => 'DWARF GALAXY MIXED',
    6161 => 'DWARF GALAXY MIXED FLAT RADIO SP',
    6162 => 'DWARF GALAXY MIXED STEEP RADIO SP',
    6163 => 'DWARF GALAXY MIXED INVERTED RADIO SP',
    6164 => 'DWARF GALAXY MIXED',
    6165 => 'DWARF GALAXY MIXED',
    6166 => 'DWARF GALAXY MIXED',
    6167 => 'DWARF GALAXY MIXED',
    6168 => 'DWARF GALAXY MIXED',
    6169 => 'DWARF GALAXY MIXED',
    6170 => 'DWARF GALAXY NEBULOUS REGION',
    6171 => 'DWARF GALAXY NEBULOUS REGION FLAT RADIO SP',
    6172 => 'DWARF GALAXY NEBULOUS REGION STEEP RADIO SP',
    6173 => 'DWARF GALAXY NEBULOUS REGION INVERTED RADIO SP',
    6174 => 'DWARF GALAXY NEBULOUS REGION',
    6175 => 'DWARF GALAXY NEBULOUS REGION',
    6176 => 'DWARF GALAXY NEBULOUS REGION',
    6177 => 'DWARF GALAXY NEBULOUS REGION',
    6178 => 'DWARF GALAXY NEBULOUS REGION',
    6179 => 'DWARF GALAXY NEBULOUS REGION',
    6180 => 'DWARF GALAXY',
    6181 => 'DWARF GALAXY FLAT RADIO SP',
    6182 => 'DWARF GALAXY STEEP RADIO SP',
    6183 => 'DWARF GALAXY INVERTED RADIO SP',
    6184 => 'DWARF GALAXY',
    6185 => 'DWARF GALAXY',
    6186 => 'DWARF GALAXY',
    6187 => 'DWARF GALAXY',
    6188 => 'DWARF GALAXY',
    6189 => 'DWARF GALAXY',
    6190 => 'DWARF GALAXY',
    6191 => 'DWARF GALAXY FLAT RADIO SP',
    6192 => 'DWARF GALAXY STEEP RADIO SP',
    6193 => 'DWARF GALAXY INVERTED RADIO SP',
    6194 => 'DWARF GALAXY',
    6195 => 'DWARF GALAXY',
    6196 => 'DWARF GALAXY',
    6197 => 'DWARF GALAXY',
    6198 => 'DWARF GALAXY',
    6199 => 'DWARF GALAXY',
    6200 => 'SPIRAL GALAXY',
    6201 => 'SPIRAL GALAXY FLAT RADIO SP',
    6202 => 'SPIRAL GALAXY STEEP RADIO SP',
    6203 => 'SPIRAL GALAXY INVERTED RADIO SP',
    6204 => 'SPIRAL GALAXY',
    6205 => 'SPIRAL GALAXY',
    6206 => 'SPIRAL GALAXY',
    6207 => 'SPIRAL GALAXY',
    6208 => 'SPIRAL GALAXY',
    6209 => 'SPIRAL GALAXY',
    6210 => 'SPIRAL GALAXY RADIO LOUD',
    6211 => 'SPIRAL GALAXY RADIO LOUD FLAT RADIO SP',
    6212 => 'SPIRAL GALAXY RADIO LOUD STEEP RADIO SP',
    6213 => 'SPIRAL GALAXY RADIO LOUD INVERTED RADIO SP',
    6214 => 'SPIRAL GALAXY RADIO LOUD',
    6215 => 'SPIRAL GALAXY RADIO LOUD',
    6216 => 'SPIRAL GALAXY RADIO LOUD',
    6217 => 'SPIRAL GALAXY RADIO LOUD',
    6218 => 'SPIRAL GALAXY RADIO LOUD',
    6219 => 'SPIRAL GALAXY RADIO LOUD',
    6220 => 'SPIRAL GALAXY HII REGION',
    6221 => 'SPIRAL GALAXY HII REGION FLAT RADIO SP',
    6222 => 'SPIRAL GALAXY HII REGION STEEP RADIO SP',
    6223 => 'SPIRAL GALAXY HII REGION INVERTED RADIO SP',
    6224 => 'SPIRAL GALAXY HII REGION',
    6225 => 'SPIRAL GALAXY HII REGION',
    6226 => 'SPIRAL GALAXY HII REGION',
    6227 => 'SPIRAL GALAXY HII REGION',
    6228 => 'SPIRAL GALAXY HII REGION',
    6229 => 'SPIRAL GALAXY HII REGION',
    6230 => 'SPIRAL GALAXY MULTIPLE NUCLEI',
    6231 => 'SPIRAL GALAXY MULTIPLE NUCLEI FLAT RADIO SP',
    6232 => 'SPIRAL GALAXY MULTIPLE NUCLEI STEEP RADIO SP',
    6233 => 'SPIRAL GALAXY MULTIPLE NUCLEI INVERTED RADIO SP',
    6234 => 'SPIRAL GALAXY MULTIPLE NUCLEI',
    6235 => 'SPIRAL GALAXY MULTIPLE NUCLEI',
    6236 => 'SPIRAL GALAXY MULTIPLE NUCLEI',
    6237 => 'SPIRAL GALAXY MULTIPLE NUCLEI',
    6238 => 'SPIRAL GALAXY MULTIPLE NUCLEI',
    6239 => 'SPIRAL GALAXY MULTIPLE NUCLEI',
    6240 => 'SPIRAL GALAXY BARRED',
    6241 => 'SPIRAL GALAXY BARRED FLAT RADIO SP',
    6242 => 'SPIRAL GALAXY BARRED STEEP RADIO SP',
    6243 => 'SPIRAL GALAXY BARRED INVERTED RADIO SP',
    6244 => 'SPIRAL GALAXY BARRED',
    6245 => 'SPIRAL GALAXY BARRED',
    6246 => 'SPIRAL GALAXY BARRED',
    6247 => 'SPIRAL GALAXY BARRED',
    6248 => 'SPIRAL GALAXY BARRED',
    6249 => 'SPIRAL GALAXY BARRED',
    6250 => 'SPIRAL GALAXY UNBARRED',
    6251 => 'SPIRAL GALAXY UNBARRED FLAT RADIO SP',
    6252 => 'SPIRAL GALAXY UNBARRED STEEP RADIO SP',
    6253 => 'SPIRAL GALAXY UNBARRED INVERTED RADIO SP',
    6254 => 'SPIRAL GALAXY UNBARRED',
    6255 => 'SPIRAL GALAXY UNBARRED',
    6256 => 'SPIRAL GALAXY UNBARRED',
    6257 => 'SPIRAL GALAXY UNBARRED',
    6258 => 'SPIRAL GALAXY UNBARRED',
    6259 => 'SPIRAL GALAXY UNBARRED',
    6260 => 'SPIRAL GALAXY MIXED',
    6261 => 'SPIRAL GALAXY MIXED FLAT RADIO SP',
    6262 => 'SPIRAL GALAXY MIXED STEEP RADIO SP',
    6263 => 'SPIRAL GALAXY MIXED INVERTED RADIO SP',
    6264 => 'SPIRAL GALAXY MIXED',
    6265 => 'SPIRAL GALAXY MIXED',
    6266 => 'SPIRAL GALAXY MIXED',
    6267 => 'SPIRAL GALAXY MIXED',
    6268 => 'SPIRAL GALAXY MIXED',
    6269 => 'SPIRAL GALAXY MIXED',
    6270 => 'SPIRAL GALAXY NEBULOUS REGION',
    6271 => 'SPIRAL GALAXY NEBULOUS REGION FLAT RADIO SP',
    6272 => 'SPIRAL GALAXY NEBULOUS REGION STEEP RADIO SP',
    6273 => 'SPIRAL GALAXY NEBULOUS REGION INVERTED RADIO SP',
    6274 => 'SPIRAL GALAXY NEBULOUS REGION',
    6275 => 'SPIRAL GALAXY NEBULOUS REGION',
    6276 => 'SPIRAL GALAXY NEBULOUS REGION',
    6277 => 'SPIRAL GALAXY NEBULOUS REGION',
    6278 => 'SPIRAL GALAXY NEBULOUS REGION',
    6279 => 'SPIRAL GALAXY NEBULOUS REGION',
    6280 => 'SPIRAL GALAXY',
    6281 => 'SPIRAL GALAXY FLAT RADIO SP',
    6282 => 'SPIRAL GALAXY STEEP RADIO SP',
    6283 => 'SPIRAL GALAXY INVERTED RADIO SP',
    6284 => 'SPIRAL GALAXY',
    6285 => 'SPIRAL GALAXY',
    6286 => 'SPIRAL GALAXY',
    6287 => 'SPIRAL GALAXY',
    6288 => 'SPIRAL GALAXY',
    6289 => 'SPIRAL GALAXY',
    6290 => 'SPIRAL GALAXY',
    6291 => 'SPIRAL GALAXY FLAT RADIO SP',
    6292 => 'SPIRAL GALAXY STEEP RADIO SP',
    6293 => 'SPIRAL GALAXY INVERTED RADIO SP',
    6294 => 'SPIRAL GALAXY',
    6295 => 'SPIRAL GALAXY',
    6296 => 'SPIRAL GALAXY',
    6297 => 'SPIRAL GALAXY',
    6298 => 'SPIRAL GALAXY',
    6299 => 'SPIRAL GALAXY',
    6300 => 'ELLIPTICAL GALAXY',
    6301 => 'ELLIPTICAL GALAXY FLAT RADIO SP',
    6302 => 'ELLIPTICAL GALAXY STEEP RADIO SP',
    6303 => 'ELLIPTICAL GALAXY INVERTED RADIO SP',
    6304 => 'ELLIPTICAL GALAXY',
    6305 => 'ELLIPTICAL GALAXY',
    6306 => 'ELLIPTICAL GALAXY',
    6307 => 'ELLIPTICAL GALAXY',
    6308 => 'ELLIPTICAL GALAXY',
    6309 => 'ELLIPTICAL GALAXY',
    6310 => 'ELLIPTICAL GALAXY RADIO LOUD',
    6311 => 'ELLIPTICAL GALAXY RADIO LOUD FLAT RADIO SP',
    6312 => 'ELLIPTICAL GALAXY RADIO LOUD STEEP RADIO SP',
    6313 => 'ELLIPTICAL GALAXY RADIO LOUD INVERTED RADIO SP',
    6314 => 'ELLIPTICAL GALAXY RADIO LOUD',
    6315 => 'ELLIPTICAL GALAXY RADIO LOUD',
    6316 => 'ELLIPTICAL GALAXY RADIO LOUD',
    6317 => 'ELLIPTICAL GALAXY RADIO LOUD',
    6318 => 'ELLIPTICAL GALAXY RADIO LOUD',
    6319 => 'ELLIPTICAL GALAXY RADIO LOUD',
    6320 => 'ELLIPTICAL GALAXY HII REGION',
    6321 => 'ELLIPTICAL GALAXY HII REGION FLAT RADIO SP',
    6322 => 'ELLIPTICAL GALAXY HII REGION STEEP RADIO SP',
    6323 => 'ELLIPTICAL GALAXY HII REGION INVERTED RADIO SP',
    6324 => 'ELLIPTICAL GALAXY HII REGION',
    6325 => 'ELLIPTICAL GALAXY HII REGION',
    6326 => 'ELLIPTICAL GALAXY HII REGION',
    6327 => 'ELLIPTICAL GALAXY HII REGION',
    6328 => 'ELLIPTICAL GALAXY HII REGION',
    6329 => 'ELLIPTICAL GALAXY HII REGION',
    6330 => 'ELLIPTICAL GALAXY MULTIPLE NUCLEI',
    6331 => 'ELLIPTICAL GALAXY MULTIPLE NUCLEI FLAT RADIO SP',
    6332 => 'ELLIPTICAL GALAXY MULTIPLE NUCLEI STEEP RADIO SP',
    6333 => 'ELLIPTICAL GALAXY MULTIPLE NUCLEI INVERTED RADIO SP',
    6334 => 'ELLIPTICAL GALAXY MULTIPLE NUCLEI',
    6335 => 'ELLIPTICAL GALAXY MULTIPLE NUCLEI',
    6336 => 'ELLIPTICAL GALAXY MULTIPLE NUCLEI',
    6337 => 'ELLIPTICAL GALAXY MULTIPLE NUCLEI',
    6338 => 'ELLIPTICAL GALAXY MULTIPLE NUCLEI',
    6339 => 'ELLIPTICAL GALAXY MULTIPLE NUCLEI',
    6340 => 'ELLIPTICAL GALAXY BARRED',
    6341 => 'ELLIPTICAL GALAXY BARRED FLAT RADIO SP',
    6342 => 'ELLIPTICAL GALAXY BARRED STEEP RADIO SP',
    6343 => 'ELLIPTICAL GALAXY BARRED INVERTED RADIO SP',
    6344 => 'ELLIPTICAL GALAXY BARRED',
    6345 => 'ELLIPTICAL GALAXY BARRED',
    6346 => 'ELLIPTICAL GALAXY BARRED',
    6347 => 'ELLIPTICAL GALAXY BARRED',
    6348 => 'ELLIPTICAL GALAXY BARRED',
    6349 => 'ELLIPTICAL GALAXY BARRED',
    6350 => 'ELLIPTICAL GALAXY UNBARRED',
    6351 => 'ELLIPTICAL GALAXY UNBARRED FLAT RADIO SP',
    6352 => 'ELLIPTICAL GALAXY UNBARRED STEEP RADIO SP',
    6353 => 'ELLIPTICAL GALAXY UNBARRED INVERTED RADIO SP',
    6354 => 'ELLIPTICAL GALAXY UNBARRED',
    6355 => 'ELLIPTICAL GALAXY UNBARRED',
    6356 => 'ELLIPTICAL GALAXY UNBARRED',
    6357 => 'ELLIPTICAL GALAXY UNBARRED',
    6358 => 'ELLIPTICAL GALAXY UNBARRED',
    6359 => 'ELLIPTICAL GALAXY UNBARRED',
    6360 => 'ELLIPTICAL GALAXY MIXED',
    6361 => 'ELLIPTICAL GALAXY MIXED FLAT RADIO SP',
    6362 => 'ELLIPTICAL GALAXY MIXED STEEP RADIO SP',
    6363 => 'ELLIPTICAL GALAXY MIXED INVERTED RADIO SP',
    6364 => 'ELLIPTICAL GALAXY MIXED',
    6365 => 'ELLIPTICAL GALAXY MIXED',
    6366 => 'ELLIPTICAL GALAXY MIXED',
    6367 => 'ELLIPTICAL GALAXY MIXED',
    6368 => 'ELLIPTICAL GALAXY MIXED',
    6369 => 'ELLIPTICAL GALAXY MIXED',
    6370 => 'ELLIPTICAL GALAXY NEBULOUS REGION',
    6371 => 'ELLIPTICAL GALAXY NEBULOUS REGION FLAT RADIO SP',
    6372 => 'ELLIPTICAL GALAXY NEBULOUS REGION STEEP RADIO SP',
    6373 => 'ELLIPTICAL GALAXY NEBULOUS REGION INVERTED RADIO SP',
    6374 => 'ELLIPTICAL GALAXY NEBULOUS REGION',
    6375 => 'ELLIPTICAL GALAXY NEBULOUS REGION',
    6376 => 'ELLIPTICAL GALAXY NEBULOUS REGION',
    6377 => 'ELLIPTICAL GALAXY NEBULOUS REGION',
    6378 => 'ELLIPTICAL GALAXY NEBULOUS REGION',
    6379 => 'ELLIPTICAL GALAXY NEBULOUS REGION',
    6380 => 'ELLIPTICAL GALAXY',
    6381 => 'ELLIPTICAL GALAXY FLAT RADIO SP',
    6382 => 'ELLIPTICAL GALAXY STEEP RADIO SP',
    6383 => 'ELLIPTICAL GALAXY INVERTED RADIO SP',
    6384 => 'ELLIPTICAL GALAXY',
    6385 => 'ELLIPTICAL GALAXY',
    6386 => 'ELLIPTICAL GALAXY',
    6387 => 'ELLIPTICAL GALAXY',
    6388 => 'ELLIPTICAL GALAXY',
    6389 => 'ELLIPTICAL GALAXY',
    6390 => 'ELLIPTICAL GALAXY',
    6391 => 'ELLIPTICAL GALAXY FLAT RADIO SP',
    6392 => 'ELLIPTICAL GALAXY STEEP RADIO SP',
    6393 => 'ELLIPTICAL GALAXY INVERTED RADIO SP',
    6394 => 'ELLIPTICAL GALAXY',
    6395 => 'ELLIPTICAL GALAXY',
    6396 => 'ELLIPTICAL GALAXY',
    6397 => 'ELLIPTICAL GALAXY',
    6398 => 'ELLIPTICAL GALAXY',
    6399 => 'ELLIPTICAL GALAXY',
    6400 => 'STARBURST GALAXY',
    6401 => 'STARBURST GALAXY FLAT RADIO SP',
    6402 => 'STARBURST GALAXY STEEP RADIO SP',
    6403 => 'STARBURST GALAXY INVERTED RADIO SP',
    6404 => 'STARBURST GALAXY',
    6405 => 'STARBURST GALAXY',
    6406 => 'STARBURST GALAXY',
    6407 => 'STARBURST GALAXY',
    6408 => 'STARBURST GALAXY',
    6409 => 'STARBURST GALAXY',
    6410 => 'STARBURST GALAXY RADIO LOUD',
    6411 => 'STARBURST GALAXY RADIO LOUD FLAT RADIO SP',
    6412 => 'STARBURST GALAXY RADIO LOUD STEEP RADIO SP',
    6413 => 'STARBURST GALAXY RADIO LOUD INVERTED RADIO SP',
    6414 => 'STARBURST GALAXY RADIO LOUD',
    6415 => 'STARBURST GALAXY RADIO LOUD',
    6416 => 'STARBURST GALAXY RADIO LOUD',
    6417 => 'STARBURST GALAXY RADIO LOUD',
    6418 => 'STARBURST GALAXY RADIO LOUD',
    6419 => 'STARBURST GALAXY RADIO LOUD',
    6420 => 'STARBURST GALAXY HII REGION',
    6421 => 'STARBURST GALAXY HII REGION FLAT RADIO SP',
    6422 => 'STARBURST GALAXY HII REGION STEEP RADIO SP',
    6423 => 'STARBURST GALAXY HII REGION INVERTED RADIO SP',
    6424 => 'STARBURST GALAXY HII REGION',
    6425 => 'STARBURST GALAXY HII REGION',
    6426 => 'STARBURST GALAXY HII REGION',
    6427 => 'STARBURST GALAXY HII REGION',
    6428 => 'STARBURST GALAXY HII REGION',
    6429 => 'STARBURST GALAXY HII REGION',
    6430 => 'STARBURST GALAXY MULTIPLE NUCLEI',
    6431 => 'STARBURST GALAXY MULTIPLE NUCLEI FLAT RADIO SP',
    6432 => 'STARBURST GALAXY MULTIPLE NUCLEI STEEP RADIO SP',
    6433 => 'STARBURST GALAXY MULTIPLE NUCLEI INVERTED RADIO SP',
    6434 => 'STARBURST GALAXY MULTIPLE NUCLEI',
    6435 => 'STARBURST GALAXY MULTIPLE NUCLEI',
    6436 => 'STARBURST GALAXY MULTIPLE NUCLEI',
    6437 => 'STARBURST GALAXY MULTIPLE NUCLEI',
    6438 => 'STARBURST GALAXY MULTIPLE NUCLEI',
    6439 => 'STARBURST GALAXY MULTIPLE NUCLEI',
    6440 => 'STARBURST GALAXY BARRED',
    6441 => 'STARBURST GALAXY BARRED FLAT RADIO SP',
    6442 => 'STARBURST GALAXY BARRED STEEP RADIO SP',
    6443 => 'STARBURST GALAXY BARRED INVERTED RADIO SP',
    6444 => 'STARBURST GALAXY BARRED',
    6445 => 'STARBURST GALAXY BARRED',
    6446 => 'STARBURST GALAXY BARRED',
    6447 => 'STARBURST GALAXY BARRED',
    6448 => 'STARBURST GALAXY BARRED',
    6449 => 'STARBURST GALAXY BARRED',
    6450 => 'STARBURST GALAXY UNBARRED',
    6451 => 'STARBURST GALAXY UNBARRED FLAT RADIO SP',
    6452 => 'STARBURST GALAXY UNBARRED STEEP RADIO SP',
    6453 => 'STARBURST GALAXY UNBARRED INVERTED RADIO SP',
    6454 => 'STARBURST GALAXY UNBARRED',
    6455 => 'STARBURST GALAXY UNBARRED',
    6456 => 'STARBURST GALAXY UNBARRED',
    6457 => 'STARBURST GALAXY UNBARRED',
    6458 => 'STARBURST GALAXY UNBARRED',
    6459 => 'STARBURST GALAXY UNBARRED',
    6460 => 'STARBURST GALAXY MIXED',
    6461 => 'STARBURST GALAXY MIXED FLAT RADIO SP',
    6462 => 'STARBURST GALAXY MIXED STEEP RADIO SP',
    6463 => 'STARBURST GALAXY MIXED INVERTED RADIO SP',
    6464 => 'STARBURST GALAXY MIXED',
    6465 => 'STARBURST GALAXY MIXED',
    6466 => 'STARBURST GALAXY MIXED',
    6467 => 'STARBURST GALAXY MIXED',
    6468 => 'STARBURST GALAXY MIXED',
    6469 => 'STARBURST GALAXY MIXED',
    6470 => 'STARBURST GALAXY NEBULOUS REGION',
    6471 => 'STARBURST GALAXY NEBULOUS REGION FLAT RADIO SP',
    6472 => 'STARBURST GALAXY NEBULOUS REGION STEEP RADIO SP',
    6473 => 'STARBURST GALAXY NEBULOUS REGION INVERTED RADIO SP',
    6474 => 'STARBURST GALAXY NEBULOUS REGION',
    6475 => 'STARBURST GALAXY NEBULOUS REGION',
    6476 => 'STARBURST GALAXY NEBULOUS REGION',
    6477 => 'STARBURST GALAXY NEBULOUS REGION',
    6478 => 'STARBURST GALAXY NEBULOUS REGION',
    6479 => 'STARBURST GALAXY NEBULOUS REGION',
    6480 => 'STARBURST GALAXY',
    6481 => 'STARBURST GALAXY FLAT RADIO SP',
    6482 => 'STARBURST GALAXY STEEP RADIO SP',
    6483 => 'STARBURST GALAXY INVERTED RADIO SP',
    6484 => 'STARBURST GALAXY',
    6485 => 'STARBURST GALAXY',
    6486 => 'STARBURST GALAXY',
    6487 => 'STARBURST GALAXY',
    6488 => 'STARBURST GALAXY',
    6489 => 'STARBURST GALAXY',
    6490 => 'STARBURST GALAXY',
    6491 => 'STARBURST GALAXY FLAT RADIO SP',
    6492 => 'STARBURST GALAXY STEEP RADIO SP',
    6493 => 'STARBURST GALAXY INVERTED RADIO SP',
    6494 => 'STARBURST GALAXY',
    6495 => 'STARBURST GALAXY',
    6496 => 'STARBURST GALAXY',
    6497 => 'STARBURST GALAXY',
    6498 => 'STARBURST GALAXY',
    6499 => 'STARBURST GALAXY',
    6500 => 'INTERACTING GALAXY',
    6501 => 'INTERACTING GALAXY FLAT RADIO SP',
    6502 => 'INTERACTING GALAXY STEEP RADIO SP',
    6503 => 'INTERACTING GALAXY INVERTED RADIO SP',
    6504 => 'INTERACTING GALAXY',
    6505 => 'INTERACTING GALAXY',
    6506 => 'INTERACTING GALAXY',
    6507 => 'INTERACTING GALAXY',
    6508 => 'INTERACTING GALAXY',
    6509 => 'INTERACTING GALAXY',
    6510 => 'INTERACTING GALAXY RADIO LOUD',
    6511 => 'INTERACTING GALAXY RADIO LOUD FLAT RADIO SP',
    6512 => 'INTERACTING GALAXY RADIO LOUD STEEP RADIO SP',
    6513 => 'INTERACTING GALAXY RADIO LOUD INVERTED RADIO SP',
    6514 => 'INTERACTING GALAXY RADIO LOUD',
    6515 => 'INTERACTING GALAXY RADIO LOUD',
    6516 => 'INTERACTING GALAXY RADIO LOUD',
    6517 => 'INTERACTING GALAXY RADIO LOUD',
    6518 => 'INTERACTING GALAXY RADIO LOUD',
    6519 => 'INTERACTING GALAXY RADIO LOUD',
    6520 => 'INTERACTING GALAXY HII REGION',
    6521 => 'INTERACTING GALAXY HII REGION FLAT RADIO SP',
    6522 => 'INTERACTING GALAXY HII REGION STEEP RADIO SP',
    6523 => 'INTERACTING GALAXY HII REGION INVERTED RADIO SP',
    6524 => 'INTERACTING GALAXY HII REGION',
    6525 => 'INTERACTING GALAXY HII REGION',
    6526 => 'INTERACTING GALAXY HII REGION',
    6527 => 'INTERACTING GALAXY HII REGION',
    6528 => 'INTERACTING GALAXY HII REGION',
    6529 => 'INTERACTING GALAXY HII REGION',
    6530 => 'INTERACTING GALAXY MULTIPLE NUCLEI',
    6531 => 'INTERACTING GALAXY MULTIPLE NUCLEI FLAT RADIO SP',
    6532 => 'INTERACTING GALAXY MULTIPLE NUCLEI STEEP RADIO SP',
    6533 => 'INTERACTING GALAXY MULTIPLE NUCLEI INVERTED RADIO SP',
    6534 => 'INTERACTING GALAXY MULTIPLE NUCLEI',
    6535 => 'INTERACTING GALAXY MULTIPLE NUCLEI',
    6536 => 'INTERACTING GALAXY MULTIPLE NUCLEI',
    6537 => 'INTERACTING GALAXY MULTIPLE NUCLEI',
    6538 => 'INTERACTING GALAXY MULTIPLE NUCLEI',
    6539 => 'INTERACTING GALAXY MULTIPLE NUCLEI',
    6540 => 'INTERACTING GALAXY BARRED',
    6541 => 'INTERACTING GALAXY BARRED FLAT RADIO SP',
    6542 => 'INTERACTING GALAXY BARRED STEEP RADIO SP',
    6543 => 'INTERACTING GALAXY BARRED INVERTED RADIO SP',
    6544 => 'INTERACTING GALAXY BARRED',
    6545 => 'INTERACTING GALAXY BARRED',
    6546 => 'INTERACTING GALAXY BARRED',
    6547 => 'INTERACTING GALAXY BARRED',
    6548 => 'INTERACTING GALAXY BARRED',
    6549 => 'INTERACTING GALAXY BARRED',
    6550 => 'INTERACTING GALAXY UNBARRED',
    6551 => 'INTERACTING GALAXY UNBARRED FLAT RADIO SP',
    6552 => 'INTERACTING GALAXY UNBARRED STEEP RADIO SP',
    6553 => 'INTERACTING GALAXY UNBARRED INVERTED RADIO SP',
    6554 => 'INTERACTING GALAXY UNBARRED',
    6555 => 'INTERACTING GALAXY UNBARRED',
    6556 => 'INTERACTING GALAXY UNBARRED',
    6557 => 'INTERACTING GALAXY UNBARRED',
    6558 => 'INTERACTING GALAXY UNBARRED',
    6559 => 'INTERACTING GALAXY UNBARRED',
    6560 => 'INTERACTING GALAXY MIXED',
    6561 => 'INTERACTING GALAXY MIXED FLAT RADIO SP',
    6562 => 'INTERACTING GALAXY MIXED STEEP RADIO SP',
    6563 => 'INTERACTING GALAXY MIXED INVERTED RADIO SP',
    6564 => 'INTERACTING GALAXY MIXED',
    6565 => 'INTERACTING GALAXY MIXED',
    6566 => 'INTERACTING GALAXY MIXED',
    6567 => 'INTERACTING GALAXY MIXED',
    6568 => 'INTERACTING GALAXY MIXED',
    6569 => 'INTERACTING GALAXY MIXED',
    6570 => 'INTERACTING GALAXY NEBULOUS REGION',
    6571 => 'INTERACTING GALAXY NEBULOUS REGION FLAT RADIO SP',
    6572 => 'INTERACTING GALAXY NEBULOUS REGION STEEP RADIO SP',
    6573 => 'INTERACTING GALAXY NEBULOUS REGION INVERTED RADIO SP',
    6574 => 'INTERACTING GALAXY NEBULOUS REGION',
    6575 => 'INTERACTING GALAXY NEBULOUS REGION',
    6576 => 'INTERACTING GALAXY NEBULOUS REGION',
    6577 => 'INTERACTING GALAXY NEBULOUS REGION',
    6578 => 'INTERACTING GALAXY NEBULOUS REGION',
    6579 => 'INTERACTING GALAXY NEBULOUS REGION',
    6580 => 'INTERACTING GALAXY',
    6581 => 'INTERACTING GALAXY FLAT RADIO SP',
    6582 => 'INTERACTING GALAXY STEEP RADIO SP',
    6583 => 'INTERACTING GALAXY INVERTED RADIO SP',
    6584 => 'INTERACTING GALAXY',
    6585 => 'INTERACTING GALAXY',
    6586 => 'INTERACTING GALAXY',
    6587 => 'INTERACTING GALAXY',
    6588 => 'INTERACTING GALAXY',
    6589 => 'INTERACTING GALAXY',
    6590 => 'INTERACTING GALAXY',
    6591 => 'INTERACTING GALAXY FLAT RADIO SP',
    6592 => 'INTERACTING GALAXY STEEP RADIO SP',
    6593 => 'INTERACTING GALAXY INVERTED RADIO SP',
    6594 => 'INTERACTING GALAXY',
    6595 => 'INTERACTING GALAXY',
    6596 => 'INTERACTING GALAXY',
    6597 => 'INTERACTING GALAXY',
    6598 => 'INTERACTING GALAXY',
    6599 => 'INTERACTING GALAXY',
    6600 => 'IRREGULAR GALAXY',
    6601 => 'IRREGULAR GALAXY FLAT RADIO SP',
    6602 => 'IRREGULAR GALAXY STEEP RADIO SP',
    6603 => 'IRREGULAR GALAXY INVERTED RADIO SP',
    6604 => 'IRREGULAR GALAXY',
    6605 => 'IRREGULAR GALAXY',
    6606 => 'IRREGULAR GALAXY',
    6607 => 'IRREGULAR GALAXY',
    6608 => 'IRREGULAR GALAXY',
    6609 => 'IRREGULAR GALAXY',
    6610 => 'IRREGULAR GALAXY RADIO LOUD',
    6611 => 'IRREGULAR GALAXY RADIO LOUD FLAT RADIO SP',
    6612 => 'IRREGULAR GALAXY RADIO LOUD STEEP RADIO SP',
    6613 => 'IRREGULAR GALAXY RADIO LOUD INVERTED RADIO SP',
    6614 => 'IRREGULAR GALAXY RADIO LOUD',
    6615 => 'IRREGULAR GALAXY RADIO LOUD',
    6616 => 'IRREGULAR GALAXY RADIO LOUD',
    6617 => 'IRREGULAR GALAXY RADIO LOUD',
    6618 => 'IRREGULAR GALAXY RADIO LOUD',
    6619 => 'IRREGULAR GALAXY RADIO LOUD',
    6620 => 'IRREGULAR GALAXY HII REGION',
    6621 => 'IRREGULAR GALAXY HII REGION FLAT RADIO SP',
    6622 => 'IRREGULAR GALAXY HII REGION STEEP RADIO SP',
    6623 => 'IRREGULAR GALAXY HII REGION INVERTED RADIO SP',
    6624 => 'IRREGULAR GALAXY HII REGION',
    6625 => 'IRREGULAR GALAXY HII REGION',
    6626 => 'IRREGULAR GALAXY HII REGION',
    6627 => 'IRREGULAR GALAXY HII REGION',
    6628 => 'IRREGULAR GALAXY HII REGION',
    6629 => 'IRREGULAR GALAXY HII REGION',
    6630 => 'IRREGULAR GALAXY MULTIPLE NUCLEI',
    6631 => 'IRREGULAR GALAXY MULTIPLE NUCLEI FLAT RADIO SP',
    6632 => 'IRREGULAR GALAXY MULTIPLE NUCLEI STEEP RADIO SP',
    6633 => 'IRREGULAR GALAXY MULTIPLE NUCLEI INVERTED RADIO SP',
    6634 => 'IRREGULAR GALAXY MULTIPLE NUCLEI',
    6635 => 'IRREGULAR GALAXY MULTIPLE NUCLEI',
    6636 => 'IRREGULAR GALAXY MULTIPLE NUCLEI',
    6637 => 'IRREGULAR GALAXY MULTIPLE NUCLEI',
    6638 => 'IRREGULAR GALAXY MULTIPLE NUCLEI',
    6639 => 'IRREGULAR GALAXY MULTIPLE NUCLEI',
    6640 => 'IRREGULAR GALAXY BARRED',
    6641 => 'IRREGULAR GALAXY BARRED FLAT RADIO SP',
    6642 => 'IRREGULAR GALAXY BARRED STEEP RADIO SP',
    6643 => 'IRREGULAR GALAXY BARRED INVERTED RADIO SP',
    6644 => 'IRREGULAR GALAXY BARRED',
    6645 => 'IRREGULAR GALAXY BARRED',
    6646 => 'IRREGULAR GALAXY BARRED',
    6647 => 'IRREGULAR GALAXY BARRED',
    6648 => 'IRREGULAR GALAXY BARRED',
    6649 => 'IRREGULAR GALAXY BARRED',
    6650 => 'IRREGULAR GALAXY UNBARRED',
    6651 => 'IRREGULAR GALAXY UNBARRED FLAT RADIO SP',
    6652 => 'IRREGULAR GALAXY UNBARRED STEEP RADIO SP',
    6653 => 'IRREGULAR GALAXY UNBARRED INVERTED RADIO SP',
    6654 => 'IRREGULAR GALAXY UNBARRED',
    6655 => 'IRREGULAR GALAXY UNBARRED',
    6656 => 'IRREGULAR GALAXY UNBARRED',
    6657 => 'IRREGULAR GALAXY UNBARRED',
    6658 => 'IRREGULAR GALAXY UNBARRED',
    6659 => 'IRREGULAR GALAXY UNBARRED',
    6660 => 'IRREGULAR GALAXY MIXED',
    6661 => 'IRREGULAR GALAXY MIXED FLAT RADIO SP',
    6662 => 'IRREGULAR GALAXY MIXED STEEP RADIO SP',
    6663 => 'IRREGULAR GALAXY MIXED INVERTED RADIO SP',
    6664 => 'IRREGULAR GALAXY MIXED',
    6665 => 'IRREGULAR GALAXY MIXED',
    6666 => 'IRREGULAR GALAXY MIXED',
    6667 => 'IRREGULAR GALAXY MIXED',
    6668 => 'IRREGULAR GALAXY MIXED',
    6669 => 'IRREGULAR GALAXY MIXED',
    6670 => 'IRREGULAR GALAXY NEBULOUS REGION',
    6671 => 'IRREGULAR GALAXY NEBULOUS REGION FLAT RADIO SP',
    6672 => 'IRREGULAR GALAXY NEBULOUS REGION STEEP RADIO SP',
    6673 => 'IRREGULAR GALAXY NEBULOUS REGION INVERTED RADIO SP',
    6674 => 'IRREGULAR GALAXY NEBULOUS REGION',
    6675 => 'IRREGULAR GALAXY NEBULOUS REGION',
    6676 => 'IRREGULAR GALAXY NEBULOUS REGION',
    6677 => 'IRREGULAR GALAXY NEBULOUS REGION',
    6678 => 'IRREGULAR GALAXY NEBULOUS REGION',
    6679 => 'IRREGULAR GALAXY NEBULOUS REGION',
    6680 => 'IRREGULAR GALAXY',
    6681 => 'IRREGULAR GALAXY FLAT RADIO SP',
    6682 => 'IRREGULAR GALAXY STEEP RADIO SP',
    6683 => 'IRREGULAR GALAXY INVERTED RADIO SP',
    6684 => 'IRREGULAR GALAXY',
    6685 => 'IRREGULAR GALAXY',
    6686 => 'IRREGULAR GALAXY',
    6687 => 'IRREGULAR GALAXY',
    6688 => 'IRREGULAR GALAXY',
    6689 => 'IRREGULAR GALAXY',
    6690 => 'IRREGULAR GALAXY',
    6691 => 'IRREGULAR GALAXY FLAT RADIO SP',
    6692 => 'IRREGULAR GALAXY STEEP RADIO SP',
    6693 => 'IRREGULAR GALAXY INVERTED RADIO SP',
    6694 => 'IRREGULAR GALAXY',
    6695 => 'IRREGULAR GALAXY',
    6696 => 'IRREGULAR GALAXY',
    6697 => 'IRREGULAR GALAXY',
    6698 => 'IRREGULAR GALAXY',
    6699 => 'IRREGULAR GALAXY',
    6700 => 'GALAXY',
    6701 => 'GALAXY FLAT RADIO SP',
    6702 => 'GALAXY STEEP RADIO SP',
    6703 => 'GALAXY INVERTED RADIO SP',
    6704 => 'GALAXY',
    6705 => 'GALAXY',
    6706 => 'GALAXY',
    6707 => 'GALAXY',
    6708 => 'GALAXY',
    6709 => 'GALAXY',
    6710 => 'GALAXY RADIO LOUD',
    6711 => 'GALAXY RADIO LOUD FLAT RADIO SP',
    6712 => 'GALAXY RADIO LOUD STEEP RADIO SP',
    6713 => 'GALAXY RADIO LOUD INVERTED RADIO SP',
    6714 => 'GALAXY RADIO LOUD',
    6715 => 'GALAXY RADIO LOUD',
    6716 => 'GALAXY RADIO LOUD',
    6717 => 'GALAXY RADIO LOUD',
    6718 => 'GALAXY RADIO LOUD',
    6719 => 'GALAXY RADIO LOUD',
    6720 => 'GALAXY HII REGION',
    6721 => 'GALAXY HII REGION FLAT RADIO SP',
    6722 => 'GALAXY HII REGION STEEP RADIO SP',
    6723 => 'GALAXY HII REGION INVERTED RADIO SP',
    6724 => 'GALAXY HII REGION',
    6725 => 'GALAXY HII REGION',
    6726 => 'GALAXY HII REGION',
    6727 => 'GALAXY HII REGION',
    6728 => 'GALAXY HII REGION',
    6729 => 'GALAXY HII REGION',
    6730 => 'GALAXY MULTIPLE NUCLEI',
    6731 => 'GALAXY MULTIPLE NUCLEI FLAT RADIO SP',
    6732 => 'GALAXY MULTIPLE NUCLEI STEEP RADIO SP',
    6733 => 'GALAXY MULTIPLE NUCLEI INVERTED RADIO SP',
    6734 => 'GALAXY MULTIPLE NUCLEI',
    6735 => 'GALAXY MULTIPLE NUCLEI',
    6736 => 'GALAXY MULTIPLE NUCLEI',
    6737 => 'GALAXY MULTIPLE NUCLEI',
    6738 => 'GALAXY MULTIPLE NUCLEI',
    6739 => 'GALAXY MULTIPLE NUCLEI',
    6740 => 'GALAXY BARRED',
    6741 => 'GALAXY BARRED FLAT RADIO SP',
    6742 => 'GALAXY BARRED STEEP RADIO SP',
    6743 => 'GALAXY BARRED INVERTED RADIO SP',
    6744 => 'GALAXY BARRED',
    6745 => 'GALAXY BARRED',
    6746 => 'GALAXY BARRED',
    6747 => 'GALAXY BARRED',
    6748 => 'GALAXY BARRED',
    6749 => 'GALAXY BARRED',
    6750 => 'GALAXY UNBARRED',
    6751 => 'GALAXY UNBARRED FLAT RADIO SP',
    6752 => 'GALAXY UNBARRED STEEP RADIO SP',
    6753 => 'GALAXY UNBARRED INVERTED RADIO SP',
    6754 => 'GALAXY UNBARRED',
    6755 => 'GALAXY UNBARRED',
    6756 => 'GALAXY UNBARRED',
    6757 => 'GALAXY UNBARRED',
    6758 => 'GALAXY UNBARRED',
    6759 => 'GALAXY UNBARRED',
    6760 => 'GALAXY MIXED',
    6761 => 'GALAXY MIXED FLAT RADIO SP',
    6762 => 'GALAXY MIXED STEEP RADIO SP',
    6763 => 'GALAXY MIXED INVERTED RADIO SP',
    6764 => 'GALAXY MIXED',
    6765 => 'GALAXY MIXED',
    6766 => 'GALAXY MIXED',
    6767 => 'GALAXY MIXED',
    6768 => 'GALAXY MIXED',
    6769 => 'GALAXY MIXED',
    6770 => 'GALAXY NEBULOUS REGION',
    6771 => 'GALAXY NEBULOUS REGION FLAT RADIO SP',
    6772 => 'GALAXY NEBULOUS REGION STEEP RADIO SP',
    6773 => 'GALAXY NEBULOUS REGION INVERTED RADIO SP',
    6774 => 'GALAXY NEBULOUS REGION',
    6775 => 'GALAXY NEBULOUS REGION',
    6776 => 'GALAXY NEBULOUS REGION',
    6777 => 'GALAXY NEBULOUS REGION',
    6778 => 'GALAXY NEBULOUS REGION',
    6779 => 'GALAXY NEBULOUS REGION',
    6780 => 'GALAXY',
    6781 => 'GALAXY FLAT RADIO SP',
    6782 => 'GALAXY STEEP RADIO SP',
    6783 => 'GALAXY INVERTED RADIO SP',
    6784 => 'GALAXY',
    6785 => 'GALAXY',
    6786 => 'GALAXY',
    6787 => 'GALAXY',
    6788 => 'GALAXY',
    6789 => 'GALAXY',
    6790 => 'GALAXY',
    6791 => 'GALAXY FLAT RADIO SP',
    6792 => 'GALAXY STEEP RADIO SP',
    6793 => 'GALAXY INVERTED RADIO SP',
    6794 => 'GALAXY',
    6795 => 'GALAXY',
    6796 => 'GALAXY',
    6797 => 'GALAXY',
    6798 => 'GALAXY',
    6799 => 'GALAXY',
    6800 => 'LENTICULAR GALAXY',
    6801 => 'LENTICULAR GALAXY FLAT RADIO SP',
    6802 => 'LENTICULAR GALAXY STEEP RADIO SP',
    6803 => 'LENTICULAR GALAXY INVERTED RADIO SP',
    6804 => 'LENTICULAR GALAXY',
    6805 => 'LENTICULAR GALAXY',
    6806 => 'LENTICULAR GALAXY',
    6807 => 'LENTICULAR GALAXY',
    6808 => 'LENTICULAR GALAXY',
    6809 => 'LENTICULAR GALAXY',
    6810 => 'LENTICULAR GALAXY RADIO LOUD',
    6811 => 'LENTICULAR GALAXY RADIO LOUD FLAT RADIO SP',
    6812 => 'LENTICULAR GALAXY RADIO LOUD STEEP RADIO SP',
    6813 => 'LENTICULAR GALAXY RADIO LOUD INVERTED RADIO SP',
    6814 => 'LENTICULAR GALAXY RADIO LOUD',
    6815 => 'LENTICULAR GALAXY RADIO LOUD',
    6816 => 'LENTICULAR GALAXY RADIO LOUD',
    6817 => 'LENTICULAR GALAXY RADIO LOUD',
    6818 => 'LENTICULAR GALAXY RADIO LOUD',
    6819 => 'LENTICULAR GALAXY RADIO LOUD',
    6820 => 'LENTICULAR GALAXY HII REGION',
    6821 => 'LENTICULAR GALAXY HII REGION FLAT RADIO SP',
    6822 => 'LENTICULAR GALAXY HII REGION STEEP RADIO SP',
    6823 => 'LENTICULAR GALAXY HII REGION INVERTED RADIO SP',
    6824 => 'LENTICULAR GALAXY HII REGION',
    6825 => 'LENTICULAR GALAXY HII REGION',
    6826 => 'LENTICULAR GALAXY HII REGION',
    6827 => 'LENTICULAR GALAXY HII REGION',
    6828 => 'LENTICULAR GALAXY HII REGION',
    6829 => 'LENTICULAR GALAXY HII REGION',
    6830 => 'LENTICULAR GALAXY MULTIPLE NUCLEI',
    6831 => 'LENTICULAR GALAXY MULTIPLE NUCLEI FLAT RADIO SP',
    6832 => 'LENTICULAR GALAXY MULTIPLE NUCLEI STEEP RADIO SP',
    6833 => 'LENTICULAR GALAXY MULTIPLE NUCLEI INVERTED RADIO SP',
    6834 => 'LENTICULAR GALAXY MULTIPLE NUCLEI',
    6835 => 'LENTICULAR GALAXY MULTIPLE NUCLEI',
    6836 => 'LENTICULAR GALAXY MULTIPLE NUCLEI',
    6837 => 'LENTICULAR GALAXY MULTIPLE NUCLEI',
    6838 => 'LENTICULAR GALAXY MULTIPLE NUCLEI',
    6839 => 'LENTICULAR GALAXY MULTIPLE NUCLEI',
    6840 => 'LENTICULAR GALAXY BARRED',
    6841 => 'LENTICULAR GALAXY BARRED FLAT RADIO SP',
    6842 => 'LENTICULAR GALAXY BARRED STEEP RADIO SP',
    6843 => 'LENTICULAR GALAXY BARRED INVERTED RADIO SP',
    6844 => 'LENTICULAR GALAXY BARRED',
    6845 => 'LENTICULAR GALAXY BARRED',
    6846 => 'LENTICULAR GALAXY BARRED',
    6847 => 'LENTICULAR GALAXY BARRED',
    6848 => 'LENTICULAR GALAXY BARRED',
    6849 => 'LENTICULAR GALAXY BARRED',
    6850 => 'LENTICULAR GALAXY UNBARRED',
    6851 => 'LENTICULAR GALAXY UNBARRED FLAT RADIO SP',
    6852 => 'LENTICULAR GALAXY UNBARRED STEEP RADIO SP',
    6853 => 'LENTICULAR GALAXY UNBARRED INVERTED RADIO SP',
    6854 => 'LENTICULAR GALAXY UNBARRED',
    6855 => 'LENTICULAR GALAXY UNBARRED',
    6856 => 'LENTICULAR GALAXY UNBARRED',
    6857 => 'LENTICULAR GALAXY UNBARRED',
    6858 => 'LENTICULAR GALAXY UNBARRED',
    6859 => 'LENTICULAR GALAXY UNBARRED',
    6860 => 'LENTICULAR GALAXY MIXED',
    6861 => 'LENTICULAR GALAXY MIXED FLAT RADIO SP',
    6862 => 'LENTICULAR GALAXY MIXED STEEP RADIO SP',
    6863 => 'LENTICULAR GALAXY MIXED INVERTED RADIO SP',
    6864 => 'LENTICULAR GALAXY MIXED',
    6865 => 'LENTICULAR GALAXY MIXED',
    6866 => 'LENTICULAR GALAXY MIXED',
    6867 => 'LENTICULAR GALAXY MIXED',
    6868 => 'LENTICULAR GALAXY MIXED',
    6869 => 'LENTICULAR GALAXY MIXED',
    6870 => 'LENTICULAR GALAXY NEBULOUS REGION',
    6871 => 'LENTICULAR GALAXY NEBULOUS REGION FLAT RADIO SP',
    6872 => 'LENTICULAR GALAXY NEBULOUS REGION STEEP RADIO SP',
    6873 => 'LENTICULAR GALAXY NEBULOUS REGION INVERTED RADIO SP',
    6874 => 'LENTICULAR GALAXY NEBULOUS REGION',
    6875 => 'LENTICULAR GALAXY NEBULOUS REGION',
    6876 => 'LENTICULAR GALAXY NEBULOUS REGION',
    6877 => 'LENTICULAR GALAXY NEBULOUS REGION',
    6878 => 'LENTICULAR GALAXY NEBULOUS REGION',
    6879 => 'LENTICULAR GALAXY NEBULOUS REGION',
    6880 => 'LENTICULAR GALAXY',
    6881 => 'LENTICULAR GALAXY FLAT RADIO SP',
    6882 => 'LENTICULAR GALAXY STEEP RADIO SP',
    6883 => 'LENTICULAR GALAXY INVERTED RADIO SP',
    6884 => 'LENTICULAR GALAXY',
    6885 => 'LENTICULAR GALAXY',
    6886 => 'LENTICULAR GALAXY',
    6887 => 'LENTICULAR GALAXY',
    6888 => 'LENTICULAR GALAXY',
    6889 => 'LENTICULAR GALAXY',
    6890 => 'LENTICULAR GALAXY',
    6891 => 'LENTICULAR GALAXY FLAT RADIO SP',
    6892 => 'LENTICULAR GALAXY STEEP RADIO SP',
    6893 => 'LENTICULAR GALAXY INVERTED RADIO SP',
    6894 => 'LENTICULAR GALAXY',
    6895 => 'LENTICULAR GALAXY',
    6896 => 'LENTICULAR GALAXY',
    6897 => 'LENTICULAR GALAXY',
    6898 => 'LENTICULAR GALAXY',
    6899 => 'LENTICULAR GALAXY',
    6900 => 'NORMAL GALAXY',
    6901 => 'NORMAL GALAXY FLAT RADIO SP',
    6902 => 'NORMAL GALAXY STEEP RADIO SP',
    6903 => 'NORMAL GALAXY INVERTED RADIO SP',
    6904 => 'NORMAL GALAXY',
    6905 => 'NORMAL GALAXY',
    6906 => 'NORMAL GALAXY',
    6907 => 'NORMAL GALAXY',
    6908 => 'NORMAL GALAXY',
    6909 => 'NORMAL GALAXY',
    6910 => 'NORMAL GALAXY RADIO LOUD',
    6911 => 'NORMAL GALAXY RADIO LOUD FLAT RADIO SP',
    6912 => 'NORMAL GALAXY RADIO LOUD STEEP RADIO SP',
    6913 => 'NORMAL GALAXY RADIO LOUD INVERTED RADIO SP',
    6914 => 'NORMAL GALAXY RADIO LOUD',
    6915 => 'NORMAL GALAXY RADIO LOUD',
    6916 => 'NORMAL GALAXY RADIO LOUD',
    6917 => 'NORMAL GALAXY RADIO LOUD',
    6918 => 'NORMAL GALAXY RADIO LOUD',
    6919 => 'NORMAL GALAXY RADIO LOUD',
    6920 => 'NORMAL GALAXY HII REGION',
    6921 => 'NORMAL GALAXY HII REGION FLAT RADIO SP',
    6922 => 'NORMAL GALAXY HII REGION STEEP RADIO SP',
    6923 => 'NORMAL GALAXY HII REGION INVERTED RADIO SP',
    6924 => 'NORMAL GALAXY HII REGION',
    6925 => 'NORMAL GALAXY HII REGION',
    6926 => 'NORMAL GALAXY HII REGION',
    6927 => 'NORMAL GALAXY HII REGION',
    6928 => 'NORMAL GALAXY HII REGION',
    6929 => 'NORMAL GALAXY HII REGION',
    6930 => 'NORMAL GALAXY MULTIPLE NUCLEI',
    6931 => 'NORMAL GALAXY MULTIPLE NUCLEI FLAT RADIO SP',
    6932 => 'NORMAL GALAXY MULTIPLE NUCLEI STEEP RADIO SP',
    6933 => 'NORMAL GALAXY MULTIPLE NUCLEI INVERTED RADIO SP',
    6934 => 'NORMAL GALAXY MULTIPLE NUCLEI',
    6935 => 'NORMAL GALAXY MULTIPLE NUCLEI',
    6936 => 'NORMAL GALAXY MULTIPLE NUCLEI',
    6937 => 'NORMAL GALAXY MULTIPLE NUCLEI',
    6938 => 'NORMAL GALAXY MULTIPLE NUCLEI',
    6939 => 'NORMAL GALAXY MULTIPLE NUCLEI',
    6940 => 'NORMAL GALAXY BARRED',
    6941 => 'NORMAL GALAXY BARRED FLAT RADIO SP',
    6942 => 'NORMAL GALAXY BARRED STEEP RADIO SP',
    6943 => 'NORMAL GALAXY BARRED INVERTED RADIO SP',
    6944 => 'NORMAL GALAXY BARRED',
    6945 => 'NORMAL GALAXY BARRED',
    6946 => 'NORMAL GALAXY BARRED',
    6947 => 'NORMAL GALAXY BARRED',
    6948 => 'NORMAL GALAXY BARRED',
    6949 => 'NORMAL GALAXY BARRED',
    6950 => 'NORMAL GALAXY UNBARRED',
    6951 => 'NORMAL GALAXY UNBARRED FLAT RADIO SP',
    6952 => 'NORMAL GALAXY UNBARRED STEEP RADIO SP',
    6953 => 'NORMAL GALAXY UNBARRED INVERTED RADIO SP',
    6954 => 'NORMAL GALAXY UNBARRED',
    6955 => 'NORMAL GALAXY UNBARRED',
    6956 => 'NORMAL GALAXY UNBARRED',
    6957 => 'NORMAL GALAXY UNBARRED',
    6958 => 'NORMAL GALAXY UNBARRED',
    6959 => 'NORMAL GALAXY UNBARRED',
    6960 => 'NORMAL GALAXY MIXED',
    6961 => 'NORMAL GALAXY MIXED FLAT RADIO SP',
    6962 => 'NORMAL GALAXY MIXED STEEP RADIO SP',
    6963 => 'NORMAL GALAXY MIXED INVERTED RADIO SP',
    6964 => 'NORMAL GALAXY MIXED',
    6965 => 'NORMAL GALAXY MIXED',
    6966 => 'NORMAL GALAXY MIXED',
    6967 => 'NORMAL GALAXY MIXED',
    6968 => 'NORMAL GALAXY MIXED',
    6969 => 'NORMAL GALAXY MIXED',
    6970 => 'NORMAL GALAXY NEBULOUS REGION',
    6971 => 'NORMAL GALAXY NEBULOUS REGION FLAT RADIO SP',
    6972 => 'NORMAL GALAXY NEBULOUS REGION STEEP RADIO SP',
    6973 => 'NORMAL GALAXY NEBULOUS REGION INVERTED RADIO SP',
    6974 => 'NORMAL GALAXY NEBULOUS REGION',
    6975 => 'NORMAL GALAXY NEBULOUS REGION',
    6976 => 'NORMAL GALAXY NEBULOUS REGION',
    6977 => 'NORMAL GALAXY NEBULOUS REGION',
    6978 => 'NORMAL GALAXY NEBULOUS REGION',
    6979 => 'NORMAL GALAXY NEBULOUS REGION',
    6980 => 'NORMAL GALAXY',
    6981 => 'NORMAL GALAXY FLAT RADIO SP',
    6982 => 'NORMAL GALAXY STEEP RADIO SP',
    6983 => 'NORMAL GALAXY INVERTED RADIO SP',
    6984 => 'NORMAL GALAXY',
    6985 => 'NORMAL GALAXY',
    6986 => 'NORMAL GALAXY',
    6987 => 'NORMAL GALAXY',
    6988 => 'NORMAL GALAXY',
    6989 => 'NORMAL GALAXY',
    6990 => 'NORMAL GALAXY',
    6991 => 'NORMAL GALAXY FLAT RADIO SP',
    6992 => 'NORMAL GALAXY STEEP RADIO SP',
    6993 => 'NORMAL GALAXY INVERTED RADIO SP',
    6994 => 'NORMAL GALAXY',
    6995 => 'NORMAL GALAXY',
    6996 => 'NORMAL GALAXY',
    6997 => 'NORMAL GALAXY',
    6998 => 'NORMAL GALAXY',
    6999 => 'NORMAL GALAXY',
    7000 => 'AGN UNCLASSIFIED',
    7001 => 'AGN UNCLASSIFIED FLAT RADIO SP',
    7002 => 'AGN UNCLASSIFIED STEEP RADIO SP',
    7003 => 'AGN UNCLASSIFIED INVERTED RADIO SP',
    7004 => 'AGN UNCLASSIFIED TYPE 1',
    7005 => 'AGN UNCLASSIFIED TYPE 1.5',
    7006 => 'AGN UNCLASSIFIED TYPE 2',
    7007 => 'AGN UNCLASSIFIED',
    7008 => 'AGN UNCLASSIFIED',
    7009 => 'AGN UNCLASSIFIED',
    7010 => 'AGN UNCLASSIFIED RADIO LOUD',
    7011 => 'AGN UNCLASSIFIED RADIO LOUD FLAT RADIO SP',
    7012 => 'AGN UNCLASSIFIED RADIO LOUD STEEP RADIO SP',
    7013 => 'AGN UNCLASSIFIED RADIO LOUD INVERTED RADIO SP',
    7014 => 'AGN UNCLASSIFIED RADIO LOUD TYPE 1',
    7015 => 'AGN UNCLASSIFIED RADIO LOUD TYPE 1.5',
    7016 => 'AGN UNCLASSIFIED RADIO LOUD TYPE 2',
    7017 => 'AGN UNCLASSIFIED RADIO LOUD',
    7018 => 'AGN UNCLASSIFIED RADIO LOUD',
    7019 => 'AGN UNCLASSIFIED RADIO LOUD',
    7020 => 'AGN UNCLASSIFIED RADIO LOUD/POLARISED',
    7021 => 'AGN UNCLASSIFIED RADIO LOUD/POLARISED FLAT RADIO SP',
    7022 => 'AGN UNCLASSIFIED RADIO LOUD/POLARISED STEEP RADIO SP',
    7023 => 'AGN UNCLASSIFIED RADIO LOUD/POLARISED INVERTED RADIO SP',
    7024 => 'AGN UNCLASSIFIED RADIO LOUD/POLARISED TYPE 1',
    7025 => 'AGN UNCLASSIFIED RADIO LOUD/POLARISED TYPE 1.5',
    7026 => 'AGN UNCLASSIFIED RADIO LOUD/POLARISED TYPE 2',
    7027 => 'AGN UNCLASSIFIED RADIO LOUD/POLARISED',
    7028 => 'AGN UNCLASSIFIED RADIO LOUD/POLARISED',
    7029 => 'AGN UNCLASSIFIED RADIO LOUD/POLARISED',
    7030 => 'AGN UNCLASSIFIED RADIO QUIET',
    7031 => 'AGN UNCLASSIFIED RADIO QUIET FLAT RADIO SP',
    7032 => 'AGN UNCLASSIFIED RADIO QUIET STEEP RADIO SP',
    7033 => 'AGN UNCLASSIFIED RADIO QUIET INVERTED RADIO SP',
    7034 => 'AGN UNCLASSIFIED RADIO QUIET TYPE 1',
    7035 => 'AGN UNCLASSIFIED RADIO QUIET TYPE 1.5',
    7036 => 'AGN UNCLASSIFIED RADIO QUIET TYPE 2',
    7037 => 'AGN UNCLASSIFIED RADIO QUIET',
    7038 => 'AGN UNCLASSIFIED RADIO QUIET',
    7039 => 'AGN UNCLASSIFIED RADIO QUIET',
    7040 => 'AGN UNCLASSIFIED RADIO LOUD/INVERT SPECT',
    7041 => 'AGN UNCLASSIFIED RADIO LOUD/INVERT SPECT FLAT RADIO SP',
    7042 => 'AGN UNCLASSIFIED RADIO LOUD/INVERT SPECT STEEP RADIO SP',
    7043 => 'AGN UNCLASSIFIED RADIO LOUD/INVERT SPECT INVERTED RADIO SP',
    7044 => 'AGN UNCLASSIFIED RADIO LOUD/INVERT SPECT TYPE 1',
    7045 => 'AGN UNCLASSIFIED RADIO LOUD/INVERT SPECT TYPE 1.5',
    7046 => 'AGN UNCLASSIFIED RADIO LOUD/INVERT SPECT TYPE 2',
    7047 => 'AGN UNCLASSIFIED RADIO LOUD/INVERT SPECT',
    7048 => 'AGN UNCLASSIFIED RADIO LOUD/INVERT SPECT',
    7049 => 'AGN UNCLASSIFIED RADIO LOUD/INVERT SPECT',
    7050 => 'AGN UNCLASSIFIED RADIO LOUD/FLAT SPECT',
    7051 => 'AGN UNCLASSIFIED RADIO LOUD/FLAT SPECT FLAT RADIO SP',
    7052 => 'AGN UNCLASSIFIED RADIO LOUD/FLAT SPECT STEEP RADIO SP',
    7053 => 'AGN UNCLASSIFIED RADIO LOUD/FLAT SPECT INVERTED RADIO SP',
    7054 => 'AGN UNCLASSIFIED RADIO LOUD/FLAT SPECT TYPE 1',
    7055 => 'AGN UNCLASSIFIED RADIO LOUD/FLAT SPECT TYPE 1.5',
    7056 => 'AGN UNCLASSIFIED RADIO LOUD/FLAT SPECT TYPE 2',
    7057 => 'AGN UNCLASSIFIED RADIO LOUD/FLAT SPECT',
    7058 => 'AGN UNCLASSIFIED RADIO LOUD/FLAT SPECT',
    7059 => 'AGN UNCLASSIFIED RADIO LOUD/FLAT SPECT',
    7060 => 'AGN UNCLASSIFIED RADIO LOUD/STEEP SPECT',
    7061 => 'AGN UNCLASSIFIED RADIO LOUD/STEEP SPECT FLAT RADIO SP',
    7062 => 'AGN UNCLASSIFIED RADIO LOUD/STEEP SPECT STEEP RADIO SP',
    7063 => 'AGN UNCLASSIFIED RADIO LOUD/STEEP SPECT INVERTED RADIO SP',
    7064 => 'AGN UNCLASSIFIED RADIO LOUD/STEEP SPECT TYPE 1',
    7065 => 'AGN UNCLASSIFIED RADIO LOUD/STEEP SPECT TYPE 1.5',
    7066 => 'AGN UNCLASSIFIED RADIO LOUD/STEEP SPECT TYPE 2',
    7067 => 'AGN UNCLASSIFIED RADIO LOUD/STEEP SPECT',
    7068 => 'AGN UNCLASSIFIED RADIO LOUD/STEEP SPECT',
    7069 => 'AGN UNCLASSIFIED RADIO LOUD/STEEP SPECT',
    7070 => 'AGN UNCLASSIFIED RADIO POLAR/INVERT SPECT',
    7071 => 'AGN UNCLASSIFIED RADIO POLAR/INVERT SPECT FLAT RADIO SP',
    7072 => 'AGN UNCLASSIFIED RADIO POLAR/INVERT SPECT STEEP RADIO SP',
    7073 => 'AGN UNCLASSIFIED RADIO POLAR/INVERT SPECT INVERTED RADIO SP',
    7074 => 'AGN UNCLASSIFIED RADIO POLAR/INVERT SPECT TYPE 1',
    7075 => 'AGN UNCLASSIFIED RADIO POLAR/INVERT SPECT TYPE 1.5',
    7076 => 'AGN UNCLASSIFIED RADIO POLAR/INVERT SPECT TYPE 2',
    7077 => 'AGN UNCLASSIFIED RADIO POLAR/INVERT SPECT',
    7078 => 'AGN UNCLASSIFIED RADIO POLAR/INVERT SPECT',
    7079 => 'AGN UNCLASSIFIED RADIO POLAR/INVERT SPECT',
    7080 => 'AGN UNCLASSIFIED RADIO POLAR/FLAT SPECT',
    7081 => 'AGN UNCLASSIFIED RADIO POLAR/FLAT SPECT FLAT RADIO SP',
    7082 => 'AGN UNCLASSIFIED RADIO POLAR/FLAT SPECT STEEP RADIO SP',
    7083 => 'AGN UNCLASSIFIED RADIO POLAR/FLAT SPECT INVERTED RADIO SP',
    7084 => 'AGN UNCLASSIFIED RADIO POLAR/FLAT SPECT TYPE 1',
    7085 => 'AGN UNCLASSIFIED RADIO POLAR/FLAT SPECT TYPE 1.5',
    7086 => 'AGN UNCLASSIFIED RADIO POLAR/FLAT SPECT TYPE 2',
    7087 => 'AGN UNCLASSIFIED RADIO POLAR/FLAT SPECT',
    7088 => 'AGN UNCLASSIFIED RADIO POLAR/FLAT SPECT',
    7089 => 'AGN UNCLASSIFIED RADIO POLAR/FLAT SPECT',
    7090 => 'AGN UNCLASSIFIED RADIO POLAR/STEEP SPECT',
    7091 => 'AGN UNCLASSIFIED RADIO POLAR/STEEP SPECT FLAT RADIO SP',
    7092 => 'AGN UNCLASSIFIED RADIO POLAR/STEEP SPECT STEEP RADIO SP',
    7093 => 'AGN UNCLASSIFIED RADIO POLAR/STEEP SPECT INVERTED RADIO SP',
    7094 => 'AGN UNCLASSIFIED RADIO POLAR/STEEP SPECT TYPE 1',
    7095 => 'AGN UNCLASSIFIED RADIO POLAR/STEEP SPECT TYPE 1.5',
    7096 => 'AGN UNCLASSIFIED RADIO POLAR/STEEP SPECT TYPE 2',
    7097 => 'AGN UNCLASSIFIED RADIO POLAR/STEEP SPECT',
    7098 => 'AGN UNCLASSIFIED RADIO POLAR/STEEP SPECT',
    7099 => 'AGN UNCLASSIFIED RADIO POLAR/STEEP SPECT',
    7100 => 'SEYFERT',
    7101 => 'SEYFERT FLAT RADIO SP',
    7102 => 'SEYFERT STEEP RADIO SP',
    7103 => 'SEYFERT INVERTED RADIO SP',
    7104 => 'SEYFERT TYPE 1',
    7105 => 'SEYFERT TYPE 1.5',
    7106 => 'SEYFERT TYPE 2',
    7107 => 'SEYFERT',
    7108 => 'SEYFERT',
    7109 => 'SEYFERT',
    7110 => 'SEYFERT RADIO LOUD',
    7111 => 'SEYFERT RADIO LOUD FLAT RADIO SP',
    7112 => 'SEYFERT RADIO LOUD STEEP RADIO SP',
    7113 => 'SEYFERT RADIO LOUD INVERTED RADIO SP',
    7114 => 'SEYFERT RADIO LOUD TYPE 1',
    7115 => 'SEYFERT RADIO LOUD TYPE 1.5',
    7116 => 'SEYFERT RADIO LOUD TYPE 2',
    7117 => 'SEYFERT RADIO LOUD',
    7118 => 'SEYFERT RADIO LOUD',
    7119 => 'SEYFERT RADIO LOUD',
    7120 => 'SEYFERT RADIO LOUD/POLARISED',
    7121 => 'SEYFERT RADIO LOUD/POLARISED FLAT RADIO SP',
    7122 => 'SEYFERT RADIO LOUD/POLARISED STEEP RADIO SP',
    7123 => 'SEYFERT RADIO LOUD/POLARISED INVERTED RADIO SP',
    7124 => 'SEYFERT RADIO LOUD/POLARISED TYPE 1',
    7125 => 'SEYFERT RADIO LOUD/POLARISED TYPE 1.5',
    7126 => 'SEYFERT RADIO LOUD/POLARISED TYPE 2',
    7127 => 'SEYFERT RADIO LOUD/POLARISED',
    7128 => 'SEYFERT RADIO LOUD/POLARISED',
    7129 => 'SEYFERT RADIO LOUD/POLARISED',
    7130 => 'SEYFERT RADIO QUIET',
    7131 => 'SEYFERT RADIO QUIET FLAT RADIO SP',
    7132 => 'SEYFERT RADIO QUIET STEEP RADIO SP',
    7133 => 'SEYFERT RADIO QUIET INVERTED RADIO SP',
    7134 => 'SEYFERT RADIO QUIET TYPE 1',
    7135 => 'SEYFERT RADIO QUIET TYPE 1.5',
    7136 => 'SEYFERT RADIO QUIET TYPE 2',
    7137 => 'SEYFERT RADIO QUIET',
    7138 => 'SEYFERT RADIO QUIET',
    7139 => 'SEYFERT RADIO QUIET',
    7140 => 'SEYFERT RADIO LOUD/INVERT SPECT',
    7141 => 'SEYFERT RADIO LOUD/INVERT SPECT FLAT RADIO SP',
    7142 => 'SEYFERT RADIO LOUD/INVERT SPECT STEEP RADIO SP',
    7143 => 'SEYFERT RADIO LOUD/INVERT SPECT INVERTED RADIO SP',
    7144 => 'SEYFERT RADIO LOUD/INVERT SPECT TYPE 1',
    7145 => 'SEYFERT RADIO LOUD/INVERT SPECT TYPE 1.5',
    7146 => 'SEYFERT RADIO LOUD/INVERT SPECT TYPE 2',
    7147 => 'SEYFERT RADIO LOUD/INVERT SPECT',
    7148 => 'SEYFERT RADIO LOUD/INVERT SPECT',
    7149 => 'SEYFERT RADIO LOUD/INVERT SPECT',
    7150 => 'SEYFERT RADIO LOUD/FLAT SPECT',
    7151 => 'SEYFERT RADIO LOUD/FLAT SPECT FLAT RADIO SP',
    7152 => 'SEYFERT RADIO LOUD/FLAT SPECT STEEP RADIO SP',
    7153 => 'SEYFERT RADIO LOUD/FLAT SPECT INVERTED RADIO SP',
    7154 => 'SEYFERT RADIO LOUD/FLAT SPECT TYPE 1',
    7155 => 'SEYFERT RADIO LOUD/FLAT SPECT TYPE 1.5',
    7156 => 'SEYFERT RADIO LOUD/FLAT SPECT TYPE 2',
    7157 => 'SEYFERT RADIO LOUD/FLAT SPECT',
    7158 => 'SEYFERT RADIO LOUD/FLAT SPECT',
    7159 => 'SEYFERT RADIO LOUD/FLAT SPECT',
    7160 => 'SEYFERT RADIO LOUD/STEEP SPECT',
    7161 => 'SEYFERT RADIO LOUD/STEEP SPECT FLAT RADIO SP',
    7162 => 'SEYFERT RADIO LOUD/STEEP SPECT STEEP RADIO SP',
    7163 => 'SEYFERT RADIO LOUD/STEEP SPECT INVERTED RADIO SP',
    7164 => 'SEYFERT RADIO LOUD/STEEP SPECT TYPE 1',
    7165 => 'SEYFERT RADIO LOUD/STEEP SPECT TYPE 1.5',
    7166 => 'SEYFERT RADIO LOUD/STEEP SPECT TYPE 2',
    7167 => 'SEYFERT RADIO LOUD/STEEP SPECT',
    7168 => 'SEYFERT RADIO LOUD/STEEP SPECT',
    7169 => 'SEYFERT RADIO LOUD/STEEP SPECT',
    7170 => 'SEYFERT RADIO POLAR/INVERT SPECT',
    7171 => 'SEYFERT RADIO POLAR/INVERT SPECT FLAT RADIO SP',
    7172 => 'SEYFERT RADIO POLAR/INVERT SPECT STEEP RADIO SP',
    7173 => 'SEYFERT RADIO POLAR/INVERT SPECT INVERTED RADIO SP',
    7174 => 'SEYFERT RADIO POLAR/INVERT SPECT TYPE 1',
    7175 => 'SEYFERT RADIO POLAR/INVERT SPECT TYPE 1.5',
    7176 => 'SEYFERT RADIO POLAR/INVERT SPECT TYPE 2',
    7177 => 'SEYFERT RADIO POLAR/INVERT SPECT',
    7178 => 'SEYFERT RADIO POLAR/INVERT SPECT',
    7179 => 'SEYFERT RADIO POLAR/INVERT SPECT',
    7180 => 'SEYFERT RADIO POLAR/FLAT SPECT',
    7181 => 'SEYFERT RADIO POLAR/FLAT SPECT FLAT RADIO SP',
    7182 => 'SEYFERT RADIO POLAR/FLAT SPECT STEEP RADIO SP',
    7183 => 'SEYFERT RADIO POLAR/FLAT SPECT INVERTED RADIO SP',
    7184 => 'SEYFERT RADIO POLAR/FLAT SPECT TYPE 1',
    7185 => 'SEYFERT RADIO POLAR/FLAT SPECT TYPE 1.5',
    7186 => 'SEYFERT RADIO POLAR/FLAT SPECT TYPE 2',
    7187 => 'SEYFERT RADIO POLAR/FLAT SPECT',
    7188 => 'SEYFERT RADIO POLAR/FLAT SPECT',
    7189 => 'SEYFERT RADIO POLAR/FLAT SPECT',
    7190 => 'SEYFERT RADIO POLAR/STEEP SPECT',
    7191 => 'SEYFERT RADIO POLAR/STEEP SPECT FLAT RADIO SP',
    7192 => 'SEYFERT RADIO POLAR/STEEP SPECT STEEP RADIO SP',
    7193 => 'SEYFERT RADIO POLAR/STEEP SPECT INVERTED RADIO SP',
    7194 => 'SEYFERT RADIO POLAR/STEEP SPECT TYPE 1',
    7195 => 'SEYFERT RADIO POLAR/STEEP SPECT TYPE 1.5',
    7196 => 'SEYFERT RADIO POLAR/STEEP SPECT TYPE 2',
    7197 => 'SEYFERT RADIO POLAR/STEEP SPECT',
    7198 => 'SEYFERT RADIO POLAR/STEEP SPECT',
    7199 => 'SEYFERT RADIO POLAR/STEEP SPECT',
    7200 => 'QSO',
    7201 => 'QSO FLAT RADIO SP',
    7202 => 'QSO STEEP RADIO SP',
    7203 => 'QSO INVERTED RADIO SP',
    7204 => 'QSO TYPE 1',
    7205 => 'QSO TYPE 1.5',
    7206 => 'QSO TYPE 2',
    7207 => 'QSO',
    7208 => 'QSO',
    7209 => 'QSO',
    7210 => 'QSO RADIO LOUD',
    7211 => 'QSO RADIO LOUD FLAT RADIO SP',
    7212 => 'QSO RADIO LOUD STEEP RADIO SP',
    7213 => 'QSO RADIO LOUD INVERTED RADIO SP',
    7214 => 'QSO RADIO LOUD TYPE 1',
    7215 => 'QSO RADIO LOUD TYPE 1.5',
    7216 => 'QSO RADIO LOUD TYPE 2',
    7217 => 'QSO RADIO LOUD',
    7218 => 'QSO RADIO LOUD',
    7219 => 'QSO RADIO LOUD',
    7220 => 'QSO RADIO LOUD/POLARISED',
    7221 => 'QSO RADIO LOUD/POLARISED FLAT RADIO SP',
    7222 => 'QSO RADIO LOUD/POLARISED STEEP RADIO SP',
    7223 => 'QSO RADIO LOUD/POLARISED INVERTED RADIO SP',
    7224 => 'QSO RADIO LOUD/POLARISED TYPE 1',
    7225 => 'QSO RADIO LOUD/POLARISED TYPE 1.5',
    7226 => 'QSO RADIO LOUD/POLARISED TYPE 2',
    7227 => 'QSO RADIO LOUD/POLARISED',
    7228 => 'QSO RADIO LOUD/POLARISED',
    7229 => 'QSO RADIO LOUD/POLARISED',
    7230 => 'QSO RADIO QUIET',
    7231 => 'QSO RADIO QUIET FLAT RADIO SP',
    7232 => 'QSO RADIO QUIET STEEP RADIO SP',
    7233 => 'QSO RADIO QUIET INVERTED RADIO SP',
    7234 => 'QSO RADIO QUIET TYPE 1',
    7235 => 'QSO RADIO QUIET TYPE 1.5',
    7236 => 'QSO RADIO QUIET TYPE 2',
    7237 => 'QSO RADIO QUIET',
    7238 => 'QSO RADIO QUIET',
    7239 => 'QSO RADIO QUIET',
    7240 => 'QSO RADIO LOUD/INVERT SPECT',
    7241 => 'QSO RADIO LOUD/INVERT SPECT FLAT RADIO SP',
    7242 => 'QSO RADIO LOUD/INVERT SPECT STEEP RADIO SP',
    7243 => 'QSO RADIO LOUD/INVERT SPECT INVERTED RADIO SP',
    7244 => 'QSO RADIO LOUD/INVERT SPECT TYPE 1',
    7245 => 'QSO RADIO LOUD/INVERT SPECT TYPE 1.5',
    7246 => 'QSO RADIO LOUD/INVERT SPECT TYPE 2',
    7247 => 'QSO RADIO LOUD/INVERT SPECT',
    7248 => 'QSO RADIO LOUD/INVERT SPECT',
    7249 => 'QSO RADIO LOUD/INVERT SPECT',
    7250 => 'QSO RADIO LOUD/FLAT SPECT',
    7251 => 'QSO RADIO LOUD/FLAT SPECT FLAT RADIO SP',
    7252 => 'QSO RADIO LOUD/FLAT SPECT STEEP RADIO SP',
    7253 => 'QSO RADIO LOUD/FLAT SPECT INVERTED RADIO SP',
    7254 => 'QSO RADIO LOUD/FLAT SPECT TYPE 1',
    7255 => 'QSO RADIO LOUD/FLAT SPECT TYPE 1.5',
    7256 => 'QSO RADIO LOUD/FLAT SPECT TYPE 2',
    7257 => 'QSO RADIO LOUD/FLAT SPECT',
    7258 => 'QSO RADIO LOUD/FLAT SPECT',
    7259 => 'QSO RADIO LOUD/FLAT SPECT',
    7260 => 'QSO RADIO LOUD/STEEP SPECT',
    7261 => 'QSO RADIO LOUD/STEEP SPECT FLAT RADIO SP',
    7262 => 'QSO RADIO LOUD/STEEP SPECT STEEP RADIO SP',
    7263 => 'QSO RADIO LOUD/STEEP SPECT INVERTED RADIO SP',
    7264 => 'QSO RADIO LOUD/STEEP SPECT TYPE 1',
    7265 => 'QSO RADIO LOUD/STEEP SPECT TYPE 1.5',
    7266 => 'QSO RADIO LOUD/STEEP SPECT TYPE 2',
    7267 => 'QSO RADIO LOUD/STEEP SPECT',
    7268 => 'QSO RADIO LOUD/STEEP SPECT',
    7269 => 'QSO RADIO LOUD/STEEP SPECT',
    7270 => 'QSO RADIO POLAR/INVERT SPECT',
    7271 => 'QSO RADIO POLAR/INVERT SPECT FLAT RADIO SP',
    7272 => 'QSO RADIO POLAR/INVERT SPECT STEEP RADIO SP',
    7273 => 'QSO RADIO POLAR/INVERT SPECT INVERTED RADIO SP',
    7274 => 'QSO RADIO POLAR/INVERT SPECT TYPE 1',
    7275 => 'QSO RADIO POLAR/INVERT SPECT TYPE 1.5',
    7276 => 'QSO RADIO POLAR/INVERT SPECT TYPE 2',
    7277 => 'QSO RADIO POLAR/INVERT SPECT',
    7278 => 'QSO RADIO POLAR/INVERT SPECT',
    7279 => 'QSO RADIO POLAR/INVERT SPECT',
    7280 => 'QSO RADIO POLAR/FLAT SPECT',
    7281 => 'QSO RADIO POLAR/FLAT SPECT FLAT RADIO SP',
    7282 => 'QSO RADIO POLAR/FLAT SPECT STEEP RADIO SP',
    7283 => 'QSO RADIO POLAR/FLAT SPECT INVERTED RADIO SP',
    7284 => 'QSO RADIO POLAR/FLAT SPECT TYPE 1',
    7285 => 'QSO RADIO POLAR/FLAT SPECT TYPE 1.5',
    7286 => 'QSO RADIO POLAR/FLAT SPECT TYPE 2',
    7287 => 'QSO RADIO POLAR/FLAT SPECT',
    7288 => 'QSO RADIO POLAR/FLAT SPECT',
    7289 => 'QSO RADIO POLAR/FLAT SPECT',
    7290 => 'QSO RADIO POLAR/STEEP SPECT',
    7291 => 'QSO RADIO POLAR/STEEP SPECT FLAT RADIO SP',
    7292 => 'QSO RADIO POLAR/STEEP SPECT STEEP RADIO SP',
    7293 => 'QSO RADIO POLAR/STEEP SPECT INVERTED RADIO SP',
    7294 => 'QSO RADIO POLAR/STEEP SPECT TYPE 1',
    7295 => 'QSO RADIO POLAR/STEEP SPECT TYPE 1.5',
    7296 => 'QSO RADIO POLAR/STEEP SPECT TYPE 2',
    7297 => 'QSO RADIO POLAR/STEEP SPECT',
    7298 => 'QSO RADIO POLAR/STEEP SPECT',
    7299 => 'QSO RADIO POLAR/STEEP SPECT',
    7300 => 'BL LAC',
    7301 => 'BL LAC FLAT RADIO SP',
    7302 => 'BL LAC STEEP RADIO SP',
    7303 => 'BL LAC INVERTED RADIO SP',
    7304 => 'BL LAC TYPE 1',
    7305 => 'BL LAC TYPE 1.5',
    7306 => 'BL LAC TYPE 2',
    7307 => 'BL LAC',
    7308 => 'BL LAC',
    7309 => 'BL LAC',
    7310 => 'BL LAC RADIO LOUD',
    7311 => 'BL LAC RADIO LOUD FLAT RADIO SP',
    7312 => 'BL LAC RADIO LOUD STEEP RADIO SP',
    7313 => 'BL LAC RADIO LOUD INVERTED RADIO SP',
    7314 => 'BL LAC RADIO LOUD TYPE 1',
    7315 => 'BL LAC RADIO LOUD TYPE 1.5',
    7316 => 'BL LAC RADIO LOUD TYPE 2',
    7317 => 'BL LAC RADIO LOUD',
    7318 => 'BL LAC RADIO LOUD',
    7319 => 'BL LAC RADIO LOUD',
    7320 => 'BL LAC RADIO LOUD/POLARISED',
    7321 => 'BL LAC RADIO LOUD/POLARISED FLAT RADIO SP',
    7322 => 'BL LAC RADIO LOUD/POLARISED STEEP RADIO SP',
    7323 => 'BL LAC RADIO LOUD/POLARISED INVERTED RADIO SP',
    7324 => 'BL LAC RADIO LOUD/POLARISED TYPE 1',
    7325 => 'BL LAC RADIO LOUD/POLARISED TYPE 1.5',
    7326 => 'BL LAC RADIO LOUD/POLARISED TYPE 2',
    7327 => 'BL LAC RADIO LOUD/POLARISED',
    7328 => 'BL LAC RADIO LOUD/POLARISED',
    7329 => 'BL LAC RADIO LOUD/POLARISED',
    7330 => 'BL LAC RADIO QUIET',
    7331 => 'BL LAC RADIO QUIET FLAT RADIO SP',
    7332 => 'BL LAC RADIO QUIET STEEP RADIO SP',
    7333 => 'BL LAC RADIO QUIET INVERTED RADIO SP',
    7334 => 'BL LAC RADIO QUIET TYPE 1',
    7335 => 'BL LAC RADIO QUIET TYPE 1.5',
    7336 => 'BL LAC RADIO QUIET TYPE 2',
    7337 => 'BL LAC RADIO QUIET',
    7338 => 'BL LAC RADIO QUIET',
    7339 => 'BL LAC RADIO QUIET',
    7340 => 'BL LAC RADIO LOUD/INVERT SPECT',
    7341 => 'BL LAC RADIO LOUD/INVERT SPECT FLAT RADIO SP',
    7342 => 'BL LAC RADIO LOUD/INVERT SPECT STEEP RADIO SP',
    7343 => 'BL LAC RADIO LOUD/INVERT SPECT INVERTED RADIO SP',
    7344 => 'BL LAC RADIO LOUD/INVERT SPECT TYPE 1',
    7345 => 'BL LAC RADIO LOUD/INVERT SPECT TYPE 1.5',
    7346 => 'BL LAC RADIO LOUD/INVERT SPECT TYPE 2',
    7347 => 'BL LAC RADIO LOUD/INVERT SPECT',
    7348 => 'BL LAC RADIO LOUD/INVERT SPECT',
    7349 => 'BL LAC RADIO LOUD/INVERT SPECT',
    7350 => 'BL LAC RADIO LOUD/FLAT SPECT',
    7351 => 'BL LAC RADIO LOUD/FLAT SPECT FLAT RADIO SP',
    7352 => 'BL LAC RADIO LOUD/FLAT SPECT STEEP RADIO SP',
    7353 => 'BL LAC RADIO LOUD/FLAT SPECT INVERTED RADIO SP',
    7354 => 'BL LAC RADIO LOUD/FLAT SPECT TYPE 1',
    7355 => 'BL LAC RADIO LOUD/FLAT SPECT TYPE 1.5',
    7356 => 'BL LAC RADIO LOUD/FLAT SPECT TYPE 2',
    7357 => 'BL LAC RADIO LOUD/FLAT SPECT',
    7358 => 'BL LAC RADIO LOUD/FLAT SPECT',
    7359 => 'BL LAC RADIO LOUD/FLAT SPECT',
    7360 => 'BL LAC RADIO LOUD/STEEP SPECT',
    7361 => 'BL LAC RADIO LOUD/STEEP SPECT FLAT RADIO SP',
    7362 => 'BL LAC RADIO LOUD/STEEP SPECT STEEP RADIO SP',
    7363 => 'BL LAC RADIO LOUD/STEEP SPECT INVERTED RADIO SP',
    7364 => 'BL LAC RADIO LOUD/STEEP SPECT TYPE 1',
    7365 => 'BL LAC RADIO LOUD/STEEP SPECT TYPE 1.5',
    7366 => 'BL LAC RADIO LOUD/STEEP SPECT TYPE 2',
    7367 => 'BL LAC RADIO LOUD/STEEP SPECT',
    7368 => 'BL LAC RADIO LOUD/STEEP SPECT',
    7369 => 'BL LAC RADIO LOUD/STEEP SPECT',
    7370 => 'BL LAC RADIO POLAR/INVERT SPECT',
    7371 => 'BL LAC RADIO POLAR/INVERT SPECT FLAT RADIO SP',
    7372 => 'BL LAC RADIO POLAR/INVERT SPECT STEEP RADIO SP',
    7373 => 'BL LAC RADIO POLAR/INVERT SPECT INVERTED RADIO SP',
    7374 => 'BL LAC RADIO POLAR/INVERT SPECT TYPE 1',
    7375 => 'BL LAC RADIO POLAR/INVERT SPECT TYPE 1.5',
    7376 => 'BL LAC RADIO POLAR/INVERT SPECT TYPE 2',
    7377 => 'BL LAC RADIO POLAR/INVERT SPECT',
    7378 => 'BL LAC RADIO POLAR/INVERT SPECT',
    7379 => 'BL LAC RADIO POLAR/INVERT SPECT',
    7380 => 'BL LAC RADIO POLAR/FLAT SPECT',
    7381 => 'BL LAC RADIO POLAR/FLAT SPECT FLAT RADIO SP',
    7382 => 'BL LAC RADIO POLAR/FLAT SPECT STEEP RADIO SP',
    7383 => 'BL LAC RADIO POLAR/FLAT SPECT INVERTED RADIO SP',
    7384 => 'BL LAC RADIO POLAR/FLAT SPECT TYPE 1',
    7385 => 'BL LAC RADIO POLAR/FLAT SPECT TYPE 1.5',
    7386 => 'BL LAC RADIO POLAR/FLAT SPECT TYPE 2',
    7387 => 'BL LAC RADIO POLAR/FLAT SPECT',
    7388 => 'BL LAC RADIO POLAR/FLAT SPECT',
    7389 => 'BL LAC RADIO POLAR/FLAT SPECT',
    7390 => 'BL LAC RADIO POLAR/STEEP SPECT',
    7391 => 'BL LAC RADIO POLAR/STEEP SPECT FLAT RADIO SP',
    7392 => 'BL LAC RADIO POLAR/STEEP SPECT STEEP RADIO SP',
    7393 => 'BL LAC RADIO POLAR/STEEP SPECT INVERTED RADIO SP',
    7394 => 'BL LAC RADIO POLAR/STEEP SPECT TYPE 1',
    7395 => 'BL LAC RADIO POLAR/STEEP SPECT TYPE 1.5',
    7396 => 'BL LAC RADIO POLAR/STEEP SPECT TYPE 2',
    7397 => 'BL LAC RADIO POLAR/STEEP SPECT',
    7398 => 'BL LAC RADIO POLAR/STEEP SPECT',
    7399 => 'BL LAC RADIO POLAR/STEEP SPECT',
    7400 => 'LINER',
    7401 => 'LINER FLAT RADIO SP',
    7402 => 'LINER STEEP RADIO SP',
    7403 => 'LINER INVERTED RADIO SP',
    7404 => 'LINER TYPE 1',
    7405 => 'LINER TYPE 1.5',
    7406 => 'LINER TYPE 2',
    7407 => 'LINER',
    7408 => 'LINER',
    7409 => 'LINER',
    7410 => 'LINER RADIO LOUD',
    7411 => 'LINER RADIO LOUD FLAT RADIO SP',
    7412 => 'LINER RADIO LOUD STEEP RADIO SP',
    7413 => 'LINER RADIO LOUD INVERTED RADIO SP',
    7414 => 'LINER RADIO LOUD TYPE 1',
    7415 => 'LINER RADIO LOUD TYPE 1.5',
    7416 => 'LINER RADIO LOUD TYPE 2',
    7417 => 'LINER RADIO LOUD',
    7418 => 'LINER RADIO LOUD',
    7419 => 'LINER RADIO LOUD',
    7420 => 'LINER RADIO LOUD/POLARISED',
    7421 => 'LINER RADIO LOUD/POLARISED FLAT RADIO SP',
    7422 => 'LINER RADIO LOUD/POLARISED STEEP RADIO SP',
    7423 => 'LINER RADIO LOUD/POLARISED INVERTED RADIO SP',
    7424 => 'LINER RADIO LOUD/POLARISED TYPE 1',
    7425 => 'LINER RADIO LOUD/POLARISED TYPE 1.5',
    7426 => 'LINER RADIO LOUD/POLARISED TYPE 2',
    7427 => 'LINER RADIO LOUD/POLARISED',
    7428 => 'LINER RADIO LOUD/POLARISED',
    7429 => 'LINER RADIO LOUD/POLARISED',
    7430 => 'LINER RADIO QUIET',
    7431 => 'LINER RADIO QUIET FLAT RADIO SP',
    7432 => 'LINER RADIO QUIET STEEP RADIO SP',
    7433 => 'LINER RADIO QUIET INVERTED RADIO SP',
    7434 => 'LINER RADIO QUIET TYPE 1',
    7435 => 'LINER RADIO QUIET TYPE 1.5',
    7436 => 'LINER RADIO QUIET TYPE 2',
    7437 => 'LINER RADIO QUIET',
    7438 => 'LINER RADIO QUIET',
    7439 => 'LINER RADIO QUIET',
    7440 => 'LINER RADIO LOUD/INVERT SPECT',
    7441 => 'LINER RADIO LOUD/INVERT SPECT FLAT RADIO SP',
    7442 => 'LINER RADIO LOUD/INVERT SPECT STEEP RADIO SP',
    7443 => 'LINER RADIO LOUD/INVERT SPECT INVERTED RADIO SP',
    7444 => 'LINER RADIO LOUD/INVERT SPECT TYPE 1',
    7445 => 'LINER RADIO LOUD/INVERT SPECT TYPE 1.5',
    7446 => 'LINER RADIO LOUD/INVERT SPECT TYPE 2',
    7447 => 'LINER RADIO LOUD/INVERT SPECT',
    7448 => 'LINER RADIO LOUD/INVERT SPECT',
    7449 => 'LINER RADIO LOUD/INVERT SPECT',
    7450 => 'LINER RADIO LOUD/FLAT SPECT',
    7451 => 'LINER RADIO LOUD/FLAT SPECT FLAT RADIO SP',
    7452 => 'LINER RADIO LOUD/FLAT SPECT STEEP RADIO SP',
    7453 => 'LINER RADIO LOUD/FLAT SPECT INVERTED RADIO SP',
    7454 => 'LINER RADIO LOUD/FLAT SPECT TYPE 1',
    7455 => 'LINER RADIO LOUD/FLAT SPECT TYPE 1.5',
    7456 => 'LINER RADIO LOUD/FLAT SPECT TYPE 2',
    7457 => 'LINER RADIO LOUD/FLAT SPECT',
    7458 => 'LINER RADIO LOUD/FLAT SPECT',
    7459 => 'LINER RADIO LOUD/FLAT SPECT',
    7460 => 'LINER RADIO LOUD/STEEP SPECT',
    7461 => 'LINER RADIO LOUD/STEEP SPECT FLAT RADIO SP',
    7462 => 'LINER RADIO LOUD/STEEP SPECT STEEP RADIO SP',
    7463 => 'LINER RADIO LOUD/STEEP SPECT INVERTED RADIO SP',
    7464 => 'LINER RADIO LOUD/STEEP SPECT TYPE 1',
    7465 => 'LINER RADIO LOUD/STEEP SPECT TYPE 1.5',
    7466 => 'LINER RADIO LOUD/STEEP SPECT TYPE 2',
    7467 => 'LINER RADIO LOUD/STEEP SPECT',
    7468 => 'LINER RADIO LOUD/STEEP SPECT',
    7469 => 'LINER RADIO LOUD/STEEP SPECT',
    7470 => 'LINER RADIO POLAR/INVERT SPECT',
    7471 => 'LINER RADIO POLAR/INVERT SPECT FLAT RADIO SP',
    7472 => 'LINER RADIO POLAR/INVERT SPECT STEEP RADIO SP',
    7473 => 'LINER RADIO POLAR/INVERT SPECT INVERTED RADIO SP',
    7474 => 'LINER RADIO POLAR/INVERT SPECT TYPE 1',
    7475 => 'LINER RADIO POLAR/INVERT SPECT TYPE 1.5',
    7476 => 'LINER RADIO POLAR/INVERT SPECT TYPE 2',
    7477 => 'LINER RADIO POLAR/INVERT SPECT',
    7478 => 'LINER RADIO POLAR/INVERT SPECT',
    7479 => 'LINER RADIO POLAR/INVERT SPECT',
    7480 => 'LINER RADIO POLAR/FLAT SPECT',
    7481 => 'LINER RADIO POLAR/FLAT SPECT FLAT RADIO SP',
    7482 => 'LINER RADIO POLAR/FLAT SPECT STEEP RADIO SP',
    7483 => 'LINER RADIO POLAR/FLAT SPECT INVERTED RADIO SP',
    7484 => 'LINER RADIO POLAR/FLAT SPECT TYPE 1',
    7485 => 'LINER RADIO POLAR/FLAT SPECT TYPE 1.5',
    7486 => 'LINER RADIO POLAR/FLAT SPECT TYPE 2',
    7487 => 'LINER RADIO POLAR/FLAT SPECT',
    7488 => 'LINER RADIO POLAR/FLAT SPECT',
    7489 => 'LINER RADIO POLAR/FLAT SPECT',
    7490 => 'LINER RADIO POLAR/STEEP SPECT',
    7491 => 'LINER RADIO POLAR/STEEP SPECT FLAT RADIO SP',
    7492 => 'LINER RADIO POLAR/STEEP SPECT STEEP RADIO SP',
    7493 => 'LINER RADIO POLAR/STEEP SPECT INVERTED RADIO SP',
    7494 => 'LINER RADIO POLAR/STEEP SPECT TYPE 1',
    7495 => 'LINER RADIO POLAR/STEEP SPECT TYPE 1.5',
    7496 => 'LINER RADIO POLAR/STEEP SPECT TYPE 2',
    7497 => 'LINER RADIO POLAR/STEEP SPECT',
    7498 => 'LINER RADIO POLAR/STEEP SPECT',
    7499 => 'LINER RADIO POLAR/STEEP SPECT',
    7500 => 'RADIO GALAXY',
    7501 => 'RADIO GALAXY FLAT RADIO SP',
    7502 => 'RADIO GALAXY STEEP RADIO SP',
    7503 => 'RADIO GALAXY INVERTED RADIO SP',
    7504 => 'RADIO GALAXY TYPE 1',
    7505 => 'RADIO GALAXY TYPE 1.5',
    7506 => 'RADIO GALAXY TYPE 2',
    7507 => 'RADIO GALAXY',
    7508 => 'RADIO GALAXY',
    7509 => 'RADIO GALAXY',
    7510 => 'RADIO GALAXY RADIO LOUD',
    7511 => 'RADIO GALAXY RADIO LOUD FLAT RADIO SP',
    7512 => 'RADIO GALAXY RADIO LOUD STEEP RADIO SP',
    7513 => 'RADIO GALAXY RADIO LOUD INVERTED RADIO SP',
    7514 => 'RADIO GALAXY RADIO LOUD TYPE 1',
    7515 => 'RADIO GALAXY RADIO LOUD TYPE 1.5',
    7516 => 'RADIO GALAXY RADIO LOUD TYPE 2',
    7517 => 'RADIO GALAXY RADIO LOUD',
    7518 => 'RADIO GALAXY RADIO LOUD',
    7519 => 'RADIO GALAXY RADIO LOUD',
    7520 => 'RADIO GALAXY RADIO LOUD/POLARISED',
    7521 => 'RADIO GALAXY RADIO LOUD/POLARISED FLAT RADIO SP',
    7522 => 'RADIO GALAXY RADIO LOUD/POLARISED STEEP RADIO SP',
    7523 => 'RADIO GALAXY RADIO LOUD/POLARISED INVERTED RADIO SP',
    7524 => 'RADIO GALAXY RADIO LOUD/POLARISED TYPE 1',
    7525 => 'RADIO GALAXY RADIO LOUD/POLARISED TYPE 1.5',
    7526 => 'RADIO GALAXY RADIO LOUD/POLARISED TYPE 2',
    7527 => 'RADIO GALAXY RADIO LOUD/POLARISED',
    7528 => 'RADIO GALAXY RADIO LOUD/POLARISED',
    7529 => 'RADIO GALAXY RADIO LOUD/POLARISED',
    7530 => 'RADIO GALAXY RADIO QUIET',
    7531 => 'RADIO GALAXY RADIO QUIET FLAT RADIO SP',
    7532 => 'RADIO GALAXY RADIO QUIET STEEP RADIO SP',
    7533 => 'RADIO GALAXY RADIO QUIET INVERTED RADIO SP',
    7534 => 'RADIO GALAXY RADIO QUIET TYPE 1',
    7535 => 'RADIO GALAXY RADIO QUIET TYPE 1.5',
    7536 => 'RADIO GALAXY RADIO QUIET TYPE 2',
    7537 => 'RADIO GALAXY RADIO QUIET',
    7538 => 'RADIO GALAXY RADIO QUIET',
    7539 => 'RADIO GALAXY RADIO QUIET',
    7540 => 'RADIO GALAXY RADIO LOUD/INVERT SPECT',
    7541 => 'RADIO GALAXY RADIO LOUD/INVERT SPECT FLAT RADIO SP',
    7542 => 'RADIO GALAXY RADIO LOUD/INVERT SPECT STEEP RADIO SP',
    7543 => 'RADIO GALAXY RADIO LOUD/INVERT SPECT INVERTED RADIO SP',
    7544 => 'RADIO GALAXY RADIO LOUD/INVERT SPECT TYPE 1',
    7545 => 'RADIO GALAXY RADIO LOUD/INVERT SPECT TYPE 1.5',
    7546 => 'RADIO GALAXY RADIO LOUD/INVERT SPECT TYPE 2',
    7547 => 'RADIO GALAXY RADIO LOUD/INVERT SPECT',
    7548 => 'RADIO GALAXY RADIO LOUD/INVERT SPECT',
    7549 => 'RADIO GALAXY RADIO LOUD/INVERT SPECT',
    7550 => 'RADIO GALAXY RADIO LOUD/FLAT SPECT',
    7551 => 'RADIO GALAXY RADIO LOUD/FLAT SPECT FLAT RADIO SP',
    7552 => 'RADIO GALAXY RADIO LOUD/FLAT SPECT STEEP RADIO SP',
    7553 => 'RADIO GALAXY RADIO LOUD/FLAT SPECT INVERTED RADIO SP',
    7554 => 'RADIO GALAXY RADIO LOUD/FLAT SPECT TYPE 1',
    7555 => 'RADIO GALAXY RADIO LOUD/FLAT SPECT TYPE 1.5',
    7556 => 'RADIO GALAXY RADIO LOUD/FLAT SPECT TYPE 2',
    7557 => 'RADIO GALAXY RADIO LOUD/FLAT SPECT',
    7558 => 'RADIO GALAXY RADIO LOUD/FLAT SPECT',
    7559 => 'RADIO GALAXY RADIO LOUD/FLAT SPECT',
    7560 => 'RADIO GALAXY RADIO LOUD/STEEP SPECT',
    7561 => 'RADIO GALAXY RADIO LOUD/STEEP SPECT FLAT RADIO SP',
    7562 => 'RADIO GALAXY RADIO LOUD/STEEP SPECT STEEP RADIO SP',
    7563 => 'RADIO GALAXY RADIO LOUD/STEEP SPECT INVERTED RADIO SP',
    7564 => 'RADIO GALAXY RADIO LOUD/STEEP SPECT TYPE 1',
    7565 => 'RADIO GALAXY RADIO LOUD/STEEP SPECT TYPE 1.5',
    7566 => 'RADIO GALAXY RADIO LOUD/STEEP SPECT TYPE 2',
    7567 => 'RADIO GALAXY RADIO LOUD/STEEP SPECT',
    7568 => 'RADIO GALAXY RADIO LOUD/STEEP SPECT',
    7569 => 'RADIO GALAXY RADIO LOUD/STEEP SPECT',
    7570 => 'RADIO GALAXY RADIO POLAR/INVERT SPECT',
    7571 => 'RADIO GALAXY RADIO POLAR/INVERT SPECT FLAT RADIO SP',
    7572 => 'RADIO GALAXY RADIO POLAR/INVERT SPECT STEEP RADIO SP',
    7573 => 'RADIO GALAXY RADIO POLAR/INVERT SPECT INVERTED RADIO SP',
    7574 => 'RADIO GALAXY RADIO POLAR/INVERT SPECT TYPE 1',
    7575 => 'RADIO GALAXY RADIO POLAR/INVERT SPECT TYPE 1.5',
    7576 => 'RADIO GALAXY RADIO POLAR/INVERT SPECT TYPE 2',
    7577 => 'RADIO GALAXY RADIO POLAR/INVERT SPECT',
    7578 => 'RADIO GALAXY RADIO POLAR/INVERT SPECT',
    7579 => 'RADIO GALAXY RADIO POLAR/INVERT SPECT',
    7580 => 'RADIO GALAXY RADIO POLAR/FLAT SPECT',
    7581 => 'RADIO GALAXY RADIO POLAR/FLAT SPECT FLAT RADIO SP',
    7582 => 'RADIO GALAXY RADIO POLAR/FLAT SPECT STEEP RADIO SP',
    7583 => 'RADIO GALAXY RADIO POLAR/FLAT SPECT INVERTED RADIO SP',
    7584 => 'RADIO GALAXY RADIO POLAR/FLAT SPECT TYPE 1',
    7585 => 'RADIO GALAXY RADIO POLAR/FLAT SPECT TYPE 1.5',
    7586 => 'RADIO GALAXY RADIO POLAR/FLAT SPECT TYPE 2',
    7587 => 'RADIO GALAXY RADIO POLAR/FLAT SPECT',
    7588 => 'RADIO GALAXY RADIO POLAR/FLAT SPECT',
    7589 => 'RADIO GALAXY RADIO POLAR/FLAT SPECT',
    7590 => 'RADIO GALAXY RADIO POLAR/STEEP SPECT',
    7591 => 'RADIO GALAXY RADIO POLAR/STEEP SPECT FLAT RADIO SP',
    7592 => 'RADIO GALAXY RADIO POLAR/STEEP SPECT STEEP RADIO SP',
    7593 => 'RADIO GALAXY RADIO POLAR/STEEP SPECT INVERTED RADIO SP',
    7594 => 'RADIO GALAXY RADIO POLAR/STEEP SPECT TYPE 1',
    7595 => 'RADIO GALAXY RADIO POLAR/STEEP SPECT TYPE 1.5',
    7596 => 'RADIO GALAXY RADIO POLAR/STEEP SPECT TYPE 2',
    7597 => 'RADIO GALAXY RADIO POLAR/STEEP SPECT',
    7598 => 'RADIO GALAXY RADIO POLAR/STEEP SPECT',
    7599 => 'RADIO GALAXY RADIO POLAR/STEEP SPECT',
    7600 => 'IR GALAXY',
    7601 => 'IR GALAXY FLAT RADIO SP',
    7602 => 'IR GALAXY STEEP RADIO SP',
    7603 => 'IR GALAXY INVERTED RADIO SP',
    7604 => 'IR GALAXY TYPE 1',
    7605 => 'IR GALAXY TYPE 1.5',
    7606 => 'IR GALAXY TYPE 2',
    7607 => 'IR GALAXY',
    7608 => 'IR GALAXY',
    7609 => 'IR GALAXY',
    7610 => 'IR GALAXY RADIO LOUD',
    7611 => 'IR GALAXY RADIO LOUD FLAT RADIO SP',
    7612 => 'IR GALAXY RADIO LOUD STEEP RADIO SP',
    7613 => 'IR GALAXY RADIO LOUD INVERTED RADIO SP',
    7614 => 'IR GALAXY RADIO LOUD TYPE 1',
    7615 => 'IR GALAXY RADIO LOUD TYPE 1.5',
    7616 => 'IR GALAXY RADIO LOUD TYPE 2',
    7617 => 'IR GALAXY RADIO LOUD',
    7618 => 'IR GALAXY RADIO LOUD',
    7619 => 'IR GALAXY RADIO LOUD',
    7620 => 'IR GALAXY RADIO LOUD/POLARISED',
    7621 => 'IR GALAXY RADIO LOUD/POLARISED FLAT RADIO SP',
    7622 => 'IR GALAXY RADIO LOUD/POLARISED STEEP RADIO SP',
    7623 => 'IR GALAXY RADIO LOUD/POLARISED INVERTED RADIO SP',
    7624 => 'IR GALAXY RADIO LOUD/POLARISED TYPE 1',
    7625 => 'IR GALAXY RADIO LOUD/POLARISED TYPE 1.5',
    7626 => 'IR GALAXY RADIO LOUD/POLARISED TYPE 2',
    7627 => 'IR GALAXY RADIO LOUD/POLARISED',
    7628 => 'IR GALAXY RADIO LOUD/POLARISED',
    7629 => 'IR GALAXY RADIO LOUD/POLARISED',
    7630 => 'IR GALAXY RADIO QUIET',
    7631 => 'IR GALAXY RADIO QUIET FLAT RADIO SP',
    7632 => 'IR GALAXY RADIO QUIET STEEP RADIO SP',
    7633 => 'IR GALAXY RADIO QUIET INVERTED RADIO SP',
    7634 => 'IR GALAXY RADIO QUIET TYPE 1',
    7635 => 'IR GALAXY RADIO QUIET TYPE 1.5',
    7636 => 'IR GALAXY RADIO QUIET TYPE 2',
    7637 => 'IR GALAXY RADIO QUIET',
    7638 => 'IR GALAXY RADIO QUIET',
    7639 => 'IR GALAXY RADIO QUIET',
    7640 => 'IR GALAXY RADIO LOUD/INVERT SPECT',
    7641 => 'IR GALAXY RADIO LOUD/INVERT SPECT FLAT RADIO SP',
    7642 => 'IR GALAXY RADIO LOUD/INVERT SPECT STEEP RADIO SP',
    7643 => 'IR GALAXY RADIO LOUD/INVERT SPECT INVERTED RADIO SP',
    7644 => 'IR GALAXY RADIO LOUD/INVERT SPECT TYPE 1',
    7645 => 'IR GALAXY RADIO LOUD/INVERT SPECT TYPE 1.5',
    7646 => 'IR GALAXY RADIO LOUD/INVERT SPECT TYPE 2',
    7647 => 'IR GALAXY RADIO LOUD/INVERT SPECT',
    7648 => 'IR GALAXY RADIO LOUD/INVERT SPECT',
    7649 => 'IR GALAXY RADIO LOUD/INVERT SPECT',
    7650 => 'IR GALAXY RADIO LOUD/FLAT SPECT',
    7651 => 'IR GALAXY RADIO LOUD/FLAT SPECT FLAT RADIO SP',
    7652 => 'IR GALAXY RADIO LOUD/FLAT SPECT STEEP RADIO SP',
    7653 => 'IR GALAXY RADIO LOUD/FLAT SPECT INVERTED RADIO SP',
    7654 => 'IR GALAXY RADIO LOUD/FLAT SPECT TYPE 1',
    7655 => 'IR GALAXY RADIO LOUD/FLAT SPECT TYPE 1.5',
    7656 => 'IR GALAXY RADIO LOUD/FLAT SPECT TYPE 2',
    7657 => 'IR GALAXY RADIO LOUD/FLAT SPECT',
    7658 => 'IR GALAXY RADIO LOUD/FLAT SPECT',
    7659 => 'IR GALAXY RADIO LOUD/FLAT SPECT',
    7660 => 'IR GALAXY RADIO LOUD/STEEP SPECT',
    7661 => 'IR GALAXY RADIO LOUD/STEEP SPECT FLAT RADIO SP',
    7662 => 'IR GALAXY RADIO LOUD/STEEP SPECT STEEP RADIO SP',
    7663 => 'IR GALAXY RADIO LOUD/STEEP SPECT INVERTED RADIO SP',
    7664 => 'IR GALAXY RADIO LOUD/STEEP SPECT TYPE 1',
    7665 => 'IR GALAXY RADIO LOUD/STEEP SPECT TYPE 1.5',
    7666 => 'IR GALAXY RADIO LOUD/STEEP SPECT TYPE 2',
    7667 => 'IR GALAXY RADIO LOUD/STEEP SPECT',
    7668 => 'IR GALAXY RADIO LOUD/STEEP SPECT',
    7669 => 'IR GALAXY RADIO LOUD/STEEP SPECT',
    7670 => 'IR GALAXY RADIO POLAR/INVERT SPECT',
    7671 => 'IR GALAXY RADIO POLAR/INVERT SPECT FLAT RADIO SP',
    7672 => 'IR GALAXY RADIO POLAR/INVERT SPECT STEEP RADIO SP',
    7673 => 'IR GALAXY RADIO POLAR/INVERT SPECT INVERTED RADIO SP',
    7674 => 'IR GALAXY RADIO POLAR/INVERT SPECT TYPE 1',
    7675 => 'IR GALAXY RADIO POLAR/INVERT SPECT TYPE 1.5',
    7676 => 'IR GALAXY RADIO POLAR/INVERT SPECT TYPE 2',
    7677 => 'IR GALAXY RADIO POLAR/INVERT SPECT',
    7678 => 'IR GALAXY RADIO POLAR/INVERT SPECT',
    7679 => 'IR GALAXY RADIO POLAR/INVERT SPECT',
    7680 => 'IR GALAXY RADIO POLAR/FLAT SPECT',
    7681 => 'IR GALAXY RADIO POLAR/FLAT SPECT FLAT RADIO SP',
    7682 => 'IR GALAXY RADIO POLAR/FLAT SPECT STEEP RADIO SP',
    7683 => 'IR GALAXY RADIO POLAR/FLAT SPECT INVERTED RADIO SP',
    7684 => 'IR GALAXY RADIO POLAR/FLAT SPECT TYPE 1',
    7685 => 'IR GALAXY RADIO POLAR/FLAT SPECT TYPE 1.5',
    7686 => 'IR GALAXY RADIO POLAR/FLAT SPECT TYPE 2',
    7687 => 'IR GALAXY RADIO POLAR/FLAT SPECT',
    7688 => 'IR GALAXY RADIO POLAR/FLAT SPECT',
    7689 => 'IR GALAXY RADIO POLAR/FLAT SPECT',
    7690 => 'IR GALAXY RADIO POLAR/STEEP SPECT',
    7691 => 'IR GALAXY RADIO POLAR/STEEP SPECT FLAT RADIO SP',
    7692 => 'IR GALAXY RADIO POLAR/STEEP SPECT STEEP RADIO SP',
    7693 => 'IR GALAXY RADIO POLAR/STEEP SPECT INVERTED RADIO SP',
    7694 => 'IR GALAXY RADIO POLAR/STEEP SPECT TYPE 1',
    7695 => 'IR GALAXY RADIO POLAR/STEEP SPECT TYPE 1.5',
    7696 => 'IR GALAXY RADIO POLAR/STEEP SPECT TYPE 2',
    7697 => 'IR GALAXY RADIO POLAR/STEEP SPECT',
    7698 => 'IR GALAXY RADIO POLAR/STEEP SPECT',
    7699 => 'IR GALAXY RADIO POLAR/STEEP SPECT',
    7700 => 'OVV',
    7701 => 'OVV FLAT RADIO SP',
    7702 => 'OVV STEEP RADIO SP',
    7703 => 'OVV INVERTED RADIO SP',
    7704 => 'OVV TYPE 1',
    7705 => 'OVV TYPE 1.5',
    7706 => 'OVV TYPE 2',
    7707 => 'OVV',
    7708 => 'OVV',
    7709 => 'OVV',
    7710 => 'OVV RADIO LOUD',
    7711 => 'OVV RADIO LOUD FLAT RADIO SP',
    7712 => 'OVV RADIO LOUD STEEP RADIO SP',
    7713 => 'OVV RADIO LOUD INVERTED RADIO SP',
    7714 => 'OVV RADIO LOUD TYPE 1',
    7715 => 'OVV RADIO LOUD TYPE 1.5',
    7716 => 'OVV RADIO LOUD TYPE 2',
    7717 => 'OVV RADIO LOUD',
    7718 => 'OVV RADIO LOUD',
    7719 => 'OVV RADIO LOUD',
    7720 => 'OVV RADIO LOUD/POLARISED',
    7721 => 'OVV RADIO LOUD/POLARISED FLAT RADIO SP',
    7722 => 'OVV RADIO LOUD/POLARISED STEEP RADIO SP',
    7723 => 'OVV RADIO LOUD/POLARISED INVERTED RADIO SP',
    7724 => 'OVV RADIO LOUD/POLARISED TYPE 1',
    7725 => 'OVV RADIO LOUD/POLARISED TYPE 1.5',
    7726 => 'OVV RADIO LOUD/POLARISED TYPE 2',
    7727 => 'OVV RADIO LOUD/POLARISED',
    7728 => 'OVV RADIO LOUD/POLARISED',
    7729 => 'OVV RADIO LOUD/POLARISED',
    7730 => 'OVV RADIO QUIET',
    7731 => 'OVV RADIO QUIET FLAT RADIO SP',
    7732 => 'OVV RADIO QUIET STEEP RADIO SP',
    7733 => 'OVV RADIO QUIET INVERTED RADIO SP',
    7734 => 'OVV RADIO QUIET TYPE 1',
    7735 => 'OVV RADIO QUIET TYPE 1.5',
    7736 => 'OVV RADIO QUIET TYPE 2',
    7737 => 'OVV RADIO QUIET',
    7738 => 'OVV RADIO QUIET',
    7739 => 'OVV RADIO QUIET',
    7740 => 'OVV RADIO LOUD/INVERT SPECT',
    7741 => 'OVV RADIO LOUD/INVERT SPECT FLAT RADIO SP',
    7742 => 'OVV RADIO LOUD/INVERT SPECT STEEP RADIO SP',
    7743 => 'OVV RADIO LOUD/INVERT SPECT INVERTED RADIO SP',
    7744 => 'OVV RADIO LOUD/INVERT SPECT TYPE 1',
    7745 => 'OVV RADIO LOUD/INVERT SPECT TYPE 1.5',
    7746 => 'OVV RADIO LOUD/INVERT SPECT TYPE 2',
    7747 => 'OVV RADIO LOUD/INVERT SPECT',
    7748 => 'OVV RADIO LOUD/INVERT SPECT',
    7749 => 'OVV RADIO LOUD/INVERT SPECT',
    7750 => 'OVV RADIO LOUD/FLAT SPECT',
    7751 => 'OVV RADIO LOUD/FLAT SPECT FLAT RADIO SP',
    7752 => 'OVV RADIO LOUD/FLAT SPECT STEEP RADIO SP',
    7753 => 'OVV RADIO LOUD/FLAT SPECT INVERTED RADIO SP',
    7754 => 'OVV RADIO LOUD/FLAT SPECT TYPE 1',
    7755 => 'OVV RADIO LOUD/FLAT SPECT TYPE 1.5',
    7756 => 'OVV RADIO LOUD/FLAT SPECT TYPE 2',
    7757 => 'OVV RADIO LOUD/FLAT SPECT',
    7758 => 'OVV RADIO LOUD/FLAT SPECT',
    7759 => 'OVV RADIO LOUD/FLAT SPECT',
    7760 => 'OVV RADIO LOUD/STEEP SPECT',
    7761 => 'OVV RADIO LOUD/STEEP SPECT FLAT RADIO SP',
    7762 => 'OVV RADIO LOUD/STEEP SPECT STEEP RADIO SP',
    7763 => 'OVV RADIO LOUD/STEEP SPECT INVERTED RADIO SP',
    7764 => 'OVV RADIO LOUD/STEEP SPECT TYPE 1',
    7765 => 'OVV RADIO LOUD/STEEP SPECT TYPE 1.5',
    7766 => 'OVV RADIO LOUD/STEEP SPECT TYPE 2',
    7767 => 'OVV RADIO LOUD/STEEP SPECT',
    7768 => 'OVV RADIO LOUD/STEEP SPECT',
    7769 => 'OVV RADIO LOUD/STEEP SPECT',
    7770 => 'OVV RADIO POLAR/INVERT SPECT',
    7771 => 'OVV RADIO POLAR/INVERT SPECT FLAT RADIO SP',
    7772 => 'OVV RADIO POLAR/INVERT SPECT STEEP RADIO SP',
    7773 => 'OVV RADIO POLAR/INVERT SPECT INVERTED RADIO SP',
    7774 => 'OVV RADIO POLAR/INVERT SPECT TYPE 1',
    7775 => 'OVV RADIO POLAR/INVERT SPECT TYPE 1.5',
    7776 => 'OVV RADIO POLAR/INVERT SPECT TYPE 2',
    7777 => 'OVV RADIO POLAR/INVERT SPECT',
    7778 => 'OVV RADIO POLAR/INVERT SPECT',
    7779 => 'OVV RADIO POLAR/INVERT SPECT',
    7780 => 'OVV RADIO POLAR/FLAT SPECT',
    7781 => 'OVV RADIO POLAR/FLAT SPECT FLAT RADIO SP',
    7782 => 'OVV RADIO POLAR/FLAT SPECT STEEP RADIO SP',
    7783 => 'OVV RADIO POLAR/FLAT SPECT INVERTED RADIO SP',
    7784 => 'OVV RADIO POLAR/FLAT SPECT TYPE 1',
    7785 => 'OVV RADIO POLAR/FLAT SPECT TYPE 1.5',
    7786 => 'OVV RADIO POLAR/FLAT SPECT TYPE 2',
    7787 => 'OVV RADIO POLAR/FLAT SPECT',
    7788 => 'OVV RADIO POLAR/FLAT SPECT',
    7789 => 'OVV RADIO POLAR/FLAT SPECT',
    7790 => 'OVV RADIO POLAR/STEEP SPECT',
    7791 => 'OVV RADIO POLAR/STEEP SPECT FLAT RADIO SP',
    7792 => 'OVV RADIO POLAR/STEEP SPECT STEEP RADIO SP',
    7793 => 'OVV RADIO POLAR/STEEP SPECT INVERTED RADIO SP',
    7794 => 'OVV RADIO POLAR/STEEP SPECT TYPE 1',
    7795 => 'OVV RADIO POLAR/STEEP SPECT TYPE 1.5',
    7796 => 'OVV RADIO POLAR/STEEP SPECT TYPE 2',
    7797 => 'OVV RADIO POLAR/STEEP SPECT',
    7798 => 'OVV RADIO POLAR/STEEP SPECT',
    7799 => 'OVV RADIO POLAR/STEEP SPECT',
    7800 => 'NEL GALAXY',
    7801 => 'NEL GALAXY FLAT RADIO SP',
    7802 => 'NEL GALAXY STEEP RADIO SP',
    7803 => 'NEL GALAXY INVERTED RADIO SP',
    7804 => 'NEL GALAXY TYPE 1',
    7805 => 'NEL GALAXY TYPE 1.5',
    7806 => 'NEL GALAXY TYPE 2',
    7807 => 'NEL GALAXY',
    7808 => 'NEL GALAXY',
    7809 => 'NEL GALAXY',
    7810 => 'NEL GALAXY RADIO LOUD',
    7811 => 'NEL GALAXY RADIO LOUD FLAT RADIO SP',
    7812 => 'NEL GALAXY RADIO LOUD STEEP RADIO SP',
    7813 => 'NEL GALAXY RADIO LOUD INVERTED RADIO SP',
    7814 => 'NEL GALAXY RADIO LOUD TYPE 1',
    7815 => 'NEL GALAXY RADIO LOUD TYPE 1.5',
    7816 => 'NEL GALAXY RADIO LOUD TYPE 2',
    7817 => 'NEL GALAXY RADIO LOUD',
    7818 => 'NEL GALAXY RADIO LOUD',
    7819 => 'NEL GALAXY RADIO LOUD',
    7820 => 'NEL GALAXY RADIO LOUD/POLARISED',
    7821 => 'NEL GALAXY RADIO LOUD/POLARISED FLAT RADIO SP',
    7822 => 'NEL GALAXY RADIO LOUD/POLARISED STEEP RADIO SP',
    7823 => 'NEL GALAXY RADIO LOUD/POLARISED INVERTED RADIO SP',
    7824 => 'NEL GALAXY RADIO LOUD/POLARISED TYPE 1',
    7825 => 'NEL GALAXY RADIO LOUD/POLARISED TYPE 1.5',
    7826 => 'NEL GALAXY RADIO LOUD/POLARISED TYPE 2',
    7827 => 'NEL GALAXY RADIO LOUD/POLARISED',
    7828 => 'NEL GALAXY RADIO LOUD/POLARISED',
    7829 => 'NEL GALAXY RADIO LOUD/POLARISED',
    7830 => 'NEL GALAXY RADIO QUIET',
    7831 => 'NEL GALAXY RADIO QUIET FLAT RADIO SP',
    7832 => 'NEL GALAXY RADIO QUIET STEEP RADIO SP',
    7833 => 'NEL GALAXY RADIO QUIET INVERTED RADIO SP',
    7834 => 'NEL GALAXY RADIO QUIET TYPE 1',
    7835 => 'NEL GALAXY RADIO QUIET TYPE 1.5',
    7836 => 'NEL GALAXY RADIO QUIET TYPE 2',
    7837 => 'NEL GALAXY RADIO QUIET',
    7838 => 'NEL GALAXY RADIO QUIET',
    7839 => 'NEL GALAXY RADIO QUIET',
    7840 => 'NEL GALAXY RADIO LOUD/INVERT SPECT',
    7841 => 'NEL GALAXY RADIO LOUD/INVERT SPECT FLAT RADIO SP',
    7842 => 'NEL GALAXY RADIO LOUD/INVERT SPECT STEEP RADIO SP',
    7843 => 'NEL GALAXY RADIO LOUD/INVERT SPECT INVERTED RADIO SP',
    7844 => 'NEL GALAXY RADIO LOUD/INVERT SPECT TYPE 1',
    7845 => 'NEL GALAXY RADIO LOUD/INVERT SPECT TYPE 1.5',
    7846 => 'NEL GALAXY RADIO LOUD/INVERT SPECT TYPE 2',
    7847 => 'NEL GALAXY RADIO LOUD/INVERT SPECT',
    7848 => 'NEL GALAXY RADIO LOUD/INVERT SPECT',
    7849 => 'NEL GALAXY RADIO LOUD/INVERT SPECT',
    7850 => 'NEL GALAXY RADIO LOUD/FLAT SPECT',
    7851 => 'NEL GALAXY RADIO LOUD/FLAT SPECT FLAT RADIO SP',
    7852 => 'NEL GALAXY RADIO LOUD/FLAT SPECT STEEP RADIO SP',
    7853 => 'NEL GALAXY RADIO LOUD/FLAT SPECT INVERTED RADIO SP',
    7854 => 'NEL GALAXY RADIO LOUD/FLAT SPECT TYPE 1',
    7855 => 'NEL GALAXY RADIO LOUD/FLAT SPECT TYPE 1.5',
    7856 => 'NEL GALAXY RADIO LOUD/FLAT SPECT TYPE 2',
    7857 => 'NEL GALAXY RADIO LOUD/FLAT SPECT',
    7858 => 'NEL GALAXY RADIO LOUD/FLAT SPECT',
    7859 => 'NEL GALAXY RADIO LOUD/FLAT SPECT',
    7860 => 'NEL GALAXY RADIO LOUD/STEEP SPECT',
    7861 => 'NEL GALAXY RADIO LOUD/STEEP SPECT FLAT RADIO SP',
    7862 => 'NEL GALAXY RADIO LOUD/STEEP SPECT STEEP RADIO SP',
    7863 => 'NEL GALAXY RADIO LOUD/STEEP SPECT INVERTED RADIO SP',
    7864 => 'NEL GALAXY RADIO LOUD/STEEP SPECT TYPE 1',
    7865 => 'NEL GALAXY RADIO LOUD/STEEP SPECT TYPE 1.5',
    7866 => 'NEL GALAXY RADIO LOUD/STEEP SPECT TYPE 2',
    7867 => 'NEL GALAXY RADIO LOUD/STEEP SPECT',
    7868 => 'NEL GALAXY RADIO LOUD/STEEP SPECT',
    7869 => 'NEL GALAXY RADIO LOUD/STEEP SPECT',
    7870 => 'NEL GALAXY RADIO POLAR/INVERT SPECT',
    7871 => 'NEL GALAXY RADIO POLAR/INVERT SPECT FLAT RADIO SP',
    7872 => 'NEL GALAXY RADIO POLAR/INVERT SPECT STEEP RADIO SP',
    7873 => 'NEL GALAXY RADIO POLAR/INVERT SPECT INVERTED RADIO SP',
    7874 => 'NEL GALAXY RADIO POLAR/INVERT SPECT TYPE 1',
    7875 => 'NEL GALAXY RADIO POLAR/INVERT SPECT TYPE 1.5',
    7876 => 'NEL GALAXY RADIO POLAR/INVERT SPECT TYPE 2',
    7877 => 'NEL GALAXY RADIO POLAR/INVERT SPECT',
    7878 => 'NEL GALAXY RADIO POLAR/INVERT SPECT',
    7879 => 'NEL GALAXY RADIO POLAR/INVERT SPECT',
    7880 => 'NEL GALAXY RADIO POLAR/FLAT SPECT',
    7881 => 'NEL GALAXY RADIO POLAR/FLAT SPECT FLAT RADIO SP',
    7882 => 'NEL GALAXY RADIO POLAR/FLAT SPECT STEEP RADIO SP',
    7883 => 'NEL GALAXY RADIO POLAR/FLAT SPECT INVERTED RADIO SP',
    7884 => 'NEL GALAXY RADIO POLAR/FLAT SPECT TYPE 1',
    7885 => 'NEL GALAXY RADIO POLAR/FLAT SPECT TYPE 1.5',
    7886 => 'NEL GALAXY RADIO POLAR/FLAT SPECT TYPE 2',
    7887 => 'NEL GALAXY RADIO POLAR/FLAT SPECT',
    7888 => 'NEL GALAXY RADIO POLAR/FLAT SPECT',
    7889 => 'NEL GALAXY RADIO POLAR/FLAT SPECT',
    7890 => 'NEL GALAXY RADIO POLAR/STEEP SPECT',
    7891 => 'NEL GALAXY RADIO POLAR/STEEP SPECT FLAT RADIO SP',
    7892 => 'NEL GALAXY RADIO POLAR/STEEP SPECT STEEP RADIO SP',
    7893 => 'NEL GALAXY RADIO POLAR/STEEP SPECT INVERTED RADIO SP',
    7894 => 'NEL GALAXY RADIO POLAR/STEEP SPECT TYPE 1',
    7895 => 'NEL GALAXY RADIO POLAR/STEEP SPECT TYPE 1.5',
    7896 => 'NEL GALAXY RADIO POLAR/STEEP SPECT TYPE 2',
    7897 => 'NEL GALAXY RADIO POLAR/STEEP SPECT',
    7898 => 'NEL GALAXY RADIO POLAR/STEEP SPECT',
    7899 => 'NEL GALAXY RADIO POLAR/STEEP SPECT',
    7900 => 'NL SEYFERT 1',
    7901 => 'NL SEYFERT 1 FLAT RADIO SP',
    7902 => 'NL SEYFERT 1 STEEP RADIO SP',
    7903 => 'NL SEYFERT 1 INVERTED RADIO SP',
    7904 => 'NL SEYFERT 1 TYPE 1',
    7905 => 'NL SEYFERT 1 TYPE 1.5',
    7906 => 'NL SEYFERT 1 TYPE 2',
    7907 => 'NL SEYFERT 1',
    7908 => 'NL SEYFERT 1',
    7909 => 'NL SEYFERT 1',
    7910 => 'NL SEYFERT 1 RADIO LOUD',
    7911 => 'NL SEYFERT 1 RADIO LOUD FLAT RADIO SP',
    7912 => 'NL SEYFERT 1 RADIO LOUD STEEP RADIO SP',
    7913 => 'NL SEYFERT 1 RADIO LOUD INVERTED RADIO SP',
    7914 => 'NL SEYFERT 1 RADIO LOUD TYPE 1',
    7915 => 'NL SEYFERT 1 RADIO LOUD TYPE 1.5',
    7916 => 'NL SEYFERT 1 RADIO LOUD TYPE 2',
    7917 => 'NL SEYFERT 1 RADIO LOUD',
    7918 => 'NL SEYFERT 1 RADIO LOUD',
    7919 => 'NL SEYFERT 1 RADIO LOUD',
    7920 => 'NL SEYFERT 1 RADIO LOUD/POLARISED',
    7921 => 'NL SEYFERT 1 RADIO LOUD/POLARISED FLAT RADIO SP',
    7922 => 'NL SEYFERT 1 RADIO LOUD/POLARISED STEEP RADIO SP',
    7923 => 'NL SEYFERT 1 RADIO LOUD/POLARISED INVERTED RADIO SP',
    7924 => 'NL SEYFERT 1 RADIO LOUD/POLARISED TYPE 1',
    7925 => 'NL SEYFERT 1 RADIO LOUD/POLARISED TYPE 1.5',
    7926 => 'NL SEYFERT 1 RADIO LOUD/POLARISED TYPE 2',
    7927 => 'NL SEYFERT 1 RADIO LOUD/POLARISED',
    7928 => 'NL SEYFERT 1 RADIO LOUD/POLARISED',
    7929 => 'NL SEYFERT 1 RADIO LOUD/POLARISED',
    7930 => 'NL SEYFERT 1 RADIO QUIET',
    7931 => 'NL SEYFERT 1 RADIO QUIET FLAT RADIO SP',
    7932 => 'NL SEYFERT 1 RADIO QUIET STEEP RADIO SP',
    7933 => 'NL SEYFERT 1 RADIO QUIET INVERTED RADIO SP',
    7934 => 'NL SEYFERT 1 RADIO QUIET TYPE 1',
    7935 => 'NL SEYFERT 1 RADIO QUIET TYPE 1.5',
    7936 => 'NL SEYFERT 1 RADIO QUIET TYPE 2',
    7937 => 'NL SEYFERT 1 RADIO QUIET',
    7938 => 'NL SEYFERT 1 RADIO QUIET',
    7939 => 'NL SEYFERT 1 RADIO QUIET',
    7940 => 'NL SEYFERT 1 RADIO LOUD/INVERT SPECT',
    7941 => 'NL SEYFERT 1 RADIO LOUD/INVERT SPECT FLAT RADIO SP',
    7942 => 'NL SEYFERT 1 RADIO LOUD/INVERT SPECT STEEP RADIO SP',
    7943 => 'NL SEYFERT 1 RADIO LOUD/INVERT SPECT INVERTED RADIO SP',
    7944 => 'NL SEYFERT 1 RADIO LOUD/INVERT SPECT TYPE 1',
    7945 => 'NL SEYFERT 1 RADIO LOUD/INVERT SPECT TYPE 1.5',
    7946 => 'NL SEYFERT 1 RADIO LOUD/INVERT SPECT TYPE 2',
    7947 => 'NL SEYFERT 1 RADIO LOUD/INVERT SPECT',
    7948 => 'NL SEYFERT 1 RADIO LOUD/INVERT SPECT',
    7949 => 'NL SEYFERT 1 RADIO LOUD/INVERT SPECT',
    7950 => 'NL SEYFERT 1 RADIO LOUD/FLAT SPECT',
    7951 => 'NL SEYFERT 1 RADIO LOUD/FLAT SPECT FLAT RADIO SP',
    7952 => 'NL SEYFERT 1 RADIO LOUD/FLAT SPECT STEEP RADIO SP',
    7953 => 'NL SEYFERT 1 RADIO LOUD/FLAT SPECT INVERTED RADIO SP',
    7954 => 'NL SEYFERT 1 RADIO LOUD/FLAT SPECT TYPE 1',
    7955 => 'NL SEYFERT 1 RADIO LOUD/FLAT SPECT TYPE 1.5',
    7956 => 'NL SEYFERT 1 RADIO LOUD/FLAT SPECT TYPE 2',
    7957 => 'NL SEYFERT 1 RADIO LOUD/FLAT SPECT',
    7958 => 'NL SEYFERT 1 RADIO LOUD/FLAT SPECT',
    7959 => 'NL SEYFERT 1 RADIO LOUD/FLAT SPECT',
    7960 => 'NL SEYFERT 1 RADIO LOUD/STEEP SPECT',
    7961 => 'NL SEYFERT 1 RADIO LOUD/STEEP SPECT FLAT RADIO SP',
    7962 => 'NL SEYFERT 1 RADIO LOUD/STEEP SPECT STEEP RADIO SP',
    7963 => 'NL SEYFERT 1 RADIO LOUD/STEEP SPECT INVERTED RADIO SP',
    7964 => 'NL SEYFERT 1 RADIO LOUD/STEEP SPECT TYPE 1',
    7965 => 'NL SEYFERT 1 RADIO LOUD/STEEP SPECT TYPE 1.5',
    7966 => 'NL SEYFERT 1 RADIO LOUD/STEEP SPECT TYPE 2',
    7967 => 'NL SEYFERT 1 RADIO LOUD/STEEP SPECT',
    7968 => 'NL SEYFERT 1 RADIO LOUD/STEEP SPECT',
    7969 => 'NL SEYFERT 1 RADIO LOUD/STEEP SPECT',
    7970 => 'NL SEYFERT 1 RADIO POLAR/INVERT SPECT',
    7971 => 'NL SEYFERT 1 RADIO POLAR/INVERT SPECT FLAT RADIO SP',
    7972 => 'NL SEYFERT 1 RADIO POLAR/INVERT SPECT STEEP RADIO SP',
    7973 => 'NL SEYFERT 1 RADIO POLAR/INVERT SPECT INVERTED RADIO SP',
    7974 => 'NL SEYFERT 1 RADIO POLAR/INVERT SPECT TYPE 1',
    7975 => 'NL SEYFERT 1 RADIO POLAR/INVERT SPECT TYPE 1.5',
    7976 => 'NL SEYFERT 1 RADIO POLAR/INVERT SPECT TYPE 2',
    7977 => 'NL SEYFERT 1 RADIO POLAR/INVERT SPECT',
    7978 => 'NL SEYFERT 1 RADIO POLAR/INVERT SPECT',
    7979 => 'NL SEYFERT 1 RADIO POLAR/INVERT SPECT',
    7980 => 'NL SEYFERT 1 RADIO POLAR/FLAT SPECT',
    7981 => 'NL SEYFERT 1 RADIO POLAR/FLAT SPECT FLAT RADIO SP',
    7982 => 'NL SEYFERT 1 RADIO POLAR/FLAT SPECT STEEP RADIO SP',
    7983 => 'NL SEYFERT 1 RADIO POLAR/FLAT SPECT INVERTED RADIO SP',
    7984 => 'NL SEYFERT 1 RADIO POLAR/FLAT SPECT TYPE 1',
    7985 => 'NL SEYFERT 1 RADIO POLAR/FLAT SPECT TYPE 1.5',
    7986 => 'NL SEYFERT 1 RADIO POLAR/FLAT SPECT TYPE 2',
    7987 => 'NL SEYFERT 1 RADIO POLAR/FLAT SPECT',
    7988 => 'NL SEYFERT 1 RADIO POLAR/FLAT SPECT',
    7989 => 'NL SEYFERT 1 RADIO POLAR/FLAT SPECT',
    7990 => 'NL SEYFERT 1 RADIO POLAR/STEEP SPECT',
    7991 => 'NL SEYFERT 1 RADIO POLAR/STEEP SPECT FLAT RADIO SP',
    7992 => 'NL SEYFERT 1 RADIO POLAR/STEEP SPECT STEEP RADIO SP',
    7993 => 'NL SEYFERT 1 RADIO POLAR/STEEP SPECT INVERTED RADIO SP',
    7994 => 'NL SEYFERT 1 RADIO POLAR/STEEP SPECT TYPE 1',
    7995 => 'NL SEYFERT 1 RADIO POLAR/STEEP SPECT TYPE 1.5',
    7996 => 'NL SEYFERT 1 RADIO POLAR/STEEP SPECT TYPE 2',
    7997 => 'NL SEYFERT 1 RADIO POLAR/STEEP SPECT',
    7998 => 'NL SEYFERT 1 RADIO POLAR/STEEP SPECT',
    7999 => 'NL SEYFERT 1 RADIO POLAR/STEEP SPECT',
    8000 => 'SOLAR SYSTEM OBJECT',
    8001 => 'SOLAR SYSTEM OBJECT',
    8002 => 'SOLAR SYSTEM OBJECT',
    8003 => 'SOLAR SYSTEM OBJECT',
    8004 => 'SOLAR SYSTEM OBJECT',
    8005 => 'SOLAR SYSTEM OBJECT',
    8006 => 'SOLAR SYSTEM OBJECT',
    8007 => 'SOLAR SYSTEM OBJECT',
    8008 => 'SOLAR SYSTEM OBJECT',
    8009 => 'SOLAR SYSTEM OBJECT',
    8010 => 'SOLAR SYSTEM OBJECT',
    8011 => 'SOLAR SYSTEM OBJECT',
    8012 => 'SOLAR SYSTEM OBJECT',
    8013 => 'SOLAR SYSTEM OBJECT',
    8014 => 'SOLAR SYSTEM OBJECT',
    8015 => 'SOLAR SYSTEM OBJECT',
    8016 => 'SOLAR SYSTEM OBJECT',
    8017 => 'SOLAR SYSTEM OBJECT',
    8018 => 'SOLAR SYSTEM OBJECT',
    8019 => 'SOLAR SYSTEM OBJECT',
    8020 => 'SOLAR SYSTEM OBJECT',
    8021 => 'SOLAR SYSTEM OBJECT',
    8022 => 'SOLAR SYSTEM OBJECT',
    8023 => 'SOLAR SYSTEM OBJECT',
    8024 => 'SOLAR SYSTEM OBJECT',
    8025 => 'SOLAR SYSTEM OBJECT',
    8026 => 'SOLAR SYSTEM OBJECT',
    8027 => 'SOLAR SYSTEM OBJECT',
    8028 => 'SOLAR SYSTEM OBJECT',
    8029 => 'SOLAR SYSTEM OBJECT',
    8030 => 'SOLAR SYSTEM OBJECT',
    8031 => 'SOLAR SYSTEM OBJECT',
    8032 => 'SOLAR SYSTEM OBJECT',
    8033 => 'SOLAR SYSTEM OBJECT',
    8034 => 'SOLAR SYSTEM OBJECT',
    8035 => 'SOLAR SYSTEM OBJECT',
    8036 => 'SOLAR SYSTEM OBJECT',
    8037 => 'SOLAR SYSTEM OBJECT',
    8038 => 'SOLAR SYSTEM OBJECT',
    8039 => 'SOLAR SYSTEM OBJECT',
    8040 => 'SOLAR SYSTEM OBJECT',
    8041 => 'SOLAR SYSTEM OBJECT',
    8042 => 'SOLAR SYSTEM OBJECT',
    8043 => 'SOLAR SYSTEM OBJECT',
    8044 => 'SOLAR SYSTEM OBJECT',
    8045 => 'SOLAR SYSTEM OBJECT',
    8046 => 'SOLAR SYSTEM OBJECT',
    8047 => 'SOLAR SYSTEM OBJECT',
    8048 => 'SOLAR SYSTEM OBJECT',
    8049 => 'SOLAR SYSTEM OBJECT',
    8050 => 'SOLAR SYSTEM OBJECT',
    8051 => 'SOLAR SYSTEM OBJECT',
    8052 => 'SOLAR SYSTEM OBJECT',
    8053 => 'SOLAR SYSTEM OBJECT',
    8054 => 'SOLAR SYSTEM OBJECT',
    8055 => 'SOLAR SYSTEM OBJECT',
    8056 => 'SOLAR SYSTEM OBJECT',
    8057 => 'SOLAR SYSTEM OBJECT',
    8058 => 'SOLAR SYSTEM OBJECT',
    8059 => 'SOLAR SYSTEM OBJECT',
    8060 => 'SOLAR SYSTEM OBJECT',
    8061 => 'SOLAR SYSTEM OBJECT',
    8062 => 'SOLAR SYSTEM OBJECT',
    8063 => 'SOLAR SYSTEM OBJECT',
    8064 => 'SOLAR SYSTEM OBJECT',
    8065 => 'SOLAR SYSTEM OBJECT',
    8066 => 'SOLAR SYSTEM OBJECT',
    8067 => 'SOLAR SYSTEM OBJECT',
    8068 => 'SOLAR SYSTEM OBJECT',
    8069 => 'SOLAR SYSTEM OBJECT',
    8070 => 'SOLAR SYSTEM OBJECT',
    8071 => 'SOLAR SYSTEM OBJECT',
    8072 => 'SOLAR SYSTEM OBJECT',
    8073 => 'SOLAR SYSTEM OBJECT',
    8074 => 'SOLAR SYSTEM OBJECT',
    8075 => 'SOLAR SYSTEM OBJECT',
    8076 => 'SOLAR SYSTEM OBJECT',
    8077 => 'SOLAR SYSTEM OBJECT',
    8078 => 'SOLAR SYSTEM OBJECT',
    8079 => 'SOLAR SYSTEM OBJECT',
    8080 => 'SOLAR SYSTEM OBJECT',
    8081 => 'SOLAR SYSTEM OBJECT',
    8082 => 'SOLAR SYSTEM OBJECT',
    8083 => 'SOLAR SYSTEM OBJECT',
    8084 => 'SOLAR SYSTEM OBJECT',
    8085 => 'SOLAR SYSTEM OBJECT',
    8086 => 'SOLAR SYSTEM OBJECT',
    8087 => 'SOLAR SYSTEM OBJECT',
    8088 => 'SOLAR SYSTEM OBJECT',
    8089 => 'SOLAR SYSTEM OBJECT',
    8090 => 'SOLAR SYSTEM OBJECT',
    8091 => 'SOLAR SYSTEM OBJECT',
    8092 => 'SOLAR SYSTEM OBJECT',
    8093 => 'SOLAR SYSTEM OBJECT',
    8094 => 'SOLAR SYSTEM OBJECT',
    8095 => 'SOLAR SYSTEM OBJECT',
    8096 => 'SOLAR SYSTEM OBJECT',
    8097 => 'SOLAR SYSTEM OBJECT',
    8098 => 'SOLAR SYSTEM OBJECT',
    8099 => 'SOLAR SYSTEM OBJECT',
    8100 => 'PLANET',
    8101 => 'PLANET',
    8102 => 'PLANET',
    8103 => 'PLANET',
    8104 => 'PLANET',
    8105 => 'PLANET',
    8106 => 'PLANET',
    8107 => 'PLANET',
    8108 => 'PLANET',
    8109 => 'PLANET',
    8110 => 'PLANET MERCURY',
    8111 => 'PLANET MERCURY',
    8112 => 'PLANET MERCURY',
    8113 => 'PLANET MERCURY',
    8114 => 'PLANET MERCURY',
    8115 => 'PLANET MERCURY',
    8116 => 'PLANET MERCURY',
    8117 => 'PLANET MERCURY',
    8118 => 'PLANET MERCURY',
    8119 => 'PLANET MERCURY',
    8120 => 'PLANET VENUS',
    8121 => 'PLANET VENUS',
    8122 => 'PLANET VENUS',
    8123 => 'PLANET VENUS',
    8124 => 'PLANET VENUS',
    8125 => 'PLANET VENUS',
    8126 => 'PLANET VENUS',
    8127 => 'PLANET VENUS',
    8128 => 'PLANET VENUS',
    8129 => 'PLANET VENUS',
    8130 => 'PLANET EARTH',
    8131 => 'PLANET EARTH',
    8132 => 'PLANET EARTH',
    8133 => 'PLANET EARTH',
    8134 => 'PLANET EARTH',
    8135 => 'PLANET EARTH',
    8136 => 'PLANET EARTH',
    8137 => 'PLANET EARTH',
    8138 => 'PLANET EARTH',
    8139 => 'PLANET EARTH',
    8140 => 'PLANET MARS',
    8141 => 'PLANET MARS',
    8142 => 'PLANET MARS',
    8143 => 'PLANET MARS',
    8144 => 'PLANET MARS',
    8145 => 'PLANET MARS',
    8146 => 'PLANET MARS',
    8147 => 'PLANET MARS',
    8148 => 'PLANET MARS',
    8149 => 'PLANET MARS',
    8150 => 'PLANET JUPITER',
    8151 => 'PLANET JUPITER',
    8152 => 'PLANET JUPITER',
    8153 => 'PLANET JUPITER',
    8154 => 'PLANET JUPITER',
    8155 => 'PLANET JUPITER',
    8156 => 'PLANET JUPITER',
    8157 => 'PLANET JUPITER',
    8158 => 'PLANET JUPITER',
    8159 => 'PLANET JUPITER',
    8160 => 'PLANET SATURN',
    8161 => 'PLANET SATURN',
    8162 => 'PLANET SATURN',
    8163 => 'PLANET SATURN',
    8164 => 'PLANET SATURN',
    8165 => 'PLANET SATURN',
    8166 => 'PLANET SATURN',
    8167 => 'PLANET SATURN',
    8168 => 'PLANET SATURN',
    8169 => 'PLANET SATURN',
    8170 => 'PLANET URANUS',
    8171 => 'PLANET URANUS',
    8172 => 'PLANET URANUS',
    8173 => 'PLANET URANUS',
    8174 => 'PLANET URANUS',
    8175 => 'PLANET URANUS',
    8176 => 'PLANET URANUS',
    8177 => 'PLANET URANUS',
    8178 => 'PLANET URANUS',
    8179 => 'PLANET URANUS',
    8180 => 'PLANET NEPTUNE',
    8181 => 'PLANET NEPTUNE',
    8182 => 'PLANET NEPTUNE',
    8183 => 'PLANET NEPTUNE',
    8184 => 'PLANET NEPTUNE',
    8185 => 'PLANET NEPTUNE',
    8186 => 'PLANET NEPTUNE',
    8187 => 'PLANET NEPTUNE',
    8188 => 'PLANET NEPTUNE',
    8189 => 'PLANET NEPTUNE',
    8190 => 'PLANET PLUTO',
    8191 => 'PLANET PLUTO',
    8192 => 'PLANET PLUTO',
    8193 => 'PLANET PLUTO',
    8194 => 'PLANET PLUTO',
    8195 => 'PLANET PLUTO',
    8196 => 'PLANET PLUTO',
    8197 => 'PLANET PLUTO',
    8198 => 'PLANET PLUTO',
    8199 => 'PLANET PLUTO',
    8200 => 'SOLAR FEATURE',
    8201 => 'SOLAR FEATURE',
    8202 => 'SOLAR FEATURE',
    8203 => 'SOLAR FEATURE',
    8204 => 'SOLAR FEATURE',
    8205 => 'SOLAR FEATURE',
    8206 => 'SOLAR FEATURE',
    8207 => 'SOLAR FEATURE',
    8208 => 'SOLAR FEATURE',
    8209 => 'SOLAR FEATURE',
    8210 => 'SOLAR FEATURE: QUIET SUN',
    8211 => 'SOLAR FEATURE: QUIET SUN',
    8212 => 'SOLAR FEATURE: QUIET SUN',
    8213 => 'SOLAR FEATURE: QUIET SUN',
    8214 => 'SOLAR FEATURE: QUIET SUN',
    8215 => 'SOLAR FEATURE: QUIET SUN',
    8216 => 'SOLAR FEATURE: QUIET SUN',
    8217 => 'SOLAR FEATURE: QUIET SUN',
    8218 => 'SOLAR FEATURE: QUIET SUN',
    8219 => 'SOLAR FEATURE: QUIET SUN',
    8220 => 'SOLAR FEATURE: ACTIVE SUN',
    8221 => 'SOLAR FEATURE: ACTIVE SUN',
    8222 => 'SOLAR FEATURE: ACTIVE SUN',
    8223 => 'SOLAR FEATURE: ACTIVE SUN',
    8224 => 'SOLAR FEATURE: ACTIVE SUN',
    8225 => 'SOLAR FEATURE: ACTIVE SUN',
    8226 => 'SOLAR FEATURE: ACTIVE SUN',
    8227 => 'SOLAR FEATURE: ACTIVE SUN',
    8228 => 'SOLAR FEATURE: ACTIVE SUN',
    8229 => 'SOLAR FEATURE: ACTIVE SUN',
    8230 => 'SOLAR FEATURE: SUNSPOT',
    8231 => 'SOLAR FEATURE: SUNSPOT',
    8232 => 'SOLAR FEATURE: SUNSPOT',
    8233 => 'SOLAR FEATURE: SUNSPOT',
    8234 => 'SOLAR FEATURE: SUNSPOT',
    8235 => 'SOLAR FEATURE: SUNSPOT',
    8236 => 'SOLAR FEATURE: SUNSPOT',
    8237 => 'SOLAR FEATURE: SUNSPOT',
    8238 => 'SOLAR FEATURE: SUNSPOT',
    8239 => 'SOLAR FEATURE: SUNSPOT',
    8240 => 'SOLAR FEATURE: PLAGE/ACTIVE REGION',
    8241 => 'SOLAR FEATURE: PLAGE/ACTIVE REGION',
    8242 => 'SOLAR FEATURE: PLAGE/ACTIVE REGION',
    8243 => 'SOLAR FEATURE: PLAGE/ACTIVE REGION',
    8244 => 'SOLAR FEATURE: PLAGE/ACTIVE REGION',
    8245 => 'SOLAR FEATURE: PLAGE/ACTIVE REGION',
    8246 => 'SOLAR FEATURE: PLAGE/ACTIVE REGION',
    8247 => 'SOLAR FEATURE: PLAGE/ACTIVE REGION',
    8248 => 'SOLAR FEATURE: PLAGE/ACTIVE REGION',
    8249 => 'SOLAR FEATURE: PLAGE/ACTIVE REGION',
    8250 => 'SOLAR FEATURE: NORTH POLE',
    8251 => 'SOLAR FEATURE: NORTH POLE',
    8252 => 'SOLAR FEATURE: NORTH POLE',
    8253 => 'SOLAR FEATURE: NORTH POLE',
    8254 => 'SOLAR FEATURE: NORTH POLE',
    8255 => 'SOLAR FEATURE: NORTH POLE',
    8256 => 'SOLAR FEATURE: NORTH POLE',
    8257 => 'SOLAR FEATURE: NORTH POLE',
    8258 => 'SOLAR FEATURE: NORTH POLE',
    8259 => 'SOLAR FEATURE: NORTH POLE',
    8260 => 'SOLAR FEATURE: SOUTH POLE',
    8261 => 'SOLAR FEATURE: SOUTH POLE',
    8262 => 'SOLAR FEATURE: SOUTH POLE',
    8263 => 'SOLAR FEATURE: SOUTH POLE',
    8264 => 'SOLAR FEATURE: SOUTH POLE',
    8265 => 'SOLAR FEATURE: SOUTH POLE',
    8266 => 'SOLAR FEATURE: SOUTH POLE',
    8267 => 'SOLAR FEATURE: SOUTH POLE',
    8268 => 'SOLAR FEATURE: SOUTH POLE',
    8269 => 'SOLAR FEATURE: SOUTH POLE',
    8270 => 'SOLAR FEATURE: EQUATORIAL REGION',
    8271 => 'SOLAR FEATURE: EQUATORIAL REGION',
    8272 => 'SOLAR FEATURE: EQUATORIAL REGION',
    8273 => 'SOLAR FEATURE: EQUATORIAL REGION',
    8274 => 'SOLAR FEATURE: EQUATORIAL REGION',
    8275 => 'SOLAR FEATURE: EQUATORIAL REGION',
    8276 => 'SOLAR FEATURE: EQUATORIAL REGION',
    8277 => 'SOLAR FEATURE: EQUATORIAL REGION',
    8278 => 'SOLAR FEATURE: EQUATORIAL REGION',
    8279 => 'SOLAR FEATURE: EQUATORIAL REGION',
    8280 => 'SOLAR FEATURE: MID-LATITUDE REGION',
    8281 => 'SOLAR FEATURE: MID-LATITUDE REGION',
    8282 => 'SOLAR FEATURE: MID-LATITUDE REGION',
    8283 => 'SOLAR FEATURE: MID-LATITUDE REGION',
    8284 => 'SOLAR FEATURE: MID-LATITUDE REGION',
    8285 => 'SOLAR FEATURE: MID-LATITUDE REGION',
    8286 => 'SOLAR FEATURE: MID-LATITUDE REGION',
    8287 => 'SOLAR FEATURE: MID-LATITUDE REGION',
    8288 => 'SOLAR FEATURE: MID-LATITUDE REGION',
    8289 => 'SOLAR FEATURE: MID-LATITUDE REGION',
    8290 => 'SOLAR FEATURE: FLARE',
    8291 => 'SOLAR FEATURE: FLARE',
    8292 => 'SOLAR FEATURE: FLARE',
    8293 => 'SOLAR FEATURE: FLARE',
    8294 => 'SOLAR FEATURE: FLARE',
    8295 => 'SOLAR FEATURE: FLARE',
    8296 => 'SOLAR FEATURE: FLARE',
    8297 => 'SOLAR FEATURE: FLARE',
    8298 => 'SOLAR FEATURE: FLARE',
    8299 => 'SOLAR FEATURE: FLARE',
    8300 => 'ASTEROID',
    8301 => 'ASTEROID',
    8302 => 'ASTEROID',
    8303 => 'ASTEROID',
    8304 => 'ASTEROID',
    8305 => 'ASTEROID',
    8306 => 'ASTEROID',
    8307 => 'ASTEROID',
    8308 => 'ASTEROID',
    8309 => 'ASTEROID',
    8310 => 'ASTEROID MAIN BELT OBJECT',
    8311 => 'ASTEROID MAIN BELT OBJECT',
    8312 => 'ASTEROID MAIN BELT OBJECT',
    8313 => 'ASTEROID MAIN BELT OBJECT',
    8314 => 'ASTEROID MAIN BELT OBJECT',
    8315 => 'ASTEROID MAIN BELT OBJECT',
    8316 => 'ASTEROID MAIN BELT OBJECT',
    8317 => 'ASTEROID MAIN BELT OBJECT',
    8318 => 'ASTEROID MAIN BELT OBJECT',
    8319 => 'ASTEROID MAIN BELT OBJECT',
    8320 => 'ASTEROID CENTAUR',
    8321 => 'ASTEROID CENTAUR',
    8322 => 'ASTEROID CENTAUR',
    8323 => 'ASTEROID CENTAUR',
    8324 => 'ASTEROID CENTAUR',
    8325 => 'ASTEROID CENTAUR',
    8326 => 'ASTEROID CENTAUR',
    8327 => 'ASTEROID CENTAUR',
    8328 => 'ASTEROID CENTAUR',
    8329 => 'ASTEROID CENTAUR',
    8330 => 'ASTEROID KUIPER BELT OBJECT',
    8331 => 'ASTEROID KUIPER BELT OBJECT',
    8332 => 'ASTEROID KUIPER BELT OBJECT',
    8333 => 'ASTEROID KUIPER BELT OBJECT',
    8334 => 'ASTEROID KUIPER BELT OBJECT',
    8335 => 'ASTEROID KUIPER BELT OBJECT',
    8336 => 'ASTEROID KUIPER BELT OBJECT',
    8337 => 'ASTEROID KUIPER BELT OBJECT',
    8338 => 'ASTEROID KUIPER BELT OBJECT',
    8339 => 'ASTEROID KUIPER BELT OBJECT',
    8340 => 'ASTEROID NEAR-EARTH',
    8341 => 'ASTEROID NEAR-EARTH',
    8342 => 'ASTEROID NEAR-EARTH',
    8343 => 'ASTEROID NEAR-EARTH',
    8344 => 'ASTEROID NEAR-EARTH',
    8345 => 'ASTEROID NEAR-EARTH',
    8346 => 'ASTEROID NEAR-EARTH',
    8347 => 'ASTEROID NEAR-EARTH',
    8348 => 'ASTEROID NEAR-EARTH',
    8349 => 'ASTEROID NEAR-EARTH',
    8350 => 'ASTEROID',
    8351 => 'ASTEROID',
    8352 => 'ASTEROID',
    8353 => 'ASTEROID',
    8354 => 'ASTEROID',
    8355 => 'ASTEROID',
    8356 => 'ASTEROID',
    8357 => 'ASTEROID',
    8358 => 'ASTEROID',
    8359 => 'ASTEROID',
    8360 => 'ASTEROID',
    8361 => 'ASTEROID',
    8362 => 'ASTEROID',
    8363 => 'ASTEROID',
    8364 => 'ASTEROID',
    8365 => 'ASTEROID',
    8366 => 'ASTEROID',
    8367 => 'ASTEROID',
    8368 => 'ASTEROID',
    8369 => 'ASTEROID',
    8370 => 'ASTEROID',
    8371 => 'ASTEROID',
    8372 => 'ASTEROID',
    8373 => 'ASTEROID',
    8374 => 'ASTEROID',
    8375 => 'ASTEROID',
    8376 => 'ASTEROID',
    8377 => 'ASTEROID',
    8378 => 'ASTEROID',
    8379 => 'ASTEROID',
    8380 => 'ASTEROID',
    8381 => 'ASTEROID',
    8382 => 'ASTEROID',
    8383 => 'ASTEROID',
    8384 => 'ASTEROID',
    8385 => 'ASTEROID',
    8386 => 'ASTEROID',
    8387 => 'ASTEROID',
    8388 => 'ASTEROID',
    8389 => 'ASTEROID',
    8390 => 'ASTEROID',
    8391 => 'ASTEROID',
    8392 => 'ASTEROID',
    8393 => 'ASTEROID',
    8394 => 'ASTEROID',
    8395 => 'ASTEROID',
    8396 => 'ASTEROID',
    8397 => 'ASTEROID',
    8398 => 'ASTEROID',
    8399 => 'ASTEROID',
    8400 => 'COMET',
    8401 => 'COMET',
    8402 => 'COMET',
    8403 => 'COMET',
    8404 => 'COMET',
    8405 => 'COMET',
    8406 => 'COMET',
    8407 => 'COMET',
    8408 => 'COMET',
    8409 => 'COMET',
    8410 => 'COMET (PERIODIC)',
    8411 => 'COMET (PERIODIC)',
    8412 => 'COMET (PERIODIC)',
    8413 => 'COMET (PERIODIC)',
    8414 => 'COMET (PERIODIC)',
    8415 => 'COMET (PERIODIC)',
    8416 => 'COMET (PERIODIC)',
    8417 => 'COMET (PERIODIC)',
    8418 => 'COMET (PERIODIC)',
    8419 => 'COMET (PERIODIC)',
    8420 => 'COMET (NON-PERIODIC)',
    8421 => 'COMET (NON-PERIODIC)',
    8422 => 'COMET (NON-PERIODIC)',
    8423 => 'COMET (NON-PERIODIC)',
    8424 => 'COMET (NON-PERIODIC)',
    8425 => 'COMET (NON-PERIODIC)',
    8426 => 'COMET (NON-PERIODIC)',
    8427 => 'COMET (NON-PERIODIC)',
    8428 => 'COMET (NON-PERIODIC)',
    8429 => 'COMET (NON-PERIODIC)',
    8430 => 'COMET (SUN-GRAZING)',
    8431 => 'COMET (SUN-GRAZING)',
    8432 => 'COMET (SUN-GRAZING)',
    8433 => 'COMET (SUN-GRAZING)',
    8434 => 'COMET (SUN-GRAZING)',
    8435 => 'COMET (SUN-GRAZING)',
    8436 => 'COMET (SUN-GRAZING)',
    8437 => 'COMET (SUN-GRAZING)',
    8438 => 'COMET (SUN-GRAZING)',
    8439 => 'COMET (SUN-GRAZING)',
    8440 => 'COMET',
    8441 => 'COMET',
    8442 => 'COMET',
    8443 => 'COMET',
    8444 => 'COMET',
    8445 => 'COMET',
    8446 => 'COMET',
    8447 => 'COMET',
    8448 => 'COMET',
    8449 => 'COMET',
    8450 => 'COMET',
    8451 => 'COMET',
    8452 => 'COMET',
    8453 => 'COMET',
    8454 => 'COMET',
    8455 => 'COMET',
    8456 => 'COMET',
    8457 => 'COMET',
    8458 => 'COMET',
    8459 => 'COMET',
    8460 => 'COMET',
    8461 => 'COMET',
    8462 => 'COMET',
    8463 => 'COMET',
    8464 => 'COMET',
    8465 => 'COMET',
    8466 => 'COMET',
    8467 => 'COMET',
    8468 => 'COMET',
    8469 => 'COMET',
    8470 => 'COMET',
    8471 => 'COMET',
    8472 => 'COMET',
    8473 => 'COMET',
    8474 => 'COMET',
    8475 => 'COMET',
    8476 => 'COMET',
    8477 => 'COMET',
    8478 => 'COMET',
    8479 => 'COMET',
    8480 => 'COMET',
    8481 => 'COMET',
    8482 => 'COMET',
    8483 => 'COMET',
    8484 => 'COMET',
    8485 => 'COMET',
    8486 => 'COMET',
    8487 => 'COMET',
    8488 => 'COMET',
    8489 => 'COMET',
    8490 => 'COMET',
    8491 => 'COMET',
    8492 => 'COMET',
    8493 => 'COMET',
    8494 => 'COMET',
    8495 => 'COMET',
    8496 => 'COMET',
    8497 => 'COMET',
    8498 => 'COMET',
    8499 => 'COMET',
    8500 => 'MOON',
    8501 => 'MOON',
    8502 => 'MOON',
    8503 => 'MOON',
    8504 => 'MOON',
    8505 => 'MOON',
    8506 => 'MOON',
    8507 => 'MOON',
    8508 => 'MOON',
    8509 => 'MOON',
    8510 => 'MOON',
    8511 => 'MOON',
    8512 => 'MOON',
    8513 => 'MOON',
    8514 => 'MOON',
    8515 => 'MOON',
    8516 => 'MOON',
    8517 => 'MOON',
    8518 => 'MOON',
    8519 => 'MOON',
    8520 => 'MOON',
    8521 => 'MOON',
    8522 => 'MOON',
    8523 => 'MOON',
    8524 => 'MOON',
    8525 => 'MOON',
    8526 => 'MOON',
    8527 => 'MOON',
    8528 => 'MOON',
    8529 => 'MOON',
    8530 => 'MOON',
    8531 => 'MOON',
    8532 => 'MOON',
    8533 => 'MOON',
    8534 => 'MOON',
    8535 => 'MOON',
    8536 => 'MOON',
    8537 => 'MOON',
    8538 => 'MOON',
    8539 => 'MOON',
    8540 => 'MOON',
    8541 => 'MOON',
    8542 => 'MOON',
    8543 => 'MOON',
    8544 => 'MOON',
    8545 => 'MOON',
    8546 => 'MOON',
    8547 => 'MOON',
    8548 => 'MOON',
    8549 => 'MOON',
    8550 => 'MOON',
    8551 => 'MOON',
    8552 => 'MOON',
    8553 => 'MOON',
    8554 => 'MOON',
    8555 => 'MOON',
    8556 => 'MOON',
    8557 => 'MOON',
    8558 => 'MOON',
    8559 => 'MOON',
    8560 => 'MOON',
    8561 => 'MOON',
    8562 => 'MOON',
    8563 => 'MOON',
    8564 => 'MOON',
    8565 => 'MOON',
    8566 => 'MOON',
    8567 => 'MOON',
    8568 => 'MOON',
    8569 => 'MOON',
    8570 => 'MOON',
    8571 => 'MOON',
    8572 => 'MOON',
    8573 => 'MOON',
    8574 => 'MOON',
    8575 => 'MOON',
    8576 => 'MOON',
    8577 => 'MOON',
    8578 => 'MOON',
    8579 => 'MOON',
    8580 => 'MOON',
    8581 => 'MOON',
    8582 => 'MOON',
    8583 => 'MOON',
    8584 => 'MOON',
    8585 => 'MOON',
    8586 => 'MOON',
    8587 => 'MOON',
    8588 => 'MOON',
    8589 => 'MOON',
    8590 => 'MOON',
    8591 => 'MOON',
    8592 => 'MOON',
    8593 => 'MOON',
    8594 => 'MOON',
    8595 => 'MOON',
    8596 => 'MOON',
    8597 => 'MOON',
    8598 => 'MOON',
    8599 => 'MOON',
    8600 => 'UNCLASSIFIED',
    8601 => 'UNCLASSIFIED',
    8602 => 'UNCLASSIFIED',
    8603 => 'UNCLASSIFIED',
    8604 => 'UNCLASSIFIED',
    8605 => 'UNCLASSIFIED',
    8606 => 'UNCLASSIFIED',
    8607 => 'UNCLASSIFIED',
    8608 => 'UNCLASSIFIED',
    8609 => 'UNCLASSIFIED',
    8610 => 'UNCLASSIFIED',
    8611 => 'UNCLASSIFIED',
    8612 => 'UNCLASSIFIED',
    8613 => 'UNCLASSIFIED',
    8614 => 'UNCLASSIFIED',
    8615 => 'UNCLASSIFIED',
    8616 => 'UNCLASSIFIED',
    8617 => 'UNCLASSIFIED',
    8618 => 'UNCLASSIFIED',
    8619 => 'UNCLASSIFIED',
    8620 => 'UNCLASSIFIED',
    8621 => 'UNCLASSIFIED',
    8622 => 'UNCLASSIFIED',
    8623 => 'UNCLASSIFIED',
    8624 => 'UNCLASSIFIED',
    8625 => 'UNCLASSIFIED',
    8626 => 'UNCLASSIFIED',
    8627 => 'UNCLASSIFIED',
    8628 => 'UNCLASSIFIED',
    8629 => 'UNCLASSIFIED',
    8630 => 'UNCLASSIFIED',
    8631 => 'UNCLASSIFIED',
    8632 => 'UNCLASSIFIED',
    8633 => 'UNCLASSIFIED',
    8634 => 'UNCLASSIFIED',
    8635 => 'UNCLASSIFIED',
    8636 => 'UNCLASSIFIED',
    8637 => 'UNCLASSIFIED',
    8638 => 'UNCLASSIFIED',
    8639 => 'UNCLASSIFIED',
    8640 => 'UNCLASSIFIED',
    8641 => 'UNCLASSIFIED',
    8642 => 'UNCLASSIFIED',
    8643 => 'UNCLASSIFIED',
    8644 => 'UNCLASSIFIED',
    8645 => 'UNCLASSIFIED',
    8646 => 'UNCLASSIFIED',
    8647 => 'UNCLASSIFIED',
    8648 => 'UNCLASSIFIED',
    8649 => 'UNCLASSIFIED',
    8650 => 'UNCLASSIFIED',
    8651 => 'UNCLASSIFIED',
    8652 => 'UNCLASSIFIED',
    8653 => 'UNCLASSIFIED',
    8654 => 'UNCLASSIFIED',
    8655 => 'UNCLASSIFIED',
    8656 => 'UNCLASSIFIED',
    8657 => 'UNCLASSIFIED',
    8658 => 'UNCLASSIFIED',
    8659 => 'UNCLASSIFIED',
    8660 => 'UNCLASSIFIED',
    8661 => 'UNCLASSIFIED',
    8662 => 'UNCLASSIFIED',
    8663 => 'UNCLASSIFIED',
    8664 => 'UNCLASSIFIED',
    8665 => 'UNCLASSIFIED',
    8666 => 'UNCLASSIFIED',
    8667 => 'UNCLASSIFIED',
    8668 => 'UNCLASSIFIED',
    8669 => 'UNCLASSIFIED',
    8670 => 'UNCLASSIFIED',
    8671 => 'UNCLASSIFIED',
    8672 => 'UNCLASSIFIED',
    8673 => 'UNCLASSIFIED',
    8674 => 'UNCLASSIFIED',
    8675 => 'UNCLASSIFIED',
    8676 => 'UNCLASSIFIED',
    8677 => 'UNCLASSIFIED',
    8678 => 'UNCLASSIFIED',
    8679 => 'UNCLASSIFIED',
    8680 => 'UNCLASSIFIED',
    8681 => 'UNCLASSIFIED',
    8682 => 'UNCLASSIFIED',
    8683 => 'UNCLASSIFIED',
    8684 => 'UNCLASSIFIED',
    8685 => 'UNCLASSIFIED',
    8686 => 'UNCLASSIFIED',
    8687 => 'UNCLASSIFIED',
    8688 => 'UNCLASSIFIED',
    8689 => 'UNCLASSIFIED',
    8690 => 'UNCLASSIFIED',
    8691 => 'UNCLASSIFIED',
    8692 => 'UNCLASSIFIED',
    8693 => 'UNCLASSIFIED',
    8694 => 'UNCLASSIFIED',
    8695 => 'UNCLASSIFIED',
    8696 => 'UNCLASSIFIED',
    8697 => 'UNCLASSIFIED',
    8698 => 'UNCLASSIFIED',
    8699 => 'UNCLASSIFIED',
    8700 => 'UNCLASSIFIED',
    8701 => 'UNCLASSIFIED',
    8702 => 'UNCLASSIFIED',
    8703 => 'UNCLASSIFIED',
    8704 => 'UNCLASSIFIED',
    8705 => 'UNCLASSIFIED',
    8706 => 'UNCLASSIFIED',
    8707 => 'UNCLASSIFIED',
    8708 => 'UNCLASSIFIED',
    8709 => 'UNCLASSIFIED',
    8710 => 'UNCLASSIFIED',
    8711 => 'UNCLASSIFIED',
    8712 => 'UNCLASSIFIED',
    8713 => 'UNCLASSIFIED',
    8714 => 'UNCLASSIFIED',
    8715 => 'UNCLASSIFIED',
    8716 => 'UNCLASSIFIED',
    8717 => 'UNCLASSIFIED',
    8718 => 'UNCLASSIFIED',
    8719 => 'UNCLASSIFIED',
    8720 => 'UNCLASSIFIED',
    8721 => 'UNCLASSIFIED',
    8722 => 'UNCLASSIFIED',
    8723 => 'UNCLASSIFIED',
    8724 => 'UNCLASSIFIED',
    8725 => 'UNCLASSIFIED',
    8726 => 'UNCLASSIFIED',
    8727 => 'UNCLASSIFIED',
    8728 => 'UNCLASSIFIED',
    8729 => 'UNCLASSIFIED',
    8730 => 'UNCLASSIFIED',
    8731 => 'UNCLASSIFIED',
    8732 => 'UNCLASSIFIED',
    8733 => 'UNCLASSIFIED',
    8734 => 'UNCLASSIFIED',
    8735 => 'UNCLASSIFIED',
    8736 => 'UNCLASSIFIED',
    8737 => 'UNCLASSIFIED',
    8738 => 'UNCLASSIFIED',
    8739 => 'UNCLASSIFIED',
    8740 => 'UNCLASSIFIED',
    8741 => 'UNCLASSIFIED',
    8742 => 'UNCLASSIFIED',
    8743 => 'UNCLASSIFIED',
    8744 => 'UNCLASSIFIED',
    8745 => 'UNCLASSIFIED',
    8746 => 'UNCLASSIFIED',
    8747 => 'UNCLASSIFIED',
    8748 => 'UNCLASSIFIED',
    8749 => 'UNCLASSIFIED',
    8750 => 'UNCLASSIFIED',
    8751 => 'UNCLASSIFIED',
    8752 => 'UNCLASSIFIED',
    8753 => 'UNCLASSIFIED',
    8754 => 'UNCLASSIFIED',
    8755 => 'UNCLASSIFIED',
    8756 => 'UNCLASSIFIED',
    8757 => 'UNCLASSIFIED',
    8758 => 'UNCLASSIFIED',
    8759 => 'UNCLASSIFIED',
    8760 => 'UNCLASSIFIED',
    8761 => 'UNCLASSIFIED',
    8762 => 'UNCLASSIFIED',
    8763 => 'UNCLASSIFIED',
    8764 => 'UNCLASSIFIED',
    8765 => 'UNCLASSIFIED',
    8766 => 'UNCLASSIFIED',
    8767 => 'UNCLASSIFIED',
    8768 => 'UNCLASSIFIED',
    8769 => 'UNCLASSIFIED',
    8770 => 'UNCLASSIFIED',
    8771 => 'UNCLASSIFIED',
    8772 => 'UNCLASSIFIED',
    8773 => 'UNCLASSIFIED',
    8774 => 'UNCLASSIFIED',
    8775 => 'UNCLASSIFIED',
    8776 => 'UNCLASSIFIED',
    8777 => 'UNCLASSIFIED',
    8778 => 'UNCLASSIFIED',
    8779 => 'UNCLASSIFIED',
    8780 => 'UNCLASSIFIED',
    8781 => 'UNCLASSIFIED',
    8782 => 'UNCLASSIFIED',
    8783 => 'UNCLASSIFIED',
    8784 => 'UNCLASSIFIED',
    8785 => 'UNCLASSIFIED',
    8786 => 'UNCLASSIFIED',
    8787 => 'UNCLASSIFIED',
    8788 => 'UNCLASSIFIED',
    8789 => 'UNCLASSIFIED',
    8790 => 'UNCLASSIFIED',
    8791 => 'UNCLASSIFIED',
    8792 => 'UNCLASSIFIED',
    8793 => 'UNCLASSIFIED',
    8794 => 'UNCLASSIFIED',
    8795 => 'UNCLASSIFIED',
    8796 => 'UNCLASSIFIED',
    8797 => 'UNCLASSIFIED',
    8798 => 'UNCLASSIFIED',
    8799 => 'UNCLASSIFIED',
    8800 => 'UNCLASSIFIED',
    8801 => 'UNCLASSIFIED',
    8802 => 'UNCLASSIFIED',
    8803 => 'UNCLASSIFIED',
    8804 => 'UNCLASSIFIED',
    8805 => 'UNCLASSIFIED',
    8806 => 'UNCLASSIFIED',
    8807 => 'UNCLASSIFIED',
    8808 => 'UNCLASSIFIED',
    8809 => 'UNCLASSIFIED',
    8810 => 'UNCLASSIFIED',
    8811 => 'UNCLASSIFIED',
    8812 => 'UNCLASSIFIED',
    8813 => 'UNCLASSIFIED',
    8814 => 'UNCLASSIFIED',
    8815 => 'UNCLASSIFIED',
    8816 => 'UNCLASSIFIED',
    8817 => 'UNCLASSIFIED',
    8818 => 'UNCLASSIFIED',
    8819 => 'UNCLASSIFIED',
    8820 => 'UNCLASSIFIED',
    8821 => 'UNCLASSIFIED',
    8822 => 'UNCLASSIFIED',
    8823 => 'UNCLASSIFIED',
    8824 => 'UNCLASSIFIED',
    8825 => 'UNCLASSIFIED',
    8826 => 'UNCLASSIFIED',
    8827 => 'UNCLASSIFIED',
    8828 => 'UNCLASSIFIED',
    8829 => 'UNCLASSIFIED',
    8830 => 'UNCLASSIFIED',
    8831 => 'UNCLASSIFIED',
    8832 => 'UNCLASSIFIED',
    8833 => 'UNCLASSIFIED',
    8834 => 'UNCLASSIFIED',
    8835 => 'UNCLASSIFIED',
    8836 => 'UNCLASSIFIED',
    8837 => 'UNCLASSIFIED',
    8838 => 'UNCLASSIFIED',
    8839 => 'UNCLASSIFIED',
    8840 => 'UNCLASSIFIED',
    8841 => 'UNCLASSIFIED',
    8842 => 'UNCLASSIFIED',
    8843 => 'UNCLASSIFIED',
    8844 => 'UNCLASSIFIED',
    8845 => 'UNCLASSIFIED',
    8846 => 'UNCLASSIFIED',
    8847 => 'UNCLASSIFIED',
    8848 => 'UNCLASSIFIED',
    8849 => 'UNCLASSIFIED',
    8850 => 'UNCLASSIFIED',
    8851 => 'UNCLASSIFIED',
    8852 => 'UNCLASSIFIED',
    8853 => 'UNCLASSIFIED',
    8854 => 'UNCLASSIFIED',
    8855 => 'UNCLASSIFIED',
    8856 => 'UNCLASSIFIED',
    8857 => 'UNCLASSIFIED',
    8858 => 'UNCLASSIFIED',
    8859 => 'UNCLASSIFIED',
    8860 => 'UNCLASSIFIED',
    8861 => 'UNCLASSIFIED',
    8862 => 'UNCLASSIFIED',
    8863 => 'UNCLASSIFIED',
    8864 => 'UNCLASSIFIED',
    8865 => 'UNCLASSIFIED',
    8866 => 'UNCLASSIFIED',
    8867 => 'UNCLASSIFIED',
    8868 => 'UNCLASSIFIED',
    8869 => 'UNCLASSIFIED',
    8870 => 'UNCLASSIFIED',
    8871 => 'UNCLASSIFIED',
    8872 => 'UNCLASSIFIED',
    8873 => 'UNCLASSIFIED',
    8874 => 'UNCLASSIFIED',
    8875 => 'UNCLASSIFIED',
    8876 => 'UNCLASSIFIED',
    8877 => 'UNCLASSIFIED',
    8878 => 'UNCLASSIFIED',
    8879 => 'UNCLASSIFIED',
    8880 => 'UNCLASSIFIED',
    8881 => 'UNCLASSIFIED',
    8882 => 'UNCLASSIFIED',
    8883 => 'UNCLASSIFIED',
    8884 => 'UNCLASSIFIED',
    8885 => 'UNCLASSIFIED',
    8886 => 'UNCLASSIFIED',
    8887 => 'UNCLASSIFIED',
    8888 => 'UNCLASSIFIED',
    8889 => 'UNCLASSIFIED',
    8890 => 'UNCLASSIFIED',
    8891 => 'UNCLASSIFIED',
    8892 => 'UNCLASSIFIED',
    8893 => 'UNCLASSIFIED',
    8894 => 'UNCLASSIFIED',
    8895 => 'UNCLASSIFIED',
    8896 => 'UNCLASSIFIED',
    8897 => 'UNCLASSIFIED',
    8898 => 'UNCLASSIFIED',
    8899 => 'UNCLASSIFIED',
    8900 => 'UNCLASSIFIED',
    8901 => 'UNCLASSIFIED',
    8902 => 'UNCLASSIFIED',
    8903 => 'UNCLASSIFIED',
    8904 => 'UNCLASSIFIED',
    8905 => 'UNCLASSIFIED',
    8906 => 'UNCLASSIFIED',
    8907 => 'UNCLASSIFIED',
    8908 => 'UNCLASSIFIED',
    8909 => 'UNCLASSIFIED',
    8910 => 'UNCLASSIFIED',
    8911 => 'UNCLASSIFIED',
    8912 => 'UNCLASSIFIED',
    8913 => 'UNCLASSIFIED',
    8914 => 'UNCLASSIFIED',
    8915 => 'UNCLASSIFIED',
    8916 => 'UNCLASSIFIED',
    8917 => 'UNCLASSIFIED',
    8918 => 'UNCLASSIFIED',
    8919 => 'UNCLASSIFIED',
    8920 => 'UNCLASSIFIED',
    8921 => 'UNCLASSIFIED',
    8922 => 'UNCLASSIFIED',
    8923 => 'UNCLASSIFIED',
    8924 => 'UNCLASSIFIED',
    8925 => 'UNCLASSIFIED',
    8926 => 'UNCLASSIFIED',
    8927 => 'UNCLASSIFIED',
    8928 => 'UNCLASSIFIED',
    8929 => 'UNCLASSIFIED',
    8930 => 'UNCLASSIFIED',
    8931 => 'UNCLASSIFIED',
    8932 => 'UNCLASSIFIED',
    8933 => 'UNCLASSIFIED',
    8934 => 'UNCLASSIFIED',
    8935 => 'UNCLASSIFIED',
    8936 => 'UNCLASSIFIED',
    8937 => 'UNCLASSIFIED',
    8938 => 'UNCLASSIFIED',
    8939 => 'UNCLASSIFIED',
    8940 => 'UNCLASSIFIED',
    8941 => 'UNCLASSIFIED',
    8942 => 'UNCLASSIFIED',
    8943 => 'UNCLASSIFIED',
    8944 => 'UNCLASSIFIED',
    8945 => 'UNCLASSIFIED',
    8946 => 'UNCLASSIFIED',
    8947 => 'UNCLASSIFIED',
    8948 => 'UNCLASSIFIED',
    8949 => 'UNCLASSIFIED',
    8950 => 'UNCLASSIFIED',
    8951 => 'UNCLASSIFIED',
    8952 => 'UNCLASSIFIED',
    8953 => 'UNCLASSIFIED',
    8954 => 'UNCLASSIFIED',
    8955 => 'UNCLASSIFIED',
    8956 => 'UNCLASSIFIED',
    8957 => 'UNCLASSIFIED',
    8958 => 'UNCLASSIFIED',
    8959 => 'UNCLASSIFIED',
    8960 => 'UNCLASSIFIED',
    8961 => 'UNCLASSIFIED',
    8962 => 'UNCLASSIFIED',
    8963 => 'UNCLASSIFIED',
    8964 => 'UNCLASSIFIED',
    8965 => 'UNCLASSIFIED',
    8966 => 'UNCLASSIFIED',
    8967 => 'UNCLASSIFIED',
    8968 => 'UNCLASSIFIED',
    8969 => 'UNCLASSIFIED',
    8970 => 'UNCLASSIFIED',
    8971 => 'UNCLASSIFIED',
    8972 => 'UNCLASSIFIED',
    8973 => 'UNCLASSIFIED',
    8974 => 'UNCLASSIFIED',
    8975 => 'UNCLASSIFIED',
    8976 => 'UNCLASSIFIED',
    8977 => 'UNCLASSIFIED',
    8978 => 'UNCLASSIFIED',
    8979 => 'UNCLASSIFIED',
    8980 => 'UNCLASSIFIED',
    8981 => 'UNCLASSIFIED',
    8982 => 'UNCLASSIFIED',
    8983 => 'UNCLASSIFIED',
    8984 => 'UNCLASSIFIED',
    8985 => 'UNCLASSIFIED',
    8986 => 'UNCLASSIFIED',
    8987 => 'UNCLASSIFIED',
    8988 => 'UNCLASSIFIED',
    8989 => 'UNCLASSIFIED',
    8990 => 'UNCLASSIFIED',
    8991 => 'UNCLASSIFIED',
    8992 => 'UNCLASSIFIED',
    8993 => 'UNCLASSIFIED',
    8994 => 'UNCLASSIFIED',
    8995 => 'UNCLASSIFIED',
    8996 => 'UNCLASSIFIED',
    8997 => 'UNCLASSIFIED',
    8998 => 'UNCLASSIFIED',
    8999 => 'UNCLASSIFIED',
    9000 => 'UNUSUAL OBJECT',
    9001 => 'UNUSUAL OBJECT',
    9002 => 'UNUSUAL OBJECT',
    9003 => 'UNUSUAL OBJECT',
    9004 => 'UNUSUAL OBJECT',
    9005 => 'UNUSUAL OBJECT',
    9006 => 'UNUSUAL OBJECT',
    9007 => 'UNUSUAL OBJECT',
    9008 => 'UNUSUAL OBJECT',
    9009 => 'UNUSUAL OBJECT',
    9010 => 'UNUSUAL OBJECT',
    9011 => 'UNUSUAL OBJECT',
    9012 => 'UNUSUAL OBJECT',
    9013 => 'UNUSUAL OBJECT',
    9014 => 'UNUSUAL OBJECT',
    9015 => 'UNUSUAL OBJECT',
    9016 => 'UNUSUAL OBJECT',
    9017 => 'UNUSUAL OBJECT',
    9018 => 'UNUSUAL OBJECT',
    9019 => 'UNUSUAL OBJECT',
    9020 => 'UNUSUAL OBJECT',
    9021 => 'UNUSUAL OBJECT',
    9022 => 'UNUSUAL OBJECT',
    9023 => 'UNUSUAL OBJECT',
    9024 => 'UNUSUAL OBJECT',
    9025 => 'UNUSUAL OBJECT',
    9026 => 'UNUSUAL OBJECT',
    9027 => 'UNUSUAL OBJECT',
    9028 => 'UNUSUAL OBJECT',
    9029 => 'UNUSUAL OBJECT',
    9030 => 'UNUSUAL OBJECT',
    9031 => 'UNUSUAL OBJECT',
    9032 => 'UNUSUAL OBJECT',
    9033 => 'UNUSUAL OBJECT',
    9034 => 'UNUSUAL OBJECT',
    9035 => 'UNUSUAL OBJECT',
    9036 => 'UNUSUAL OBJECT',
    9037 => 'UNUSUAL OBJECT',
    9038 => 'UNUSUAL OBJECT',
    9039 => 'UNUSUAL OBJECT',
    9040 => 'UNUSUAL OBJECT',
    9041 => 'UNUSUAL OBJECT',
    9042 => 'UNUSUAL OBJECT',
    9043 => 'UNUSUAL OBJECT',
    9044 => 'UNUSUAL OBJECT',
    9045 => 'UNUSUAL OBJECT',
    9046 => 'UNUSUAL OBJECT',
    9047 => 'UNUSUAL OBJECT',
    9048 => 'UNUSUAL OBJECT',
    9049 => 'UNUSUAL OBJECT',
    9050 => 'UNUSUAL OBJECT',
    9051 => 'UNUSUAL OBJECT',
    9052 => 'UNUSUAL OBJECT',
    9053 => 'UNUSUAL OBJECT',
    9054 => 'UNUSUAL OBJECT',
    9055 => 'UNUSUAL OBJECT',
    9056 => 'UNUSUAL OBJECT',
    9057 => 'UNUSUAL OBJECT',
    9058 => 'UNUSUAL OBJECT',
    9059 => 'UNUSUAL OBJECT',
    9060 => 'UNUSUAL OBJECT',
    9061 => 'UNUSUAL OBJECT',
    9062 => 'UNUSUAL OBJECT',
    9063 => 'UNUSUAL OBJECT',
    9064 => 'UNUSUAL OBJECT',
    9065 => 'UNUSUAL OBJECT',
    9066 => 'UNUSUAL OBJECT',
    9067 => 'UNUSUAL OBJECT',
    9068 => 'UNUSUAL OBJECT',
    9069 => 'UNUSUAL OBJECT',
    9070 => 'UNUSUAL OBJECT',
    9071 => 'UNUSUAL OBJECT',
    9072 => 'UNUSUAL OBJECT',
    9073 => 'UNUSUAL OBJECT',
    9074 => 'UNUSUAL OBJECT',
    9075 => 'UNUSUAL OBJECT',
    9076 => 'UNUSUAL OBJECT',
    9077 => 'UNUSUAL OBJECT',
    9078 => 'UNUSUAL OBJECT',
    9079 => 'UNUSUAL OBJECT',
    9080 => 'UNUSUAL OBJECT',
    9081 => 'UNUSUAL OBJECT',
    9082 => 'UNUSUAL OBJECT',
    9083 => 'UNUSUAL OBJECT',
    9084 => 'UNUSUAL OBJECT',
    9085 => 'UNUSUAL OBJECT',
    9086 => 'UNUSUAL OBJECT',
    9087 => 'UNUSUAL OBJECT',
    9088 => 'UNUSUAL OBJECT',
    9089 => 'UNUSUAL OBJECT',
    9090 => 'UNUSUAL OBJECT',
    9091 => 'UNUSUAL OBJECT',
    9092 => 'UNUSUAL OBJECT',
    9093 => 'UNUSUAL OBJECT',
    9094 => 'UNUSUAL OBJECT',
    9095 => 'UNUSUAL OBJECT',
    9096 => 'UNUSUAL OBJECT',
    9097 => 'UNUSUAL OBJECT',
    9098 => 'UNUSUAL OBJECT',
    9099 => 'UNUSUAL OBJECT',
    9100 => 'SUPERNOVA',
    9101 => 'SUPERNOVA',
    9102 => 'SUPERNOVA',
    9103 => 'SUPERNOVA',
    9104 => 'SUPERNOVA',
    9105 => 'SUPERNOVA',
    9106 => 'SUPERNOVA',
    9107 => 'SUPERNOVA',
    9108 => 'SUPERNOVA',
    9109 => 'SUPERNOVA',
    9110 => 'SUPERNOVA TYPE I',
    9111 => 'SUPERNOVA TYPE I',
    9112 => 'SUPERNOVA TYPE I',
    9113 => 'SUPERNOVA TYPE I',
    9114 => 'SUPERNOVA TYPE I',
    9115 => 'SUPERNOVA TYPE I',
    9116 => 'SUPERNOVA TYPE I',
    9117 => 'SUPERNOVA TYPE I',
    9118 => 'SUPERNOVA TYPE I',
    9119 => 'SUPERNOVA TYPE I',
    9120 => 'SUPERNOVA TYPE II',
    9121 => 'SUPERNOVA TYPE II',
    9122 => 'SUPERNOVA TYPE II',
    9123 => 'SUPERNOVA TYPE II',
    9124 => 'SUPERNOVA TYPE II',
    9125 => 'SUPERNOVA TYPE II',
    9126 => 'SUPERNOVA TYPE II',
    9127 => 'SUPERNOVA TYPE II',
    9128 => 'SUPERNOVA TYPE II',
    9129 => 'SUPERNOVA TYPE II',
    9130 => 'SUPERNOVA',
    9131 => 'SUPERNOVA',
    9132 => 'SUPERNOVA',
    9133 => 'SUPERNOVA',
    9134 => 'SUPERNOVA',
    9135 => 'SUPERNOVA',
    9136 => 'SUPERNOVA',
    9137 => 'SUPERNOVA',
    9138 => 'SUPERNOVA',
    9139 => 'SUPERNOVA',
    9140 => 'SUPERNOVA',
    9141 => 'SUPERNOVA',
    9142 => 'SUPERNOVA',
    9143 => 'SUPERNOVA',
    9144 => 'SUPERNOVA',
    9145 => 'SUPERNOVA',
    9146 => 'SUPERNOVA',
    9147 => 'SUPERNOVA',
    9148 => 'SUPERNOVA',
    9149 => 'SUPERNOVA',
    9150 => 'SUPERNOVA',
    9151 => 'SUPERNOVA',
    9152 => 'SUPERNOVA',
    9153 => 'SUPERNOVA',
    9154 => 'SUPERNOVA',
    9155 => 'SUPERNOVA',
    9156 => 'SUPERNOVA',
    9157 => 'SUPERNOVA',
    9158 => 'SUPERNOVA',
    9159 => 'SUPERNOVA',
    9160 => 'SUPERNOVA',
    9161 => 'SUPERNOVA',
    9162 => 'SUPERNOVA',
    9163 => 'SUPERNOVA',
    9164 => 'SUPERNOVA',
    9165 => 'SUPERNOVA',
    9166 => 'SUPERNOVA',
    9167 => 'SUPERNOVA',
    9168 => 'SUPERNOVA',
    9169 => 'SUPERNOVA',
    9170 => 'SUPERNOVA',
    9171 => 'SUPERNOVA',
    9172 => 'SUPERNOVA',
    9173 => 'SUPERNOVA',
    9174 => 'SUPERNOVA',
    9175 => 'SUPERNOVA',
    9176 => 'SUPERNOVA',
    9177 => 'SUPERNOVA',
    9178 => 'SUPERNOVA',
    9179 => 'SUPERNOVA',
    9180 => 'SUPERNOVA',
    9181 => 'SUPERNOVA',
    9182 => 'SUPERNOVA',
    9183 => 'SUPERNOVA',
    9184 => 'SUPERNOVA',
    9185 => 'SUPERNOVA',
    9186 => 'SUPERNOVA',
    9187 => 'SUPERNOVA',
    9188 => 'SUPERNOVA',
    9189 => 'SUPERNOVA',
    9190 => 'SUPERNOVA',
    9191 => 'SUPERNOVA',
    9192 => 'SUPERNOVA',
    9193 => 'SUPERNOVA',
    9194 => 'SUPERNOVA',
    9195 => 'SUPERNOVA',
    9196 => 'SUPERNOVA',
    9197 => 'SUPERNOVA',
    9198 => 'SUPERNOVA',
    9199 => 'SUPERNOVA',
    9200 => 'HYPERNOVA',
    9201 => 'HYPERNOVA',
    9202 => 'HYPERNOVA',
    9203 => 'HYPERNOVA',
    9204 => 'HYPERNOVA',
    9205 => 'HYPERNOVA',
    9206 => 'HYPERNOVA',
    9207 => 'HYPERNOVA',
    9208 => 'HYPERNOVA',
    9209 => 'HYPERNOVA',
    9210 => 'HYPERNOVA',
    9211 => 'HYPERNOVA',
    9212 => 'HYPERNOVA',
    9213 => 'HYPERNOVA',
    9214 => 'HYPERNOVA',
    9215 => 'HYPERNOVA',
    9216 => 'HYPERNOVA',
    9217 => 'HYPERNOVA',
    9218 => 'HYPERNOVA',
    9219 => 'HYPERNOVA',
    9220 => 'HYPERNOVA',
    9221 => 'HYPERNOVA',
    9222 => 'HYPERNOVA',
    9223 => 'HYPERNOVA',
    9224 => 'HYPERNOVA',
    9225 => 'HYPERNOVA',
    9226 => 'HYPERNOVA',
    9227 => 'HYPERNOVA',
    9228 => 'HYPERNOVA',
    9229 => 'HYPERNOVA',
    9230 => 'HYPERNOVA',
    9231 => 'HYPERNOVA',
    9232 => 'HYPERNOVA',
    9233 => 'HYPERNOVA',
    9234 => 'HYPERNOVA',
    9235 => 'HYPERNOVA',
    9236 => 'HYPERNOVA',
    9237 => 'HYPERNOVA',
    9238 => 'HYPERNOVA',
    9239 => 'HYPERNOVA',
    9240 => 'HYPERNOVA',
    9241 => 'HYPERNOVA',
    9242 => 'HYPERNOVA',
    9243 => 'HYPERNOVA',
    9244 => 'HYPERNOVA',
    9245 => 'HYPERNOVA',
    9246 => 'HYPERNOVA',
    9247 => 'HYPERNOVA',
    9248 => 'HYPERNOVA',
    9249 => 'HYPERNOVA',
    9250 => 'HYPERNOVA',
    9251 => 'HYPERNOVA',
    9252 => 'HYPERNOVA',
    9253 => 'HYPERNOVA',
    9254 => 'HYPERNOVA',
    9255 => 'HYPERNOVA',
    9256 => 'HYPERNOVA',
    9257 => 'HYPERNOVA',
    9258 => 'HYPERNOVA',
    9259 => 'HYPERNOVA',
    9260 => 'HYPERNOVA',
    9261 => 'HYPERNOVA',
    9262 => 'HYPERNOVA',
    9263 => 'HYPERNOVA',
    9264 => 'HYPERNOVA',
    9265 => 'HYPERNOVA',
    9266 => 'HYPERNOVA',
    9267 => 'HYPERNOVA',
    9268 => 'HYPERNOVA',
    9269 => 'HYPERNOVA',
    9270 => 'HYPERNOVA',
    9271 => 'HYPERNOVA',
    9272 => 'HYPERNOVA',
    9273 => 'HYPERNOVA',
    9274 => 'HYPERNOVA',
    9275 => 'HYPERNOVA',
    9276 => 'HYPERNOVA',
    9277 => 'HYPERNOVA',
    9278 => 'HYPERNOVA',
    9279 => 'HYPERNOVA',
    9280 => 'HYPERNOVA',
    9281 => 'HYPERNOVA',
    9282 => 'HYPERNOVA',
    9283 => 'HYPERNOVA',
    9284 => 'HYPERNOVA',
    9285 => 'HYPERNOVA',
    9286 => 'HYPERNOVA',
    9287 => 'HYPERNOVA',
    9288 => 'HYPERNOVA',
    9289 => 'HYPERNOVA',
    9290 => 'HYPERNOVA',
    9291 => 'HYPERNOVA',
    9292 => 'HYPERNOVA',
    9293 => 'HYPERNOVA',
    9294 => 'HYPERNOVA',
    9295 => 'HYPERNOVA',
    9296 => 'HYPERNOVA',
    9297 => 'HYPERNOVA',
    9298 => 'HYPERNOVA',
    9299 => 'HYPERNOVA',
    9300 => 'UNCLASSIFIED',
    9301 => 'UNCLASSIFIED',
    9302 => 'UNCLASSIFIED',
    9303 => 'UNCLASSIFIED',
    9304 => 'UNCLASSIFIED',
    9305 => 'UNCLASSIFIED',
    9306 => 'UNCLASSIFIED',
    9307 => 'UNCLASSIFIED',
    9308 => 'UNCLASSIFIED',
    9309 => 'UNCLASSIFIED',
    9310 => 'UNCLASSIFIED',
    9311 => 'UNCLASSIFIED',
    9312 => 'UNCLASSIFIED',
    9313 => 'UNCLASSIFIED',
    9314 => 'UNCLASSIFIED',
    9315 => 'UNCLASSIFIED',
    9316 => 'UNCLASSIFIED',
    9317 => 'UNCLASSIFIED',
    9318 => 'UNCLASSIFIED',
    9319 => 'UNCLASSIFIED',
    9320 => 'UNCLASSIFIED',
    9321 => 'UNCLASSIFIED',
    9322 => 'UNCLASSIFIED',
    9323 => 'UNCLASSIFIED',
    9324 => 'UNCLASSIFIED',
    9325 => 'UNCLASSIFIED',
    9326 => 'UNCLASSIFIED',
    9327 => 'UNCLASSIFIED',
    9328 => 'UNCLASSIFIED',
    9329 => 'UNCLASSIFIED',
    9330 => 'UNCLASSIFIED',
    9331 => 'UNCLASSIFIED',
    9332 => 'UNCLASSIFIED',
    9333 => 'UNCLASSIFIED',
    9334 => 'UNCLASSIFIED',
    9335 => 'UNCLASSIFIED',
    9336 => 'UNCLASSIFIED',
    9337 => 'UNCLASSIFIED',
    9338 => 'UNCLASSIFIED',
    9339 => 'UNCLASSIFIED',
    9340 => 'UNCLASSIFIED',
    9341 => 'UNCLASSIFIED',
    9342 => 'UNCLASSIFIED',
    9343 => 'UNCLASSIFIED',
    9344 => 'UNCLASSIFIED',
    9345 => 'UNCLASSIFIED',
    9346 => 'UNCLASSIFIED',
    9347 => 'UNCLASSIFIED',
    9348 => 'UNCLASSIFIED',
    9349 => 'UNCLASSIFIED',
    9350 => 'UNCLASSIFIED',
    9351 => 'UNCLASSIFIED',
    9352 => 'UNCLASSIFIED',
    9353 => 'UNCLASSIFIED',
    9354 => 'UNCLASSIFIED',
    9355 => 'UNCLASSIFIED',
    9356 => 'UNCLASSIFIED',
    9357 => 'UNCLASSIFIED',
    9358 => 'UNCLASSIFIED',
    9359 => 'UNCLASSIFIED',
    9360 => 'UNCLASSIFIED',
    9361 => 'UNCLASSIFIED',
    9362 => 'UNCLASSIFIED',
    9363 => 'UNCLASSIFIED',
    9364 => 'UNCLASSIFIED',
    9365 => 'UNCLASSIFIED',
    9366 => 'UNCLASSIFIED',
    9367 => 'UNCLASSIFIED',
    9368 => 'UNCLASSIFIED',
    9369 => 'UNCLASSIFIED',
    9370 => 'UNCLASSIFIED',
    9371 => 'UNCLASSIFIED',
    9372 => 'UNCLASSIFIED',
    9373 => 'UNCLASSIFIED',
    9374 => 'UNCLASSIFIED',
    9375 => 'UNCLASSIFIED',
    9376 => 'UNCLASSIFIED',
    9377 => 'UNCLASSIFIED',
    9378 => 'UNCLASSIFIED',
    9379 => 'UNCLASSIFIED',
    9380 => 'UNCLASSIFIED',
    9381 => 'UNCLASSIFIED',
    9382 => 'UNCLASSIFIED',
    9383 => 'UNCLASSIFIED',
    9384 => 'UNCLASSIFIED',
    9385 => 'UNCLASSIFIED',
    9386 => 'UNCLASSIFIED',
    9387 => 'UNCLASSIFIED',
    9388 => 'UNCLASSIFIED',
    9389 => 'UNCLASSIFIED',
    9390 => 'UNCLASSIFIED',
    9391 => 'UNCLASSIFIED',
    9392 => 'UNCLASSIFIED',
    9393 => 'UNCLASSIFIED',
    9394 => 'UNCLASSIFIED',
    9395 => 'UNCLASSIFIED',
    9396 => 'UNCLASSIFIED',
    9397 => 'UNCLASSIFIED',
    9398 => 'UNCLASSIFIED',
    9399 => 'UNCLASSIFIED',
    9400 => 'UNCLASSIFIED',
    9401 => 'UNCLASSIFIED',
    9402 => 'UNCLASSIFIED',
    9403 => 'UNCLASSIFIED',
    9404 => 'UNCLASSIFIED',
    9405 => 'UNCLASSIFIED',
    9406 => 'UNCLASSIFIED',
    9407 => 'UNCLASSIFIED',
    9408 => 'UNCLASSIFIED',
    9409 => 'UNCLASSIFIED',
    9410 => 'UNCLASSIFIED',
    9411 => 'UNCLASSIFIED',
    9412 => 'UNCLASSIFIED',
    9413 => 'UNCLASSIFIED',
    9414 => 'UNCLASSIFIED',
    9415 => 'UNCLASSIFIED',
    9416 => 'UNCLASSIFIED',
    9417 => 'UNCLASSIFIED',
    9418 => 'UNCLASSIFIED',
    9419 => 'UNCLASSIFIED',
    9420 => 'UNCLASSIFIED',
    9421 => 'UNCLASSIFIED',
    9422 => 'UNCLASSIFIED',
    9423 => 'UNCLASSIFIED',
    9424 => 'UNCLASSIFIED',
    9425 => 'UNCLASSIFIED',
    9426 => 'UNCLASSIFIED',
    9427 => 'UNCLASSIFIED',
    9428 => 'UNCLASSIFIED',
    9429 => 'UNCLASSIFIED',
    9430 => 'UNCLASSIFIED',
    9431 => 'UNCLASSIFIED',
    9432 => 'UNCLASSIFIED',
    9433 => 'UNCLASSIFIED',
    9434 => 'UNCLASSIFIED',
    9435 => 'UNCLASSIFIED',
    9436 => 'UNCLASSIFIED',
    9437 => 'UNCLASSIFIED',
    9438 => 'UNCLASSIFIED',
    9439 => 'UNCLASSIFIED',
    9440 => 'UNCLASSIFIED',
    9441 => 'UNCLASSIFIED',
    9442 => 'UNCLASSIFIED',
    9443 => 'UNCLASSIFIED',
    9444 => 'UNCLASSIFIED',
    9445 => 'UNCLASSIFIED',
    9446 => 'UNCLASSIFIED',
    9447 => 'UNCLASSIFIED',
    9448 => 'UNCLASSIFIED',
    9449 => 'UNCLASSIFIED',
    9450 => 'UNCLASSIFIED',
    9451 => 'UNCLASSIFIED',
    9452 => 'UNCLASSIFIED',
    9453 => 'UNCLASSIFIED',
    9454 => 'UNCLASSIFIED',
    9455 => 'UNCLASSIFIED',
    9456 => 'UNCLASSIFIED',
    9457 => 'UNCLASSIFIED',
    9458 => 'UNCLASSIFIED',
    9459 => 'UNCLASSIFIED',
    9460 => 'UNCLASSIFIED',
    9461 => 'UNCLASSIFIED',
    9462 => 'UNCLASSIFIED',
    9463 => 'UNCLASSIFIED',
    9464 => 'UNCLASSIFIED',
    9465 => 'UNCLASSIFIED',
    9466 => 'UNCLASSIFIED',
    9467 => 'UNCLASSIFIED',
    9468 => 'UNCLASSIFIED',
    9469 => 'UNCLASSIFIED',
    9470 => 'UNCLASSIFIED',
    9471 => 'UNCLASSIFIED',
    9472 => 'UNCLASSIFIED',
    9473 => 'UNCLASSIFIED',
    9474 => 'UNCLASSIFIED',
    9475 => 'UNCLASSIFIED',
    9476 => 'UNCLASSIFIED',
    9477 => 'UNCLASSIFIED',
    9478 => 'UNCLASSIFIED',
    9479 => 'UNCLASSIFIED',
    9480 => 'UNCLASSIFIED',
    9481 => 'UNCLASSIFIED',
    9482 => 'UNCLASSIFIED',
    9483 => 'UNCLASSIFIED',
    9484 => 'UNCLASSIFIED',
    9485 => 'UNCLASSIFIED',
    9486 => 'UNCLASSIFIED',
    9487 => 'UNCLASSIFIED',
    9488 => 'UNCLASSIFIED',
    9489 => 'UNCLASSIFIED',
    9490 => 'UNCLASSIFIED',
    9491 => 'UNCLASSIFIED',
    9492 => 'UNCLASSIFIED',
    9493 => 'UNCLASSIFIED',
    9494 => 'UNCLASSIFIED',
    9495 => 'UNCLASSIFIED',
    9496 => 'UNCLASSIFIED',
    9497 => 'UNCLASSIFIED',
    9498 => 'UNCLASSIFIED',
    9499 => 'UNCLASSIFIED',
    9500 => 'UNCLASSIFIED',
    9501 => 'UNCLASSIFIED',
    9502 => 'UNCLASSIFIED',
    9503 => 'UNCLASSIFIED',
    9504 => 'UNCLASSIFIED',
    9505 => 'UNCLASSIFIED',
    9506 => 'UNCLASSIFIED',
    9507 => 'UNCLASSIFIED',
    9508 => 'UNCLASSIFIED',
    9509 => 'UNCLASSIFIED',
    9510 => 'UNCLASSIFIED',
    9511 => 'UNCLASSIFIED',
    9512 => 'UNCLASSIFIED',
    9513 => 'UNCLASSIFIED',
    9514 => 'UNCLASSIFIED',
    9515 => 'UNCLASSIFIED',
    9516 => 'UNCLASSIFIED',
    9517 => 'UNCLASSIFIED',
    9518 => 'UNCLASSIFIED',
    9519 => 'UNCLASSIFIED',
    9520 => 'UNCLASSIFIED',
    9521 => 'UNCLASSIFIED',
    9522 => 'UNCLASSIFIED',
    9523 => 'UNCLASSIFIED',
    9524 => 'UNCLASSIFIED',
    9525 => 'UNCLASSIFIED',
    9526 => 'UNCLASSIFIED',
    9527 => 'UNCLASSIFIED',
    9528 => 'UNCLASSIFIED',
    9529 => 'UNCLASSIFIED',
    9530 => 'UNCLASSIFIED',
    9531 => 'UNCLASSIFIED',
    9532 => 'UNCLASSIFIED',
    9533 => 'UNCLASSIFIED',
    9534 => 'UNCLASSIFIED',
    9535 => 'UNCLASSIFIED',
    9536 => 'UNCLASSIFIED',
    9537 => 'UNCLASSIFIED',
    9538 => 'UNCLASSIFIED',
    9539 => 'UNCLASSIFIED',
    9540 => 'UNCLASSIFIED',
    9541 => 'UNCLASSIFIED',
    9542 => 'UNCLASSIFIED',
    9543 => 'UNCLASSIFIED',
    9544 => 'UNCLASSIFIED',
    9545 => 'UNCLASSIFIED',
    9546 => 'UNCLASSIFIED',
    9547 => 'UNCLASSIFIED',
    9548 => 'UNCLASSIFIED',
    9549 => 'UNCLASSIFIED',
    9550 => 'UNCLASSIFIED',
    9551 => 'UNCLASSIFIED',
    9552 => 'UNCLASSIFIED',
    9553 => 'UNCLASSIFIED',
    9554 => 'UNCLASSIFIED',
    9555 => 'UNCLASSIFIED',
    9556 => 'UNCLASSIFIED',
    9557 => 'UNCLASSIFIED',
    9558 => 'UNCLASSIFIED',
    9559 => 'UNCLASSIFIED',
    9560 => 'UNCLASSIFIED',
    9561 => 'UNCLASSIFIED',
    9562 => 'UNCLASSIFIED',
    9563 => 'UNCLASSIFIED',
    9564 => 'UNCLASSIFIED',
    9565 => 'UNCLASSIFIED',
    9566 => 'UNCLASSIFIED',
    9567 => 'UNCLASSIFIED',
    9568 => 'UNCLASSIFIED',
    9569 => 'UNCLASSIFIED',
    9570 => 'UNCLASSIFIED',
    9571 => 'UNCLASSIFIED',
    9572 => 'UNCLASSIFIED',
    9573 => 'UNCLASSIFIED',
    9574 => 'UNCLASSIFIED',
    9575 => 'UNCLASSIFIED',
    9576 => 'UNCLASSIFIED',
    9577 => 'UNCLASSIFIED',
    9578 => 'UNCLASSIFIED',
    9579 => 'UNCLASSIFIED',
    9580 => 'UNCLASSIFIED',
    9581 => 'UNCLASSIFIED',
    9582 => 'UNCLASSIFIED',
    9583 => 'UNCLASSIFIED',
    9584 => 'UNCLASSIFIED',
    9585 => 'UNCLASSIFIED',
    9586 => 'UNCLASSIFIED',
    9587 => 'UNCLASSIFIED',
    9588 => 'UNCLASSIFIED',
    9589 => 'UNCLASSIFIED',
    9590 => 'UNCLASSIFIED',
    9591 => 'UNCLASSIFIED',
    9592 => 'UNCLASSIFIED',
    9593 => 'UNCLASSIFIED',
    9594 => 'UNCLASSIFIED',
    9595 => 'UNCLASSIFIED',
    9596 => 'UNCLASSIFIED',
    9597 => 'UNCLASSIFIED',
    9598 => 'UNCLASSIFIED',
    9599 => 'UNCLASSIFIED',
    9600 => 'UNCLASSIFIED',
    9601 => 'UNCLASSIFIED',
    9602 => 'UNCLASSIFIED',
    9603 => 'UNCLASSIFIED',
    9604 => 'UNCLASSIFIED',
    9605 => 'UNCLASSIFIED',
    9606 => 'UNCLASSIFIED',
    9607 => 'UNCLASSIFIED',
    9608 => 'UNCLASSIFIED',
    9609 => 'UNCLASSIFIED',
    9610 => 'UNCLASSIFIED',
    9611 => 'UNCLASSIFIED',
    9612 => 'UNCLASSIFIED',
    9613 => 'UNCLASSIFIED',
    9614 => 'UNCLASSIFIED',
    9615 => 'UNCLASSIFIED',
    9616 => 'UNCLASSIFIED',
    9617 => 'UNCLASSIFIED',
    9618 => 'UNCLASSIFIED',
    9619 => 'UNCLASSIFIED',
    9620 => 'UNCLASSIFIED',
    9621 => 'UNCLASSIFIED',
    9622 => 'UNCLASSIFIED',
    9623 => 'UNCLASSIFIED',
    9624 => 'UNCLASSIFIED',
    9625 => 'UNCLASSIFIED',
    9626 => 'UNCLASSIFIED',
    9627 => 'UNCLASSIFIED',
    9628 => 'UNCLASSIFIED',
    9629 => 'UNCLASSIFIED',
    9630 => 'UNCLASSIFIED',
    9631 => 'UNCLASSIFIED',
    9632 => 'UNCLASSIFIED',
    9633 => 'UNCLASSIFIED',
    9634 => 'UNCLASSIFIED',
    9635 => 'UNCLASSIFIED',
    9636 => 'UNCLASSIFIED',
    9637 => 'UNCLASSIFIED',
    9638 => 'UNCLASSIFIED',
    9639 => 'UNCLASSIFIED',
    9640 => 'UNCLASSIFIED',
    9641 => 'UNCLASSIFIED',
    9642 => 'UNCLASSIFIED',
    9643 => 'UNCLASSIFIED',
    9644 => 'UNCLASSIFIED',
    9645 => 'UNCLASSIFIED',
    9646 => 'UNCLASSIFIED',
    9647 => 'UNCLASSIFIED',
    9648 => 'UNCLASSIFIED',
    9649 => 'UNCLASSIFIED',
    9650 => 'UNCLASSIFIED',
    9651 => 'UNCLASSIFIED',
    9652 => 'UNCLASSIFIED',
    9653 => 'UNCLASSIFIED',
    9654 => 'UNCLASSIFIED',
    9655 => 'UNCLASSIFIED',
    9656 => 'UNCLASSIFIED',
    9657 => 'UNCLASSIFIED',
    9658 => 'UNCLASSIFIED',
    9659 => 'UNCLASSIFIED',
    9660 => 'UNCLASSIFIED',
    9661 => 'UNCLASSIFIED',
    9662 => 'UNCLASSIFIED',
    9663 => 'UNCLASSIFIED',
    9664 => 'UNCLASSIFIED',
    9665 => 'UNCLASSIFIED',
    9666 => 'UNCLASSIFIED',
    9667 => 'UNCLASSIFIED',
    9668 => 'UNCLASSIFIED',
    9669 => 'UNCLASSIFIED',
    9670 => 'UNCLASSIFIED',
    9671 => 'UNCLASSIFIED',
    9672 => 'UNCLASSIFIED',
    9673 => 'UNCLASSIFIED',
    9674 => 'UNCLASSIFIED',
    9675 => 'UNCLASSIFIED',
    9676 => 'UNCLASSIFIED',
    9677 => 'UNCLASSIFIED',
    9678 => 'UNCLASSIFIED',
    9679 => 'UNCLASSIFIED',
    9680 => 'UNCLASSIFIED',
    9681 => 'UNCLASSIFIED',
    9682 => 'UNCLASSIFIED',
    9683 => 'UNCLASSIFIED',
    9684 => 'UNCLASSIFIED',
    9685 => 'UNCLASSIFIED',
    9686 => 'UNCLASSIFIED',
    9687 => 'UNCLASSIFIED',
    9688 => 'UNCLASSIFIED',
    9689 => 'UNCLASSIFIED',
    9690 => 'UNCLASSIFIED',
    9691 => 'UNCLASSIFIED',
    9692 => 'UNCLASSIFIED',
    9693 => 'UNCLASSIFIED',
    9694 => 'UNCLASSIFIED',
    9695 => 'UNCLASSIFIED',
    9696 => 'UNCLASSIFIED',
    9697 => 'UNCLASSIFIED',
    9698 => 'UNCLASSIFIED',
    9699 => 'UNCLASSIFIED',
    9700 => 'UNCLASSIFIED',
    9701 => 'UNCLASSIFIED',
    9702 => 'UNCLASSIFIED',
    9703 => 'UNCLASSIFIED',
    9704 => 'UNCLASSIFIED',
    9705 => 'UNCLASSIFIED',
    9706 => 'UNCLASSIFIED',
    9707 => 'UNCLASSIFIED',
    9708 => 'UNCLASSIFIED',
    9709 => 'UNCLASSIFIED',
    9710 => 'UNCLASSIFIED',
    9711 => 'UNCLASSIFIED',
    9712 => 'UNCLASSIFIED',
    9713 => 'UNCLASSIFIED',
    9714 => 'UNCLASSIFIED',
    9715 => 'UNCLASSIFIED',
    9716 => 'UNCLASSIFIED',
    9717 => 'UNCLASSIFIED',
    9718 => 'UNCLASSIFIED',
    9719 => 'UNCLASSIFIED',
    9720 => 'UNCLASSIFIED',
    9721 => 'UNCLASSIFIED',
    9722 => 'UNCLASSIFIED',
    9723 => 'UNCLASSIFIED',
    9724 => 'UNCLASSIFIED',
    9725 => 'UNCLASSIFIED',
    9726 => 'UNCLASSIFIED',
    9727 => 'UNCLASSIFIED',
    9728 => 'UNCLASSIFIED',
    9729 => 'UNCLASSIFIED',
    9730 => 'UNCLASSIFIED',
    9731 => 'UNCLASSIFIED',
    9732 => 'UNCLASSIFIED',
    9733 => 'UNCLASSIFIED',
    9734 => 'UNCLASSIFIED',
    9735 => 'UNCLASSIFIED',
    9736 => 'UNCLASSIFIED',
    9737 => 'UNCLASSIFIED',
    9738 => 'UNCLASSIFIED',
    9739 => 'UNCLASSIFIED',
    9740 => 'UNCLASSIFIED',
    9741 => 'UNCLASSIFIED',
    9742 => 'UNCLASSIFIED',
    9743 => 'UNCLASSIFIED',
    9744 => 'UNCLASSIFIED',
    9745 => 'UNCLASSIFIED',
    9746 => 'UNCLASSIFIED',
    9747 => 'UNCLASSIFIED',
    9748 => 'UNCLASSIFIED',
    9749 => 'UNCLASSIFIED',
    9750 => 'UNCLASSIFIED',
    9751 => 'UNCLASSIFIED',
    9752 => 'UNCLASSIFIED',
    9753 => 'UNCLASSIFIED',
    9754 => 'UNCLASSIFIED',
    9755 => 'UNCLASSIFIED',
    9756 => 'UNCLASSIFIED',
    9757 => 'UNCLASSIFIED',
    9758 => 'UNCLASSIFIED',
    9759 => 'UNCLASSIFIED',
    9760 => 'UNCLASSIFIED',
    9761 => 'UNCLASSIFIED',
    9762 => 'UNCLASSIFIED',
    9763 => 'UNCLASSIFIED',
    9764 => 'UNCLASSIFIED',
    9765 => 'UNCLASSIFIED',
    9766 => 'UNCLASSIFIED',
    9767 => 'UNCLASSIFIED',
    9768 => 'UNCLASSIFIED',
    9769 => 'UNCLASSIFIED',
    9770 => 'UNCLASSIFIED',
    9771 => 'UNCLASSIFIED',
    9772 => 'UNCLASSIFIED',
    9773 => 'UNCLASSIFIED',
    9774 => 'UNCLASSIFIED',
    9775 => 'UNCLASSIFIED',
    9776 => 'UNCLASSIFIED',
    9777 => 'UNCLASSIFIED',
    9778 => 'UNCLASSIFIED',
    9779 => 'UNCLASSIFIED',
    9780 => 'UNCLASSIFIED',
    9781 => 'UNCLASSIFIED',
    9782 => 'UNCLASSIFIED',
    9783 => 'UNCLASSIFIED',
    9784 => 'UNCLASSIFIED',
    9785 => 'UNCLASSIFIED',
    9786 => 'UNCLASSIFIED',
    9787 => 'UNCLASSIFIED',
    9788 => 'UNCLASSIFIED',
    9789 => 'UNCLASSIFIED',
    9790 => 'UNCLASSIFIED',
    9791 => 'UNCLASSIFIED',
    9792 => 'UNCLASSIFIED',
    9793 => 'UNCLASSIFIED',
    9794 => 'UNCLASSIFIED',
    9795 => 'UNCLASSIFIED',
    9796 => 'UNCLASSIFIED',
    9797 => 'UNCLASSIFIED',
    9798 => 'UNCLASSIFIED',
    9799 => 'UNCLASSIFIED',
    9800 => 'UNCLASSIFIED',
    9801 => 'UNCLASSIFIED',
    9802 => 'UNCLASSIFIED',
    9803 => 'UNCLASSIFIED',
    9804 => 'UNCLASSIFIED',
    9805 => 'UNCLASSIFIED',
    9806 => 'UNCLASSIFIED',
    9807 => 'UNCLASSIFIED',
    9808 => 'UNCLASSIFIED',
    9809 => 'UNCLASSIFIED',
    9810 => 'UNCLASSIFIED',
    9811 => 'UNCLASSIFIED',
    9812 => 'UNCLASSIFIED',
    9813 => 'UNCLASSIFIED',
    9814 => 'UNCLASSIFIED',
    9815 => 'UNCLASSIFIED',
    9816 => 'UNCLASSIFIED',
    9817 => 'UNCLASSIFIED',
    9818 => 'UNCLASSIFIED',
    9819 => 'UNCLASSIFIED',
    9820 => 'UNCLASSIFIED',
    9821 => 'UNCLASSIFIED',
    9822 => 'UNCLASSIFIED',
    9823 => 'UNCLASSIFIED',
    9824 => 'UNCLASSIFIED',
    9825 => 'UNCLASSIFIED',
    9826 => 'UNCLASSIFIED',
    9827 => 'UNCLASSIFIED',
    9828 => 'UNCLASSIFIED',
    9829 => 'UNCLASSIFIED',
    9830 => 'UNCLASSIFIED',
    9831 => 'UNCLASSIFIED',
    9832 => 'UNCLASSIFIED',
    9833 => 'UNCLASSIFIED',
    9834 => 'UNCLASSIFIED',
    9835 => 'UNCLASSIFIED',
    9836 => 'UNCLASSIFIED',
    9837 => 'UNCLASSIFIED',
    9838 => 'UNCLASSIFIED',
    9839 => 'UNCLASSIFIED',
    9840 => 'UNCLASSIFIED',
    9841 => 'UNCLASSIFIED',
    9842 => 'UNCLASSIFIED',
    9843 => 'UNCLASSIFIED',
    9844 => 'UNCLASSIFIED',
    9845 => 'UNCLASSIFIED',
    9846 => 'UNCLASSIFIED',
    9847 => 'UNCLASSIFIED',
    9848 => 'UNCLASSIFIED',
    9849 => 'UNCLASSIFIED',
    9850 => 'UNCLASSIFIED',
    9851 => 'UNCLASSIFIED',
    9852 => 'UNCLASSIFIED',
    9853 => 'UNCLASSIFIED',
    9854 => 'UNCLASSIFIED',
    9855 => 'UNCLASSIFIED',
    9856 => 'UNCLASSIFIED',
    9857 => 'UNCLASSIFIED',
    9858 => 'UNCLASSIFIED',
    9859 => 'UNCLASSIFIED',
    9860 => 'UNCLASSIFIED',
    9861 => 'UNCLASSIFIED',
    9862 => 'UNCLASSIFIED',
    9863 => 'UNCLASSIFIED',
    9864 => 'UNCLASSIFIED',
    9865 => 'UNCLASSIFIED',
    9866 => 'UNCLASSIFIED',
    9867 => 'UNCLASSIFIED',
    9868 => 'UNCLASSIFIED',
    9869 => 'UNCLASSIFIED',
    9870 => 'UNCLASSIFIED',
    9871 => 'UNCLASSIFIED',
    9872 => 'UNCLASSIFIED',
    9873 => 'UNCLASSIFIED',
    9874 => 'UNCLASSIFIED',
    9875 => 'UNCLASSIFIED',
    9876 => 'UNCLASSIFIED',
    9877 => 'UNCLASSIFIED',
    9878 => 'UNCLASSIFIED',
    9879 => 'UNCLASSIFIED',
    9880 => 'UNCLASSIFIED',
    9881 => 'UNCLASSIFIED',
    9882 => 'UNCLASSIFIED',
    9883 => 'UNCLASSIFIED',
    9884 => 'UNCLASSIFIED',
    9885 => 'UNCLASSIFIED',
    9886 => 'UNCLASSIFIED',
    9887 => 'UNCLASSIFIED',
    9888 => 'UNCLASSIFIED',
    9889 => 'UNCLASSIFIED',
    9890 => 'UNCLASSIFIED',
    9891 => 'UNCLASSIFIED',
    9892 => 'UNCLASSIFIED',
    9893 => 'UNCLASSIFIED',
    9894 => 'UNCLASSIFIED',
    9895 => 'UNCLASSIFIED',
    9896 => 'UNCLASSIFIED',
    9897 => 'UNCLASSIFIED',
    9898 => 'UNCLASSIFIED',
    9899 => 'UNCLASSIFIED',
    9900 => 'UNCLASSIFIED',
    9901 => 'UNCLASSIFIED',
    9902 => 'UNCLASSIFIED',
    9903 => 'UNCLASSIFIED',
    9904 => 'UNCLASSIFIED',
    9905 => 'UNCLASSIFIED',
    9906 => 'UNCLASSIFIED',
    9907 => 'UNCLASSIFIED',
    9908 => 'UNCLASSIFIED',
    9909 => 'UNCLASSIFIED',
    9910 => 'UNCLASSIFIED',
    9911 => 'UNCLASSIFIED',
    9912 => 'UNCLASSIFIED',
    9913 => 'UNCLASSIFIED',
    9914 => 'UNCLASSIFIED',
    9915 => 'UNCLASSIFIED',
    9916 => 'UNCLASSIFIED',
    9917 => 'UNCLASSIFIED',
    9918 => 'UNCLASSIFIED',
    9919 => 'UNCLASSIFIED',
    9920 => 'UNCLASSIFIED',
    9921 => 'UNCLASSIFIED',
    9922 => 'UNCLASSIFIED',
    9923 => 'UNCLASSIFIED',
    9924 => 'UNCLASSIFIED',
    9925 => 'UNCLASSIFIED',
    9926 => 'UNCLASSIFIED',
    9927 => 'UNCLASSIFIED',
    9928 => 'UNCLASSIFIED',
    9929 => 'UNCLASSIFIED',
    9930 => 'UNCLASSIFIED',
    9931 => 'UNCLASSIFIED',
    9932 => 'UNCLASSIFIED',
    9933 => 'UNCLASSIFIED',
    9934 => 'UNCLASSIFIED',
    9935 => 'UNCLASSIFIED',
    9936 => 'UNCLASSIFIED',
    9937 => 'UNCLASSIFIED',
    9938 => 'UNCLASSIFIED',
    9939 => 'UNCLASSIFIED',
    9940 => 'UNCLASSIFIED',
    9941 => 'UNCLASSIFIED',
    9942 => 'UNCLASSIFIED',
    9943 => 'UNCLASSIFIED',
    9944 => 'UNCLASSIFIED',
    9945 => 'UNCLASSIFIED',
    9946 => 'UNCLASSIFIED',
    9947 => 'UNCLASSIFIED',
    9948 => 'UNCLASSIFIED',
    9949 => 'UNCLASSIFIED',
    9950 => 'UNCLASSIFIED',
    9951 => 'UNCLASSIFIED',
    9952 => 'UNCLASSIFIED',
    9953 => 'UNCLASSIFIED',
    9954 => 'UNCLASSIFIED',
    9955 => 'UNCLASSIFIED',
    9956 => 'UNCLASSIFIED',
    9957 => 'UNCLASSIFIED',
    9958 => 'UNCLASSIFIED',
    9959 => 'UNCLASSIFIED',
    9960 => 'UNCLASSIFIED',
    9961 => 'UNCLASSIFIED',
    9962 => 'UNCLASSIFIED',
    9963 => 'UNCLASSIFIED',
    9964 => 'UNCLASSIFIED',
    9965 => 'UNCLASSIFIED',
    9966 => 'UNCLASSIFIED',
    9967 => 'UNCLASSIFIED',
    9968 => 'UNCLASSIFIED',
    9969 => 'UNCLASSIFIED',
    9970 => 'UNCLASSIFIED',
    9971 => 'UNCLASSIFIED',
    9972 => 'UNCLASSIFIED',
    9973 => 'UNCLASSIFIED',
    9974 => 'UNCLASSIFIED',
    9975 => 'UNCLASSIFIED',
    9976 => 'UNCLASSIFIED',
    9977 => 'UNCLASSIFIED',
    9978 => 'UNCLASSIFIED',
    9979 => 'UNCLASSIFIED',
    9980 => 'UNCLASSIFIED',
    9981 => 'UNCLASSIFIED',
    9982 => 'UNCLASSIFIED',
    9983 => 'UNCLASSIFIED',
    9984 => 'UNCLASSIFIED',
    9985 => 'UNCLASSIFIED',
    9986 => 'UNCLASSIFIED',
    9987 => 'UNCLASSIFIED',
    9988 => 'UNCLASSIFIED',
    9989 => 'UNCLASSIFIED',
    9990 => 'UNCLASSIFIED',
    9991 => 'UNCLASSIFIED',
    9992 => 'UNCLASSIFIED',
    9993 => 'UNCLASSIFIED',
    9994 => 'UNCLASSIFIED',
    9995 => 'UNCLASSIFIED',
    9996 => 'UNCLASSIFIED',
    9997 => 'UNCLASSIFIED',
    9998 => 'UNCLASSIFIED',
    9999 => 'UNIDENTIFIED'
);
#------------------------------------------------------------------------------
sub num2name {

    my($num) = @_;

    if ($num < 0) {
	$num = -$num;
    }
    return('UNCLASSIFIED') if ($num > 9999);
    return($name{$num});
    
} #eosub--num2name
#------------------------------------------------------------------------------
#########
#  EOF  #
#########
1;
